# @(#)48        1.27 src/43haes/lib/perl/cl_testtool/Event.pm, hacmp, 52haes_r540, r5400622c 4/24/06 17:51:49
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package Event;

use 5.006;
use strict;
use warnings;
use POSIX ":sys_wait_h";  # For WNOHANG flag to waitpid()

require Exporter;

use lib '/usr/es/lib/perl';
use lib '/usr/es/lib/perl/cl_testtool';

use MsgCatalog;
use StateData;
use StaticConfig;
use Logger;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Event ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '1.27';

=head1 NAME

Event --
Perl extension providing the parent object class for all test events.

=head1 SYNOPSIS

    use Event;

    my $event = Event->new( $logger, \@params );
    
    if ( $event->is_valid() eq 1 && $event->is_rational() eq 1 ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() unless $eventRC eq 0;
        $eventRC = $event->check_status() unless $eventRC eq 0;;
    }

=head1 DESCRIPTION

Event is the parent class of all Cluster Test Tool event-specific classes. It
contains the default definitions for all public methods that will be
over-ridden in the child classes, as well as providing a series of general
helper methods.

=head1 CONSTANTS

=head2  Continue -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::Continue".

=head3      Arguments:

    none.

=head3      Returns:

    Event::Continue

=cut

sub Continue()
{
    return 1;
}

=head2  Complete -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::Complete".

=head3      Arguments:

    none.

=head3      Returns:

    Event::Complete

=cut

sub Complete()
{
    return 0;
}

=head2  Fatal -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::Fatal".

=head3      Arguments:

    none.

=head3      Returns:

    Event::Fatal

=cut

sub Fatal()
{
    return 2;
}

=head2  Failed -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::Failed".

=head3      Arguments:

    none.

=head3      Returns:

    Event::Failed

=cut

sub Failed()
{
    return 3;
}

=head2  Timeout -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::Timeout".

=head3      Arguments:

    none.

=head3      Returns:

    Event::Timeout

=cut

sub Timeout()
{
    return 4;
}

=head2  NotRational -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::NotRational".

=head3      Arguments:

    none.

=head3      Returns:

    Event::NotRational

=cut

sub NotRational()
{
    return 5;
}


=head2  Waiting -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::Waiting".

=head3      Arguments:

    none.

=head3      Returns:

    Event::Waiting

=cut

sub Waiting()
{
    return 6;
}


=head2  Unstable -- function acts like a constant for package

=head3      Description:

Return contant value represented by symbol "Event::Unstable".

=head3      Arguments:

    none.

=head3      Returns:

    Event::Unstable

=cut

sub Unstable()
{
    return 7;
}


=head1 CLASS/SUBCLASS METHODS

The methods listed here represent the required interface for all sub-classes
of the Event class. Basic methods are provided by the Event class itself, but
most will do nothing. This approach is taken in order to ensure that the
subclasses will function without fatal errors, even if a required method has
not been specifically over-ridden. It is the expectation, however, that most
subclasses will over-ride all of the methods in this section. Exceptions to
this rule will be noted elsewhere.

=head2 new -- create a new instance of Event.

=head3     Description:

The new() method creates an hashtable, blesses it into the Event class, and
returns a reference to it. It accepts a reference to a parameters array, but
since this is simply a generic Event, it does not store any of the parameters.

This method must be over-ridden by all children of the Event class.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event object.

=head3     Examples:

    my $event = Event->new( $logger, \@parameters );

=cut

sub new ( )
{
    my $proto  = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{logger} = shift;
    validate($self);
    return bless $self, $class;
}


=head2 validate -- Validate the event parameters.

=head3     Description:

validate() will simply set the object's 'valid' data member, since the base
event class has no parameters to validate.

All children of this class should over-ride this method and call it before
returning from new().

=head3     Arguments:

    $event: the event to validate.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    $event->validate();
    
=cut

sub validate()
{
    my $self = shift;
    $self->{valid} = 1;
}


=head2 is_valid -- Check whether the event is valid.

=head3     Description:

is_valid() returns the value of the event's 'valid' data member. This method
does not need to be over-ridden by subclasses.

=head3     Arguments:

    none.

=head3     Returns:

    1  == event is valid.
    0 == event is not valid.

=head3     Examples:

    if ( $event->is_valid() ) {
        print "Event is valid.\n";
    }
    
=cut

sub is_valid()
{
    my $self = shift;
    return $self->{valid};
}


=head2 is_rational -- Check whether it is rational to run the event.

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
This version for the base class will always return 1. While it is not
rational to run an event that does nothing, there is no reason not to run
it either.

This method should be over-ridden by all subclasses and called explicitly from
the subclass's execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1 == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational()
{
    return 1;
}


=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of an event. For the
base class, there is no action to be taken, and so execute() will simply return
1 immediately.

This event will be over-ridden by virtually all events. The one exception is
the CLSTRMGR_KILL_COMPLETE event, which is not required to perform any actions
(since the required action will have been performed before the cluster manager
was killed).

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue == event was correctly executed

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    return Event::Continue;
}


=head2  wait_for_stable -- waits until the cluster is stable.

=head3     Description:

wait_for_stable() will loop, polling the cluster manager on all
cluster nodes to determine whether event execution is complete.

=head3     Algorithm:

    I. set startwait value to current time and wait 30 seconds
    
   II. collect current lssrc output from all nodes

        if the command returns fatal error from any node,
        output all error messages and return Event::Fatal.

  III. for each node; check the state of the cluster manager

        A. ST_STABLE

            check for event timers that indicate we should
            continue to wait. if none, this node is considered
            stable. otherwise, save current state as previous
            state and reset lastchange value. node state is
            considered "waiting".

        B. NONE (clstrmgrES is not active)

            node is considered stable.

        C. ST_RP_FAILED

            node is in error state and requires user intervention. 

        D. OTHER

            node is considered unstable

   IV. check whether the node is active

        A. is this the first check?

                yes: save the current state as previous state
                     and set lastchange value to current time

        B. has the state changed since the last check?

                yes: node has not hung. save the current state
                     as prev state and set lastchange.

        C. has the TE_NODE pointer changed since the last check?

                yes: node has not hung. save the current state
                     as prev state and reset lastchange.

    V. examine the node states

        A. all are stable

            return Event::Continue

        B. one or more nodes are waiting; rest are stable

            continue waiting (return to step II)

        C. one or more nodes have failed

            output error message and return Event::Fatal

        D. DEFAULT CASE: one or more nodes are unstable but none have failed

            1. have we waited longer than the maximum timeout value?

                yes: output error message and return Event::Timeout

            2. has it been longer than the base timeout value since
               the state of any of the nodes has changed?

                yes: output error message and return Event::Timeout

            3. it doesn't look like the cluster is hung. return to
               step II.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue == cluster reached the stable state.
    Event::Timeout  == cluster did not reach the stable state, and the 
                       timeout for this event was reached.
    Event::Fatal    == a fatal error occured testing for stability

=head3     Examples:

    if ( $event->wait_for_stable() eq Event::Timeout ) {
        print "Timeout waiting for cluster to stabilize.\n".
    }
    
=cut

sub wait_for_stable ( $ )
{
    my $self = shift;

    my $startwait = time;
    my $lastchange = $startwait;
    my $prevstate = "";
    my $retval = Event::Unstable;
    my $node_states = initialize_node_states( $self );

    $self->{logger}->phase_header("Entering wait_for_stable");

    LOOP: while ( $retval == Event::Unstable || $retval == Event::Waiting ) {

        sleep_for_polling_interval($self);

        # collect the current data set
        $self->{stable_data} = "";
        if ( ! collect_stable_data ( $self ) ) {
            return Event::Fatal;
        }

        # need current time to judge stabilization timeout
        my $looptime = time;

        # check node states (the node_states hash will be updated
        # during processing to reflect the "real" state value for
        # testing purposes)
        my $lssrc_states = $self->{stable_data}->get_node_states();
        $self->{logger}->verb_log_msg("\tChecking Node States:\n");
        foreach my $node ( sort( keys( %{ $lssrc_states} ))) {

            $self->{logger}->verb_log_msg("\t\tNode %s: %s\n",
                $node, $lssrc_states->{$node});

            SWITCH: for ( $lssrc_states->{$node} ) {
                /^ST_STABLE$/       &&
                do {    # the cluster manager is active _and_ stable
                    if( $self->{stable_data}->get_ibcast_count($node)         ||
                        $self->{stable_data}->get_rmibcast_count($node)       ||
                        scalar( $self->{stable_data}->get_event_queue($node)) ||
                        node_has_active_timers ( $self, $node ) ) {

                        $node_states->{$node} = Event::Waiting;

                    } else {
                        $node_states->{$node} = Event::Continue;
                    }
                    last SWITCH;
                };

                # If the node is in ST_INIT or NOT_CONFIGURED state,
                # then it went down or was never started.
                # Most likely OK, but each EVENT_NAME.pm class will
                # be responsible for making sure this is correct.
                /^(ST_INIT|NOT_CONFIGURED)$/    &&
                do {
                    $node_states->{$node} = Event::Continue;
                    last SWITCH;
                };

                /^$/                &&
                do {    # no value means the cluster manager is inactive

                    $node_states->{$node} = Event::Continue;
                    last SWITCH;
                };

                /^ST_RP_FAILED$/    &&
                do {    # the node requires user intervention
                    $node_states->{$node} = Event::Fatal;
                    $self->{logger}->err_msg( 1, 19,
                 "Event failure detected waiting for cluster stabilization.\n" .
                 "Event processing FAILED on node '%s'. " .
                 "The node requires user intervention.\n" .
                 "View the hacmp.out file on node '%s' to determine " .
                 "the cause of the failure.\n", $node, $node);
                    last SWITCH;
                };

                #DEFAULT CASE: cluster is considered unstable
                $node_states->{$node} = Event::Unstable;
                last SWITCH;
            }
            # no matter what the node's state, make sure we remember
            # whether there was a change since the last check
            if(node_is_active($self, $node, $prevstate, $self->{stable_data})) {
                    $lastchange = $looptime;
            }
        }

        $retval = calculate_wait_for_stable_return( 
                $self, $node_states, $startwait, $lastchange, $looptime );

        if ( $lastchange == $looptime ) {
            # there's been a change, so keep the current data
            $prevstate = $self->{stable_data};
        }
    }
    return $retval;
}

=head2 initialize_node_state -- initialize hash of node state

=head3      Description:

wait_for_stable uses a hashtable to store the current node states for use
in calculate_wait_for_stable_return. This method creates and initializes
the hashtable.

=head3      Arguments:

    none

=head3      Returns:

    reference to hash

=cut

sub initialize_node_states ( $ )
{
    my $self = shift;
    my $retval = {};

    foreach my $node ( @{ $self->{StaticConfig}->{node_names} } ) {
        $retval->{$node} = Event::Waiting;
    }
    return $retval;
}

=head2 sleep_for_polling_interval -- wait 30 seconds

=head3      Description:

sleep_for_polling_interval will sleep for 30 seconds, outputting appropriate
messaging to reassure the user that the test tool is not hung.

=head3      Arguments:

    none

=head3      Returns:

    none

=cut

sub sleep_for_polling_interval ( $ )
{
    my $self = shift;

    # wait a reasonable amount of time for the cluster state to change
    my $wait_interval = 30;
    $self->{logger}->info_msg( 1, 20, 
        "Waiting %ld seconds for cluster to stabilize.\n", $wait_interval);
    $| = 1; # autoflush
    while ( $wait_interval ) {
        print ". ";
        sleep 2;
        $wait_interval = $wait_interval - 2;
    }
    print "\n";
    $| = 0;
}

=head2 node_has_active_timers

=head3      Description:

Check an event's stable_data member to determine whether there are any
active MIT timers that impact the wait_for_stable calculation.

=head3      Arguments:

    $node: node to check

=head3      Returns:

    0   no timers
    1   active timers found

=cut

sub node_has_active_timers ( $$ )
{
    my $self = shift;
    my $node = shift;

    my @timers = $self->{stable_data}->get_active_timers( $node );
    if(scalar(@timers)) {
	$self->{logger}->verb_log_msg("\t\tActive Timers\n");
	foreach (@timers) {

	    $self->{logger}->verb_log_msg("\t\t\t$_\n");

	    # all of the following timer types are important
	    # wait_for_stable won't return (but won't timeout either)
	    # while we're waiting for these timers to expire
	    /^mark affected resource groups timer$/     && return 1;
	    /^stable callback timer$/                   && return 1;
	    /^monitor \w+:\w*:StabilizationTimer:\w*$/  && return 1;
	    # geo network is misspelled in the lssrc output
	    # match the misspelling here
	    /^check geo netowrk .* timer$/              && return 1;
        
	    # we purposefully ignore the app monitor warning timer,
	    # the app monitor restart timer, the fallback timer, and the
	    # settling timer. the last three of these could be arbitrarily
	    # long, and we don't want to hold up testing "forever". A
	    # future version of the test tool could take these timers into
	    # account when determining the success or failure of an event,
	    # but it will likely never make sense to wait indefinitely long
	    # for the timers to expire.
	}
    } else {
	$self->{logger}->verb_log_msg("\t\tActive Timers: None\n");
    }
    return 0;
}

=head2 node_is_active -- is the cluster processing events?

=head3      Description:

Check two sets of state data to determine whether a node is actively
processing events, or if it is potentially hung. If the node state has
not changed and the TE_NODE pointer address has not changed, then the
node is not considered active.

=head3      Arguments:

    $node:  nodename

    $prev:  previous state data

    $cur:   current state data

=head3      Returns:

    o   node is not active
    1   node is active

=cut

sub node_is_active ( $$$$ )
{
    my $self  = shift;
    my $node  = shift;
    my $prev  = shift;
    my $cur   = shift;

    if ( ! $prev ) {
        # if there was no previous check, assume the cluster is active
        return 1;
    }

    $self->{logger}->verb_log_msg("\t\tChecking if node $node is active\n");

    # did the node state change?
    my $pstate = $prev->get_node_state($node);
    my $cstate = $cur->get_node_state($node);
    $self->{logger}->verb_log_msg(
            "\t\t\tprevstate = %s, curstate = %s\n", $pstate, $cstate);
    if ( $pstate ne $cstate ) {
        return 1;
    }

    #has the event pointer changed?
    my $pevent = $prev->get_TE_NODE_pointer($node);
    my $cevent = $cur->get_TE_NODE_pointer($node);
    $self->{logger}->verb_log_msg(
            "\t\t\tprevtp = %s, curtp = %s\n", $pevent, $cevent);
    if ( $pevent ne $cevent ) {
        return 1;
    }

    return 0;
}

=head2 calculate_wait_for_stable_return

=head3      Description:

Based on current node states, how long we've been waiting, and how long it's
been since the last change in node state, calculate the return value for
wait_for_stable (or whether wait_for_stable should continue waiting).

=head3      Arguments:

    $node_state:    hashtable with node=state values

    $startwait:     time we started waiting

    $lastchange:    the last time a state change was observed

    $curtime:       the time now.

=head3      Returns:

    Event::Continue     cluster is stable, move on to next phase

    Event::Fatal        fatal error collecting stable data

    Event::Unstable     cluster is still active, keep waiting

    Event::Waiting      cluster is still, but at least one node shows an
                        indication that more events might occur

    Event::Timeout      stabilization timeout has occured

=cut

sub calculate_wait_for_stable_return ( $$$$$ )
{
    my $self        = shift;
    my $node_states = shift;
    my $startwait   = shift;
    my $lastchange  = shift;
    my $curtime     = shift;
    my $retval = Event::Continue;

    LOOP: foreach my $node ( sort( keys( %{ $node_states } ))) {

        if ( $node_states->{$node} == Event::Unstable ) {
            $retval = Event::Unstable;
        } elsif ( $node_states->{$node} == Event::Waiting ) {
            $retval = Event::Waiting unless $retval == Event::Unstable;
        } elsif ( $node_states->{$node} == Event::Fatal ) {
            $retval = Event::Fatal;
            last LOOP;
        }
    }

    if ( $retval == Event::Unstable ) {
        
        my $event_time = $curtime - $lastchange;
        my $total_time = $curtime - $startwait;

        if ( $event_time > $self->{StaticConfig}->{timeout} ) {
            $self->{logger}->err_msg( 1, 21,
            "Cluster state has not changed for %ld seconds.\n" .
            "This exceeds the test tool event timeout value of %ld seconds.\n" .
            "Increasing the cluster time until warning values will also\n" . 
            "increase the test tool event timeout value.\n",
            $event_time, $self->{StaticConfig}->{timeout});
            $retval = Event::Timeout;
        } elsif ( $total_time > $self->{StaticConfig}->{max_timeout} ) {
            $self->{logger}->err_msg( 1, 22,
            "Cluster has not stabilized in %ld seconds.\n" .
            "This exceeds the test tool stabilization timeout value of %ld.\n" .
            "This is most likely an indication that the cluster is in an event\n" .
            "loop and will not stabilize.\n" .
            "Increasing the cluster time until warning values will also\n" . 
            "increase the test tool event timeout value.\n",
            $total_time, $self->{StaticConfig}->{max_timeout});
            $retval = Event::Timeout;
        } else {
            $self->{logger}->info_msg( 1, 56, "Cluster is not yet stable.\n");
        }
    } elsif ( $retval == Event::Waiting ) {
        $self->{logger}->info_msg( 1, 56, "Cluster is not yet stable.\n");
    }

    return $retval;
}


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console. Since the base Event class
does nothing on its own, this method will simply return 1.

All subclasses of Event should over-ride this method.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue == event was judged to have passed.

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Event Status: FAILED\n";
    }
    
=cut

sub check_status()
{
    return Event::Continue;
}


=head2 generic_check_status -- checks the event's completion status.

=head3     Description:

generic_check_status() will run any checks that need to be run after any
event is run.

All subclasses of Event should over-ride this method.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue == event checks passed
    Event::Failed == event checks failed

=cut

sub generic_check_status($)
{
    my $self = shift;

    my $rc = Event::Continue;
    my $event_name = ref($self);
    $event_name = (split(/:/,$event_name))[2] || $event_name;

    # Do not run any checks if the event is WAIT.
    if ($event_name eq "WAIT") {
        return $rc;
    }

    # Check that each non-concurrent RG is not online on any more than 1 node.
    foreach my $resgrp (@{$self->{StaticConfig}->{resource_groups}}) {
        my $RGname = $resgrp->{group};
        my $count = Event::count_rg_instances($self, $RGname, "postevent");
        if($self->{StaticConfig}->rg_is_nonconcurrent($RGname) &&
            ($count > 1))
        {
            $self->{logger}->err_msg(2, 60,
                "%s: Bad Post Event Status\n" .
                "Nonconcurrent Resource Group %s is online on multiple nodes\n", 
                $event_name, $RGname);
            $rc = Event::Failed;
        }
    }
    
    return $rc;
}


=head2 to_string -- returns the event as the original Test string

=head3     Description:

to_string() will return the event and all of its parameters as a string
Since the base Event class is not a true test event, it returns an
empty string.

All subclasses of Event should over-ride this method.

=head3     Arguments:

    none

=head3     Returns:

    event as a string

=head3     Examples:

    for my $event (@event_list) {
        print EVENT_FILE, $event->to_string() . "\n";
    }
    
=cut

sub to_string() {
    return "";
}


=head1 HELPER METHODS

The methods listed here are common helper methods intended to be used by the
subclasses of the Event class. When called, these helper methods will not use
object-oriented notation, but instead will use the following notation, for
improved clarity:

    Event::method_name ( $eventReference );

This notation has the same net effect as using the object-oriented "->"
notation, but is easier to follow because it explicitly references the module
in which the actual function to be run resides.

=head2 execute_cmd -- use the event controller to execute a command.

=head3     Description:

execute_cmd() will use the event controller to execute a command
serially on one or more nodes. The command clrsh will be run locally in
order to execute cl_testtool_ctrl on the specified node(s). While it could
be possible to reuse C-SPOC functionality to control the serial execution
of commands, this function will allow the Test Tool to avoid the additional
overhead associated with creating the C-SPOC process and initializing the
C-SPOC environment.

=head3     Arguments:

    $command: the argument string to be provided to cl_testtool_ctrl.

    @nodelist: list of nodes on which the cmd should be run.

=head3     Returns:

    Event::Continue     == command to run was executed successfully on all 
                           required nodes.
    Event::NotRational  == command to run was not executed successfully on all
                           required nodes, but the error was not fatal.
    Event::Fatal        == command to run encountered a fatal error, which means
                           that the Test Tool should abort.

=head3     Examples:

    Event::execute_cmd( $command, @nodelist );

=head3     Notes:

May need to consider adding codd to guard against the external command
hanging. There aren't currently any commands called that should ever
hang. However, if such a problem ever occurs in the field, this would be
one place to fix it.
    
=cut

sub execute_cmd ( $$@ )
{
    my $nodes = shift;
    my $event = shift;
    my @args  = @_;
    my $retCode = Event::Continue;
    
    my @nodelist = split /,/, $nodes;
    my $event_name = ref($event);
    $event_name = (split(/:/,$event_name))[2] || $event_name;
    my $event_to_run;
    if (defined $event->{event_to_run}) {
        $event_to_run = $event->{event_to_run};
    }
    else {
        $event_to_run = $event_name;
    }
 
    $event->{logger}->phase_header("Executing Command for $event_name");

    my $arg_string = join("' '", @args);
    if ( $arg_string ) {
        $arg_string = "\'$arg_string\'";
    }

    my $ctrl_cmd = 
        "/usr/es/sbin/cluster/cl_testtool/cl_testtool_ctrl " .
        "-e $event_to_run -m execute $arg_string";

    my %pid_cmd_hash;
    foreach my $node ( @nodelist ) {
        my $cmd_to_exec = $ctrl_cmd;
        if ( $node ne $event->{StaticConfig}->{local_node} ) {
            $cmd_to_exec = 
                "/usr/es/sbin/cluster/utilities/clrsh -n $node $ctrl_cmd";
        }

        $event->{logger}->verb_log_msg("\t$cmd_to_exec\n");

        my $pid = fork();
        if (!defined $pid) {
            $event->{logger}->err_msg(1, 57,
                "Could not fork child process.\n");
            return Event::Fatal;
        }
        if ($pid == 0) {
            # Child process.
            exec($cmd_to_exec);
        }
        $pid_cmd_hash{$pid} = $cmd_to_exec;
    }
    
    my $MAX_CHILD_WAIT = 1800;  # 30 minutes
    my $count = 0;
    my $child_timeout = 0;
    my $irrational_count = 0;
    foreach my $pidcnt (keys %pid_cmd_hash) {
        my $found = 0;
        while ($count < $MAX_CHILD_WAIT) {
            my $retpid = waitpid(-1, &WNOHANG);
            # $? is set to the return code of the child
            my $systemRC = $?;
            if ($retpid > 0) {
                if ( $systemRC < 0 ) {
                    $event->{logger}->err_msg(1, 23,
                        "Internal error executing Perl system function.\n");
                    $retCode = Event::Fatal;
                }
                $systemRC = $systemRC / 256;
                if ( $systemRC == 1 ) {  # fatal error from cl_testtool_ctrl
                    $event->{logger}->err_msg(1, 24,
                        "Internal error running command: %s\n",
                        $pid_cmd_hash{$retpid});
                    $retCode = Event::Fatal;
                } elsif ( $systemRC == 2 ) { # non-fatal error
                    $irrational_count++;
                } elsif ( $systemRC ) {      # any other return code is fatal
                    $event->{logger}->err_msg(1, 24,
                        "Internal error running command: %s\n",
                        $pid_cmd_hash{$retpid});
                    $retCode = Event::Fatal;
                }
                $found = 1;
                last;
            }
            elsif ($retpid == 0) {
                sleep 1;
            }
            #elsif ($retpid == -1) {
            #    # No more children to wait for.
            #    last;
            #}
            $count++;
        }
        # If we got here than we waited for $MAX_CHILD_WAIT seconds and the
        # current child still hasn't returned.
        if (!$found) {
            $child_timeout = 1;
            last;
        }
    }

    # Only consider this event irrational if it was irrational on each
    # node in the node list.  Otherwise, if the event worked somewhere,
    # it's ok.  E.g., this might happen during NETWORK_UP event when a node
    # in the network already has all its interfaces on the specified
    # network up.
    #if ($retCode == Event::NotRational)
    if ($irrational_count == scalar(@nodelist)) {
        $event->{logger}->warn_msg(1, 25,
            "%s: Cannot be executed anywhere.\n" .
            "The event is not rational on all nodes.\n",
            $event_name);
        $retCode = Event::NotRational;
    }

    if ($child_timeout) {
        $event->{logger}->err_msg(1, 58,
            "Some commands failed to return after %s seconds.\n",
            $MAX_CHILD_WAIT);
        $retCode = Event::Fatal;
    }
    
    return $retCode;
}


=head2 choose_random_node -- select a random node from a nodelist.

=head3     Description:

choose_random_node() will select a node at random from a list of nodes.

=head3     Arguments:

    @nodelist: list of nodes to choose from.

=head3     Returns:

    undef    == failure.
    "string" == name of selected node.

=head3     Examples:

    Event::choose_random_node( @nodelist );
    
=cut

sub choose_random_node ( @ )
{
    my @nodes = @_;
    my $nodeix = rand $#nodes;
    return $nodes[$nodeix];
}

=head2 collect_preevent -- run prevent data collector

=head3      Description:

Run the pre-event data collector for a specified event.

=head3      Arguments:

    $event

=head3      Returns:

    1   success
    0   failure

=cut

sub collect_preevent ( $ )
{
    my $event = shift;
    my $event_name = ref($event);
    my $retval = 1;

    $event_name = (split(/:/, $event_name))[2] || $event_name;

    # get a new StateData object if there isn't one already
    if ( ! $event->{preevent} ) {
        my $data = StateData->new( $event->{logger}, $event->{StaticConfig} );
        $data->set_active_nodes();
        $event->{preevent} = $data;
    }

    foreach my $nodename (@{$event->{StaticConfig}->{node_names}}) {

        $retval = load_data($event, $event_name, $nodename, "preevent");
        last unless $retval;
    }
    return $retval;
}


=head2 collect_postevent -- run postevent data collector

=head3      Description:

run post-event data collector for the specified event.

=head3      Arguments:

    $event

=head3      Returns:

    1   success
    0   failure

=cut

sub collect_postevent ( $$ )
{
    my $event = shift;
    my $param = shift;
    my $retval = 1;
    my $event_name = ref($event);

    $event_name = (split(/:/, $event_name))[2] || $event_name;
    if ( $event_name eq "CLSTRMGR_KILL_COMPLETE" ) {
        $event_name = " CLSTRMGR_KILL";
    }
    my $tag = " $event_name:$param";


    # get a new StateData object if there isn't one already
    if ( ! $event->{postevent} ) {
        my $data = StateData->new( $event->{logger}, $event->{StaticConfig} );
        $data->set_active_nodes();
        $event->{postevent} = $data;
    }

    foreach my $nodename (@{$event->{StaticConfig}->{node_names}}) {
        $retval = load_data(
            $event, $event_name, $nodename, "postevent", $tag);
        last unless $retval;
    }
    return $retval;
}


=head2 collect_stable_data -- run stable_data collector on all nodes

=head3      Description:

Collect data for cluster stability check.

=head3      Arguments:

    $event  = event that is being run

=head3      Returns:

    1   success
    0   failure

=cut

sub collect_stable_data ( $ )
{
    my $event = shift;
    my $retval = 1;

    # get a new StateData object if there isn't one already
    if ( ! $event->{stable_data} ) {
        my $data = StateData->new( $event->{logger}, $event->{StaticConfig} );
        $data->set_active_nodes();
        $event->{stable_data} = $data;
    }

    # This used to loop over the active nodes returned above, but the active
    # nodes are returned by running clgetactivenodes on each node until a list
    # is returned.  The problem is that just because isn't included in the
    # active node list doesn't mean it isn't in the ST_JOINING state.
    # So loop over all nodes instead.
    foreach my $nodename (@{$event->{StaticConfig}->{node_names}}) {
        $retval = load_data($event, "collect", $nodename, "stable_data");
        last unless $retval;
    }
    return $retval;
}


=head2 load_data -- load data from a collector into event object

=head3      Description:

Run a data collector and load it's data into a StateData object.

=head3      Arguments:

    $event          event that's currently running
    $event_name     name to use for the event (may be different than real
                    name of $event object class)
    $nodename       node to collect data from
    $mode           the type of collector being run

=head3      Returns:

    1   success
    0   failure

=head3     Notes:

May need to consider adding codd to guard against the external command
hanging. There aren't currently any commands called that should ever
hang. However, if such a problem ever occurs in the field, this would be
one place to fix it.

=cut

sub load_data ( $$$$;$ )
{
    my $event = shift;
    my $event_name = shift;
    my $nodename = shift;
    my $mode = shift;
    my $param = shift || "";

    # build the command to run
    my $command = 
            "/usr/es/sbin/cluster/utilities/clrsh -n $nodename" . 
            " /usr/es/sbin/cluster/cl_testtool/cl_testtool_ctrl " .
            "-e $event_name -m $mode$param";

    # open a pipe from the command
    my $childPID = open ( DATA, "-|", "$command" );
    if ( ! $childPID ) {
        $event->{logger}->err_msg( 1, 26,
            "Internal error opening pipe for input from command.\n%s\n",
            $command);
        return 0;
    } 
    
    my $data =  $event->{$mode};

    # now load the data
    my $dataRC = $data->process_data( \*DATA );
    if ( ! $dataRC ) {
        
        # if process_data doesn't return true, it's a fatal error
        # in that case, not all of the data was read from the pipe,
        # so closing the file now might cause SIG_PIPE. we'll just
        # leave it open, since the program will be exiting fairly
        # soon, and we won'y get SIG_PIPE if the file is closed
        # by Perl's automatic clean-up activity
        return 0;
        
    } else {
        # inside the else, process_data was successful, so we'll 
        # explicitly close the file
        close(DATA);
    }

    return 1;
}

=head2 get_node_statename -- replace empty statename with INOPERATIVE

=head3      Description:

Provides consistent replacement of empty-string statename with the string
INOPERATIVE. All other cluster states are left as they are.

=head3      Arguments:

    $state_name     name of state.

=head3      Returns:

    $state_name or INOPERATIVE

=cut

sub get_node_statename ( $ )
{
    my $statename = shift;
    return (($statename) ? $statename : "INOPERATIVE");
}


sub handle_invalid_interface_label($$$) {
    my $self = shift;
    my $label = shift || return;
    my $errcode = shift || return;

    if ($errcode == 0) {
        $self->{valid} = 0;
        $self->{logger}->err_msg(2, 36,
            "%s: Invalid Test\n".
            "Interface label %s not in cluster configuration\n",
            $self->{event_name}, $label);
    }
    elsif ($errcode == -1) {
        $self->{valid} = 0;
        $self->{logger}->err_msg(2, 37,
            "%s: Invalid Test\nInterface label %s is a persistent label.\n".
            "Persistent labels aren't allowed for test %s.\n",
            $self->{event_name}, $label, $self->{event_name});
    }
}


sub handle_invalid_network($$$)
{
    my $self = shift;
    my $network = shift || return;
    my $errcode = shift || return;

    if ($errcode == -1) {
        $self->{logger}->err_msg(2, 28,
            "%s: Invalid Test\n".
            "Geo, atm, and hps networks are not supported.\n",
            $self->{event_name});
    }
    elsif ($errcode == -2) {
        $self->{logger}->err_msg(2, 29,
            "%s: Invalid Test\n".
            "Network %s not in cluster configuration\n",
            $self->{event_name}, $network);
    }
    elsif ($errcode == -3) {
        $self->{logger}->err_msg(2, 30,
            "%s: Invalid Test\n".
            "Cannot find two IP networks in cluster configuration\n" .
            "The %s test requires at least two IP networks.\n",
            $self->{event_name}, $self->{event_name});
    } 
}
 

=head2 is_node_active -- checks if node is online 

=head3      Description:

Checks the preevent state data if a provided node is online

=head3      Arguments:

    $nodename     the name of the node

=head3      Returns:

    1 if the node is online, 0 otherwise

=cut

sub is_node_active($$)
{
    my $self = shift || return 0;
    my $nodename = shift || return 0;

    my $node_state = $self->{preevent}->get_node_state($nodename);
    if ($node_state eq "" || $node_state eq "NOT_CONFIGURED" ||
        $node_state eq "ST_INIT")
    {
        return 0;
    }

    return 1;
}


sub get_node_hosting_interface_label($$)
{
    my $self = shift || return "";
    my $label = shift || return "";
    my $nodename = "";

    foreach my $iface (@{$self->{StaticConfig}->{interfaces}}) {
        if ($iface->{ip_label} eq $label) {
            if ($iface->{function} ne "shared" &&
                $iface->{function} ne "service")
            {
                $nodename = $iface->{nodename};
            }
            else {
                $nodename = find_node_hosting_service_label($self, $iface->{identifier});
            }
            last;  # break out of foreach iface
        }
    }

    return $nodename;
}


sub find_node_hosting_service_label($$)
{
    my $self = shift || return "";
    my $ipaddr = shift || return "";

    foreach my $node (@{$self->{StaticConfig}->{node_names}}) {
        my $cmd = "/usr/es/sbin/cluster/utilities/clrsh -n $node /usr/es/sbin/cluster/utilities/cl_netstat -in";
        my $output = qx($cmd);
        if ($output =~ /\s+$ipaddr\s+/) {
            return $node;
        }
    }

    # If we got here, we didn't find the service label hosted anywhere.
    return "";
}


sub is_network_up_on_node($$$)
{
    my $self = shift || return 0;
    my $network = shift || return 0;
    my $node = shift || return 0;

    my $output = qx(/usr/es/sbin/cluster/utilities/cllscompstat -c);
    my @compstates = split('\n', $output);
    foreach my $line (@compstates) {
        if ($line eq "NETWORK:$network:UP:$node") {
            return 1;
        }
    }

    return 0;
}


sub is_network_up_anywhere($$)
{
    my $self = shift || return 0;
    my $network = shift || return 0;

    foreach my $node (@{$self->{StaticConfig}->{node_names}}) {
        my $up = is_network_up_on_node($self, $network, $node);
        if ($up) {
            $self->{logger}->verb_log_msg("is_network_up_anywhere: Network $network is UP on node $node\n");
            return 1;
        }
    }

    return 0;
}


sub is_network_down_anywhere($$)
{
    my $self = shift || return 0;
    my $network = shift || return 0;

    foreach my $node (@{$self->{StaticConfig}->{node_names}}) {
        my $up = is_network_up_on_node($self, $network, $node);
        if (!$up) {
            $self->{logger}->verb_log_msg("is_network_down_anywhere: Network $network is DOWN on node $node\n");
            return 1;
        }
    }

    return 0;
}


sub is_interface_label_up($$)
{
    my $self = shift || return 0;
    my $label = shift || return 0;

    my $output = qx(/usr/es/sbin/cluster/utilities/cllscompstat -c);
    my @compstates = split('\n', $output);
    foreach my $line (@compstates) {
        if ($line =~ /LABEL:$label:UP:(.*)/) {
            my $node = $1;
            $self->{logger}->verb_log_msg("is_interface_label_up: Label $label is UP on node $node\n");
            return 1;
        }
    }

    return 0;
}
 

sub have_nodes_changed_state($)
{
    my $self = shift || return 1;
    my $changes = 0;
#
# no node has changed state
#
    $self->{logger}->verb_log_msg("Checking node states\n");
    my $pre_node_states = $self->{preevent}->get_node_states();
    my $post_node_states = $self->{postevent}->get_node_states();

    foreach my $node ( sort( keys( %{$pre_node_states} ))) {

        $self->{logger}->verb_log_msg(
            "$node: Preevent state: $pre_node_states->{$node}, " .
            "Postevent state: $post_node_states->{$node}\n");

        if ($pre_node_states->{$node} ne $post_node_states->{$node}) {

            $self->{logger}->err_msg(2, 20,
                "%s: Unexpected state change: node %s\n" .
                "Pre-event state %s, Post-event state %s\n",
                $self->{event_name}, $node, 
                Event::get_node_statename($pre_node_states->{$node}),
                Event::get_node_statename($post_node_states->{$node}));
            $changes = 1;
        }
    }

    return $changes;
}


sub have_any_rgs_changed_state($)
{
    my $self = shift;

    $self->{logger}->verb_log_msg("Checking RG states\n");
    RGRP: foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} )
    {
        $self->{logger}->verb_log_msg("\tResource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});

        foreach my $node ( sort( keys( %{$post_states} ))) {
            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"),
                $post_states->{$node});

            if ($pre_states->{$node} ne $post_states->{$node}) {
                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s\n",
                    $self->{event_name}, $resgrp->{group},
                    $post_states->{$node}, $node);
                return 1;
            }
        }
    }

    return 0;
}


sub sleep_for_failure_detection_interval($$)
{
    my $self = shift;
    my $network = shift;

    # look up the nim object for the network
    my $nim = "";
    foreach my $net ( @{$self->{StaticConfig}->{networks}} ) {
        if ( $network eq $net->{name} ) {
            $nim = $net->{nimname};
        }
    }   
    my $odm = ODM->new("HACMPnim", ODMDIR=>ODM::DCD());
    my ($nim_obj) = $odm->get(-q => "name = $nim");
        
    # now calculate the failure detection rate
    if ($nim_obj) {
        my $hb_rate = ($nim_obj->{hbrate} / 1000000);
        # round up for sub-second hb rate
        if ($nim_obj->{hbrate} % 1000000) {
            $hb_rate += 1;
        }
        my $fail_rate = ($hb_rate * $nim_obj->{cycle} * 2);
        if ( $fail_rate > 0 ) {
            # needs new message in catalog for next release
            $self->{logger}->info_msg(2, 39,
            "%s: Waiting for failure detection interval %ld before checking stability.\n",
            $self->{event_name}, "$fail_rate");
            sleep $fail_rate;
        }
    }   
}


=head2 count_rg_instances

=head3      Description:

Counts the number of nodes where an RG is online

=head3      Arguments:

    $RGname     the name of the RG
    $prepost    "preevent" or "postevent"

=head3      Returns:

    number of nodes where RG is ONLINE.

=cut

sub count_rg_instances($$$)
{
    my $self = shift;
    my $RGname = shift;
    my $prepost = shift;

    my $count=0;

    my $post_rg_states = $self->{$prepost}->get_rg_states($RGname);

    foreach my $node (@{$self->{StaticConfig}->{node_names}}) {
        if (!defined $post_rg_states->{$node}) {
            next;
        }
	if ($post_rg_states->{$node} eq "ONLINE") {
	    $count++;
	}
    }
    return $count;
}


=head2 generic_node -- check if node is a keyword

=head3      Description:

Checks if node is one of the generic keywords supported for most events

=head3      Arguments:

    none (uses nodename in $self)

=head3      Returns:

    1 if nodename is ANY, ALL, or RESTORE
    
=cut

sub generic_node($) {
    my $self = shift || return 0;
    if ($self->{nodename} eq "ANY" || $self->{nodename} eq "ALL" || $self->{nodename} eq "RESTORE") {
	return 1;
    }
    return 0;
}


=head2 rg_is_online -- check if an RG is online

=head3      Description:

Check either the preevent or postevent structures for if the RG is in ONLINE state

=head3      Arguments:

    $prepost     "preevent" for data before the event, "postevent" for after

=head3      Returns:

    1 if RG is online somewhere, 0 otherwise.
    
=cut

sub rg_is_online($$) {
    my $self = shift || return 0;
    my $prepost = shift || return 0;

    my $RGname = $self->{RGname};
    my $states = $self->{$prepost}->get_rg_states($self->{RGname});
    foreach my $node (@{$self->{$prepost}->{active_nodes}}) {
	if( $states->{$node} eq "ONLINE" ) {
	    return 1;
	}
    }
    return 0;
}


=head2 get_random_online_node -- return a random node where RG is ONLINE

=head3      Description:

return a random node where RG is ONLINE

=head3      Arguments:

    none     (uses RGname defined in $self)

=head3      Returns:

    a nodename where the node is online, or an empty list if none available
    
=cut

sub get_random_online_node($) {
    my $self = shift || return 0;
    my $RGname=$self->{RGname};

    my @nodes = $self->{preevent}->get_nodes_for_rg($RGname,"ONLINE");
    return (choose_random_node(@nodes));
}
    

=head2 have_other_rgs_changed_state

=head3      Description:

Check all RGs except the event RG for state changes

=head3      Arguments:

    none     (uses RGname defined in $self)

=head3      Returns:

    1 if another RG has changed state, 0 otherwise.
    
=cut

sub have_other_rgs_changed_state($) {
    my $self = shift || return 1;
    $self->{logger}->verb_log_msg("Checking RG states\n");
    my $retval=0;

    foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {
	
	if ($self->{RGname} ne $resgrp->{group}) {
	    $self->{logger}->verb_log_msg("Resource Group: $resgrp->{group}\n");
	    
	    my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
	    my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});

	    foreach my $node ($self->{StaticConfig}->get_nodes_for_rg($resgrp->{group})) {
		if ($pre_states->{$node} ne $post_states->{$node}) {
		    $self->{logger}->err_msg(2, 25,
					     "%s: Unexpected state change\n".
					     "Resource Group %s changed to %s state on node %s.\n",
					     $self->{event_name}, $resgrp, 
					     $post_states->{$node},$node);
		    $retval=1;
		}
	    }
	}
    }
    return $retval;
}


=head2 get_allowed_nodes_for_rg -- return nodes where RG can be hosted

=head3      Description:

Uses data from a clsetenvgrp node query to determine a list of nodes
that can host an RG

=head3      Arguments:

    none     (uses RGname defined in $self)

=head3      Returns:

    a list of allowed nodes, an empty list if none
    
=cut

sub get_allowed_nodes_for_rg($$) {
  my $self = shift || return ();
  my $rg = shift || return ();

  my @setenvnodes = ();
  foreach my $node (keys %{$self->{preevent}->{candidates}}) {
    foreach my $rgname (keys %{$self->{preevent}->{candidates}->{$node}}) {
      if ($rg eq $rgname) {
	if ($self->{preevent}->{candidates}->{$node}->{$rg}->{nodelist} ne "") {
	  @setenvnodes = split " ",$self->{preevent}->{candidates}->{$node}->{$rg}->{nodelist};
	  return @setenvnodes;
	}
      }
    }
  }
  return @setenvnodes;
}


=head2 get_allowed_sites_for_rg -- return sitees where RG can be hosted

=head3      Description:

Uses data from a clsetenvgrp site query to determine a list of sites
that can host an RG

=head3      Arguments:

    none     (uses RGname defined in $self)

=head3      Returns:

    a list of allowed sites, an empty list if none
    
=cut

sub get_allowed_sites_for_rg($$) {
  my $self = shift || return ();
  my $rg = shift || return ();

  my @setenvsites = ();
  foreach my $node (keys %{$self->{preevent}->{candidates}}) {
    foreach my $rgname (keys %{$self->{preevent}->{candidates}->{$node}}) {
      if ($rg eq $rgname) {
	if ($self->{preevent}->{candidates}->{$node}->{$rg}->{sitelist} ne "") {
	  @setenvsites = split " ",$self->{preevent}->{candidates}->{$node}->{$rg}->{sitelist};
	  return @setenvsites;
	}
      }
    }
  }
  return @setenvsites;
}


=head2 get_site_for_rg -- return a site the RG is online

=head3      Description:

=head3      Arguments:

    $prepost     "preevent" for data before the event, "postevent" for after

=head3      Returns:

    a sitename where the RG is online, null string otherwise
    
=cut

sub get_site_for_rg($$) {
    my $self = shift || return 0;
    my $prepost = shift || return 0;
    my $RGname = $self->{RGname};
    my $states = $self->{$prepost}->get_rg_states($self->{RGname});
    foreach my $site (@{$self->{StaticConfig}->{sites}}) {
      my $site_nodes = $self->{StaticConfig}->get_site_nodes($site->{name});
      foreach my $node (@{$site_nodes}) {
	if( $states->{$node} eq "ONLINE" ) {
	  return $site->{name};
	}
      }
    }
    return "";
}


=head2 get_nodes_where_rg_is_online -- return a list of nodes RG is online

=head3      Description:

    loops over all nodes in the cluster and saves the nodes where the RG
    is online

=head3      Arguments:

    $prepost     "preevent" for data before the event, "postevent" for after

=head3      Returns:

    a reference to a list of nodes where the RG is online, a null list if none
    
=cut

sub get_nodes_where_rg_is_online($$) {
    my $self = shift || return ();
    my $prepost = shift || return ();
    my $RGname = $self->{RGname};
    my $states = $self->{$prepost}->get_rg_states($self->{RGname});
    my @nodes=();
    foreach my $node (@{$self->{$prepost}->{active_nodes}}) {
      if( $states->{$node} eq "ONLINE" ) {
	push @nodes, $node;
      }
    }
    return \@nodes;
}


=head2 node_is_allowed_for_rg -- check if a node is allowed to host RG

=head3      Description:

    uses get_allowed_nodes_for_rg and sees if the specified node is in the
    list of nodes allowed as returned by a clsetenvgrp candidate node query

=head3      Arguments:

    $node     a node to verify is in the allowed node list

=head3      Returns:

    1 if the node is allowed, 0 otherwise
    
=cut

sub node_is_allowed_for_rg {
    my $self = shift || return 0;
    my $node = shift || return 0;
    my $rg = shift || return 0;
    
    my @allowed_nodes = Event::get_allowed_nodes_for_rg($self,$rg);
    foreach my $item (@allowed_nodes) {
	if ($item eq $node) {
	    return 1;
	}
    }
    return 0;
}


sub is_node_forced_down {
    my $self = shift || return 0;
    my $node = shift || return 0;
    my $prepost = shift || return 0;

    my @active_nodes = @{$self->{$prepost}->{active_nodes}};

    foreach my $active_node (@active_nodes) {
        my @forced_nodes = $self->{$prepost}->get_forcedown_nodes($active_node);
        $self->{logger}->verb_log_msg("\tNode: $active_node, Force Down: @forced_nodes\n");
        foreach my $forced (@forced_nodes) {
            if ($forced eq $node) {
                return 1;
            }
        }
    }
    
    return 0;
}


=head2 get_EVENT_FAILED_msg

=head3      Description:

    returns the translated message for EVENT FAILED.  This needs to be used
    in hacmp.out searches since this line may be translated.
    Note- we don't translate the EVENT COMPLETED message anymore, so this routine
    will return the bare message.

=head3      Arguments:

    none

=head3      Returns:

    the translated message
    
=cut

sub get_EVENT_FAILED_msg {
    my $self = shift || return 0;
    my $EVENT_FAILED_MSG ="EVENT FAILED";
    return $EVENT_FAILED_MSG;
}


=head2 get_EVENT_COMPLETED_msg

=head3      Description:

    returns the translated message for EVENT COMPLETED.  This needs to be used
    in hacmp.out searches since this line may be translated.
    Note- we don't translate the EVENT COMPLETED message anymore, so this routine
    will return the bare message.

=head3      Arguments:

    none

=head3      Returns:

    the translated message
    
=cut

sub get_EVENT_COMPLETED_msg {
    my $self = shift || return 0;
    my $EVENT_COMPLETED_MSG = "EVENT COMPLETED";
    return $EVENT_COMPLETED_MSG;
}


=head2 rgmove_event_logged

=head3      Description:

    used by the check_status method in the RG* events to see if an rg_move
    event was successfully logged somewhere in the cluster.

=head3      Arguments:

    none

=head3      Returns:

    1 if the rg move event was logged, 0 otherwise
    
=cut

sub rgmove_event_logged {
    my $self = shift || return 0;
    my $retval = 1;
    
    my $EVENT_COMPLETED_MSG = get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = get_EVENT_FAILED_msg($self);
    
    $self->{logger}->verb_log_msg("Checking event history\n");
    my %event_complete_found;
    foreach my $comp_node (@{$self->{nodelist}}) {
	$event_complete_found{$comp_node} = 0;
    }
    my $event_history = $self->{postevent}->get_event_history("rg_move", "rg_move_complete", "external_resource_state_change", "external_resource_state_change_complete");
    
    # are there any interesting event failure messages?
    foreach my $node ( sort( keys ( %{$event_history} ))) {
    
	my @FAILED_records = 
	    grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        foreach my $record ( @FAILED_records ) {
      
	$self->{logger}->err_msg(2, 22,
				     "%s: EVENT FAILED message logged on node %s\n%s\n",
				     $self->{event_name}, $node, $record);
	    $retval = 0;
	}
    }
  
    # have we found and EVENT COMPLETED message for each event node?
  COMPCHECK: foreach my $check_node (@{$self->{nodelist}}) 
  {
    
      my $search_str = "(rg_move_complete|external_resource_state_change_complete) ";
      foreach my $node ( sort( keys ( %{$event_history} ))) {
	  
	  my @COMPLETED_records = 
	      grep {/^.* ${EVENT_COMPLETED_MSG}: $search_str/} 
	  @{$event_history->{$node}};
	  if ( scalar(@COMPLETED_records) ) {
	      $event_complete_found{$check_node} = 1;
	      next COMPCHECK;
	  }
      }
  }
    
    foreach my $msgnode (@{$self->{nodelist}}) {
	if ( ! $event_complete_found{$msgnode} ) {
	    my $search_str = "rg_move_complete or external_resource_state_change_complete";
	    $self->{logger}->err_msg(2, 26,
				     "%s: No EVENT COMPLETED message logged for node %s.\n" .
				     "Search string: %s\n",
				     $self->{event_name}, $msgnode, "${EVENT_COMPLETED_MSG}: $search_str");
	    $retval = 0;
	}
    }
    return $retval;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__
