# @(#)75        1.6 src/43haes/lib/perl/cl_testtool/Event/FAIL_LABEL.pm, hacmp.test_tool, 52haes_r540, r5400622c 3/29/06 10:44:35
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
                                                                                          
package Event::FAIL_LABEL;

use 5.006;
use strict;
use warnings;
use Event;                                                                                          
require Exporter;
                                                                                          
our @ISA = qw(Event Exporter);
                                                                                          
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
                                                                                          
# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
                                                                                          
) ] );
                                                                                          
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
                                                                                          
our @EXPORT = qw(
                                                                                          
);
our $VERSION = '1.6';
                                                                                          
##
## Put all the methods and global data for your module here
##

=head1 NAME

FAIL_LABEL -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::FAIL_LABEL;

    my $event = Event::FAIL_LABEL->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of FAIL_LABEL

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct FAIL_LABEL class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::FAIL_LABEL object.

=head3     Examples:

    my $event = Event::FAIL_LABEL->new( $logger, $staticconfig, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{interface_label} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{valid} = 1;
    $self->{event_name} = "FAIL_LABEL";
    validate( $self );
    return bless $self, $class;
}


=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to FAIL_LABEL object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->is_valid() ) {
        print "Event is valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;

    $self->{logger}->phase_header("Validate " . $self->{event_name});

    my $rc = $self->{StaticConfig}->is_valid_interface_label($self->{interface_label});
    if ($rc != 1) {
        Event::handle_invalid_interface_label($self, $self->{interface_label},
                                              $rc);
        $self->{valid} = 0;
    }

    my $network = $self->{StaticConfig}->get_network_of_interface_label($self->{interface_label});
    $self->{network} = $network;
    $rc = $self->{StaticConfig}->is_network_valid($network);
    if ($rc != 1) {
        Event::handle_invalid_network($self, $self->{network}, $rc);
        $self->{valid} = 0;
    }
} 

    
=head2 is_rational -- Check whether it is rational to run the event .

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
The entrance criteria for each event will be quite specific to that event, and
so each type of FAIL_LABEL class will most likely over-ride the is_rational()
method of the Event class. 

is_rational() will typically only be called from within the execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1 = event is rational

    0 = event is not rational

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational() {
    my $self = shift;

    $self->{logger}->phase_header("is_rational $self->{event_name}");

    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", $self->{event_name});
        return 0;
    }

    my $nodename = Event::get_node_hosting_interface_label($self, $self->{interface_label});
    if ($nodename eq "") {
        $self->{logger}->warn_msg(2, 38,
            "%s cannot be run: Not Rational\n".
            "Specified interface label %s is not hosted on any active node\n".
            "in the cluster.\n",
            $self->{event_name}, $self->{interface_label});
        return 0;
    }

    $self->{nodename} = $nodename;

    if ( ! Event::is_node_active($self, $self->{nodename})) {
        $self->{logger}->warn_msg(2, 11,
            "%s cannot be run: Not Rational\n" .
            "Specified node is not active in the cluster: %s\n",
            $self->{event_name}, $self->{nodename});
        return 0;
    }

    if ( ! Event::is_interface_label_up($self, $self->{interface_label})) {
        $self->{logger}->warn_msg(2, 76,
            "%s cannot be run: Not Rational\n" .
            "Interface label %s is not up in the cluster.\n",
            $self->{event_name}, $self->{interface_label});
        return 0;
    }
    
    return 1;
}

=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of FAIL_LABEL class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }
    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }
    return Event::execute_cmd (
        $self->{nodename}, $self, $self->{interface_label});
}


=head2  wait_for_stable -- waits until the cluster is stable.

=head3     Description:

Event::FAIL_LABEL::wait_for_stable() will determine the failure
detection rate for the NIM type of the network being stopped, and wait
that long before handing control off to Event::wait_for_stable.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue == cluster reached the stable state.
    Event::Timeout  == cluster did not reach the stable state, and the
                       timeout for this event was reached.
    Event::Fatal    == a fatal error occured testing for stability

=head3     Examples:

    if ( $event->wait_for_stable() eq Event::Timeout ) {
        print "Timeout waiting for cluster to stabilize.\n".
    }   
   
=cut

sub wait_for_stable ( )
{
    my $self = shift;
        
    Event::sleep_for_failure_detection_interval($self, $self->{network});
       
    return Event::wait_for_stable($self);
}       


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status()
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;

    $self->{logger}->phase_header(
            "$self->{event_name}: Checking post-event status"); 

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

    my @post_active_nodes = @{$self->{postevent}->{active_nodes}};
    my @pre_active_nodes = @{$self->{preevent}->{active_nodes}};

#
# no node has changed state
#
    if (Event::have_nodes_changed_state($self)) {
        $retval = Event::Failed;
    }

# Checking RG state changes:
# Check if any RGs went into ERROR.
# Check if any concurrent RGs, or non-concurrent RGs that were not ONLINE
# on the event node, changed state.
# Check if the interface label is a service label that belongs to an RG.
# If so, then check that the RG moved only if the resource's failure action
# is to fallover.
    $self->{logger}->verb_log_msg("Checking RG states\n");
    RGRP: foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {
        $self->{logger}->verb_log_msg("\tResource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});
        my $rgtype = $self->{StaticConfig}->get_rg_type($resgrp->{group});

        # have nodes moved into the ERROR state?
        foreach my $node ( sort( keys( %{$post_states} ))) {
            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"), 
                $post_states->{$node});

            if ( $post_states->{$node} eq "ERROR" &&
                 $pre_states->{$node} ne "ERROR" )
            {
                # output error message
                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s\n",
                    $self->{event_name}, $resgrp->{group}, "ERROR", "$node");
                $retval = Event::Failed;
            }
        }

        # don't bother looking at nonconcurrent RGS that were ONLINE on
        # the event node. they will be checked below.
        if ( $rgtype == StaticConfig::nonconcurrent &&
             $pre_states->{$self->{nodename}} eq "ONLINE" )
        {
            next RGRP;
        }

        # have RGs unexpectedly changed state?
        foreach my $node ( sort( keys( %{$pre_states} ))) {
            if ( $pre_states->{$node} eq "ONLINE"  &&
                 $post_states->{$node} ne "ONLINE" &&
                 $post_states->{$node} ne "ERROR"  )
            {
                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s\n",
                    $self->{event_name}, $resgrp->{group}, 
                    $post_states->{$node}, $node);
                $retval = Event::Failed;
            }
        }
    }

# check RG state/node changes for event node to make sure they're correct
#
    $self->{logger}->verb_log_msg("Checking RG state changes on the local node\n");

    my $monitor_odm = ODM->new("HACMPmonitor");
    my @pre_rgs_online_eventnode = 
            $self->{preevent}->get_rgs_for_node( $self->{nodename}, "ONLINE");
    RG: foreach my $rgpre_online ( @pre_rgs_online_eventnode ) {
        my $rgtype = $self->{StaticConfig}->get_rg_type($rgpre_online);
            
        $self->{logger}->verb_log_msg(
            "\tResource Group: $rgpre_online Type: $rgtype\n");

        if ( $rgtype == StaticConfig::concurrent ) {
            # these were checked already
            next RG;
        }

        my @post_rg_online_nodes =
            $self->{postevent}->get_nodes_for_rg( $rgpre_online, "ONLINE");
        my @svc_labels_for_rg = 
            $self->{StaticConfig}->get_resources_for_rg( 
                $rgpre_online, "SERVICE_LABEL" );

        # Check to see if the specified label is a service label that belongs
        # to this RG.
        $self->{logger}->verb_log_msg("\tChecking service label recovery action\n");
        my $is_svc_in_rg = 0;
        my $recovery_type = "notify";
        foreach my $svc ( @svc_labels_for_rg ) {
            if ($svc eq $self->{interface_label}) {
                $is_svc_in_rg = 1;
                last;
            }
        }
        if ($is_svc_in_rg) {
            my ($monitor) = $monitor_odm->get(-q =>
                "monitor = $self->{interface_label} and name = FAILURE_ACTION");
            my $mon_type = (($monitor) ? $monitor->{value} : "fallover");
            $self->{logger}->verb_log_msg(
                "Service Label: $self->{interface_label} " .
                "Monitor Type: $mon_type\n");

            if ( $mon_type ne "notify" ) {
                $recovery_type = "fallover";
            }
        }

        # if no service labels with fallover as action, 
        # or RG doesn't have the service label,
        # then there should be no change in RG state/location
        if ( $recovery_type eq "notify" || !$is_svc_in_rg ) {
            if ( ! scalar( grep {/^$self->{nodename}$/} @post_rg_online_nodes)){
                $self->{logger}->err_msg( 2, 32,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s is no longer ONLINE on node %s.\n",
                    $self->{event_name}, $rgpre_online, $self->{nodename});
                $retval = Event::Failed;
            }
        }
        # otherwise, RG should be online somewhere
        elsif ( ! scalar( @post_rg_online_nodes ) ) {
            $self->{logger}->err_msg(2, 34,
                "%s: Unexpected state change\n" .
                "Resource Group %s is no longer ONLINE on any node.\n",
                $self->{event_name}, $rgpre_online);
            $retval = Event::Failed;
        }
    }


# Process event history record
    $self->{logger}->verb_log_msg("Checking event history\n");

    my $net_complete_event = "network_down_complete";
    my @fail_swap_complete_events = ("fail_interface", "fail_standby",
                                     "swap_adapter_complete");
    my @other_events = ("network_down", "swap_adapter");

    # We need to search for the nodename or -1 for a network_down_complete.
    # -1 means that it was a global network down, which would happen in the
    # case where we just failed the last interface on this network that was
    # alive in the entire cluster.
    my $search_net_complete =
        "network_down_complete (?:-1|" . $self->{nodename} . ") " . $self->{network};
    my $search_events = join('|', @fail_swap_complete_events);
    my $addr = $self->{StaticConfig}->get_address_of_interface_label($self->{interface_label});
    my $search_fail_swap_complete =
        "(?:$search_events) " . $self->{nodename}; 
    
    my $event_complete_found = 0;
    my $event_history = $self->{postevent}->get_event_history(
        $net_complete_event, @fail_swap_complete_events, @other_events);

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);

    foreach my $node ( sort( keys ( %{$event_history} ))) {
        my @FAILED_records =
            grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        my @NET_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $search_net_complete/}
                @{$event_history->{$node}};
        my @FAIL_SWAP_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $search_fail_swap_complete/}
                @{$event_history->{$node}};
                                                                                
        foreach my $record ( @FAILED_records ) {
            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                $self->{event_name}, $node, $record);
            $retval = Event::Failed;
        }

        # A network_down_complete event should not occur during the same
        # timeframe as a fail_interface/standby or swap_adapter on the same
        # network and node.
        if (scalar(@NET_COMPLETED_records) &&
            scalar(@FAIL_SWAP_COMPLETED_records))
        {
            $self->{logger}->err_msg(2, 40,
                "%s: A swap_adapter, fail_interface, or fail_standby event\n\
                occurred for network %s on node %s during the same time\n\
                as a network_down for the same network.\n\
                A swap_adapter, fail_interface, or fail_standby event should not occur\n\
                before or after a network_down event.\n",
                $self->{event_name}, $self->{network}, $self->{nodename});
            $retval = Event::Failed;
            $event_complete_found = 1;
        }
        elsif (scalar(@NET_COMPLETED_records) ||
               scalar(@FAIL_SWAP_COMPLETED_records))
        {
            $event_complete_found = 1;
        }
    }
                                                                                
    if ( ! $event_complete_found ) {
        $self->{logger}->err_msg(2, 23,
            "%s: No EVENT COMPLETED message logged.\n" .
            "Search string: %s\n",
            $self->{event_name}, "${EVENT_COMPLETED_MSG}: $search_net_complete\n".
                                 "${EVENT_COMPLETED_MSG}: $search_fail_swap_complete");
        $retval = Event::Failed;
    }
                                                                                
    return $retval;
}


=head2 to_string -- return a string version of event

=head3     Description:

to_string() will build a Test record to insert in a Test Suite file.

=head3     Arguments:

    none.

=head3     Returns:

    Test event record as scalar value

=head3     Examples:

    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }

=cut

sub to_string() {
                                                                                       
    my $self = shift;
    my $retstring = sprintf "%s,%s,%s",
                  $self->{event_name}, $self->{interface_label},
                  $self->{description};
    return $retstring;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__

