# @(#)57        1.17 src/43haes/lib/perl/cl_testtool/Event/NETWORK_UP_LOCAL.pm, hacmp, 52haes_r540, r5400622c 4/10/06 14:04:04
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
                                                                                          
package Event::NETWORK_UP_LOCAL;

use 5.006;
use strict;
use warnings;
use Event;                                                                                          
require Exporter;
                                                                                          
our @ISA = qw(Event Exporter);
                                                                                          
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
                                                                                          
# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
                                                                                          
) ] );
                                                                                          
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
                                                                                          
our @EXPORT = qw(
                                                                                          
);
our $VERSION = '1.17';
                                                                                          
##
## Put all the methods and global data for your module here
##
                                                                                          
=head1 NAME

NETWORK_UP_LOCAL -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::NETWORK_UP_LOCAL;

    my $event = Event::NETWORK_UP_LOCAL->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of NETWORK_UP_LOCAL

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct NETWORK_UP_LOCAL class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::NETWORK_UP_LOCAL object.

=head3     Examples:

    my $event = Event::NETWORK_UP_LOCAL->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{nodename} = (shift @{$params}) || "";
    $self->{network} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{valid} = 1;
    $self->{event_name} = "NETWORK_UP_LOCAL";
    validate( $self );
    return bless $self, $class;
}

=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to NETWORK_UP_LOCAL object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->is_valid() ) {
        print "Event is valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;

    $self->{logger}->phase_header("Validate " . $self->{event_name});

# check that the $nodename field is a cluster node.
    my $nodes_configured = join " ",@{$self->{StaticConfig}->{node_names}};
    $self->{logger}->verb_log_msg("\tLocal node: $self->{nodename}");
    $self->{logger}->verb_log_msg("\t Configured nodes: $nodes_configured");

    $self->{valid} = $self->{StaticConfig}->is_node_valid($self->{nodename});
    if (!$self->{valid}) {
        $self->{logger}->err_msg(2, 2,
            "%s: Invalid Test\n".
            "Node %s not in cluster configuration\n",
            $self->{event_name}, $self->{nodename});
    }

#check to make sure we support this type of network for this event
    my $rc = $self->{StaticConfig}->is_network_valid($self->{network});
    if ($rc != 1) {
        Event::handle_invalid_network($self, $self->{network}, $rc);
        $self->{valid} = 0;
    }
    else {
        $self->{valid} = 1;
    }

#check that the specified network is configured and that the nodename
#is a member
    $self->{valid} = $self->{StaticConfig}->is_node_on_network($self->{nodename},
                                                               $self->{network});
    if (!$self->{valid}) {
        $self->{logger}->err_msg(2, 31,
            "%s: Invalid Test\n".
            "Node %s has no interfaces on network %s.\n",
            $self->{event_name}, $self->{nodename}, $self->{network});
    }
}


=head2 is_rational -- Check whether it is rational to run the event .

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
The entrance criteria for each event will be quite specific to that event, and
so each type of NETWORK_UP_LOCAL class will most likely over-ride the is_rational()
method of the Event class. 

is_rational() will typically only be called from within the execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1  == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational() {
    my $self = shift;

    $self->{logger}->phase_header("is_rational $self->{event_name}");

    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", $self->{event_name});
        return 0;
    }

    if ( ! Event::is_node_active($self, $self->{nodename})) {
        $self->{logger}->warn_msg(2, 11,
            "%s cannot be run: Not Rational\n" .
            "Specified node is not active in the cluster: %s\n",
            $self->{event_name}, $self->{nodename});
        return 0;
    }

    return 1;
}


=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of NETWORK_UP_LOCAL class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }
    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }
    return Event::execute_cmd (
        $self->{nodename}, $self, $self->{network});
}


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status()
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;

    $self->{logger}->phase_header(
            "NETWORK_UP_LOCAL: Checking post-event status"); 

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

    my @post_active_nodes = @{$self->{postevent}->{active_nodes}};
    my @pre_active_nodes = @{$self->{preevent}->{active_nodes}};

#
# no node has changed state
#
    $self->{logger}->verb_log_msg("Checking node states\n");
    my $pre_node_states = $self->{preevent}->get_node_states();
    my $post_node_states = $self->{postevent}->get_node_states();

    foreach my $node ( sort( keys( %{$pre_node_states} ))) {

        $self->{logger}->verb_log_msg(
            "\t$node: Preevent state: $pre_node_states->{$node}, " .
            "Postevent state: $post_node_states->{$node}\n");

        if ($pre_node_states->{$node} ne $post_node_states->{$node}) {

            $self->{logger}->err_msg(2, 20,
                "%s: Unexpected state change: node %s\n" .
                "Pre-event state %s, Post-event state %s\n",
                "NETWORK_UP_LOCAL", $node, 
                Event::get_node_statename($pre_node_states->{$node}),
                Event::get_node_statename($post_node_states->{$node}));
            $retval = Event::Failed;
        }
    }

# checking RG state changes
    $self->{logger}->verb_log_msg("Checking RG states\n");
    foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {

        $self->{logger}->verb_log_msg("Resource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});
        my $rgtype = $self->{StaticConfig}->get_rg_type($resgrp->{group});

        # have nodes moved into the ERROR state?
        foreach my $node ( sort( keys( %{$post_states} ))) {

            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"),
                $post_states->{$node});

            if ( $post_states->{$node} eq "ERROR" &&
                 $pre_states->{$node} ne "ERROR" ) {


                # output error message
                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s\n",
                    "NETWORK_UP_LOCAL", $resgrp->{group}, "ERROR", "$node");
                $retval = Event::Failed;
            }
        }

        # have RGs unexpectedly gone OFFLINE on nonevent nodes?
        foreach my $node ( sort( keys( %{$pre_states} ))) {
            if ( $pre_states->{$node} eq "ONLINE"  &&
                 $post_states->{$node} ne "ONLINE" &&
                 $post_states->{$node} ne "ERROR"  ) {

                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s\n",
                    "NETWORK_UP_LOCAL", $resgrp->{group}, 
                    $post_states->{$node}, $node);
                $retval = Event::Failed;
            }
        }
    }

# Process event history record
    $self->{logger}->verb_log_msg("Checking event history\n");

    my $net_complete_event = "network_up_complete";
    my @join_complete_events = ("join_interface", "join_standby");
    my @other_events = ("network_up");

    my $search_net_complete =
        "network_up_complete " . $self->{nodename} . " " . $self->{network};
    my $search_events = join('|', @join_complete_events);
    my $search_join_complete =
        "(?:$search_events) " . $self->{nodename};
    
    my $event_complete_found = 0;
    my $event_history = $self->{postevent}->get_event_history(
        $net_complete_event, @join_complete_events, @other_events);

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);

    foreach my $node ( sort( keys ( %{$event_history} ))) {
        my @FAILED_records =
            grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        my @NET_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $search_net_complete/}
                @{$event_history->{$node}};
        my @JOIN_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $search_join_complete/}
                @{$event_history->{$node}};

        foreach my $record ( @FAILED_records ) {
            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                "NETWORK_UP_LOCAL", $node, $record);
            $retval = Event::Failed;
        }
        if (scalar(@NET_COMPLETED_records) ||
            scalar(@JOIN_COMPLETED_records))
        {
            $event_complete_found = 1;
        }
    }
    if ( ! $event_complete_found ) {
        $self->{logger}->err_msg(2, 23,
            "%s: No EVENT COMPLETED message logged.\n" .
            "Search string: %s\n",
            $self->{event_name}, "${EVENT_COMPLETED_MSG}: $search_net_complete\n".
                                 "${EVENT_COMPLETED_MSG}: $search_join_complete");
        $retval = Event::Failed;
    }
    return $retval;
}

=head2 to_string -- return a string version of event
                                                                                
=head3     Description:
                                                                                
to_string() will build a Test record to insert in a Test Suite file.
                                                                                
=head3     Arguments:
                                                                                
    none.
                                                                                
=head3     Returns:
                                                                                
    Test event record as scalar value
                                                                                
=head3     Examples:
                                                                                
    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }
                                                                                
=cut
                                                                                
sub to_string() {
                                                                                
    my $self = shift;
    my $retstring = sprintf "%s,%s,%s",
                "NETWORK_UP_LOCAL", $self->{nodename}, $self->{network},
                $self->{description};
    return $retstring;
}



# it's a module, so the last statement should always just be '1;'
1;
__END__

