# @(#)58        1.14 src/43haes/lib/perl/cl_testtool/Event/NODE_DOWN_FORCED.pm, hacmp, 52haes_r540, r5400622c 3/27/06 18:43:16
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package Event::NODE_DOWN_FORCED;

use 5.006;
use strict;
use warnings;
use Event;
use StaticConfig;
require Exporter;

our @ISA = qw(Event Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);
our $VERSION = '1.14';

##
## Put all the methods and global data for your module here
##

=head1 NAME

NODE_DOWN_FORCED -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::NODE_DOWN_FORCED;

    my $event = Event::NODE_DOWN_FORCED->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of NODE_DOWN_FORCED

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct NODE_DOWN_FORCED class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::NODE_DOWN_FORCED object.

=head3     Examples:

    my $event = Event::NODE_DOWN_FORCED->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{nodename} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{valid} = 1;
    $self->{event_name} = "NODE_DOWN_FORCED";
    validate( $self );
    return bless $self, $class;
}


=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to NODE_DOWN_FORCED object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->{valid} == 0) {
        print "Event is not valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;
    
    $self->{logger}->phase_header("Validate NODE_DOWN_FORCED");

# check that the $nodename field is a cluster node.
    my $nodes_configured = join " ",@{$self->{StaticConfig}->{node_names}};
    $self->{logger}->verb_log_msg("\tEvent node: $self->{nodename}\n");
    $self->{logger}->verb_log_msg("\tConfigured nodes: $nodes_configured\n");

    my @node_configured = 
        grep {/^$self->{nodename}$/} @{$self->{StaticConfig}->{node_names}};
    if ( (scalar (@node_configured)) > 0) {
            return;
    }
    $self->{logger}->err_msg(2, 2,
        "%s: Invalid Test\n".
        "Node %s not in cluster configuration\n", 
        "NODE_DOWN_FORCED", $self->{nodename});
    $self->{valid} = 0;
}


=head2 is_rational -- Check whether it is rational to run the event .

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
The entrance criteria for each event will be quite specific to that event, and
so each type of NODE_DOWN_FORCED class will most likely over-ride the is_rational()
method of the Event class. 

is_rational() will typically only be called from within the execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1  == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational()
{
    my $self = shift;

    $self->{logger}->phase_header("is_rational NODE_DOWN_FORCED");

    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", "NODE_DOWN_FORCED");
        return 0;
    }

    my $node_state = $self->{preevent}->get_node_state( $self->{nodename} );
    if ( $node_state ne "ST_STABLE" ) {

        $self->{logger}->warn_msg(2, 11,
            "%s cannot be run: Not Rational\n" .
            "Specified node is not active in the cluster: %s\n", 
            "NODE_DOWN_FORCED", $self->{nodename});
        return 0;
    }

    if (Event::is_node_forced_down($self, $self->{nodename}, "preevent")) {
        $self->{logger}->warn_msg(2, 81,
            "%s cannot be run: Not Rational\n" .
            "Node %s is already forced down.\n",
            "NODE_DOWN_FORCED", $self->{nodename});
        return 0;
    }

    return 1;
}

=head2 is_enhanced_concurrent_vg -- Checks for enhanced concurrent VG.

=head3     Description:

is_enhanced_concurrent_vg() checks to find an enhanced concurrent VG. 
is_enhanced_concurrent_vg() will typically only be called from 
is_rational() method.

=head3     Arguments:

    none.

=head3     Returns:

    1  == enhanced concurrent VG found. 
    0 == enhanced concurrent VG not found. 

=head3     Examples:

=cut

sub is_enhanced_concurrent_vg()
{
    my @vglist = `/usr/sbin/lsvg -o 2>&1`;
    my $vg;
    my $mode;
    my $vgid;
    foreach $vg (@vglist)
    {
        print $vg;
        if ($vg ne "rootvg")
        {
            $vgid = `/usr/sbin/getlvodm -v $vg`;
print $vgid;
            $mode = `lqueryvg -g "$vgid" -X`;
print $mode;
            if ($mode == 32)
            {
                printf "JPG:NODE_DOWN_FORCED: Enanced VG found\n";
                return (1);
            }
        }
    }
    return (0);
}
=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of NODE_DOWN_FORCED class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;

    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }

    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }

    return Event::execute_cmd (
        $self->{nodename}, $self, $self->{nodename});
}


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console.

=head3     Arguments:

    $tag:   event start tag

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status()
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;

    $self->{logger}->phase_header(
            "NODE_DOWN_FORCED: Checking post-event status");

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

    my @post_active_nodes = @{$self->{postevent}->{active_nodes}};
    my @pre_active_nodes = @{$self->{preevent}->{active_nodes}};
    $self->{logger}->verb_log_msg(
            "pre-event online nodes: @pre_active_nodes\n");
    $self->{logger}->verb_log_msg(
            "post-event online nodes: @post_active_nodes\n");
#
#       Post event status checks
#

#
# is node on forced list?
#
    $self->{logger}->verb_log_msg("Checking forced down node lists\n");

    my $node_is_forced =
        Event::is_node_forced_down($self, $self->{nodename}, "postevent");
    # there must be an active node to check for forced down
    if (! scalar(@post_active_nodes) ) {
        $self->{logger}->warn_msg(2, 18,
            "%s: No active cluster nodes.\n" .
            "Unable to verify that node %s is marked forced down.\n",
            "NODE_DOWN_FORCED", $self->{nodename});
    } elsif ($node_is_forced == 0) {        
        $self->{logger}->err_msg(2, 19,
            "%s: Bad Post Event Status\n" .
            "Node %s is not marked forced down.\n",
            "NODE_DOWN_FORCED", $self->{nodename});
        $retval = Event::Failed;
    }

#
# no non-event node has changed state
#
    $self->{logger}->verb_log_msg("Checking node states\n");
    my $pre_node_states = $self->{preevent}->get_node_states();
    my $post_node_states = $self->{postevent}->get_node_states();

    STATECHECK: foreach my $node ( sort( keys( %{$pre_node_states} ))) {

        $self->{logger}->verb_log_msg(
            "$node: Preevent state: $pre_node_states->{$node}, " .
            "Postevent state: $post_node_states->{$node}\n");

        # the event node is supposed to change state, so ignore it here
        #if ($node eq $self->{nodename}) {
        #   next STATECHECK;
        #}

        if ($pre_node_states->{$node} ne $post_node_states->{$node}) {

            $self->{logger}->err_msg(2, 20,
                "%s: Unexpected state change: node %s\n" .
                "Pre-event state %s, Post-event state %s\n",
                "NODE_DOWN_FORCED", $node, 
                Event::get_node_statename($pre_node_states->{$node}),
                Event::get_node_statename($post_node_states->{$node}));
            $retval = Event::Failed;
        }
    }

#
# checking RG state changes
    $self->{logger}->verb_log_msg("Checking RG states\n");
    foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {

        $self->{logger}->verb_log_msg("\tResource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});
        my $rgtype = $self->{StaticConfig}->get_rg_type($resgrp->{group});

        foreach my $node ( sort( keys( %{$post_states} ))) {
            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"),
                $post_states->{$node});

            if ($node eq $self->{nodename} &&
                $pre_states->{$node} eq "ONLINE" &&
                $post_states->{$node} ne "UNMANAGED")
            {
                $self->{logger}->err_msg(2, 42,
                    "%s: Resource group %s did not change to the\n" .
                    "UNMANAGED state on node %s.\n",
                    $self->{event_name}, $resgrp->{group},
                    $self->{nodename});
                $self->{logger}->err_msg(2, 43,
                    "All resource groups that were ONLINE on the forced down node should\n" .
                    "change to the UNMANAGED state on that node.\n");
                $retval = Event::Failed;
            }
            elsif ($node eq $self->{nodename} &&
                   $pre_states->{$node} eq "ONLINE" &&
                   $post_states->{$node} eq "UNMANAGED")
            { 
                next;
            }
            elsif ($rgtype == StaticConfig::nonconcurrent &&
                   $post_states->{$self->{nodename}} eq "UNMANAGED" &&
                   $post_states->{$node} ne "UNMANAGED")
            {
                $self->{logger}->err_msg(2, 42,
                    "%s: Resource group %s did not change to the\n" .
                    "UNMANAGED state on node %s.\n",
                    $self->{event_name}, $resgrp->{group},
                    $self->{nodename});
                $self->{logger}->err_msg(2, 72,
                    "All non-concurrent resource groups that changed to the UNMANAGED state\n" .
                    "on the forced down node should also change to the UNMANAGED state\n" .
                    "on all other nodes.\n");
                $retval = Event::Failed;
            }
            elsif ($rgtype == StaticConfig::nonconcurrent &&
                   $post_states->{$self->{nodename}} eq "UNMANAGED" &&
                   $post_states->{$node} eq "UNMANAGED")
            {
                next;
            }
            elsif ($pre_states->{$node} ne $post_states->{$node}) {
                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s.\n",
                    $self->{event_name}, $resgrp->{group},
                    $post_states->{$node}, $node);
                $retval = Event::Failed;
            }
        }
    }

# Process event history records
    $self->{logger}->verb_log_msg("Checking event history\n");

    my $event_complete_found = 0;
    my $event_history = $self->{postevent}->get_event_history(
            "node_down", "node_down_complete");
    my $search_str = "node_down_complete " . $self->{nodename} . " forced";

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);

    foreach my $node ( sort( keys ( %{$event_history} ))) {

        my @FAILED_records = 
            grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        my @COMPLETED_records = 
            grep {/^.* ${EVENT_COMPLETED_MSG}: $search_str/} 
                @{$event_history->{$node}};

        foreach my $record ( @FAILED_records ) {

            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                "NODE_DOWN_FORCED", $node, $record);
            $retval = Event::Failed;
        }

        if ( scalar(@COMPLETED_records) ) {
            $event_complete_found = 1;
        }
    }

    if ( ! $event_complete_found ) {
        $self->{logger}->err_msg(2, 23,
            "%s: No EVENT COMPLETED message logged.\n" .
            "Search string: %s\n",
            "NODE_DOWN_FORCED", "${EVENT_COMPLETED_MSG}: $search_str");
        $retval = Event::Failed;
    }

    return $retval;
}



=head2 to_string -- return a string version of event

=head3     Description:

to_string() will build a Test record to insert in a Test Suite file.

=head3     Arguments:

    none.

=head3     Returns:

    Test event record as scalar value

=head3     Examples:

    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }
    
=cut

sub to_string() {

    my $self = shift;
    my $retstring = sprintf "%s,%s,%s", 
                "NODE_DOWN_FORCED", $self->{nodename}, $self->{description};
    return $retstring;
}


# it's a module, so the last statement should always just be '1;'
1;
__END__

