# @(#)59        1.13 src/43haes/lib/perl/cl_testtool/Event/NODE_DOWN_GRACEFUL.pm, hacmp, 52haes_r540, r5400622c 4/7/06 12:38:31
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package Event::NODE_DOWN_GRACEFUL;

use 5.006;
use strict;
use warnings;
use Event;
use StaticConfig;
require Exporter;

our @ISA = qw(Event Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);
our $VERSION = '1.13';

##
## Put all the methods and global data for your module here
##

=head1 NAME

NODE_DOWN_GRACEFUL -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::NODE_DOWN_GRACEFUL;

    my $event = Event::NODE_DOWN_GRACEFUL->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of NODE_DOWN_GRACEFUL

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct NODE_DOWN_GRACEFUL class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::NODE_DOWN_GRACEFUL object.

=head3     Examples:

    my $event = Event::NODE_DOWN_GRACEFUL->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{nodename} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{event_name} = "NODE_DOWN_GRACEFUL";
    $self->{valid} = 1;
    validate( $self );
    return bless $self, $class;
}


=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to NODE_DOWN_GRACEFUL object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->{valid} == 0 ) {
        print "Event is not valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;
    
    $self->{logger}->phase_header("Validate NODE_DOWN_GRACEFUL");

# check that the $nodename field is either "ALL" or a cluster node.
    my $nodes_configured = join " ",@{$self->{StaticConfig}->{node_names}};
    $self->{logger}->verb_log_msg("\tEvent node: $self->{nodename}\n");
    $self->{logger}->verb_log_msg("\tConfigured nodes: $nodes_configured\n");

    if ($self->{nodename} eq "ALL") {
        return;
    } 
    my @node_configured = 
        grep {/^$self->{nodename}$/} @{$self->{StaticConfig}->{node_names}};
    if ( (scalar (@node_configured)) > 0) {
            return;
    }
    $self->{logger}->err_msg(2, 2,
        "%s: Invalid Test\n".
        "Node %s not in cluster configuration\n", 
        "NODE_DOWN_GRACEFUL", $self->{nodename});
    $self->{valid} = 0;
}

=head2 is_rational -- Check whether it is rational to run the event .

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
The entrance criteria for each event will be quite specific to that event, and
so each type of NODE_DOWN_GRACEFUL class will most likely over-ride the is_rational()
method of the Event class. 

is_rational() will typically only be called from within the execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1 == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational()
{
    my $self = shift;

    $self->{logger}->phase_header("is_rational NODE_DOWN_GRACEFUL");

    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", "NODE_DOWN_GRACEFUL");
        return 0;
    }

    if ($self->{nodename} eq "ALL")
    {
        my @active_nodes = @{$self->{preevent}->{active_nodes}};
        $self->{logger}->verb_log_msg("Active Nodes: @active_nodes\n");
        if ( (scalar(@active_nodes)) < 1) {
            $self->{logger}->warn_msg(2, 12,
                "%s cannot be run: Not Rational\n" .
                "No active nodes in cluster with keyword ALL\n", 
                "NODE_DOWN_GRACEFUL");
            return 0;
        } else {
            $self->{nodename} = join ',', @active_nodes;
        }
    } else {
        my $node_state = $self->{preevent}->get_node_state( $self->{nodename} );
        $self->{logger}->verb_log_msg(
                "Node: " . $self->{nodename} . ", State: $node_state\n");
        if ( $node_state ne "ST_STABLE" ) {
            $self->{logger}->warn_msg(2, 11,
                "%s cannot be run: Not Rational\n" .
                "Specified node is not active in the cluster: %s\n", 
                "NODE_DOWN_GRACEFUL", $self->{nodename});
            return 0;
        }
    }
    return 1;
}


=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of NODE_DOWN_GRACEFUL class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }

    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }

    return Event::execute_cmd (
        $self->{StaticConfig}->{local_node}, $self, $self->{nodename});
}


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console.

=head3     Arguments:

    $tag:   event start tag

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status()
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;
    $self->{logger}->phase_header(
            "NODE_DOWN_GRACEFUL: Checking post-event status"); 
    my @event_nodes = split /,/, $self->{nodename};
    $self->{logger}->verb_log_msg("Event Nodes: @event_nodes\n");

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

    my @post_active_nodes = @{$self->{postevent}->{active_nodes}};
    my @pre_active_nodes = @{$self->{preevent}->{active_nodes}};
    $self->{logger}->verb_log_msg(
            "pre-event online nodes: @pre_active_nodes\n");
    $self->{logger}->verb_log_msg(
            "post-event online nodes: @post_active_nodes\n");

#
#       Post event status checks
#
# are event nodes online ?
#
    foreach my $event_node (@event_nodes) {
        if ((scalar (grep {/^$event_node$/} @post_active_nodes)) > 0) {
            $self->{logger}->err_msg(2, 17,
                "%s: Bad Post Event Status\n" .
                "node %s still active in cluster\n", 
                "NODE_DOWN_GRACEFUL", $event_node);
            $retval = Event::Failed;
        }
    }

#
# no other node changes state
#
    $self->{logger}->verb_log_msg("Checking node states\n");
    my $pre_node_states = $self->{preevent}->get_node_states();
    my $post_node_states = $self->{postevent}->get_node_states();

    STATECHECK: foreach my $node ( sort( keys( %{$pre_node_states} ))) {

        $self->{logger}->verb_log_msg(
            "$node: Preevent state: $pre_node_states->{$node}, " .
            "Postevent state: $post_node_states->{$node}\n");

        # the event node is supposed to change state, so ignore it here
        if ( scalar( grep {/^$node$/} @event_nodes )) {
           next STATECHECK;
        }

        if ($pre_node_states->{$node} ne $post_node_states->{$node}) {

            $self->{logger}->err_msg(2, 20,
                "%s: Unexpected state change: node %s\n" .
                "Pre-event state %s, Post-event state %s\n",
                "NODE_DOWN_GRACEFUL", $node, 
                Event::get_node_statename($pre_node_states->{$node}),
                Event::get_node_statename($post_node_states->{$node}));
            $retval = Event::Failed;
        }
    }


#
# checking RG state changes
    $self->{logger}->verb_log_msg("Checking RG states\n");
    foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {

        $self->{logger}->verb_log_msg("Resource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});
        my $rgtype = $self->{StaticConfig}->get_rg_type($resgrp->{group});

        # have nodes moved into the ERROR state?
        foreach my $node ( sort( keys( %{$post_states} ))) {

            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"),
                $post_states->{$node});

            if ( $post_states->{$node} eq "ERROR" &&
                 $pre_states->{$node} ne "ERROR" ) {

                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s\n",
                    "NODE_DOWN_GRACEFUL", $resgrp->{group}, "ERROR", $node);
                $retval = Event::Failed;
            }
        }

        # have RGs unexpectedly gone OFFLINE?
        PRENODE: foreach my $node ( sort( keys( %{$pre_states} ))) {
            # check RGs that were ONLINE on the event node
            if ( scalar( grep {/^$node$/} @event_nodes )) {
                # nonconcurrent should not be ONLINE on another node
                if ( $rgtype == StaticConfig::nonconcurrent &&
                     $pre_states->{$node} eq "ONLINE" ) {

                    foreach my $postnode ( sort( keys( %{$post_states} ))) {
                        if ( $post_states->{$postnode} eq "ONLINE" ) {
                            $self->{logger}->err_msg( 2, 24,
                                "%s: Unexpected state.\n" .
                                "Non-concurrent RG %s should be %s.\n",
                                "NODE_DOWN_GRACEFUL", $resgrp->{group},
                                "OFFLINE");
                            $retval = Event::Failed;
                            next PRENODE;
                        }
                    }
                    # if control reaches here, no ONLINE node was found
                }
                next PRENODE;
            }

            # check RGs that were ONLINE on a non-event node
            if ( $pre_states->{$node} eq "ONLINE"  &&
                 $post_states->{$node} ne "ONLINE" &&
                 $post_states->{$node} ne "ERROR"  ) {

                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s.\n",
                    "NODE_DOWN_GRACEFUL", $resgrp->{group}, "OFFLINE", $node);
                $retval = Event::Failed;
            }
        }
    }

# Process event history records
    $self->{logger}->verb_log_msg("Checking event history\n");

    my %event_complete_found;
    foreach my $comp_node (@event_nodes) {
        $event_complete_found{$comp_node} = 0;
    }
    my $event_history = $self->{postevent}->get_event_history(
            "node_down", "node_down_complete");

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);

    # are there any interesting event failure messages?
    foreach my $node ( sort( keys ( %{$event_history} ))) {

        my @FAILED_records = 
            grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        foreach my $record ( @FAILED_records ) {

            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                "NODE_DOWN_GRACEFUL", $node, $record);
            $retval = Event::Failed;
        }
    }

    # have we found and EVENT COMPLETED message for each event node?
    COMPCHECK: foreach my $check_node (@event_nodes) {

        #my $search_str = "node_down_complete " . $check_node . " graceful";
        my $search_str = "node_down_complete " . $check_node;
        foreach my $node ( sort( keys ( %{$event_history} ))) {
        
            my @COMPLETED_records = 
                grep {/^.* ${EVENT_COMPLETED_MSG}: $search_str/} 
                    @{$event_history->{$node}};
        
            if ( scalar(@COMPLETED_records) ) {
                $event_complete_found{$check_node} = 1;
                next COMPCHECK;
            }
        }
    }
    foreach my $msgnode (@event_nodes) {
        if ( ! $event_complete_found{$msgnode} ) {
            #my $search_str = "node_down_complete " . $msgnode . " graceful";
            my $search_str = "node_down_complete " . $msgnode;
            $self->{logger}->err_msg(2, 26,
                "%s: No EVENT COMPLETED message logged for node %s.\n" .
                "Search string: %s\n",
                "NODE_DOWN_GRACEFUL", $msgnode, "${EVENT_COMPLETED_MSG}: $search_str");
            $retval = Event::Failed;
        }
    }

    return $retval;
}


=head2 to_string -- return a string version of event

=head3     Description:

to_string() will build a Test record to insert in a Test Suite file.

=head3     Arguments:

    none.

=head3     Returns:

    Test event record as scalar value

=head3     Examples:

    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }
    
=cut

sub to_string() {

    my $self = shift;
    my $retstring = sprintf "%s,%s,%s", 
                "NODE_DOWN_GRACEFUL", $self->{nodename}, $self->{description};
    return $retstring;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__

