#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)16	1.8 src/43haes/lib/perl/cl_testtool/Event/RG_ONLINE.pm, hacmp.test_tool, 52haes_r540, r5400622c 4/6/06 12:31:41

package Event::RG_ONLINE;

use 5.006;
use strict;
use warnings;
use Event;
use StaticConfig;
require Exporter;

our @ISA = qw(Event Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '1.8';

##
## Put all the methods and global data for your module here
##

=head1 NAME

RG_ONLINE -- 
Perl extension providing the Event::RG_ONLINE object class.

=head1 SYNOPSIS

    use Event::RG_ONLINE;

    my $event = Event::RG_ONLINE->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

This module provides the Event::RG_ONLINE object class for use by the HACMP
Cluster Test Tool. This class inherits from the Event class, and so will use
all of the methods defined for that class unless the methods are over-ridden
here. The Event::RG_ONLINE class represents the Test Tool's RG_ONLINE event, and
will initiate starting of cluster services on one node (or on all nodes if the
keyword "ALL" is provided in place of an actual node name).

=head1 METHODS

=head2 new -- create a new instance of RG_ONLINE

=head3     Description:

new() is the constructor for the Event::RG_ONLINE class. It takes a
reference to an an array of parameters that are required for the
RG_ONLINE event: rg name node name and description. This method will
verify that the specified node name exists in the cluster
configuration on the local node, and will set the object's "valid"
variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::RG_ONLINE object.

=head3     Examples:

    my $event = Event::RG_ONLINE->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{event_name} = "RG_ONLINE";
    $self->{RGname} = (shift @{$params}) || "";
    $self->{nodename} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{valid} = 1; 
    validate( $self );
    return bless $self, $class;
}

=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the objects 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to RG_ONLINE object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->{valid} == 0 ) {
        print "Event is not valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;
    $self->{logger}->phase_header("Validate ".$self->{event_name});

    my $nodes_configured = join " ",@{$self->{StaticConfig}->{node_names}};
    $self->{logger}->verb_log_msg("\tEvent node: $self->{nodename}\n");
    $self->{logger}->verb_log_msg("\tConfigured nodes: $nodes_configured\n");

    # Check if RG is in the cluster configuration
    $self->{valid} = $self->{StaticConfig}->is_rg_valid($self->{RGname});
    if (!$self->{valid}) {
        $self->{logger}->err_msg(2, 52,
            "%s: Invalid Test\n".
            "Resource Group %s is not in the cluster configuration\n",
            $self->{event_name}, $self->{RGname});
	return;
    }

    # If node is a generic keyword return
    if (Event::generic_node($self)) {
        return;
    }

    # Otherwise confirm the node is valid
    $self->{valid} = $self->{StaticConfig}->is_node_valid($self->{nodename});
    if (!$self->{valid}) {
        $self->{logger}->err_msg(2, 2,
            "%s: Invalid Test\n".
            "Node %s not in cluster configuration\n",
            $self->{event_name}, $self->{nodename});
	return;
    }

    # and check if the node appears in RG nodelist
    $self->{valid} = $self->{StaticConfig}->is_node_in_rg_nodelist($self->{RGname}, $self->{nodename});
    if (!$self->{valid}) {
        $self->{logger}->err_msg(2, 53,
            "%s: Invalid Test\n".
            "Resource Group %s is not configured to run on Node %s\n",
            $self->{event_name}, $self->{RGname}, $self->{nodename});
	return
    }
}

=head2 is_rational -- Check whether the RG_ONLINE event is rational.

=head3     Description:

is_rational checks whether the entrance criteria for the event are satisfied,
namely that the specified node, or at least one node (if "ALL" specified)
is currently offline.

=head3     Arguments:

    none.

=head3     Returns:

    1 == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() == 0 ) {
        print "Event is not rational.\n";
    }

=cut

sub is_rational()
{
  my $self = shift;

  $self->{logger}->phase_header("is_rational ".$self->{event_name});

  # Check that the cluster is stable
  if ( ! $self->{preevent}->cluster_is_stable() ) {
    $self->{logger}->warn_msg(2, 10,
			      "%s cannot be run: Not Rational\n" .
			      "Cluster is not stable\n", $self->{event_name});
    return 0;
  }

  my $pre_node_states = $self->{preevent}->get_node_states();

  # For nonconcurrent RGs check that the RG is not online anywhere
  if ($self->{StaticConfig}->rg_is_nonconcurrent($self->{RGname}) && 
    Event::rg_is_online($self,"preevent")) {
    my $noderef = Event::get_nodes_where_rg_is_online($self,"preevent");
    $self->{logger}->warn_msg(2, 54,
			      "%s cannot be run: Not Rational\n" .
			      "Resource Group %s is already online on node %s\n", $self->{event_name}, $self->{RGname}, @{$noderef}[0]);
    return 0;
  }

  # Check if a node is specified it is online
  if (!Event::generic_node($self) && ($pre_node_states->{$self->{nodename}} ne "ST_STABLE")) {
    $self->{logger}->warn_msg(2, 55,
			      "%s cannot be run: Not Rational\n" .
			      "Node %s is inactive and cannot host resource group %s\n", 
			      $self->{event_name}, $self->{nodename}, $self->{RGname});
    return 0;
  }

  # Check if a node is specified that it can host RG
  if (!Event::generic_node($self) && !(Event::node_is_allowed_for_rg($self, $self->{nodename},$self->{RGname}))) {
    $self->{logger}->warn_msg(2, 56,
			      "%s cannot be run: Not Rational\n" .
			      "Node %s cannot host resource group %s\n", 
			      $self->{event_name}, $self->{nodename}, $self->{RGname});
    return 0;
  }

  # If ANY, ALL, RESTORE were specified for node, check that a node in the RG nodelist is online
  if (Event::generic_node($self)) {
    my @allowed_nodes = Event::get_allowed_nodes_for_rg($self,$self->{RGname});
    if (!scalar(@allowed_nodes)) {
      $self->{logger}->warn_msg(2, 57,
				"%s cannot be run: Not Rational\n" .
				"No active nodes can host resource group %s\n", 
				$self->{event_name},$self->{RGname});
      return 0;
    }
    # also check that at least one of the nodes doesn't already have the RG
    # clsetenvgrp seems to include online nodes for concurrent-style RGs
    my @target_nodes = ();
    my $pre_rg_states = $self->{preevent}->get_rg_states($self->{RGname});
    foreach my $node (@allowed_nodes) {
      if(($pre_node_states->{$node} eq "ST_STABLE") && ($pre_rg_states->{$node} ne "ONLINE") ) {
	push @target_nodes, $node;
      }
    }
    if (!scalar(@target_nodes)) {
      $self->{logger}->warn_msg(2, 57,
				"%s cannot be run: Not Rational\n" .
				"No active nodes can host resource group %s\n", 
				$self->{event_name},$self->{RGname});
      return 0;
    }
    # Finally, store the node(s) we will test in the class nodelist
    # Concurrent RGs get all allowed nodes for nodes ALL and RESTORE
    # All other cases will choose a random node.
    if ($self->{StaticConfig}->rg_is_concurrent($self->{RGname}) && 
	(($self->{nodename} eq "ALL") || $self->{nodename} eq "RESTORE")) {
      @{$self->{nodelist}} = @target_nodes;
    } else {
      push @{$self->{nodelist}}, Event::choose_random_node(@target_nodes);
    }
  }

  # if a node was specified, just add it to the list
  if (!Event::generic_node($self)) {
    push @{$self->{nodelist}}, $self->{nodename};
  }

  return 1;
}


=head2 execute -- Execute the RG_ONLINE event.

=head3     Description:

execute() will control, at a high level, the execution of the RG_ONLINE event.
It will determine the node (or list of nodes) to be started, capture the node
and RG states for the cluster, and run the necessary command (via the Test
Toll Event Controller) to start the nodes.

NOTE: The event is assumed to be rational when execute is run. It is the 
callers responsibility to call the is_rational() method before calling the
execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it does not make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut


sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }

    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }

    if ($self->{StaticConfig}->rg_is_concurrent($self->{RGname}) && ($self->{nodename} eq "ALL" || $self->{nodename} eq "RESTORE")) {
      return Event::execute_cmd (
				 @{$self->{nodelist}}[0], $self, $self->{RGname}, "ALL");
    } elsif ($self->{StaticConfig}->rg_is_nonconcurrent($self->{RGname}) && ($self->{nodename} eq "RESTORE")) {
      return Event::execute_cmd (
				 @{$self->{nodelist}}[0], $self, $self->{RGname}, "RESTORE");
    } else {
      return Event::execute_cmd (
				 @{$self->{nodelist}}[0], $self, $self->{RGname}, @{$self->{nodelist}}[0]);
    }
}

=head2 check_status -- checks the event completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether the RG_ONLINE event completed successfully. It will also report the
final event status to the log file and the console.

=head3     Arguments:

    $tag:   event start tag

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status()
{
  my $self = shift;
  my $tag  = shift;
  my $retval = Event::Continue;
  $self->{logger}->phase_header($self->{event_name}.": Checking post-event status");
  my @event_nodes = split /,/, $self->{nodename};
  $self->{logger}->verb_log_msg("Event Nodes: @event_nodes\n");
  
  if ( ! Event::collect_postevent($self, $tag) ) {
    return Event::Fatal;
  }
  
  my @pre_active_nodes = @{$self->{preevent}->{active_nodes}};
  my @post_active_nodes = @{$self->{postevent}->{active_nodes}};
  my $pre_rg_states = $self->{preevent}->get_rg_states($self->{RGname});
  my $post_rg_states = $self->{postevent}->get_rg_states($self->{RGname});
  
  $self->{logger}->verb_log_msg("pre-event online nodes: @pre_active_nodes\n");
  $self->{logger}->verb_log_msg("post-event online nodes: @post_active_nodes\n");
  
  #
  #       Post event status checks
  
  #check that RG is now online for each node specified (ignore RESTORE case until later)
  if (!($self->{StaticConfig}->rg_is_nonconcurrent($self->{RGname}) && ($self->{nodename} eq "RESTORE"))) {
    foreach my $node (@{$self->{nodelist}}) {
      if ($post_rg_states->{$node} ne "ONLINE") {
	$self->{logger}->err_msg(2, 58,
				 "%s: Bad Post Event Status\n" .
				 "Resource Group %s is not ONLINE on node %s\n", 
				 $self->{event_name}, $self->{RGname}, $node);
	$retval = Event::Failed;
      }
    }
  }
  
  # no node changes state
  if ($self->have_nodes_changed_state($self)) {
    $retval = Event::Failed;
  }
  
  # no other RG changes state
  if ($self->have_other_rgs_changed_state($self->{RGname})) {
    # errors already logged in have_other_rgs_changed_state()
    $retval = Event::Failed;
  }
  
  #if the RG is nonconcurrent, check that it is only online on one node
  my $count = Event::count_rg_instances($self, $self->{RGname}, "postevent");
  if ($self->{StaticConfig}->rg_is_nonconcurrent($self->{RGname}) && ($count == 0)) {
    $self->{logger}->err_msg(2, 59,
			     "%s: Bad Post Event Status\n" .
			     "Resource Group %s is not online on any node\n", 
			     $self->{event_name}, $self->{RGname});
    $retval = Event::Failed;
  } elsif($self->{StaticConfig}->rg_is_nonconcurrent($self->{RGname}) && ($count > 1)) {
    $self->{logger}->err_msg(2, 60,
			     "%s: Bad Post Event Status\n" .
			     "Nonconcurrent Resource Group %s is online on multiple nodes\n", 
			     $self->{event_name}, $self->{RGname});
    $retval = Event::Failed;
  }

  #if the RG is concurrent and ALL or RESTORE was the node it should be online on all online nodes
  if ($self->{StaticConfig}->rg_is_concurrent($self->{RGname})) {
    foreach my $node (@post_active_nodes) {
      if ( $self->{StaticConfig}->is_node_in_rg_nodelist($self->{RGnode},$self->{node})
	   && $post_rg_states->{$node} ne "ONLINE" ) {
	$self->{logger}->err_msg(2, 41,
				 "%s: Bad Post Event Status\n" .
				 "Resource Group %s did is not online on node %s\n", 
				 $self->{event_name}, $self->{RGname}, $self->{nodename})
      }
    }
  }
  
  # Process event history records
  if(!Event::rgmove_event_logged($self)) {
      # errors already logged in rgmove_event_logged()
      $retval = Event::Failed;
  }

  return $retval;
}

=head2 to_string -- return a string version of event

=head3     Description:

to_string() will build a Test record to insert in a Test Suite file.

=head3     Arguments:

    none.

=head3     Returns:

    Test event record as scalar value

=head3     Examples:

    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }
    
=cut

sub to_string() {

    my $self = shift;
    my $retstring = sprintf "%s,%s,%s,%s", 
                $self->{event_name}, $self->{RGname}, $self->{nodename}, $self->{description};
    return $retstring;
}


# it's a module, so the last statement should always just be '1;'
1;
__END__
