# @(#)62        1.19 src/43haes/lib/perl/cl_testtool/Event/SERVER_DOWN.pm, hacmp, 52haes_r540, r5400622c 4/4/06 15:27:00
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package Event::SERVER_DOWN;

use 5.006;
use strict;
use warnings;

use ODM;
use Logger;
use Event;

require Exporter;

our @ISA = qw(Event Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);
our $VERSION = '1.19';

##
## Put all the methods and global data for your module here
##

=head1 NAME

SERVER_DOWN -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::SERVER_DOWN;

    my $event = Event::SERVER_DOWN->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of SERVER_DOWN

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct SERVER_DOWN class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::SERVER_DOWN object.

=head3     Examples:

    my $event = Event::SERVER_DOWN->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{nodename} = (shift @{$params}) || "";
    $self->{app_server} = (shift @{$params}) || "";
    $self->{command} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{event_name} = "SERVER_DOWN";
    $self->{valid} = 1;
    validate( $self );
    return bless $self, $class;
}


=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to SERVER_DOWN object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->is_valid() ) {
        print "Event is valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;

    $self->{logger}->phase_header("Validate SERVER_DOWN");
    
# check that the $nodename field is either "ANY" or a cluster node.
    my $nodes_configured = join " ",@{$self->{StaticConfig}->{node_names}};
    $self->{logger}->verb_log_msg("\tEvent node: $self->{nodename}\n");
    $self->{logger}->verb_log_msg("\tConfigured nodes: $nodes_configured\n");
    
    if ($self->{nodename} ne "ANY") {
        my @node_configured =
            grep {/^$self->{nodename}$/} @{$self->{StaticConfig}->{node_names}};
        if ( ! (scalar (@node_configured)) ) {

            $self->{logger}->err_msg(2, 2,
                "%s: Invalid Test\n".
                "Node %s not in cluster configuration\n",
                "SERVER_DOWN", $self->{nodename});
            $self->{valid} = 0;
        }
    }

# check that appserv exists and is in an RG associated with given node
    my $rgname = $self->{StaticConfig}->get_rg_for_res(
            $self->{app_server}, "APPLICATIONS");
    $self->{logger}->verb_log_msg("\tApp: $self->{app_server}, RG Name: $rgname\n");
    if ( $rgname ) {

        # specific node specified
        if ( $self->{nodename} ne "ANY" ) {

            my @nodes = $self->{StaticConfig}->get_nodes_for_rg( $rgname );
            $self->{logger}->verb_log_msg("\tRG Nodes: @nodes\n");
            my @matches = grep {/^$self->{nodename}$/} @nodes;
            if ( scalar(@matches) < 1 ) {
                $self->{logger}->err_msg(2, 6,
                    "%s: Invalid Test\n" .
                    "Node %s is not in group %s with application server %s\n",
                    "SERVER_DOWN", $self->{nodename}, 
                    $rgname, $self->{app_server});
                $self->{valid} = 0;
            }
        }
    } else {
        $self->{logger}->err_msg(2, 7,
            "%s: Invalid Test\n" .
            "Application server %s is not in the cluster configuration.\n",
            "SERVER_DOWN", $self->{app_server});
        $self->{valid} = 0;
    }

# check that appmon exists and is associated with appserv
    my $odm = ODM->new("HACMPmonitor", ODMDIR=>"/etc/objrepos");
    my @monitor_attributes = $odm->get(-q => "type = APPLICATIONS and " .
        "name = RESOURCE_TO_MONITOR and value = " . $self->{app_server});
    if ( ! scalar( @monitor_attributes )) {
        $self->{logger}->err_msg(2, 8,
            "%s: Invalid Test\n" .
            "No Application Monitor found for Application Server %s\n" .
            "Only monitored application servers can be tested.\n",
            "SERVER_DOWN", $self->{app_server});
        $self->{valid} = 0;
    }
}

=head2 is_rational -- Check whether it is rational to run the event .

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
The entrance criteria for each event will be quite specific to that event, and
so each type of SERVER_DOWN class will most likely over-ride the is_rational()
method of the Event class. 

is_rational() will typically only be called from within the execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1 == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational()
{
    my $self = shift;

    $self->{logger}->phase_header("is_rational SERVER_DOWN");

# check that the cluster is stable
    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", "SERVER_DOWN");
        return 0;
    }

# is the specified node online?
    my $rgname = $self->{StaticConfig}->get_rg_for_res(
            $self->{app_server}, "APPLICATIONS");
    my @online_nodes = $self->{preevent}->get_nodes_for_rg($rgname, "ONLINE");
    $self->{logger}->verb_log_msg(
        "APP: $self->{app_server}, RG: $rgname, ONLINE NODES: @online_nodes\n");
    if ($self->{nodename} eq "ANY")
    {
        #if ANY then check that the RG is online somewhere
        if ( scalar(@online_nodes) < 1 ) {
            $self->{logger}->warn_msg(2, 15,
                "%s cannot be run: Not Rational\n" .
                "Resource Group %s is not online on ANY cluster node\n",
                "SERVER_DOWN",$rgname);
            return 0;
        } else {
            # decide which node to use
            $self->{nodename} = Event::choose_random_node(@online_nodes);
        }

    } else {
        #check that the RG is online to the specified node
        if ( ! scalar( grep {/^$self->{nodename}$/} @online_nodes ) ) {
            $self->{logger}->warn_msg(2, 16,
                "%s cannot be run: Not Rational\n" .
                "Resource Group %s is not ONLINE on node %s\n",
            "SERVER_DOWN", $rgname, $self->{nodename});
            return 0;
        }
    }
    return 1;
}

=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of SERVER_DOWN class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }
    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }
    return Event::execute_cmd (
        $self->{nodename}, $self, $self->{command});
}


=head2  wait_for_stable -- waits until the cluster is stable.

=head3     Description:

Event::SERVER_DOWN::wait_for_stable() will determine whether the
server to be stopped uses a custom monitor method, and if so, it
will wait for the monitor_interval before handing control off to
Event::wait_for_stable.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue == cluster reached the stable state.
    Event::Timeout  == cluster did not reach the stable state, and the 
                       timeout for this event was reached.
    Event::Fatal    == a fatal error occured testing for stability

=head3     Examples:

    if ( $event->wait_for_stable() eq Event::Timeout ) {
        print "Timeout waiting for cluster to stabilize.\n".
    }
    
=cut

sub wait_for_stable ( )
{
    my $self = shift;
    my $mon_interval = 0;

    # look up the appmon objects for the app server
    my $odm = ODM->new("HACMPmonitor", ODMDIR=>"/etc/objrepos");
    my @monitor_attributes = $odm->get(-q => 
                                    "type = APPLICATIONS and " .
                                    "name = RESOURCE_TO_MONITOR and " .
                                    "value = " . $self->{app_server});

    # now check the monitor interval of each
    foreach  my $mon ( @monitor_attributes ) {
        my ($monitor) = $odm->get(-q =>
            "name = MONITOR_INTERVAL and monitor = " . $mon->{monitor});
        if ( $monitor && $monitor->{value} > $mon_interval ) {
            $mon_interval = $monitor->{value};
        }
    }

    # now sleep for monitor interval
    if ( $mon_interval ) {
        $self->{logger}->info_msg(2, 35,
        "%s: Waiting for monitor interval %ld before checking stability.\n",
        "SERVER_DOWN", "$mon_interval");
        sleep $mon_interval;
    }

    return Event::wait_for_stable($self);
}

=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status ( )
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;
    $self->{logger}->phase_header(
            "SERVER_DOWN: Checking post-event status");

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

# get the resource and resource group information
    my $resid  = $self->{StaticConfig}->get_res_id(
            $self->{app_server}, "APPLICATIONS");
    my $rgname = $self->{StaticConfig}->get_rg_for_res(
            $self->{app_server}, "APPLICATIONS");
    my $rgtype = $self->{StaticConfig}->get_rg_type($rgname);
    my $rgid   = $self->{StaticConfig}->get_rg_id($rgname);
    $self->{logger}->verb_log_msg(
            "RESID: $resid, RG: $rgname, RGID: $rgid, TYPE: $rgtype\n");

#
# no node has changed state
#
    $self->{logger}->verb_log_msg("Checking node states\n");
    my $pre_node_states = $self->{preevent}->get_node_states();
    my $post_node_states = $self->{postevent}->get_node_states();

    foreach my $node ( sort( keys( %{$pre_node_states} ))) {

        $self->{logger}->verb_log_msg(
            "$node: Preevent state: $pre_node_states->{$node}, " .
            "Postevent state: $post_node_states->{$node}\n");

        if ($pre_node_states->{$node} ne $post_node_states->{$node}) {

            $self->{logger}->err_msg(2, 20,
                "%s: Unexpected state change: node %s\n" .
                "Pre-event state %s, Post-event state %s\n",
                "SERVER_DOWN", $node, 
                Event::get_node_statename($pre_node_states->{$node}),
                Event::get_node_statename($post_node_states->{$node}));
            $retval = Event::Failed;
        }
    }

#
# checking RG state changes
    $self->{logger}->verb_log_msg("Checking RG states\n");
    foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {

        $self->{logger}->verb_log_msg("Resource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});
        my $rgtype = $self->{StaticConfig}->get_rg_type($resgrp->{group});

        # have nodes moved into the ERROR state?
        foreach my $node ( sort( keys( %{$post_states} ))) {

            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"),
                $post_states->{$node});

            if ( $post_states->{$node} eq "ERROR" &&
                 $pre_states->{$node} ne "ERROR" ) {

                # it is OK for this RG to move into the error state if
                # the RG is concurrent, but it must stay online on all
                # other nodes
                if ( $resgrp->{group} ne $rgname ||
                     $node ne $self->{nodename}  ||
                     $rgtype == StaticConfig::nonconcurrent ) {

                    $self->{logger}->err_msg(2, 21,
                        "%s: Unexpected state change\n" .
                        "Resource Group %s changed to %s state on noded %s\n",
                        "SERVER_DOWN", $resgrp->{group}, "ERROR", $node);
                    $retval = Event::Failed;
                }
            }
        }

        # have nodes unexpectedly gone OFFLINE? come ONLINE?
        PRENODE: foreach my $node ( sort( keys( %{$pre_states} ))) {
            if ( $pre_states->{$node} eq "ONLINE" &&
                 $post_states->{$node} ne "ERROR" ) {

                if ( $resgrp->{group} ne $rgname       &&
                     $post_states->{$node} ne "ONLINE" ) {

                        $self->{logger}->err_msg(2, 21,
                           "%s: Unexpected state change\n" .
                           "Resource Group %s changed to %s state on node %s\n",
                           "SERVER_DOWN", $resgrp->{group}, "OFFLINE", $node);
                        $retval = Event::Failed;

                } elsif ( $resgrp->{group} eq $rgname ) {

                    if ( $rgtype == StaticConfig::concurrent ) {
                        if ( $self->{nodename} ne $node  &&
                             $post_states->{$node} ne "ONLINE" ) {

                        $self->{logger}->err_msg(2, 21,
                           "%s: Unexpected state change\n" .
                           "Resource Group %s changed to %s state on node %s\n",
                           "SERVER_DOWN", $resgrp->{group}, "OFFLINE", $node);
                        $retval = Event::Failed;
                        }
                        next PRENODE;
                    }

                    my $rg_is_online = 0;
                    ON: foreach my $post_node(sort(keys(%{$post_states}))) {
                        if ( $post_states->{$post_node} eq "ONLINE" ) {
                            $rg_is_online = 1;
                            last ON;
                        }
                    }
                    if ( ! $rg_is_online ) {
                        $self->{logger}->err_msg(2, 27,
                           "%s: Unexpected state change\n" .
                           "Resource Group %s changed to %s state.\n",
                           "SERVER_DOWN", $resgrp->{group}, "OFFLINE");
                        $retval = Event::Failed;
                    }
                }
            }
        }
    }

# Process event history records
    $self->{logger}->verb_log_msg("Checking event history\n");

    my @server_complete_events = (
        "server_restart_complete", "server_down_complete");
    my @rg_complete_events = (
        "rg_move_complete", "resource_state_change_complete");
    my @other_events = ("server_restart", "server_down", "rg_move");
    my $server_search_events = join('|', @server_complete_events);
    my $server_search_str =
        "(?:$server_search_events) " .  $self->{nodename} . " $resid";
    my $rg_search_events = join('|', @rg_complete_events);
    my $rg_search_str = "(?:$rg_search_events)";

    my $event_complete_found = 0;
    my $event_history = $self->{postevent}->get_event_history(
        @server_complete_events, @rg_complete_events, @other_events);

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);
    
    foreach my $node ( sort( keys ( %{$event_history} ))) {
        # are there any interesting event failure messages?
        my @FAILED_records =
            grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        foreach my $record ( @FAILED_records ) {
            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                "SERVER_DOWN", $node, $record);
            $retval = Event::Failed;
        }

        # is there an event completed record that satisfies this event?
        my @SERVER_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $server_search_str/}
                @{$event_history->{$node}};
        my @RG_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $rg_search_str/}
                @{$event_history->{$node}};
        if ( scalar(@SERVER_COMPLETED_records) ||
             scalar(@RG_COMPLETED_records) )
        {
            $event_complete_found = 1;
        }
    }
    if ( ! $event_complete_found ) {
        $self->{logger}->err_msg(2, 23,
            "%s: No EVENT COMPLETED message logged.\n" .
            "Search string: %s\n",
            "SERVER_DOWN", "${EVENT_COMPLETED_MSG}: $server_search_str");
        $self->{logger}->err_msg(2, 23,
            "%s: No EVENT COMPLETED message logged.\n" .
            "Search string: %s\n",
            "SERVER_DOWN", "${EVENT_COMPLETED_MSG}: $rg_search_str");
        $retval = Event::Failed;
    }

    return $retval;
}

=head2 to_string

=head3      Description:

=head3     Arguments:

=head3     Returns:

=head3     Examples:

=cut

sub to_string() {
    my $self = shift;
    my $retstring = sprintf "%s,%s,%s,%s,%s",
       "SERVER_DOWN", $self->{nodename}, $self->{app_server},
       $self->{command}, $self->{description};
    return $retstring;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__

