# @(#)82        1.4 src/43haes/lib/perl/cl_testtool/Event/SITE_MERGE.pm, hacmp.test_tool, 52haes_r540, r5400622c 3/31/06 12:56:30
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
                                                                                          
package Event::SITE_MERGE;

use 5.006;
use strict;
use warnings;
use Event;                                                                                          
require Exporter;
                                                                                          
our @ISA = qw(Event Exporter);
                                                                                          
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
                                                                                          
# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
                                                                                          
) ] );
                                                                                          
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
                                                                                          
our @EXPORT = qw(
                                                                                          
);
our $VERSION = '1.4';
                                                                                          
##
## Put all the methods and global data for your module here
##

=head1 NAME

SITE_MERGE -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::SITE_MERGE;

    my $event = Event::SITE_MERGE->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of SITE_MERGE

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct SITE_MERGE class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::SITE_MERGE object.

=head3     Examples:

    my $event = Event::SITE_MERGE->new( $logger, $staticconfig, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{description} = (join ',', @{$params}) || "";
    $self->{valid} = 1;
    $self->{event_name} = "SITE_MERGE";
    validate( $self );
    return bless $self, $class;
}


=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to SITE_MERGE object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->is_valid() ) {
        print "Event is valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;

    $self->{logger}->phase_header("Validate " . $self->{event_name});

    foreach my $net (@{$self->{StaticConfig}->{networks}}) {
        if ($net->{nimname} eq "XD_data") {
            push @{$self->{network_list}}, $net->{name};
        }
    }
    if (!defined $self->{network_list}) {
        $self->{logger}->err_msg(2, 46,
            "%s: Invalid Test\n" .
            "There are no XD_data networks in the cluster configuration.\n",
            $self->{event_name});
        $self->{valid} = 0;
    }

    if (scalar(@{$self->{StaticConfig}->{sites}}) < 1) {
        $self->{logger}->err_msg(2, 47,
            "%s: Invalid Test\n" .
            "There are no sites in the cluster configuration.\n",
            $self->{event_name});
        $self->{valid} = 0;
    }

    # Get the node list from each network, and stick each node in the node
    # hash.  Then store each node from the hash into a list.  This will ensure
    # that we get all nodes on all XD_data networks without building a list
    # with duplicate nodenames.
    my %node_hash;
    foreach my $net (@{$self->{network_list}}) {
        my @node_list = @{$self->{StaticConfig}->get_nodes_on_network($net)};
        foreach my $node (@node_list) {
            $node_hash{$node} = 1;
        }
    }
    foreach my $node (keys %node_hash) {
        push @{$self->{node_list}}, $node;
    }
} 

    
=head2 is_rational -- Check whether it is rational to run the event .

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
The entrance criteria for each event will be quite specific to that event, and
so each type of SITE_MERGE class will most likely over-ride the is_rational()
method of the Event class. 

is_rational() will typically only be called from within the execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1 = event is rational

    0 = event is not rational

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational() {
    my $self = shift;

    $self->{logger}->phase_header("is_rational $self->{event_name}");

    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", $self->{event_name});
        return 0;
    }

    my $nodes_alive = 0;
    foreach my $node (@{$self->{node_list}}) {
        if (Event::is_node_active($self, $node)) {
            $nodes_alive = 1;
            last;
        }
    }
    if (!$nodes_alive) {
        $self->{logger}->warn_msg(2, 78,
            "%s: cannot be run: Not Rational\n" .
            "There are no nodes active on any XD_data networks.\n",
            $self->{event_name});
        return 0;
    } 

    my $net_down = 0;
    foreach my $net (@{$self->{network_list}}) {
        if (Event::is_network_down_anywhere($self, $net)) {
            $net_down = 1;
            last;
        }
    }
    if (!$net_down) {
        $self->{logger}->warn_msg(2, 80,
            "%s cannot be run: Not Rational\n" .
            "There are no XD_data networks down anywhere in the cluster.\n",
            $self->{event_name});
        return 0;
    }

    return 1;
}

=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of SITE_MERGE class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }
    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }

    my $nodes = join(',', @{$self->{StaticConfig}->{node_names}});

    # Execute the NETWORK_UP_LOCAL test for each node for each
    # XD_data network.  Abort if a fatal error occurs.
    my $retval = Event::Continue;
    $self->{event_to_run} = "NETWORK_UP_LOCAL";
    foreach my $net (@{$self->{network_list}}) {
        my $rc = Event::execute_cmd($nodes, $self, $net);
        if ($rc != Event::Continue) {
            $retval = $rc;
            # If this was a fatal error, then stop.
            if ($rc == Event::Fatal) {
                last;
            }
        }
    }

    return $retval;
}


=head2  wait_for_stable -- waits until the cluster is stable.

=head3     Description:

Event::SITE_MERGE::wait_for_stable() will determine the failure
detection rate for the NIM type of the network being stopped, and wait
that long before handing control off to Event::wait_for_stable.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue == cluster reached the stable state.
    Event::Timeout  == cluster did not reach the stable state, and the
                       timeout for this event was reached.
    Event::Fatal    == a fatal error occured testing for stability

=head3     Examples:

    if ( $event->wait_for_stable() eq Event::Timeout ) {
        print "Timeout waiting for cluster to stabilize.\n".
    }   
   
=cut

sub wait_for_stable ( )
{
    my $self = shift;

    $self->{logger}->info_msg(2, 48,
        "%s: Waiting for %d seconds to allow a %s event to occur.\n",
        $self->{event_name}, 60, "site_merge");
    sleep 60;
        
    return Event::wait_for_stable($self);
}       


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status()
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;

    $self->{logger}->phase_header(
            "$self->{event_name}: Checking post-event status"); 

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

    my @post_active_nodes = @{$self->{postevent}->{active_nodes}};
    my @pre_active_nodes = @{$self->{preevent}->{active_nodes}};

#
# no node has changed state
#
    if (Event::have_nodes_changed_state($self)) {
        $retval = Event::Failed;
    }

# checking RG state changes
    if (Event::have_any_rgs_changed_state($self)) {
        $retval = Event::Failed;
    }

# Process event history record
    $self->{logger}->verb_log_msg("Checking event history\n");

    my $event_complete_found = 0;
    my $event_history = $self->{postevent}->get_event_history(
            "network_up", "network_up_complete",
            "join_interface", "join_standby",
            "site_merge", "site_merge_complete");
    # We need to search for -1 as the nodename.  -1 means that the event
    # was a global network down.
    my $search_str = "site_merge_complete"; 

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);

    foreach my $node ( sort( keys ( %{$event_history} ))) {
        my @FAILED_records =
            grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        my @COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $search_str/}
                @{$event_history->{$node}};
                                                                                
        foreach my $record ( @FAILED_records ) {
            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                $self->{event_name}, $node, $record);
            $retval = Event::Failed;
        }
        if ( scalar(@COMPLETED_records) ) {
            $event_complete_found = 1;
        }
    }
                                                                                
    if ( ! $event_complete_found ) {
        $self->{logger}->err_msg(2, 23,
            "%s: No EVENT COMPLETED message logged.\n" .
            "Search string: %s\n",
            $self->{event_name}, "${EVENT_COMPLETED_MSG}: $search_str");
        $retval = Event::Failed;
    }
                                                                                
    return $retval;
}


=head2 to_string -- return a string version of event

=head3     Description:

to_string() will build a Test record to insert in a Test Suite file.

=head3     Arguments:

    none.

=head3     Returns:

    Test event record as scalar value

=head3     Examples:

    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }

=cut

sub to_string() {
                                                                                       
    my $self = shift;
    my $retstring = sprintf "%s,%s",
                $self->{event_name}, $self->{description};
    return $retstring;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__

