# @(#)83        1.5 src/43haes/lib/perl/cl_testtool/Event/SITE_UP.pm, hacmp.test_tool, 52haes_r540, r5400622c 3/31/06 16:05:47
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package Event::SITE_UP;

use 5.006;
use strict;
use warnings;
use Event;
use StaticConfig;
require Exporter;

our @ISA = qw(Event Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '1.5';

##
## Put all the methods and global data for your module here
##

=head1 NAME

SITE_UP -- 
Perl extension providing the Event::SITE_UP object class.

=head1 SYNOPSIS

    use Event::SITE_UP;

    my $event = Event::SITE_UP->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

This module provides the Event::SITE_UP object class for use by the HACMP
Cluster Test Tool. This class inherits from the Event class, and so will use
all of the methods defined for that class unless the methods are over-ridden
here. The Event::SITE_UP class represents the Test Tool's SITE_UP event, and
will initiate starting of cluster services on one node (or on all nodes if the
keyword "ALL" is provided in place of an actual node name).

=head1 METHODS

=head2 new -- create a new instance of SITE_UP

=head3     Description:

new() is the constructor for the Event::SITE_UP class. It takes a reference
to an an array of parameters that are required for the SITE_UP event: node
name and description. This method will verify that the specified node name
exists in the cluster configuration on the local node, and will set the
object's "valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::SITE_UP object.

=head3     Examples:

    my $event = Event::SITE_UP->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{site} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{event_name} = "SITE_UP";
    $self->{valid} = 1; 
    validate( $self );
    return bless $self, $class;
}

=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to SITE_UP object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->{valid} == 0 ) {
        print "Event is not valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;
    
    $self->{logger}->phase_header("Validate $self->{event_name}");

    if (!$self->{StaticConfig}->is_site_valid($self->{site})) {
        $self->{valid} = 0;
        $self->{logger}->err_msg(2, 49,
            "%s: Invalid Test\n" .
            "Site %s is not in the cluster configuration.\n",
            $self->{event_name}, $self->{site});
    }
}

=head2 is_rational -- Check whether the SITE_UP event is rational.

=head3     Description:

is_rational checks whether the entrance criteria for the event are satisfied,
namely that the specified node, or at least one node (if "ALL" specified)
is currently offline.

=head3     Arguments:

    none.

=head3     Returns:

    1 == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() == 0 ) {
        print "Event is not rational.\n";
    }

=cut
    
sub is_rational()
{
    my $self = shift;

    $self->{logger}->phase_header("is_rational $self->{event_name}");

    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", $self->{event_name});
        return 0;
    }

    my $inactive_nodes = 0;
    my @nodes_array;
    foreach my $node (@{$self->{StaticConfig}->get_site_nodes($self->{site})})
    {
        if (!Event::is_node_active($self, $node)) {
            $inactive_nodes = 1;
            push @nodes_array, $node;
        }
    }
    if (!$inactive_nodes) {
        $self->{logger}->warn_msg(2, 50,
            "%s: cannot be run: Not Rational\n" .
            "No inactive nodes at site %s\n",
            $self->{event_name}, $self->{site});
        return 0;
    }

    $self->{nodelist} = join ',', @nodes_array;

    return 1;
}


=head2 execute -- Execute the SITE_UP event.

=head3     Description:

execute() will control, at a high level, the execution of the SITE_UP event.
It will determine the node (or list of nodes) to be started, capture the node
and RG states for the cluster, and run the necessary command (via the Test
Toll Event Controller) to start the nodes.

NOTE: The event is assumed to be rational when execute is run. It is the 
callers responsibility to call the is_rational() method before calling the
execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }

    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }

    $self->{event_to_run} = "NODE_UP";
    return Event::execute_cmd (
        $self->{StaticConfig}->{local_node}, $self, $self->{nodelist});
}


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether the SITE_UP event completed successfully. It will also report the
final event status to the log file and the console.

=head3     Arguments:

    $tag:   event start tag

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status()
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;
    $self->{logger}->phase_header(
           "$self->{event_name}: Checking post-event status"); 
    my @event_nodes = split /,/, $self->{nodelist};
    $self->{logger}->verb_log_msg("Event Nodes: @event_nodes\n");

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

    my @post_active_nodes = @{$self->{postevent}->{active_nodes}};
    my @pre_active_nodes = @{$self->{preevent}->{active_nodes}};
    $self->{logger}->verb_log_msg(
            "pre-event online nodes: @pre_active_nodes\n");
    $self->{logger}->verb_log_msg(
            "post-event online nodes: @post_active_nodes\n");

#
#       Post event status checks
#
# are event nodes online ?
#
    foreach my $event_node (@event_nodes) {
        if ((scalar (grep {/^$event_node$/} @post_active_nodes)) < 1) {
            $self->{logger}->err_msg(2, 25,
                "%s: Bad Post Event Status\n" .
                "Node %s is not active in cluster\n", 
                $self->{event_name}, $event_node);
            $retval = Event::Failed;
        }
    }

#
# no other node changes state
#
    $self->{logger}->verb_log_msg("Checking node states\n");
    my $pre_node_states = $self->{preevent}->get_node_states();
    my $post_node_states = $self->{postevent}->get_node_states();

    STATECHECK: foreach my $node ( sort( keys( %{$pre_node_states} ))) {
        $self->{logger}->verb_log_msg(
            "$node: Preevent state: $pre_node_states->{$node}, " .
            "Postevent state: $post_node_states->{$node}\n");

        # the event node is supposed to change state, so ignore it here
        if ( scalar( grep {/^$node$/} @event_nodes )) {
           next STATECHECK;
        }

        if ($pre_node_states->{$node} ne $post_node_states->{$node}) {
            $self->{logger}->err_msg(2, 20,
                "%s: Unexpected state change: node %s\n" .
                "Pre-event state %s, Post-event state %s\n",
                $self->{event_name}, $node, 
                Event::get_node_statename($pre_node_states->{$node}),
                Event::get_node_statename($post_node_states->{$node}));
            $retval = Event::Failed;
        }
    }

#
# checking RG state changes
    $self->{logger}->verb_log_msg("Checking RG states\n");
    foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {

        $self->{logger}->verb_log_msg("Resource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});
        my $rgtype = $self->{StaticConfig}->get_rg_type($resgrp->{group});

        # have nodes moved into the ERROR state?
        foreach my $node ( sort( keys( %{$post_states} ))) {
            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"),
                $post_states->{$node});

            if ( $post_states->{$node} eq "ERROR" &&
                 $pre_states->{$node} ne "ERROR" )
            {
                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s.\n",
                    $self->{event_name}, $resgrp->{group}, "ERROR", $node);
                $retval = Event::Failed;
            }
        }

        # have RGs unexpectedly gone OFFLINE?
        PRENODE: foreach my $node ( sort( keys( %{$pre_states} ))) {

            if ( $pre_states->{$node} eq "ONLINE"  &&
                 $post_states->{$node} ne "ERROR"  ) {

                # concurrent RG should be in ONLINE state on every node
                # where it _was_ in ONLINE state
                if ( $rgtype == StaticConfig::concurrent ) {
                    if ( $post_states->{$node} ne "ONLINE" ) {
                        $self->{logger}->err_msg(2, 21,
                          "%s: Unexpected state change\n" .
                          "Resource Group %s changed to %s state on node %s.\n",
                          $self->{event_name}, $resgrp->{group},
                          "OFFLINE", $node);
                        $retval = Event::Failed;
                    }
                    next PRENODE;
                }

                # non-concurrent RG should be ONLINE somewhere
                my $rg_is_online = 0;
                POST: foreach my $post_node ( sort( keys( %{$post_states} ))) {
                    if ( $post_states->{$post_node} eq "ONLINE" ) {
                        $rg_is_online = 1;
                        last POST;
                    }
                }
                if ( ! $rg_is_online ) {
                    $self->{logger}->err_msg(2, 21,
                        "%s: Unexpected state change\n" .
                        "Resource Group %s changed to %s state on node %s.\n",
                        $self->{event_name}, $resgrp->{group},
                        "OFFLINE", $node);
                    $retval = Event::Failed;
                }
            }
        }
    }

# Process event history records
    $self->{logger}->verb_log_msg("Checking event history\n");
    my %event_complete_found;
    foreach my $comp_node (@event_nodes) {
        $event_complete_found{$comp_node} = 0;
    }
    my $event_history = $self->{postevent}->get_event_history(
            "node_up", "node_up_complete");

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);

    # are there any interesting event failure messages?
    foreach my $node ( sort( keys ( %{$event_history} ))) {
        my @FAILED_records = 
            grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        foreach my $record ( @FAILED_records ) {
            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                $self->{event_name}, $node, $record);
            $retval = Event::Failed;
        }
    }

    # have we found an EVENT COMPLETED message for each event node?
    COMPCHECK: foreach my $check_node (@event_nodes) {
        my $search_str = "node_up_complete " . $check_node;
        foreach my $node ( sort( keys ( %{$event_history} ))) {
            my @COMPLETED_records = 
                grep {/^.* ${EVENT_COMPLETED_MSG}: $search_str/} 
                    @{$event_history->{$node}};
            if ( scalar(@COMPLETED_records) ) {
                $event_complete_found{$check_node} = 1;
                next COMPCHECK;
            }
        }
    }
    foreach my $msgnode (@event_nodes) {
        if ( ! $event_complete_found{$msgnode} ) {
            my $search_str = "node_up_complete " . $msgnode;
            $self->{logger}->err_msg(2, 26,
                "%s: No EVENT COMPLETED message logged for node %s.\n" .
                "Search string: %s\n",
                $self->{event_name}, $msgnode, "${EVENT_COMPLETED_MSG}: $search_str");
            $retval = Event::Failed;
        }
    }

    return $retval;
}

=head2 to_string -- return a string version of event

=head3     Description:

to_string() will build a Test record to insert in a Test Suite file.

=head3     Arguments:

    none.

=head3     Returns:

    Test event record as scalar value

=head3     Examples:

    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }
    
=cut

sub to_string() {

    my $self = shift;
    my $retstring = sprintf "%s,%s,%s", 
                $self->{event_name}, $self->{site}, $self->{description};
    return $retstring;
}


# it's a module, so the last statement should always just be '1;'
1;
__END__
