# @(#)84        1.8 src/43haes/lib/perl/cl_testtool/Event/VG_DOWN.pm, hacmp.test_tool, 52haes_r540, r5400622c 3/31/06 16:12:34
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package Event::VG_DOWN;

use 5.006;
use strict;
use warnings;

use ODM;
use Logger;
use Event;

require Exporter;

our @ISA = qw(Event Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);
our $VERSION = '1.8';

##
## Put all the methods and global data for your module here
##

=head1 NAME

VG_DOWN -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::VG_DOWN;

    my $event = Event::VG_DOWN->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of VG_DOWN

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct VG_DOWN class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::VG_DOWN object.

=head3     Examples:

    my $event = Event::VG_DOWN->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{volumegroup} = (shift @{$params}) || "";
    $self->{nodename} = (shift @{$params}) || "";
    $self->{description} = (join ',', @{$params}) || "";
    $self->{valid} = 1;
    $self->{event_name} = "VG_DOWN";
    validate( $self );
    return bless $self, $class;
}


=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to VG_DOWN object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->is_valid() ) {
        print "Event is valid.\n";
    }

=cut

sub validate()
{
    my $self = shift;

    $self->{logger}->phase_header("Validate " . $self->{event_name});

# check that the $nodename field is either "ANY" or "ALL" or a cluster node.
    my $nodes_configured = join " ",@{$self->{StaticConfig}->{node_names}};
    $self->{logger}->verb_log_msg("\tEvent node: $self->{nodename}\n");
    $self->{logger}->verb_log_msg("\tConfigured nodes: $nodes_configured\n");
    
    if ($self->{nodename} ne "ANY" && $self->{nodename} ne "ALL") {
        my $valid = $self->{StaticConfig}->is_node_valid($self->{nodename});
        if (!$valid) {
            $self->{logger}->err_msg(2, 2,
                "%s: Invalid Test\n".
                "Node %s not in cluster configuration\n",
                $self->{event_name}, $self->{nodename});
            $self->{valid} = 0;
        }
    }

# check that the vg exists and is in an RG associated with given node
    my $rgname = $self->{StaticConfig}->get_rg_for_res(
                $self->{volumegroup}, "VOLUME_GROUP");
    if ($rgname) {
        $self->{vg_type} = "VOLUME_GROUP";
    }
    else {
        $rgname = $self->{StaticConfig}->get_rg_for_res(
                $self->{volumegroup}, "CONCURRENT_VOLUME_GROUP");
        if ($rgname) {
            $self->{vg_type} = "CONCURRENT_VOLUME_GROUP";
        }
    }

    $self->{logger}->verb_log_msg("\tVG: $self->{volumegroup}, RG Name: $rgname\n");
    if ( $rgname ) {
        if ($self->{nodename} ne "ANY" && $self->{nodename} ne "ALL") {
            my @nodes = $self->{StaticConfig}->get_nodes_for_rg($rgname);
            $self->{logger}->verb_log_msg("\tRG Nodes: @nodes\n");
            my @matches = grep {/^$self->{nodename}$/} @nodes;
            if ( scalar(@matches) < 1 ) {
                $self->{logger}->err_msg(2, 44,
                    "%s: Invalid Test\n" .
                    "Node %s is not in resource group %s with\n" .
                    "volume group %s\n",
                    $self->{event_name}, $self->{nodename},
                    $rgname, $self->{volumegroup});
                $self->{valid} = 0;
            }
        }
    }
    else {
        $self->{logger}->err_msg(2, 45,
            "%s: Invalid Test\n" .
            "Volume group %s is not in the cluster configuration.\n",
            $self->{event_name}, $self->{volumegroup});
        $self->{valid} = 0;
    }
}

=head2 is_rational -- Check whether it is rational to run the event .

=head3     Description:

is_rational() checks whether the entrance criteria for the event are satisfied.
The entrance criteria for each event will be quite specific to that event, and
so each type of VG_DOWN class will most likely over-ride the is_rational()
method of the Event class. 

is_rational() will typically only be called from within the execute() method.

=head3     Arguments:

    none.

=head3     Returns:

    1 == event is rational.
    0 == event is not rational.

=head3     Examples:

    if ( $event->is_rational() ) {
        print "Event is rational.\n";
    }
    
=cut

sub is_rational()
{
    my $self = shift;

    $self->{logger}->phase_header("is_rational " . $self->{event_name});

# check that the cluster is stable
    if ( ! $self->{preevent}->cluster_is_stable() ) {
        $self->{logger}->warn_msg(2, 10,
            "%s cannot be run: Not Rational\n" .
            "Cluster is not stable\n", $self->{event_name});
        return 0;
    }

# is the specified node online?
    my $rgname = $self->{StaticConfig}->get_rg_for_res(
            $self->{volumegroup}, $self->{vg_type});
    my @online_nodes = $self->{preevent}->get_nodes_for_rg($rgname, "ONLINE");
    $self->{logger}->verb_log_msg(
        "VG: $self->{volumegroup}, RG: $rgname, ONLINE NODES: @online_nodes\n");
    if ($self->{nodename} eq "ANY" || $self->{nodename} eq "ALL")
    {
        #if ANY or ALL then check that the RG is online somewhere
        if ( scalar(@online_nodes) < 1 ) {
            $self->{logger}->warn_msg(2, 15,
                "%s cannot be run: Not Rational\n" .
                "Resource Group %s is not online on ANY cluster node\n",
                $self->{event_name}, $rgname);
            return 0;
        }
        elsif ($self->{nodename} eq "ANY") {
            # decide which node to use
            my $node = Event::choose_random_node(@online_nodes);
            push @{$self->{nodelist}}, $node;
            $self->{nodename} = $node;
        }
        elsif ($self->{nodename} eq "ALL") {
            $self->{nodelist} = \@online_nodes;
        }
    }
    else {
        #check that the RG is online to the specified node
        if ( ! scalar( grep {/^$self->{nodename}$/} @online_nodes ) ) {
            $self->{logger}->warn_msg(2, 16,
                "%s cannot be run: Not Rational\n" .
                "Resource Group %s is not ONLINE on node %s\n",
                $self->{event_name}, $rgname, $self->{nodename});
            return 0;
        }
        push @{$self->{nodelist}}, $self->{nodename};
    }

    return 1;
}

=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of VG_DOWN class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::FATAL if unable to collect pre-event state data

    Event::NotRational if it doesn't make sense to run the test

    or the return value from Event::execute_cmd

=head3     Examples:

    $event->execute();
    
=cut

sub execute()
{
    my $self = shift;
    if ( ! Event::collect_preevent($self) ) {
        return Event::Fatal;
    }
    if ( ! $self->is_rational() ) {
        return Event::NotRational;
    }

    my $nodes = join(',', @{$self->{nodelist}});
    return Event::execute_cmd ($nodes, $self, $self->{volumegroup});
}


=head2 check_status -- checks the event's completion status.

=head3     Description:

check_status() will collect and analyze all of the data required to determine
whether an event has completed successfully. It will also report the final 
event status to the log file and the console.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Fatal if unable to collect post-event state data

    Event::Failed if the event has failed (see below for specifics)

    Event::Continue if the event passed

=head3     Examples:

    if ( $event->check_status() eq Event::Failed ) {
        print "Test failed status check\n";
    }
    
=cut

sub check_status ( )
{
    my $self = shift;
    my $tag  = shift;
    my $retval = Event::Continue;
    $self->{logger}->phase_header(
            "$self->{event_name}: Checking post-event status");

    if ( ! Event::collect_postevent($self, $tag) ) {
        return Event::Fatal;
    }

# get the resource and resource group information
    my $resid  = $self->{StaticConfig}->get_res_id(
                        $self->{volumegroup}, $self->{vg_type});
    my $rgname = $self->{StaticConfig}->get_rg_for_res(
                        $self->{volumegroup}, $self->{vg_type});
    my $rgtype = $self->{StaticConfig}->get_rg_type($rgname);
    my $rgid   = $self->{StaticConfig}->get_rg_id($rgname);
    $self->{logger}->verb_log_msg(
            "RESID: $resid, RG: $rgname, RGID: $rgid, TYPE: $rgtype\n");

#
# no node has changed state
#
    if (Event::have_nodes_changed_state($self)) {
        $retval = Event::Failed;
    }

    my $monitor_odm = ODM->new("HACMPmonitor");
    my $failure_action = "fallover";
    my ($monitor) = $monitor_odm->get(-q =>
        "monitor=$self->{volumegroup} AND name=FAILURE_ACTION");
    if ($monitor) {
        $failure_action = $monitor->{value};
    }
    $self->{logger}->verb_log_msg(
        "Volume Group: $self->{volumegroup} " .
        "Failure Action: $failure_action\n");
#
# checking RG state changes
    $self->{logger}->verb_log_msg("Checking RG states\n");
    foreach my $resgrp ( @{$self->{StaticConfig}->{resource_groups}} ) {
        $self->{logger}->verb_log_msg("Resource Group: $resgrp->{group}\n");

        my $pre_states = $self->{preevent}->get_rg_states($resgrp->{group});
        my $post_states = $self->{postevent}->get_rg_states($resgrp->{group});
        my $rgtype = $self->{StaticConfig}->get_rg_type($resgrp->{group});

        # have nodes moved into the ERROR state?
        POSTNODE: foreach my $node ( sort( keys( %{$post_states} ))) {
            $self->{logger}->verb_log_msg(
                "\t\tNode: $node Pre Event State: %s, Post Event State: %s\n",
                (($pre_states->{$node}) ? $pre_states->{$node} : "OFFLINE"),
                $post_states->{$node});

            if ( $post_states->{$node} eq "ERROR" &&
                 $pre_states->{$node} ne "ERROR" )
            {
                # The only time an RG can correctly go into the ERROR state
                # is if it is this RG (the one containing the VG),
                # and the RG is concurrent and has a failure action of
                # "fallover".  In that case, a concurrent RG will not fallover
                # obviously, it will just go into ERROR state.
                # If the failure action is notify, then the notification script
                # should have run instead of an rg_move.
                if ($resgrp->{group} eq $rgname &&
                    $rgtype == StaticConfig::concurrent &&
                    $failure_action eq "fallover")
                {
                    next POSTNODE;
                }
                $self->{logger}->err_msg(2, 21,
                    "%s: Unexpected state change\n" .
                    "Resource Group %s changed to %s state on node %s\n",
                    $self->{event_name}, $resgrp->{group}, "ERROR", $node);
                $retval = Event::Failed;
            }
        }

        # have nodes unexpectedly gone OFFLINE? come ONLINE?
        PRENODE: foreach my $node ( sort( keys( %{$pre_states} ))) {
            if ( $pre_states->{$node} eq "ONLINE" &&
                 $post_states->{$node} ne "ERROR" )
            {
                if ( $resgrp->{group} ne $rgname       &&
                     $post_states->{$node} ne "ONLINE" )
                {
                    $self->{logger}->err_msg(2, 21,
                        "%s: Unexpected state change\n" .
                        "Resource Group %s changed to %s state on node %s\n",
                        $self->{event_name}, $resgrp->{group},
                        "OFFLINE", $node);
                    $retval = Event::Failed;

                }
                elsif ( $resgrp->{group} eq $rgname ) {
                    if (! ($rgtype == StaticConfig::nonconcurrent &&
                           $failure_action eq "fallover") &&
                        $post_states->{$node} ne "ONLINE")
                    {
                        $self->{logger}->err_msg(2, 21,
                           "%s: Unexpected state change\n" .
                           "Resource Group %s changed to %s state on node %s\n",
                           $self->{event_name}, $resgrp->{group},
                           "OFFLINE", $node);
                        $retval = Event::Failed;
                        next PRENODE;
                    }

                    my $rg_is_online = 0;
                    ON: foreach my $post_node(sort(keys(%{$post_states}))) {
                        if ( $post_states->{$post_node} eq "ONLINE" ) {
                            $rg_is_online = 1;
                            last ON;
                        }
                    }
                    if ( ! $rg_is_online ) {
                        $self->{logger}->err_msg(2, 27,
                           "%s: Unexpected state change\n" .
                           "Resource Group %s changed to %s state.\n",
                           $self->{event_name}, $resgrp->{group},
                           "OFFLINE");
                        $retval = Event::Failed;
                    }
                }
            }
        }
    }

# Process event history records
    $self->{logger}->verb_log_msg("Checking event history\n");

    my @rg_complete_events = ("rg_move_complete",
                              "resource_state_change_complete");
    my $server_complete_event = "server_down_complete";
    my @other_events = ("rg_move");
    my $rg_search_events = join('|', @rg_complete_events);
    my $rg_search_str = "(?:$rg_search_events)";
    my $server_search_str = "$server_complete_event \\w+ $resid";

    my $event_history = $self->{postevent}->get_event_history(
        @rg_complete_events, $server_complete_event, @other_events);
    my $event_complete_found = 0;
    my $rg_complete_found = 0;

    my $EVENT_COMPLETED_MSG = Event::get_EVENT_COMPLETED_msg($self);
    my $EVENT_FAILED_MSG = Event::get_EVENT_FAILED_msg($self);

    foreach my $node ( sort( keys ( %{$event_history} ))) {
        # are there any interesting event failure messages?
        my @FAILED_records =
        grep {/^.* ${EVENT_FAILED_MSG}: .*$/} @{$event_history->{$node}};
        foreach my $record ( @FAILED_records ) {
            $self->{logger}->err_msg(2, 22,
                "%s: EVENT FAILED message logged on node %s\n%s\n",
                $self->{event_name}, $node, $record);
            $retval = Event::Failed;
        }

        # is there an event completed record that satisfies this event?
        my @RG_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $rg_search_str/}
                @{$event_history->{$node}};
        my @SERVER_COMPLETED_records =
            grep {/^.* ${EVENT_COMPLETED_MSG}: $server_search_str/}
                @{$event_history->{$node}};

        if (scalar(@RG_COMPLETED_records)) {
            $rg_complete_found = 1;
        }
        if (scalar(@SERVER_COMPLETED_records) || scalar(@RG_COMPLETED_records))
        {
            $event_complete_found = 1;
        }
   }

   if ($rg_complete_found && $failure_action eq "notify") {
       $self->{logger}->err_msg(2, 82,
           "%s: The failure action for volume group %s\n" .
           "is set to run a notification script, but the volume group\n" .
           "failure caused an rg_move event to run instead.\n",
           $self->{event_name}, $self->{volumegroup});
       $retval = Event::Failed;
   }

   if ( ! $event_complete_found) {
       my $str = join(' or ', @rg_complete_events, $server_complete_event);
       $self->{logger}->err_msg(2, 26,
           "%s: No EVENT COMPLETED message logged.\n" .
           "Search string: %s\n",
           $self->{event_name}, "${EVENT_COMPLETED_MSG}: $str");
       $retval = Event::Failed;
   }
 
    return $retval;
}

=head2 to_string

=head3      Description:

=head3     Arguments:

=head3     Returns:

=head3     Examples:

=cut

sub to_string() {
    my $self = shift;
    my $retstring = sprintf "%s,%s,%s",
       $self->{event_name}, $self->{nodename}, $self->{volumegroup},
       $self->{command}, $self->{description};
    return $retstring;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__

