# @(#)63        1.8 src/43haes/lib/perl/cl_testtool/Event/WAIT.pm, hacmp, 52haes_r540, r5400622c 4/4/06 15:27:25
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package Event::WAIT;

use 5.006;
use strict;
use warnings;
use Event;
require Exporter;

our @ISA = qw(Event Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration       use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);
our $VERSION = '1.8';

##
## Put all the methods and global data for your module here
##

=head1 NAME

WAIT -- 
Perl extension providing the object class for the named event.

=head1 SYNOPSIS

    use Event::WAIT;

    my $event = Event::WAIT->new( $logger, $staticconfg, \@params );
    
    if ( $event->is_valid() ) {
        my $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status() if $eventRC eq Event::Continue;
    }

=head1 DESCRIPTION

For each supported type of test event, one Perl module is provided that
supplies the required event specific code. These classes inherit from the 
Event class, and so will use all of the methods defined for that class unless 
the methods are over-ridden by the event specific class.

=head1 METHODS

=head2 new -- create a new instance of WAIT

=head3     Description:

Each event specific class must have its own new() method. The new method will
create an instance of the correct WAIT class and will perform the
appropriate checks of the event parameters to ensure that the event parameters
do not conflict with the actual cluster configuration, setting the object's
"valid" variable accordingly.

=head3     Arguments:

    This constructor takes two arguments:

        $logger: a reference to a previously created Logger object.

        $staticconfig: a reference to a previously created StaticConfig object

        $paramters: a reference to the parameters array.

=head3     Returns:

    A new Event::WAIT object.

=head3     Examples:

    my $event = Event::WAIT->new( $logger, $staticconfg, \@parameters );

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{StaticConfig} = shift;
    my $params = shift;
    $self->{seconds} = (shift @{$params}) || "";
    $self->{description} = (shift @{$params}) || "";
    $self->{event_name} = "WAIT";
    $self->{valid} = 1;
    validate( $self );
    return bless $self, $class;
}


=head2 validate -- validates the event parameters.

=head3     Description:

validate() will validate the the event parameters to ensure that they reflect
the base HACMP configuration. For example, it will check to ensure that the
specified node name or RG name exists in the HACMP configuration. If it finds
a problem with the parameters, it will set the object's 'valid' data member
to 0. validate() will virtually always be called directly from within
the new() method.

=head3     Arguments:

    $event: reference to WAIT object.

=head3     Returns:

    none.

=head3     Examples:

    validate ( $event );
    if ( $event->is_valid() == 0 ) {
        print "Event is not valid.\n";
    }

=cut

sub validate()
{
# verify that the 'seconds' parameter is >= 0 
    my $self = shift;
    if ($self->{seconds} <= 0) {
        $self->{logger}->err_msg(2,4,
            "%s: Invalid Test.\nTime must be greater than 0.\n", "WAIT");
        $self->{valid} = 0;
    }
}

=head2 execute -- Execute the event.

=head3     Description:

execute() will control, at a high level, the execution of the specified event.
As each event performs a different set of actions, each type of WAIT class
will most likely over-ride the execute() method of the Event class.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue

=head3     Examples:

    $event->execute();
    
=cut
 
sub execute()
{
    my $self = shift;
    $self->{logger}->info_msg(2,5,
        "Test Tool will sleep for %ld seconds.\n", $self->{seconds});
    sleep $self->{seconds};
    return Event::Continue;
}


=head2 to_string -- return a string version of event

=head3     Description:

to_string() will build a Test record to insert in a Test Suite file.

=head3     Arguments:

    none.

=head3     Returns:

    Test event record as scalar value

=head3     Examples:

    for my $event (@event_list) {
        print TESTSUITE, $event->to_string() . "\n";
    }
    
=cut

sub to_string() {

    my $self = shift;
    my $retstring = sprintf "%s,%s,%s", 
                "WAIT", $self->{seconds}, $self->{description};
    return $retstring;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__

