# @(#)49        1.22 src/43haes/lib/perl/cl_testtool/EventDriver.pm, hacmp, 52haes_r541, 0736A_hacmp541 3/2/07 13:43:25
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/lib/perl/cl_testtool/EventDriver.pm 1.22 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2004,2007 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package EventDriver;

use 5.006;
use strict;
use warnings;

use lib '/usr/es/lib/perl';
use lib '/usr/es/lib/perl/cl_testtool';

use MsgCatalog;
use Logger;
use VariableTable;
use StaticConfig;
use ODM;
use File::Basename;

use Event;
use Event::CLSTRMGR_KILL;
use Event::CLSTRMGR_KILL_COMPLETE;
use Event::NETWORK_DOWN_LOCAL;
use Event::NETWORK_DOWN_GLOBAL;
use Event::NETWORK_UP_LOCAL;
use Event::NETWORK_UP_GLOBAL;
use Event::NODE_DOWN_FORCED;
use Event::NODE_DOWN_GRACEFUL;
use Event::NODE_DOWN_TAKEOVER;
use Event::NODE_UP;
use Event::SERVER_DOWN;
use Event::WAIT;
use Event::FAIL_LABEL;
use Event::JOIN_LABEL;
use Event::VG_DOWN;
use Event::SITE_ISOLATION;
use Event::SITE_MERGE;
use Event::SITE_DOWN_GRACEFUL;
use Event::SITE_DOWN_TAKEOVER;
use Event::SITE_UP;
use Event::RG_OFFLINE;
use Event::RG_ONLINE;
use Event::RG_MOVE;
use Event::RG_MOVE_SITE;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '1.22';

=head1 NAME

EventDriver -- 
Perl extension for driving test events for the HACMP Cluster Test Tool

=head1 SYNOPSIS

    use EventDriver;

    my $eventDriver = 
            EventDriver->new( $logger, $eventFileName, $varFileName );
    
    if ( $eventDriver->is_valid() )
    {
        do {
            $retVal = $eventDriver->run_next_event();
        } until ( $retVal eq "complete" || $retVal eq "failed" ) ;
    }

=head1 DESCRIPTION

This module is the component within the Cluster Test Tool that represents the
Event File. An Event File is a list of newline separated records, each of which
specifies a test event to be executed by the test tool. The EventDriver class
will process the Event File to build an ordered array of Event objects. As
illustrated above in the synopsis, the component that instantiates the
EventDriver object will have overall control over whether/when the next event
is run, but the EventDriver will decide which event to run and whether that
event was successful.

=head1 METHODS

=head2 new -- create a new instance of EventDriver

=head3     Description:

new() is the constructor for the EventDriver. This allows the developer
to separate the reading and interpretation of the event file from the
overall control of the Cluster Test Tool utility. This method will
initialize a VariableTable object using the specified variables file, 
read event records from the specified event file, substitue values as
necessary by looking them up in the VariablesTable, and convert the
resulting event data into an array of Event objects.

=head3     Arguments:

    This constructor takes three arguments:

        $logger: a reference to a previously created Logger object.

        $eventFileName: the path to an event file to read from.

        $varFileName: the path to a variables file, if one was specified.

=head3     Returns:

    A new EventDriver object.

=head3     Examples:

    my $eventDriver = EventDriver->new( $logger, $eventFile, $varFile );

=cut

sub new ( )
{
    my $proto  = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{logger}   = shift;
    $self->{fileName} = shift;
    my $varfile       = shift;
    $self->{varTable} = VariableTable->new( $varfile, $self->{logger} );
    $self->{errAbort} = shift || 0; # set to 0 if undefined
    $self->{errAbort} = (($self->{errAbort}) ? 1 : 0);
    $self->{config}   = StaticConfig->new( $self->{logger} );
    $self->{eventnum} = 0;
    $self->{valid}    = 1;
    $self->{timestamp} = time;
    $self->{event_failed} = 0;

    # call private method to initialize event queue
    buildEventQueue( $self );

    return bless $self, $class;
}


=head2 is_valid -- return value of object's "valid" parameter.

=head3     Description:

is_valid() will check the object's "valid" parameter, which is
indicates whether the object was initialized successfully. Errors
parsing the event file will set the "valid" value to "false", as will
errors creating the VariableTable.

=head3     Arguments:

    none.

=head3     Returns:

    1 == object is valid
    0 == object is invalid

=head3     Examples:

    if ( ! $eventDriver->is_valid() ) {
        print "Failed initializing Event Driver.\n";
    }

=cut

sub is_valid ( )
{
    my $self = shift;
    if ( $self->{varTable}->is_valid() && $self->{config}->is_valid() ) {
        return $self->{valid};
    }
    return 0;
}


=head2 run_next_event -- run the next event on the queue

=head3     Description:

run_next_event() will take the next event off the object's queue,
execute it, wait for the cluster to stabilize, and report the end
result of the event.

=head3     Arguments:

    none.

=head3     Returns:

    Event::Continue     == event was run (doesn't mean test passed)
    Event::Fatal        == fatal error; no more events should be run
    Event::Failed       == event failed; no more events should be run
    Event::Complete     == there are no more events on the queue to run
    Event::Timeout      == the event timed out waiting for cluster stability

=head3     Examples:

    do {
        $retVal = $eventDriver->run_next_event();
    } while ( $retVal eq Event::Continue ) ;
    
=cut

sub run_next_event ( )
{
    my $self = shift;
    my $eventRC = Event::Fatal;

    if ( ! $self->is_valid() ) {
        $eventRC = Event::Fatal;

    } elsif ( scalar( @{$self->{eventQueue}} ) == 0 ) {
        $eventRC = Event::Complete;

    } else {

        my $event = shift ( @{$self->{eventQueue}} );
        # call private method to start event logging, etc.
        init_for_event($self, $event) or return Event::Fatal;

        $eventRC = $event->execute();
        $eventRC = $event->wait_for_stable() if $eventRC eq Event::Continue;
        $eventRC = $event->check_status($self->{EST})
                                             if $eventRC eq Event::Continue;
        $eventRC = Event::generic_check_status($event)
                                             if $eventRC eq Event::Continue;

        # call private method to complete event logging, etc.
        cleanup_after_event($self, $event, $eventRC) or return Event::Fatal;
    }

    # don't return event failure if the user doesn't want to abort
    if($eventRC == Event::Failed && ! $self->{errAbort}) {
        $self->{event_failed} = 1;
        $eventRC = Event::Continue 
    }
    # return continue if the event was irrational
    # there might still be some rational events on the queue
    if ($eventRC == Event::NotRational) {
        $eventRC = Event::Continue;
    }

    return $eventRC;
}



=head2 buildEventQueue -- read test plan and build queue of events

=head3      Description:

Read test specs from the test plan and build a queue of the tests to
be run. Sets $eventDriver->{valid} to 0 if an invalid test is discovered.

=head3      Arguments:

    none

=head3      Returns:

    none

=cut

sub buildEventQueue ( )
{
    my $self = shift;

    $self->{logger}->phase_header("Building Test Queue");
    $self->{logger}->verb_log_msg("\tTest Plan: %s\n", $self->{fileName});

    # open event file
    if ( ! (open(EVENTFILE, $self->{fileName})) ) {
        $self->{logger}->err_msg(1, 27,
                "Cannot open test plan or file does not exist: %s\n", 
                $self->{fileName});
        $self->{valid} = 0;
        return;
    }

    # read events one-by-one
    my $event_count = 0;
    $self->{eventQueue} = ();
    while ( my $line = <EVENTFILE> ) {
        chomp $line;
        next if $line =~ /^\s*#.*$/;             # skip comments
        next if $line =~ /^\s*$/;                # skip blank lines
        my @tokens = split /\s*,\s*/, $line;     # get comma-separated values
        my $eventName = shift @tokens;

        $self->{logger}->verb_log_msg("\t\tEvent %ld: %s: %s\n", 
                ($event_count + 1), $eventName, $line);
        
        # create object of correct class
        # by using eval, we can dynamically select the correct class
        # to instantiate without having to use some big switch statement
        # or if/then/else block
        if ( ! ($eventName) || ! defined &{"Event::" . ${eventName} . "::new"}){
            $self->{logger}->err_msg(1, 28,
                    "Invalid Test: '%s' is not recognized.\n", "$eventName");
            $self->{valid} = 0;
            next;
        }

        # lookup variable values
        # ($_ is the variable, not a copy of the value)
        foreach ( @tokens ) {
            $_ = $self->{varTable}->lookup($_);
        }

        my $event = eval "Event::${eventName}->new(\$self->{logger}, \$self->{config}, \\\@tokens);";
        if($@) { # check for run-time errors from eval 
            $self->{logger}->err_msg(1, 29,
                    "Unexpected run-time error: %s\n", "$@");
            $self->{valid} = 0;
            next;
        }
        if ( ! $event->is_valid() ) {
            # this error was already reported
            $self->{valid} = 0;
            next;
        } else {
            push @{$self->{eventQueue}}, $event;
            $event_count++;
        }

        # if this is an automatic restart of the test tool, get the 
        # original parameter values from the event
        if ( $eventName eq "CLSTRMGR_KILL_COMPLETE" ) {
            $self->{errAbort} = $event->{errAbort};
            $self->{logger}->{verbose} = $event->{verbose};
        }
    }
    if ( ! $event_count ) {
        $self->{logger}->err_msg(1, 30,
                "Test Plan contains no valid tests: %s\n", $self->{fileName});
        $self->{valid} = 0;
    }
}


=head2 init_for_event -- initialize EventDriver for event execution

=head3      Description:

Initialize an eventDriver object for test execution. That means setting
the start time, outputing log detail, and tagging hacmp.out on all nodes.
This function will also output the necessary files to cause restart of the
test tool after a node reboot on CLSTRMGR_KILL.

=head3      Arguments:

    $event  = test to be run

=head3      Returns:

    1   success
    0   failure

=cut

sub init_for_event ( )
{
    my $self  = shift;
    my $event = shift;

    # set EST (Event Start Time) variable
    # it might need to be calculated or it might come from the event
    if( ref($event) eq "Event::CLSTRMGR_KILL_COMPLETE" ) {
        $self->{EST} = $event->{EST};
        $self->{eventnum} = $event->{eventnum};
        $self->{logger}->event_continue_msg( $self->{eventnum}, ref($event),
                                             $event->{description});
    } else {
        my ($mon, $day, $hour, $min, $year) = (localtime)[4,3,2,1,5];
        $mon += 1;
        $year += 1900;
        $year = substr ( $year, 2 ); # strip first two digits
        # calculate the EST so that it matches the format required
        # by the '-s' flag to the errpt flag
        $self->{EST} = sprintf "%02d%02d%02d%02d%02d", 
                                $mon, $day, $hour, $min, $year;
        $self->{eventnum}++;
        $self->{logger}->event_start_msg($self->{eventnum}, $event,
                                         $self->{EST});
    }


    if( ref($event) eq "Event::CLSTRMGR_KILL" && 
        $event->local_node_is_control_node() ) {

        # set up the event list to use when restarting after a local
        # clstrmgr_kill event
        my $openrc = open ( LIST, "> " . $self->{config}->{event_file} );
        if ( ! $openrc ) {
            $self->{logger}->err_msg(1, 31,
                "Unable to write to file: %s\n" .
                "CLSTRMGR_KILL test for node %s cannot be run.\n", 
                $self->{config}->{event_file}, $event->{nodename});
            return 0;
        }
        print LIST "#\n# cl_testtool_events list created by cl_testtool\n";
        print LIST "#   event start timestamp: " . $self->{EST} . "\n#\n";

        # the CLSTRMGR_KILL event to be continued should be written out
        # with the original command line parameters included
        my $eventString = $event->to_string( $self->{eventnum}, 
                $self->{errAbort}, $self->{logger}->{verbose}, $self->{EST} );
        print LIST "${eventString}\n";
        foreach my $e ( @{$self->{eventQueue}} ) {
            $eventString = $e->to_string();
            print LIST "${eventString}\n";
        }
        close LIST;

        # write out the restart flag so that clcomd knows to restart
        # the test tool
        $openrc = open ( RESTART, "> " . $self->{config}->{restart_file} );
        if ( ! $openrc ) {
            $self->{logger}->err_msg(1, 31,
                "Unable to write to file: %s\n" .
                "CLSTRMGR_KILL test for node %s cannot be run.\n", 
                $self->{config}->{restart_file}, $event->{nodename});
            return 0;
        }
        my $date = localtime();
        print RESTART "$date\n";
        close RESTART;
    }
    return 1;
}


=head2 cleanup_after_event -- handle all require post-event actions

=head3      Description:

Cleanup after a test has been run. That means outputting log file detail,
and cleaning up the flag files to prevent test tool restart.

=head3      Arguments:

    $event   = test being run
    $eventRC = completion status code for test

=head3      Returns:

    1   success
    0   failure

=cut

sub cleanup_after_event ( )
{
    my $self    = shift;
    my $event   = shift;
    my $eventRC = shift;
    my $event_name = ref($event);

    if ( $event_name eq "Event::CLSTRMGR_KILL_COMPLETE" ) {
        $event_name = "Event::CLSTRMGR_KILL";
    }

    # log event completion
    $self->{logger}->event_complete_msg( $self->{eventnum}, $event_name, 
                    $event->{description}, $self->{EST}, $eventRC);
    #$self->{logger}->event_complete_msg( $self->{eventnum}, $event_name, 
                   #$event->{description}, $self->{EST}, $eventRC);

    if ($eventRC == Event::Failed || $eventRC == Event::Fatal) {
        gather_hacmp_logs($self, $event);
    }

    # make sure that test tool won't restart automatically due
    # to an /etc/cl_testtool_events file that hasn't been removed.
    if( ref($event) eq "Event::CLSTRMGR_KILL_COMPLETE" ||
        ( ref($event) eq "Event::CLSTRMGR_KILL"  && 
          $event->local_node_is_control_node() )     ) {

        if ( -e $self->{config}->{event_file} ) {

            if ( ! rename($self->{config}->{event_file},
                    $self->{config}->{event_old}) ) {
                $self->{logger}->warn_msg(1, 32,
                    "Failed renaming %s to %s\n" .
                    "Please manually rename or remove %s.\n",
                    $self->{config}->{event_file},
                    $self->{config}->{event_old},
                    $self->{config}->{event_file});
            }
        }

        if ( -e $self->{config}->{state_file} ) {

            if ( ! rename($self->{config}->{state_file},
                    $self->{config}->{state_old}) ) {
                $self->{logger}->warn_msg(1, 32,
                    "Failed renaming %s to %s\n" .
                    "Please manually rename or remove %s.\n",
                    $self->{config}->{state_file},
                    $self->{config}->{state_old},
                    $self->{config}->{state_file});
            }
        }

        if ( -e $self->{config}->{restart_file} ) {

            if ( ! unlink($self->{config}->{restart_file}) ) {
                $self->{logger}->warn_msg(1, 33,
                    "Failed deleting file: %s\n" .
                    "Please manually remove the file.\n",
                    $self->{config}->{restart_file});
            }
        }
    }
    return 1;
}


=head2 gather_hacmp_logs -- copy hacmp.out and clstrmgr.debug from all nodes

=head3      Description:

Copy hacmp.out and clstrmgr.debug from all nodes in the cluster
to the local directory /var/hacmp/cl_testtool/<testplan>.<time>

Where:
        <testplan> = test plan name (events file name)
        <time>     = time that this testplan was started

=head3      Arguments:

    $event   = test being run

=head3      Returns:

    1   success
    0   failure

=cut

sub gather_hacmp_logs()
{
    my $self = shift;
    my $event = shift;

    my $timestamp = $self->{timestamp};
    my $path1 = "/var/hacmp/cl_testtool";
    my $tpname = basename($self->{fileName});
    my $log_path = $path1 . "/" . $tpname . "." . $timestamp;
    my $link = "$path1/lastfailed";

    if (! -d $path1) {
        if (!mkdir($path1, 755)) {
            $self->{logger}->warn_msg(1, 59,
                "Could not create directory: %s\n", $path1);
            $self->{logger}->warn_msg(1, 60,
                "Unable to gather hacmp.out and clstrmgr.debug log\n" .
                "files from remote nodes.\n");
            return 0;
        }
    }

    if (! -d $log_path) {
        if (!mkdir($log_path, 755)) {
            $self->{logger}->err_msg(1, 59,
                "Could not create directory: %s\n", $log_path);
            $self->{logger}->err_msg(1, 60,
                "Unable to gather hacmp.out and clstrmgr.debug log\n" .
                "files from remote nodes.\n");
            return 0;
        }
    }

    if (-e $link) {
        my $rc = unlink($link);
        if (!$rc) {
            $self->{logger}->err_msg(1, 63,
                "Could not delete symbolic link: %s\n",
                $link);
            return 0;
        }
    }

    my $rc = symlink($log_path, $link);
    if (!$rc) {
        $self->{logger}->err_msg(1, 64,
            "Could not create symbolic link %s\n" .
            "to link to directory %s\n",
            $link, $log_path);
        return 0;
    }

    $self->{logger}->info_msg(1, 62,
        "Copying log files %s and %s from all nodes to\n" .
        "directory %s\n" .
        "on node %s.\n",
        "hacmp.out", "clstrmgr.debug", $log_path,
        $self->{config}->{local_node});

    my $rcp = "/usr/es/sbin/cluster/utilities/clrcp";
    my $hacmp_out_path = "/var/hacmp/log";
    my $clstrmgr_debug_path = "/var/hacmp/log";

    my $odm = ODM->new("HACMPlogs", ODMDIR=>ODM::DCD());
    my ($logs_obj) = $odm->get(-q => "name=hacmp.out");
    if ($logs_obj) {
        $hacmp_out_path = $logs_obj->{value};
    }
    ($logs_obj) = $odm->get(-q => "name=clstrmgr.debug");
    if ($logs_obj) {
        $clstrmgr_debug_path = $logs_obj->{value};
    }

    foreach my $node (@{$self->{config}->{node_names}}) {
        my $remote_file = "$hacmp_out_path/hacmp.out";
        my $local_file = $self->{eventnum} . "." . $event->{event_name} . "."
                         . "$node.hacmp.out";
        my $rc = system("$rcp $node:$remote_file $log_path/$local_file");
        if (($rc >> 8) != 0) {
            $self->{logger}->warn_msg(1, 61,
                "Could not copy file %s from node %s.\n",
                $remote_file, $node);
        }

        $remote_file = "$clstrmgr_debug_path/clstrmgr.debug";
        $local_file = $self->{eventnum} . "." . $event->{event_name} . "."
                         . "$node.clstrmgr.debug";
        $rc = system("$rcp $node:$remote_file $log_path/$local_file");
        if (($rc >> 8) != 0) {
            $self->{logger}->warn_msg(1, 61,
                "Could not copy file %s from node %s.\n",
                $remote_file, $node);
        }
    }

    return 1;
}


# it's a module, so the last statement should always just be '1;'
1;
__END__
