# @(#)50        1.11 src/43haes/lib/perl/cl_testtool/Logger.pm, hacmp, 52haes_r540 3/29/04 13:06:17
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)50        1.14 src/43haes/lib/perl/cl_testtool/Logger.pm, hacmp, 52haes_r540, r5400622c 3/18/06 14:56:38

package Logger;

use 5.006;
use strict;
use warnings;

use lib '/usr/es/lib/perl';
use lib '/usr/es/lib/perl/cl_testtool';
                                                                                          
use ODM;
use Event;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '1.14';

=head1 NAME

Logger -- 
Perl extension to perform logging for the HACMP Cluster Test Tool

=head1 SYNOPSIS

    use Logger;

    my $logger = Logger->new( $cycle, $verbose );
    
    if ( $logger->is_valid() )
    {
    
        $logger->testing_start_msg(@test_tool_cmd_args);
        $logger->event_start_msg("node_up","104503120503");
        $logger->info_msg(1,342,"Event %s Completion Status %s",("rg_move","Successful"));
        $logger->warn_msg(1,75,"Event %s has run for %s minutes",("join_interface","12"));
        $logger->err_msg(1,453,"Node %s-Invalid HACMP version",("titan"));
        $logger->log_msg("Event %s continues...",("node_up"));
        $logger->event_complete_msg("node_up","104503120503);
        $logger->event_continue_msg("node_up");
        $logger->testing_complete_msg($exit_code);
    
    }


=head1 DESCRIPTION

This is the module within the Cluster Test Tool that performs requests and performs 
logging. The logger is responsible for creating and cycling log files, detecting the
context within which to log (verbose or non-verbose), and writing all Error, Warning, 
and Informational messages to the Cluster Test Tool log and SMIT console. Additionally,
the logger will call the Event Controller when necessary to insert Event Start tags into
hacmp.out and to gather between-event data for the Cluster Test Tool log.

=head1 METHODS

=head2 new -- create a new instance of Logger 

=head3     Description:

new() is the constructor for the Logger. This method will initialize
a new Logger class, and open either an existing log file or create a new one,
whichever is appropriate.

=head3     Arguments:

    The Logger takes two arguments:

    $cycle:     0 = cycle the log file
                1 = do not cycle the log file

    $verbose:   0 = do verbose logging
                1 = do not do verbose logging



=head3     Returns:

    A new Logger object.


=head3     Examples:

    my $logger = Logger::new($verbose, $cycle);
    my $logger = Logger->new($verbose, $cycle);

=cut

sub new  ( )
{
    my $proto  = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    # verbose and cycle flags actually meant _not_ verbose/_don't_ cycle
    $self->{verbose} = shift;
    $self->{verbose} = (($self->{verbose}) ? 0 : 1);
    $self->{cycle} = shift;
    $self->{cycle} = (($self->{cycle}) ? 0 : 1);

    # open catalog and get common message text
    $self->{MsgCat} = MsgCatalog->new("cl_testtool.cat");
    $self->{ERROR} = $self->{MsgCat}->getmsg(1,34,"ERROR");
    $self->{WARNING} = $self->{MsgCat}->getmsg(1,35,"WARNING");
    $self->{FAILED} = $self->{MsgCat}->getmsg(1,36,"FAILED");
    $self->{PASSED} = $self->{MsgCat}->getmsg(1,37,"PASSED");
    $self->{TIMEOUT} = $self->{MsgCat}->getmsg(1,38,"TIMEOUT");
    $self->{FATAL} = $self->{MsgCat}->getmsg(1,39,"FATAL");
    $self->{NOTRATIONAL} = $self->{MsgCat}->getmsg(1,40,"NOT RATIONAL");
    $self->{UNKNOWN} = $self->{MsgCat}->getmsg(1,41,"UNKNOWN");

    my @cluster_nodes = `/usr/es/sbin/cluster/utilities/clnodename`;
    chomp(@cluster_nodes);
    $self->{cluster_nodes} = \@cluster_nodes;

    $self->{valid} = 1;
    cycleopenLogfile( $self );
    return bless $self, $class;
}

=head2 cycleopenLogFile -- cycle the test tool log file

=head3      Description:

Cycle the test tool log file and open the new one. Marks the logger
invalid if this action fails.

=head3      Arguments:

    none

=head3      Returns:

    none

=cut

sub cycleopenLogfile 
{
    # get logfile location
    my $self = shift;
    my $logsodm = ODM->new("HACMPlogs");
    my ($logobj) = $logsodm->get(-q=>{name=>"cl_testtool.log"});
    my $logdir = $logobj->{value} || "/var/hacmp/log";
    my $logname = "$logdir/cl_testtool.log";
    my $logmode = ">>";

    # if cycling enabled
    if ($self->{cycle}) {

        if (-e "$logdir/cl_testtool.log") {

            if (-e "$logdir/cl_testtool.log.1") {

                if (-e "$logdir/cl_testtool.log.2") {
                    #remove oldest file
                    my $cnt = unlink "$logdir/cl_testtool.log.2";
                    if ($cnt < 1) {
                        $self->{MsgCat}->dspmsg(1, 42,
                            "ERROR: cannot remove file: %s\n", 
                            "$logdir/cl_testtool.log.2");
                        $self->{MsgCat}->dspmsg(1,43,
                            "Failed attempting to cycle cl_testtool.log\n");
                        $self->{valid} = 0;
                        return;
                    }
                }
                #rename middle file
                if ( ! rename("$logdir/cl_testtool.log.1",
                              "$logdir/cl_testtool.log.2") ) {
                    $self->{MsgCat}->dspmsg(1, 44,
                        "ERROR: cannot rename file: %s\n", 
                        "$logdir/cl_testtool.log.1");
                    $self->{MsgCat}->dspmsg(1,43,
                        "Failed attempting to cycle cl_testtool.log\n");
                    $self->{valid} = 0;
                    return;
                }
            } # end if -e log.1

            # rename newest file
            if ( ! rename("$logdir/cl_testtool.log",
                          "$logdir/cl_testtool.log.1") ) {
                $self->{MsgCat}->dspmsg(1, 44,
                    "ERROR: cannot rename file: %s\n", 
                    "$logdir/cl_testtool.log");
                $self->{MsgCat}->dspmsg(1,43,
                    "Failed attempting to cycle cl_testtool.log\n");
                $self->{valid} = 0;
                return;
            }
        } # end if -e log

        $logmode = ">"; # truncate on open (though the file isn't really there)
    }

    #cycling is complete, so OK to open the file
    my $logfile;
    if ( ! open($logfile, "$logmode$logname") ) {
        $self->{MsgCat}->dspmsg(1, 45,
            "ERROR: cannot open log file: %s\n", "$logname");
        $self->{valid} = 0;
        return;
    }
    $self->{LOGFILE} = $logfile;

    return;          
}

=head2 is_valid -- return value of object's "valid" parameter

=head3     Description:

is_valid() will check the object's "valid" parameter, which indicates whether 
the object was initialized successfully. Errors encountered while initializing
the Cluster Test Tool log will set the "valid" value to "false".

=head3     Arguments:

    none

=head3     Returns:

    1 == object is valid
    0 == object is invalid

=head3     Examples:

    if ( $logger->is_valid() == 0 ) {
        print "Failed initializing Logger.\n";
    }

=cut

sub is_valid ( )
{
    my $self = shift;
    return $self->{valid};
}

=head2 info_msg -- log informational message 

=head3     Description:

info_msg() Writes informational message(s) to Cluster Test Tool log and the SMIT console. 

=head3     Arguments:

    This method takes 4 arguments:

    $msgset:            The message catalog set number
    
    $msgnum:            The message number

    $default_msg_text:  The default message text

    @msg_text_subs:     The substitution args for the default message string

=head3     Returns:

    none

=head3     Examples:

    $logger->info_msg(1,342,"Event %s Completion Status %s",("rg_move","Successful"));

=cut

sub info_msg ( )
{
    my $self = shift;
    my $msgcat = $self->{MsgCat};
    my $msgset = shift;
    my $msgnum = shift;
    my $default_msg_text = shift;
    my @msg_text_subs = @_;
    $msgcat->dspmsg($msgset,$msgnum,$default_msg_text,@msg_text_subs);
    $self->log_msg($default_msg_text,@msg_text_subs);
}

=head2 warn_msg -- log warning message 

=head3     Description:

warn_msg() Writes warning message(s) to Cluster Test Tool log and the SMIT console. 
This message will prepend the string "WARNING: " to the message string.

=head3     Arguments:

    This method takes 4 arguments:

    $msgset:            The message catalog set number
    
    $msgnum:            The message number

    $default_msg_text:  The default message text

    @msg_text_subs:     The substitution args for the default message string

=head3     Returns:

    none

=head3     Examples:

    $logger->warn_msg(
        1, 75, "Event %s has run for %s minutes",("join_interface","12"));

=cut

sub warn_msg ( )
{
    my $self = shift;
    my $msgcat = $self->{MsgCat};
    my $msgset = shift;
    my $msgnum = shift;
    my $default_msg_text = shift;
    my @msg_text_subs = @_;
    my $finalmsg = $msgcat->getmsg($msgset,$msgnum,$default_msg_text,@msg_text_subs);
    my $outmsg = join ": ",$self->{WARNING},$finalmsg;
    print STDERR "$outmsg";
    $self->log_msg($self->{WARNING} . ": $default_msg_text",@msg_text_subs);
}

=head2 err_msg -- log error message 

=head3     Description:

err_msg() Writes error message(s) to Cluster Test Tool log and the SMIT console.
This method will prepend the string "ERROR: " to the message text.

=head3     Arguments:

    This method takes 4 arguments:

    $msgset:            The message catalog set number
    
    $msgnum:            The message number

    $default_msg_text:  The default message text

    @msg_text_subs:     The substitution args for the default message string

=head3     Returns:

    none

=head3     Examples:

    $logger->err_msg(1,453,"Node %s-Invalid HACMP version",("titan"));

=cut

sub err_msg ( )
{
    my $self = shift;
    my $msgcat = $self->{MsgCat};
    my $msgset = shift;
    my $msgnum = shift;
    my $default_msg_text = shift;
    my @msg_text_subs = @_;
    my $finalmsg = $msgcat->getmsg($msgset,$msgnum,$default_msg_text,@msg_text_subs);
    my $outmsg = join ": ",$self->{ERROR},$finalmsg;
    print STDERR "$outmsg";
    $self->log_msg($self->{ERROR} . ": $default_msg_text",@msg_text_subs);
}

=head2 log_msg -- write message to Cluster Test Tool log  

=head3     Description:

log_msg() Writes a message to the Cluster Test Tool log but not to the SMIT console.

=head3     Arguments:

    This method takes 2 arguments:

    $msg_text:  The message text
    @msg_format: format specifiers for message text

=head3     Returns:

    none

=head3     Examples:

    $logger->log_msg("Event %s continues...",node_up);

=cut

sub log_msg ( )
{
    my $self = shift;
    my $msgstring = shift;
    my $logfile = $self->{LOGFILE};

    my ($sec, $min, $hour, $day, $month, $year) = (localtime())[0,1,2,3,4,5];
    my $dtstamp = sprintf "%02d/%02d/%04d_%02d:%02d:%02d",
        $day, $month + 1, $year + 1900, $hour, $min, $sec;
    printf $logfile "$dtstamp: $msgstring", @_;
}

=head2 verb_log_msg -- write message to log if verbose is true

=head3     Description:

verb_log_msg() calls log_msg if verbose is true

=head3     Arguments:

    This method takes 2 arguments:

    $msg_text:  The message text
    @msg_format: format specifiers for message text

=head3     Returns:

    none

=head3     Examples:

    $logger->verb_log_msg("Event %s continues...",node_up);

=cut

sub verb_log_msg ( )
{
    my $self = shift;

    if ( $self->{verbose} ) {
        $self->log_msg(@_);
    }
}

=head2 event_start_msg --  perform event start log processing  

=head3     Description:

event_start_msg() Writes appropriate messages to the Cluster Test Tool log and to the SMIT console for event start. This method also calls the Event Controller to tag hacmp.out with the event start tag.

=head3     Arguments:

    This method takes 2 arguments:

    $event_number: Event number.

    $event_name:   The event being started
 
    $event_desc:   The description field for the event    

    $start_time:   date/time stamp.

=head3     Returns:

    none

=head3     Examples:

    $logger->event_start_msg("node_up","Start Node Tiger","104503120503");

=cut

sub event_start_msg ( )
{
    my $self = shift;
    my $eventnum = shift;
    my $event = shift;
    my $tstamp = shift;
    my $eventdesc = $event->{description};
    my $eventname = (split(/:/,ref($event)))[2] || ref($event);
    my $tagstring = "$eventname:$tstamp";

    my $banner = "=" x 75;
    print "$banner\n";
    $self->log_msg("$banner\n");
    $self->info_msg(1,46,"||\n|| Starting Test %ld - %s\n||\n",
                                    $eventnum, $event->to_string());
    print "$banner\n";
    $self->log_msg("$banner\n");

    foreach my $node (@{$self->{cluster_nodes}}) {
        # tag hacmp.out on all nodes. ignore errors. if they occur here,
        # then we'll see more errors soon enough. and we prefer not to pass
        # meaningful return codes back from these messaging functions.
        system("/usr/es/sbin/cluster/utilities/cl_rsh -n $node /usr/es/sbin/cluster/cl_testtool/cl_testtool_ctrl -m log -e tag $tagstring");
    }
}


=head2 event_complete_msg --  perform event complete log processing  

=head3     Description:

event_complete_msg() Writes appropriate messages to the Cluster Test Tool log and to the SMIT console for event completion. This method also calls the Event Controller to gather between event log data. The event_ctrl module will be asked to log verbose ONLY when verbose logging is set (!$self->{verbose}) AND status is Event::Failed.

=head3     Arguments:

    This method takes 4 arguments:

    $event_name: event being started.

    $event_desc: description from the event file

    $start_time: date/time stamp from when event was started.

    $status:     Event::Failed|Event::Continue|Event::Timeout|Event::Fatal 

=head3     Returns:

    none 

=head3     Examples:

    $logger->event_complete_msg("node_up","Start node 2","104503120503",Event::Continue);

=cut

sub event_complete_msg ( )
{
    my $self = shift;
    my $eventnum = shift;
    my $eventname = shift;
    $eventname = (split(/:/,$eventname))[2] || $eventname;
    my $eventdesc = shift;
    my $tstamp = shift;
    my $status = shift;
    my $logfile = $self->{LOGFILE};
    my $statind;

    $statind = $self->{FAILED} if ($status == Event::Failed);
    $statind = $self->{TIMEOUT} if ($status == Event::Timeout);
    $statind = $self->{PASSED} if ($status == Event::Continue);
    $statind = $self->{FATAL} if ($status == Event::Fatal);
    $statind = $self->{NOTRATIONAL} if ($status == Event::NotRational);
    $statind = $self->{UNKNOWN} if ( ! defined $statind );

    my $banner = "=" x 75;
    print "$banner\n";
    $self->log_msg("$banner\n");
    $self->info_msg(1,47,"||\n|| Test %ld Complete - %s: %s\n||\n",
                                        $eventnum, $eventname, $eventdesc);
    $self->info_msg(1,48,
                "|| Test Completion Status: %s\n||\n",$statind); 
    print "$banner\n";
    $self->log_msg("$banner\n");

    if ($self->{verbose}) {
        if ($status != Event::Continue) {
            my $tag = "$eventname:$tstamp";
            foreach my $node (@{$self->{cluster_nodes}}) {
                if (! open (CMDBUF, "/usr/es/sbin/cluster/utilities/cl_rsh $node /usr/es/sbin/cluster/cl_testtool/cl_testtool_ctrl -m log -e doverb $tag |" ) ) {
                   $self->err_msg(1,49,
                     "Unable to collect verbose logfile details from node %s\n",
                     $node);
                } else {        
                    while (<CMDBUF>) {
                       print $logfile "$_";
                    }
                    close CMDBUF;
                }
            }
        }
    }
    return 0;
}

=head2 event_continue_msg --  perform event continue log processing  

=head3     Description:

event_continue_msg() Writes appropriate messages to the Cluster Test Tool log and to the SMIT console for event continuation.

=head3     Arguments:

    $event_name: Name of event being continued.

=head3     Returns:

    none 

=head3     Examples:

    $logger->event_continue_msg("node_up");

=cut

sub event_continue_msg ( )
{
    my $self = shift;
    my $eventnum = shift;
    my $eventname = shift;
    $eventname = (split(/:/,$eventname))[2] || $eventname;
    my $eventdesc = shift;
    my $banner = "=" x 75;
    print "$banner\n";
    $self->log_msg("$banner\n");
    $self->info_msg(1,50,"||\n\t\t     || Continuing Test %ld - %s: %s\n\t\t     ||\n",
                                        $eventnum, $eventname, $eventdesc);
    print "$banner\n";
    $self->log_msg("$banner\n");
}
                                                                                          
=head2 testing_start_msg --  perform testing start log processing  

=head3     Description:

testing_start_msg() Writes appropriate messages to the Cluster Test Tool log and to the SMIT console for testing start.

=head3     Arguments:

    @testtool_cmdargs:  The command line args passed to the test tool

=head3     Returns:

    none 

=head3     Examples:

    $logger->testing_start_msg(@testtool_cmdargs);

=cut

sub testing_start_msg ( )
{
    my $self = shift;
    my $cmdargs = "@_";
    my $banner = "#" x 75;
    print "$banner\n";
    $self->log_msg("$banner\n");
    $self->info_msg(1,51,
        "##\n## Starting Cluster Test Tool: %s\n##\n",$cmdargs);
    print "$banner\n";
    $self->log_msg("$banner\n");
}


=head2 testing_complete_msg --  perform testing start log processing  

=head3     Description:

testing_complete_msg() Writes appropriate messages to the Cluster Test Tool log and to the SMIT console for testing completion.

=head3     Arguments:

    $testtool_rc:       The return code from the test tool

=head3     Returns:

    none 

=head3     Examples:

    $logger->testing_complete_msg($exit_code);

=cut

sub testing_complete_msg ( )
{
    my $self = shift;
    my $finalrc = shift;
    my $logfile = $self->{LOGFILE};

    my $banner = "#" x 75;
    print "$banner\n";
    $self->log_msg("$banner\n");
    $self->info_msg(1,52,
        "##\n## Cluster Testing Complete: Exit Code %s\n##\n",$finalrc);
    print "$banner\n";
    $self->log_msg("$banner\n");

    close $logfile;
}


=head2 phase_header -- add phase header to log

=head3     Description:

A phase header is a message with a boundary that is inserted in the
log to mark out important start points in processing.

=head3     Arguments:

    $msg

=head3     Returns:

    none 

=head3     Examples:

    $logger->phase_header($msg);

=cut

sub phase_header ( )
{
    my $self = shift;
    my $msg = shift;

    my $banner = "-" x 55;
    $self->verb_log_msg("$banner\n");
    $self->verb_log_msg("| $msg\n");
    $self->verb_log_msg("$banner\n");
}



# it's a module, so the last statement should always just be '1;'
1;
__END__
