# @(#)51        1.20 src/43haes/lib/perl/cl_testtool/StateData.pm, hacmp, 52haes_r540, r5400622c 4/20/06 09:58:24
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package StateData;

use 5.006;
use strict;
use warnings;

use MsgCatalog;
use Logger;
use Event;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '1.20';

##
## Put all the methods and global data for your module here
##

=head2 new -- create a new StateData object

=head3      Description:

Creates a new StatData object, which will be used to parse and store
state data the is returned from cluster nodes.

=head3      Arguments:

    $logger         precreated Logger object

    $static_config  precreated static_config object

=head3      Returns:

    new StateData object

=cut

sub new ( )
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{logger} = shift;
    $self->{static_config} = shift;

    if ( ! defined($class) || 
         ! defined($self->{logger}) || 
         ! defined($self->{static_config}) ) {

        # don't use logger here, because it may be the field that
        # wasn't initialized
        print STDERR "ERROR: unexpected internal error in StateData->new\n";
        $self->{valid} = 0;
    } else {
        $self->{valid} = 1;
    }

    return bless $self, $class;
}

=head2 is_valid -- is the StateData object valid

=head3      Description:

Return the 'valid' parameter of the StateData object

=head3      Arguments:

    none

=head3      Returns:

    $self->{valid}

=cut

sub is_valid ( )
{
    my $self = shift;
    return $self->{valid};
}

=head2 process_data -- process state data from remote node

=head3      Description:

Processes state data stream coming from remote node and load this object's
data structure. Data comes in with XML-style tags around the separate chunks
of data. For example:

    <command1name>
    output line 1 from command 1
    output line 2 from command 1
    </command1name>
    <command2name>
    etc.

The data should include a version number, '<version="#"/>', and it should
include a node name, '<nodename="nodename"/>'. However, the function does not
check to ensure that both have indeed been specified.

=head3      Arguments:

    $fh = file handle to read data from

=head3      Returns:

    1 on success
    0 on failure
    
=cut

sub process_data ( )
{
    my $self = shift;
    my $fh = shift;
    my $node = "";
    my $rc = 1;

    # parse data line-by-line
    LINE: while ( <$fh> ) {

        chomp;

        #line contains the version tag
        /^\s*<version="(.+)"\/>\s*$/    && do {
                                            $rc = $self->check_version ( $1 );
                                            if ( ! $rc ) {
                                                $self->{valid} = 0;
                                                last LINE;
                                            }
                                            next LINE;
                                        };
        #line contains the nodename tag
        /^\s*<nodename="(.+)"\/>\s*$/   && do {
                                            $node = $1;
                                            next LINE;
                                        };
        #skip blank lines
        /^\s*$/                         && next LINE;

        #line contains a start tag for some data
        /^\s*<([^>]+)>\s*$/             && do {
                                            $rc = process ( 
                                                $self, $1, $node, $fh );
                                            last LINE if ! $rc;
                                            next LINE;
                                        };

        # DEFAULT CASE
        $self->{logger}->err_msg(1, 53,
                "Internal Error: unexpected data encountered: %s\n", "$_");
        $rc = 0;
        last LINE;
    }

    return $rc;
}

=head2 process_state_data_file -- process state data from file

=head3      Description:

Processes state data read from file create by  write_state_data_file. This
data will be in roughly the same format as for the process_data functions,
except that each command tag will include the name of the node from which
it came (instead of the data including a <nodename> tag)..

    <command1name nodename="nodename">
    output line 1 from command 1
    output line 2 from command 1
    </command1name>
    <command2name nodename="nodename">
    etc.

=head3      Arguments:

    none

=head3      Returns:

    1 on success
    0 on failure
    
=cut

sub process_state_data_file ( )
{
    my $self = shift;
    my $rc = 1;

    open ( STATEDATA, "< " . $self->{static_config}->{state_file} );
    if ( ! defined(<STATEDATA>) ) {
        $self->{logger}->err_msg(1, 54,
            "Unable to read from file: %s\n" . 
            "Cluster State Data cannot be restored.\n",
            $self->{static_config}->{state_file});
        return 0;
    }

    # parse data line-by-line
    LINE: while ( <STATEDATA> ) {

        chomp;

        #line contains the version tag
        /^\s*<version="(.+)"\/>\s*$/    && do {
                                            $rc = $self->check_version ( $1 );
                                            if ( ! $rc ) {
                                                $self->{valid} = 0;
                                                last LINE;
                                            }
                                            next LINE;
                                        };
        #line contains the version tag
        /^\s*<active_nodes="(.+)"\/>\s*$/
                                        && do {
                                            my @nodes = split /,/, $1;
                                            $self->{active_nodes} = \@nodes;
                                            next LINE;
                                        };
        #skip blank lines
        /^\s*$/                         && next LINE;

        #line contains a start tag for some data
        /^\s*<([^> ]+) node="([^"]+)">\s*$/
                                        && do {
                                            $rc = process ( 
                                                $self, $1, $2, \*STATEDATA );
                                            last LINE if ! $rc;
                                            next LINE;
                                        };

        # DEFAULT CASE
        $self->{logger}->err_msg(1, 53,
                "Internal Error: unexpected data encountered: %s\n", "$_");
        $rc = 0;
        last LINE;
    }

    close ( STATEDATA );

    return $rc;
}

=head2 check_version -- check version number in data stream

=head3      Description:

The test tool will not run during cluster migration. The method will check
to make sure that the remote version is the same as the local version.

=head3      Arguments:

    $version = version from remote data stream

=head3      Returns:

    1   version is good
    0   version is bad

=cut

sub check_version ( )
{
    my $self = shift;
    my $version = shift;
    return 1 if ( $version == $self->{static_config}->{cluster_version} );
    return 0;
}

=head2 set_active_nodes -- set the list of active nodes

=head3      Description:

Run clgetactivenodes and set the list of active nodes.

=head3      Arguments:

    none

=head3      Returns:

    none

=cut

sub set_active_nodes ( )
{
    my $self = shift;
    my @activenodes;
    
    foreach my $node ( @{$self->{static_config}->{node_names}} ) {

        @activenodes =
            `/usr/es/sbin/cluster/utilities/clgetactivenodes -n "$node"`;
        chomp(@activenodes);
        last if ( scalar(@activenodes) );
    }

    $self->{active_nodes} = \@activenodes;
}

=head2 process -- process a tagged block of data.

=head3      Description:

Process a tagged block of data from process_data / process_state_data_file.

=head3      Arguments:

    $dataset    = name from tag that was found

    $nodename   = name of node data came from

    $fh         = file handle for input

=head3      Returns:

    $self->{valid}

=cut

sub process ( )
{
    my $self = shift;
    my $dataset = shift;
    my $nodename = shift;
    my $fh = shift;


    SWITCH: for ( $dataset ) {

        local $_ = $dataset; # this will protect $dataset from getting 
                             # over-written when $_ is changed in the 
                             # other process functions

        /^clRGinfo$/    && do {
                            $self->{$dataset}->{$nodename} =
                                process_clRGinfo ( $self, $fh );
                            last SWITCH;
                        };

        /^lssrc$/       && do {
                            $self->{$dataset}->{$nodename} =
                                process_lssrc ( $self, $fh );
                            last SWITCH;
                        };

        /^candidates$/       && do {
                            $self->{$dataset}->{$nodename} =
                                process_candidates ( $self, $fh );
                            last SWITCH;
                        };

        #DEFAULT CASE
                        do {
                            $self->{$dataset}->{$nodename} =
                                process_raw ( $self, $fh, $dataset );
                            last SWITCH;
                        };
    }

    return $self->{valid};
}

=head2 process_raw -- process an unrecognized data type

=head3      Description:

Process an unrecognized type of data. Just read until the close tag
is found and store the data.

=head3      Arguments:

    $fh         data source

    $dataset    name from data tag

=head3      Returns:

    array with all data lines in it

=cut

sub process_raw ( )
{
    my $self = shift;
    my $fh = shift;
    my $dataset = shift;
    my @retval;

    LINE: while ( <$fh> ) {

        chomp;

        #check for close of dataset
        /^\s*<\/$dataset>\s*$/      && last LINE;

        #skip blank lines
        /^\s*$/                     && next LINE;

        #DEFAULT CASE
        push @retval, $_;
    }
    return \@retval;
}

=head2 process_candidates -- process clsetenvgrp candidate node and site queries

=head3      Description:

Processes a list of site and node candidates for several resource groups.  This
is called once per node.

=head3      Arguments:

    $fh         data source

    $dataset    name from data tag

=head3      Returns:

    a hash of hashes with the rg, nodelist and sitelist information
    the nodelist and sitelist are space separated lists, as returned by clsetenvgrp

=cut


sub process_candidates ( )
{
    my $self = shift;
    my $fh = shift;
    my $dataset = shift;
    my $rgname="";
    my $retval = {};

    LINE: while ( <$fh> ) {

        chomp;

        #check for close of dataset
        /^\s*<\/candidates>\s*$/      && last LINE;

        #skip blank lines
        /^\s*$/                     && next LINE;

	#rg name tag
	if (/^\s*\<rg name=(.*?)\>\s*$/) {$rgname=$1; next LINE;}

	#end rg name tag
	if (/^\s*\<\/rg>\s*$/) {$rgname=""; next LINE;}
	
        #site list
	if(/^\s*sitelist=\s*(.+)\s*$/)
	  {$retval->{$rgname}->{sitelist} = $1; next LINE;}
	
	#node list
	if (/^\s*nodelist=\s*(.+)\s*$/)
	  {$retval->{$rgname}->{nodelist} = $1; next LINE; }

	#DEFAULT CASE (do nothing)
    }
    return $retval;
}

=head2 process_clRGinfo -- process output from clRGinfo

=head3      Description:

Processes the output from clRGinfo -s on a node. It stores the data in
a hashtable.

=head3      Arguments:

    $fh     data source

=head3      Returns:

    hashref that data is stored in.

=cut

sub process_clRGinfo ( )
{
    my $self = shift;
    my $fh = shift;
    my $retval = {};
    
    LINE: while ( <$fh> ) {

        chomp;

        #check for close of dataset
        /^\s*<\/clRGinfo>\s*$/      && last LINE;

        #skip comments and blank lines
        /^\s*#.*$/                  && next LINE;
        /^\s*$/                     && next LINE;

        #DEFAULT CASE
        my ($rgname, $rgstate, $node) = split /:/;
        if ( $rgname && $rgstate && $node ) {
            $retval->{$rgname}->{$node} = $rgstate;
        }
    }
    
    return $retval;
}

=head2 process_lssrc -- process lssrc output from a node

=head3      Description:

Process lssrc data from a node. Parse out the cluster state, list of forced
down nodes, the TE_NODE pointer address, number of events on the Ibcast queue,
list of events on the event queue, and list of active timers.

=head3      Arguments:

    $fh     data source

=head3      Returns:

    hashref with lssrc data stored

=cut

sub process_lssrc ( )
{
    my $self = shift;
    my $fh = shift;
    my $retval = {};

    LINE: while ( <$fh> ) {

        chomp;

        #check for close of dataset
        /^\s*<\/lssrc>\s*$/             && last LINE;

        #what's the cluster state?
        /^Current state: (.+)$/         && do {
                                            $retval->{state} = $1;
                                            next LINE;
                                        };
        #Are any nodes forced down?
        /^Forced down node list: (.+)$/ && do {
                                            @{$retval->{forced}} = 
                                                split /\s+/, $1;
                                            next LINE;
                                        };

        #what is the address of the current TE_NODE struct?
        /^tp is (.+)$/                 && do {
                                            $retval->{tp} = $1;
                                            next LINE;
                                        };

        #how many messages are on the ibcast queue?
        /^There are (\d+) events on the Ibcast queue$/
                                        && do {
                                            $retval->{ibcast} = $1;
                                            next LINE;
                                        };

        #how many messages are on the RM ibcast queue?
        /^There are (\d+) events on the RM Ibcast queue$/
                                        && do {
                                            $retval->{rmibcast} = $1;
                                            next LINE;
                                        };

        #get the event queue information
        /^Events on event queue:$/      && do {
                                            $retval->{events} =
                                                process_events ( $fh );
                                            #ibcast was put on end of events
                                            #array so we could get it here
                                            $retval->{ibcast} =
                                                pop(@{$retval->{events}});
                                            next LINE;
                                        };

        #get the timer information (this will also be the end
        #of the dataset)
        /^The following timer.s. are currently active:$/
                                        && do {
                                            $retval->{timers} =
                                                process_timers ( $fh );
                                            last LINE;
                                        };

        #DEFAULT CASE: skip everything else, because we don't care
        next LINE;
    }
    
    return $retval;
}

=head2 process_events -- process event queue from lssrc

=head3      Description:

Read from data source building list of events from event queue.

=head3      Arguments:

    $fh     data source

=head3      Returns:

    reference to array with event list

=cut

sub process_events ( )
{
    my $fh = shift;
    my @retval;

    LINE: while ( <$fh> ) {

        chomp;

        #is this the end of the event queue?
        /^There are (\d+) events on the Ibcast queue$/
                                        && do {
                                            #put ibcast number on end of array
                                            #so it can be retrieved by caller
                                            push(@retval, $1);
                                            last LINE;
                                        };

        #DEFAULT case
        push(@retval, $_);
        next LINE;
    }

    return \@retval;
}

=head2 process_timers -- read list of active timers from lssrc output

=head3      Description:

Read list of active timers from lssrc output

=head3      Arguments:

    $fh     data source

=head3      Returns:

    reference to array with timer list

=cut

sub process_timers ( )
{
    my $fh = shift;
    my $state = "READ";
    my @retval;

    LINE: while ( <$fh> ) {

        chomp;

        #is this the end of the dataset?
        /^\s*<\/lssrc>\s*$/             && last LINE;

        #is this the end of the timer section? skip the rest of the data.
        /^\s*Current DNP values.*$/   && do { $state = "SKIP"; };

        #DEFAULT case
        push(@retval, $_) if $state eq "READ";
        next LINE;
    }

    return \@retval;
}

=head2 get_node_state -- get the state of a particular node

=head3      Description:

Look up node state from stored lssrc output.

=head3      Arguments:

    $node

=head3      Returns:

    node state

=cut

sub get_node_state ( )
{
    my $self = shift;
    my $node = shift;

    if ( ! $self->{lssrc}                   || 
         ! $self->{lssrc}->{$node}          ||
         ! $self->{lssrc}->{$node}->{state} ) {

        return "";
    }

    return $self->{lssrc}->{$node}->{state};
}

=head2 get_node_states -- get the state of all nodes

=head3      Description:

Look up node states from stored lssrc output.

=head3      Arguments:

    none

=head3      Returns:

    hash reference with node=state pairs

=cut

sub get_node_states ( )
{
    my $self = shift;
    my $retval = {};

    foreach my $node ( @{$self->{static_config}->{node_names}} ) {

        $retval->{$node} = $self->get_node_state($node);
    }
    return $retval;
}

=head2 get_forcedown_states -- get a list of forcedown nodes

=head3      Description:

Look up list of forced down nodes from lssrc output of a specified node.

=head3      Arguments:

    $node

=head3      Returns:

    forced down node list

=cut

sub get_forcedown_nodes ( )
{
    my $self = shift;
    my $node = shift;

    if ( ! $self->{lssrc}                   || 
         ! $self->{lssrc}->{$node}          ||
         ! $self->{lssrc}->{$node}->{forced} ) {

        return ();
    }

    return @{$self->{lssrc}->{$node}->{forced}};
}

=head2 get_TE_NODE_pointer -- get TE_NODE pointer address for a node

=head3      Description:

Look up TE_NODE pointer from stored lssrc output.

=head3      Arguments:

    $node

=head3      Returns:

    TE_NODE pointer address

=cut

sub get_TE_NODE_pointer ( )
{
    my $self = shift;
    my $node = shift;

    if ( ! $self->{lssrc}                   || 
         ! $self->{lssrc}->{$node}          ||
         ! $self->{lssrc}->{$node}->{tp} ) {

        return "";
    }
    return $self->{lssrc}->{$node}->{tp};
}

=head2 get_ibcast_count -- get ibcast count for a node

=head3      Description:

Look up ibcast count from stored lssrc output.

=head3      Arguments:

    $node

=head3      Returns:

    ibcast count

=cut

sub get_ibcast_count ( )
{
    my $self = shift;
    my $node = shift;

    if ( ! $self->{lssrc}                   || 
         ! $self->{lssrc}->{$node}          ||
         ! $self->{lssrc}->{$node}->{ibcast} ) {

        return 0;
    }

    $self->{logger}->verb_log_msg("\t\t\tIbcast count: %ld\n",
           $self->{lssrc}->{$node}->{ibcast}); 

    return $self->{lssrc}->{$node}->{ibcast};
}

=head2 get_rmibcast_count -- get rmibcast count for a node

=head3      Description:

Look up rmibcast count from stored lssrc output.

=head3      Arguments:

    $node

=head3      Returns:

    rmibcast count

=cut

sub get_rmibcast_count ( )
{
    my $self = shift;
    my $node = shift;

    if ( ! $self->{lssrc}                   || 
         ! $self->{lssrc}->{$node}          ||
         ! $self->{lssrc}->{$node}->{rmibcast} ) {

        return 0;
    }

    $self->{logger}->verb_log_msg("\t\t\tRmIbcast count: %ld\n",
           $self->{lssrc}->{$node}->{rmibcast}); 

    return $self->{lssrc}->{$node}->{rmibcast};
}

=head2 get_event_queue -- get event queue for a node

=head3      Description:

Look up event queue from stored lssrc output.

=head3      Arguments:

    $node

=head3      Returns:

    array ref to event queue

=cut

sub get_event_queue ( )
{
    my $self = shift;
    my $node = shift;

    if ( ! $self->{lssrc}                   || 
         ! $self->{lssrc}->{$node}          ||
         ! $self->{lssrc}->{$node}->{events} ) {

        return ();
    }

    $self->{logger}->verb_log_msg("\t\t\tEvent Queue has %ld events\n",
        scalar(@{$self->{lssrc}->{$node}->{events}}));

    return @{$self->{lssrc}->{$node}->{events}};
}

=head2 get_active_timers -- get active timers for a node

=head3      Description:

Look up active timers from stored lssrc output.

=head3      Arguments:

    $node

=head3      Returns:

    active timers array

=cut

sub get_active_timers ( )
{
    my $self = shift;
    my $node = shift;

    if ( ! $self->{lssrc}                   || 
         ! $self->{lssrc}->{$node}          ||
         ! $self->{lssrc}->{$node}->{timers} ) {

        return ();
    }
    return @{$self->{lssrc}->{$node}->{timers}};
}

=head2 get_rg_states -- get RG state info from stored clRGinfo output

=head3      Description:

Look up RG state info from stored clRGinfo output.

=head3      Arguments:

    $rg

=head3      Returns:

    hashref with states of all nodes for RG

=cut

sub get_rg_states ( )
{
    my $self   = shift;
    my $rg     = shift;
    my $retval = undef;

    if ( ! defined $self->{clRGinfo} ) {
        return $retval;
    }

    NODE: foreach my $node ( sort( keys( %{$self->{clRGinfo}} ))) {

        if ( $self->{clRGinfo}->{$node}->{$rg} ) {
        
            $retval = $self->{clRGinfo}->{$node}->{$rg};
            last NODE;
        }
    }
    return $retval;
}

=head2 get_rgs_for_node -- get list of all RGs in specified state on node

=head3      Description:

Look up list of all RGs that are in the specified state on the specified node.

=head3      Arguments:

    $node
    $state

=head3      Returns:

    array of RGs in specified state

=cut

sub get_rgs_for_node ( )
{
    my $self = shift;
    my $node_name = shift;
    my $search_state = shift;
    my @rglist;

    foreach my $resgrp ( @{ $self->{static_config}->{resource_groups} } ) {

        my $rg_states = $self->get_rg_states( $resgrp->{group} );
        if ( $rg_states && $rg_states->{$node_name} eq $search_state ) {
            push @rglist, $resgrp->{group};
        }
    }
    return @rglist;
}

=head2 get_nodes_for_rg -- get list of all nodes with RG in specified state

=head3      Description:

Look up list of all nodes that have the specified RG in the specified state

=head3      Arguments:

    $rg_name
    $state

=head3      Returns:

    array of nodes with RG in specified state

=cut

sub get_nodes_for_rg ( )
{
    my $self = shift;
    my $rg_name = shift;
    my $search_state = shift;
    my @nodelist;

    my $rg_states = $self->get_rg_states( $rg_name );
    if ( $rg_states ) {
        foreach my $node ( sort( keys( %{$rg_states} ))) {
            if ( $rg_states->{$node} eq $search_state ) {
                push @nodelist, $node;
            }
        }
    }
    return @nodelist;
}

=head2 get_rgs_for_state -- get list of RGs in specified state on any node

=head3      Description:

Look up list of all RGs in specified state on any node

=head3      Arguments:

    $state

=head3      Returns:

    array of RGs in specified state anywhere in cluster

=cut

sub get_rgs_for_state ( )
{
    my $self = shift;
    my $search_state = shift;
    my @rglist;

    foreach my $resgrp ( @{ $self->{static_config}->{resource_groups} } ) {

        my $rgstates = $self->get_rg_states( $resgrp->{group} );
        foreach my $node ( sort( keys( %{$rgstates} ))) {
            if ( $rgstates->{$node} eq $search_state ) {
                push @rglist, $resgrp->{group};
            }
        }
    }
    return @rglist;
}

=head2 get_raw_data -- get data collected by a specific command on a node

=head3      Description:

Get the data output from a specified command on a specified node.

=head3      Arguments:

    $dataset
    $node

=head3      Returns:

    array ref to collected data

=cut

sub get_raw_data ( )
{
    my $self = shift;
    my $dataset = shift;
    my $node = shift;

    if ( ! $self->{$dataset}            || 
         ! $self->{dataset}->{$node}    ) {

        return ();
    }

    return @{$self->{$dataset}->{$node}};
}

=head2 get_event_history -- get event history from a node

=head3      Description:

Get the event history data returned from a node.

=head3      Arguments:

    list of events to search for.

=head3      Returns:

    array ref to event history records of interest

=cut

sub get_event_history ( )
{
    my $self = shift;
    my @search_events = @_;
    my $search_string = join "|", @search_events;

    my $EVENT_FAILED_MSG = "EVENT FAILED";
    my $EVENT_COMPLETED_MSG = "EVENT COMPLETED";
    
    my $pattern = "^.* (?:${EVENT_FAILED_MSG}|${EVENT_COMPLETED_MSG}): (?:$search_string) .*\$";
    my $retval = {};

    if ( ! $search_string ) {
        return $retval;
    }

    my $event_history = $self->{eventhistory};
    $self->{logger}->verb_log_msg("Begin Event History records:\n");
    foreach my $datanode ( sort( keys( %{ $event_history } ))) {
        my @history = @{ $event_history->{$datanode} };
        $self->{logger}->verb_log_msg("\tNODE: $datanode\n%s",
                (join("", (map { "\t\t\t$_\n" } @history))));
        my @matches = grep /$pattern/, @history;
        $retval->{$datanode} = \@matches;
    }
    $self->{logger}->verb_log_msg("End Event History records\n");
    return $retval;
}

=head2 cluster_is_stable -- check if cluster is stable on all nodea

=head3      Description:

Determine if all nodes are either offline or in ST_STABLE state.

=head3      Arguments:

    none

=head3      Returns:

    0 if not stable
    1 if stable

=cut

sub cluster_is_stable ( )
{
    my $self = shift;
    my $node_states = $self->get_node_states();
    $self->{logger}->verb_log_msg("Checking cluster stability\n");    
    foreach my $node ( sort( keys( %{$node_states} ))) {
        my $nodestate = Event::get_node_statename($node_states->{$node});
        $self->{logger}->verb_log_msg("\t$node: $nodestate\n");
        # node is offline or stable?
        if ( $node_states->{$node} ne "ST_STABLE" &&
             $node_states->{$node} ne "ST_INIT" &&
             $node_states->{$node} ne "NOT_CONFIGURED" ) {
                 $self->{logger}->verb_log_msg("Cluster is not stable\n");
                 return 0;
        }
    }
    $self->{logger}->verb_log_msg("Cluster is stable\n");
    return 1;
}

=head2 write_state_data_file -- write this object's data out to a file

=head3      Description:

Write this object's state data out to a file that will be read and
parsed by parse_state_data_file.

=head3      Arguments:

    none

=head3      Returns:

    1 on success
    0 on failure

=cut

sub write_state_data_file ( )
{
    my $self = shift;

    my $openrc = open( STATEDATA, "> " . $self->{static_config}->{state_file} );
    if ( ! $openrc ) {
        $self->{logger}->err_msg(1, 55,
            "Unable to write to file: %s\n" . 
            "Cluster State Data cannot be stored.\n",
            $self->{static_config}->{state_file});
        return 0;
    }

    # output version number
    printf STATEDATA "<version=\"%ld\"/>\n", 
                      $self->{static_config}->{cluster_version};

    #output list of active nodes
    my $nodelist = join ',', @{$self->{active_nodes}};
    printf STATEDATA "<active_nodes=\"%s\"/>\n", $nodelist;

    #output clRGinfo details
    foreach my $datanode ( keys( %{$self->{clRGinfo}} )) {

        printf STATEDATA "<clRGinfo node=\"%s\">\n", $datanode;
        
        foreach my $rgname ( keys( %{$self->{clRGinfo}->{$datanode}} )) {

            foreach my $rgnode ( keys(
                        %{$self->{clRGinfo}->{$datanode}->{$rgname}} )) {

                # lines from clRGinfo -s have fields in the order
                #       rgname:rgstate:nodename
                my $rgstate = 
                    $self->{clRGinfo}->{$datanode}->{$rgname}->{$rgnode};
                printf STATEDATA "%s:%s:%s\n", $rgname, $rgstate, $rgnode; 
            }
        
        }
        print STATEDATA "</clRGinfo>\n";
    }

    #output clRGinfo details
    foreach my $datanode ( sort( keys( %{$self->{lssrc}} ))) {

        printf STATEDATA "<lssrc node=\"%s\">\n", $datanode;
        printf STATEDATA "Current state: %s\n", 
                            $self->{lssrc}->{$datanode}->{state};
        print STATEDATA "</lssrc>\n";
    }

    close ( STATEDATA );

    return 1;
}


# it's a module, so the last statement should always just be '1;'
1;
__END__
