# @(#)53        1.7 src/43haes/lib/perl/cl_testtool/VariableTable.pm, hacmp, 52haes_r540, r5400622c 2/29/04 13:56:39
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2004,2005 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package VariableTable;

use 5.006;
use strict;
use warnings;

use Logger;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Event::EVENT_NAME ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '1.7';

=head1 NAME
                                                                                
VariableTable --
Perl extension providing a simple symbol table for the HACMP Cluster Test Tool
                                                                                
=head1 SYNOPSIS
                                                                                
    use VariableTable;
                                                                                
    my $vartable =
            VariableTable->new( $filename );
                                                                                
    if ( $vartable->is_valid() )
    {
        my $var_name = $vartable->lookup("node3");
    }
                                                                                
                                                                                
=head1 DESCRIPTION
                                                                                
This is the module within the Cluster Test Tool that represents the variable table.
The variable table stores the name value pairs that represent the specific cluster
attribute names in the user's cluster. These values will be set from either the parsed
information from $filename, the user's environment (%ENV), or if not found elsewhere,
the values in the event records in the event file.
                                                                                
=head1 METHODS
                                                                                
=head2 new -- create a new instance of VariableTable
                                                                                
=head3     Description:
                                                                                
new() is the constructor for the VariableTable. This method will initialize
a new VariableTable class, using either the values in the $filename (if defined)or the process environment (%ENV) to build a table of variables for use by the
Cluster Test Tool in interpretting the Test Tool Event File.
                                                                                
=head3     Arguments:
                                                                                
    new() takes one argument:

    $filename:  The name of the user supplied variables file
                                                                                
=head3     Returns:
                                                                                
    A new VariableTable object.
                                                                                
                                                                                
=head3     Examples:
                                                                                
    my $vartable = VariableTable->new($filename);
                                                                                
=cut

sub new ( )
{
    my $proto  = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{varfile} = shift;
    $self->{logger} = shift;
    $self->{valid} = 1;
    $self->{logger}->phase_header("Initializing Variable Table");
    if (defined $self->{varfile}) {
        $self->{logger}->verb_log_msg(
            "\tUsing Variable File: " . $self->{varfile} . "\n");
        $self->{symbols} = parse_var_file( $self );
    } else {
        $self->{logger}->verb_log_msg(
            "\tUsing Process Environment for Variable Table\n");
        $self->{symbols} = \%ENV;
    }
    return bless $self, $class;
}
#
# parse the variable file into a hash called symbols
#
# ARGS:     reference to hash that will be VariableTable object.
#
# RETURNS:  reference to hash (the symbol table)
#
sub parse_var_file ()
{
    my $self = shift;

    my $openerr = open(VARFILE,"$self->{varfile}");
    if ( ! $openerr ) {
        $self->{valid} = 0;
        $self->{logger}->err_msg(1, 12,
            "Cannot open variables file or file does not exist: %s\n",
            "$self->{varfile}"); 
	return undef;
    }

    my %symbols;
    while (my $rawrec = <VARFILE>) {
        chomp $rawrec;
        if ($rawrec !~ m/^[ \t]*#/ && $rawrec !~ m/^[ \t]*$/) {        
            # parse record if this isn't a comment or blank line
            my ($key, $val) = split '=',$rawrec;
            $self->{logger}->verb_log_msg("\t\tdata line: $rawrec\n");
            if ((!defined $key) || (!defined $val)) {
                $self->{valid} = 0;
                $self->{logger}->err_msg(1, 13,
                    "Invalid variable table format: %s\n", "$rawrec"); 
            } else {
                $self->{logger}->verb_log_msg("\t\tkey: $key - val: $val\n");
                $symbols{$key} = $val;
            }
        }
    }
    close(VARFILE);

    return \%symbols;
}
#

=head2 is_valid -- return value of object's "valid" parameter
                                                                                
=head3     Description:
                                                                                
is_valid() will check the object's "valid" parameter, which indicates whether
the object was initialized successfully. Errors encountered while initializing
the Cluster Test Tool log will set the "valid" value to 0.
                                                                                
=head3     Arguments:
                                                                                
    none
                                                                                
=head3     Returns:
                                                                                
    1 == object is valid
    0 == object is invalid
                                                                                
=head3     Examples:
                                                                                
    if ( $vartable->is_valid() ) {
        print "VariableTable is valid.\n";
    }
                                                                                
=cut

sub is_valid ( )
{
    my $self = shift;
    return $self->{valid}
}
#

=head2 lookup -- lookup variable in variable table
                                                                                
=head3     Description:
                                                                                
lookup() looks up a variable in the VariableTable and returns it.
                                                                                
=head3     Arguments:
                                                                                
    This method takes one argument:
                                                                                
    $variable_name:     The name of the variable to look up.
                                                                                
=head3     Returns:
                                                                                
    The value associated with $variable_name, if it is defined, else
    the $variable_name.
                                                                                
=head3     Examples:
                                                                                
    my $var_value = $vartable->lookup($var_name);
                                                                                
=cut                                                                                

sub lookup( )
{
    my $self = shift;
    my $variable_name = shift;
    my $foundvar = $self->{symbols}->{$variable_name};
    return $foundvar if defined $foundvar;
    return $variable_name;
}

# it's a module, so the last statement should always just be '1;'
1;
__END__
