#!/usr/bin/perl -w
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 1999,2005 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)66	1.6 src/43haes/usr/sbin/cluster/clconvert/cl_odmmanIO.pm.sh, hacmp.conversion, 52haes_r540, r5400622c 2/20/04 10:58:06
#
######################################################################
# cl_odmmanIO.pm
####################################################################
package cl_odmmanIO;

#####################################################################
# ODMmanager ODM I/O routines >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#####################################################################
#####################################################################
# IOclassExists
# Purpose:
#     To see if an ODM class exists or not
# Input:
#     ODMname - The name of the ODM to see if it exists
# Returns:
#     1 - exists
#     0 - does not exist 
sub IOclassExists
{
    my ($ODMname) = @_;

    open(ODM, "odmshow $ODMname 2>&1 |");
    my $retVal = <ODM>;
    close (ODM);

    if ($retVal =~ /^class/)
    {
        return 1;
    } 
    else
    {
	$ODMname .= "OLD";
    	open(ODM, "odmshow $ODMname 2>&1 |");
    	my $retVal = <ODM>;
    	close (ODM);

	if ($retVal =~ /^class/)
    	{
            return 2;
	}
        return 0;
    }
}

#####################################################################
# IOreadODMstanzas
# Purpose:
#     To read in the existing classes for an ODM and put
#     them in memory.
#
# Input:
#     ODMname - The name of the ODM class to read in
#
# Returns:
#     a list containing each ODM class.  Each ODM class is
#     stored as a hash table of field names - field values
#     were the field name is the key.
sub IOreadODMstanzas
{
    my ($ODMname) = @_;

    # This is the list of ODM classes that will be returned
    my @ODM = ();
    my $ODMstring = "";

    my $old = $/;
    undef $/;
    open(ODM, "odmget $ODMname |");
    $ODMstring = <ODM>;
    $/ = $old;
    close(ODM);

    if (! (defined $ODMstring))
    {
	::DEBUG_PRINT("\t no data in ODM $ODMname\n",
	    $::DEBUG_IOreadODMstanzas) ;
   	return @ODM; 
    }	

    ::DEBUG_PRINT("IOreadODMstanzas:\n\t$ODMstring", 
	$::DEBUG_IOreadODMstanzas) ;
    # need to concat a CR here so the pattern match
    # pulls out the last record
    $ODMstring = $ODMstring . "\n";
    while ($ODMstring =~ /$ODMname:\n(.+?)\n\n/sg)
    {
        my %class = ();
        my $class = $1 . "\n";
        ::DEBUG_PRINT("\tGOT IT>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n",
		$::DEBUG_IOreadODMstanzas);
        ::DEBUG_PRINT("\t$class", $::DEBUG_IOreadODMstanzas);
        ::DEBUG_PRINT("\t<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n", 
		$::DEBUG_IOreadODMstanzas);
        my @fields = split(/\n/,$class);
        foreach $field (@fields)
        {
            ::DEBUG_PRINT("\tGOT FIELD:$field\n", 
		$::DEBUG_IOreadODMstanzas);
            chomp $field;
            $field =~ /\s*(\S+?)\s*=\s*(.+)/;
            ::DEBUG_PRINT("\tGOT FIELD NAME:$1\n",
		$::DEBUG_IOreadODMstanzas);
            ::DEBUG_PRINT("\tGOT FIELD VALUE:$2\n",
		$::DEBUG_IOreadODMstanzas);
            # hash in this ODM class
            $class{$1} = $2;
        }

        # add this class to the ODM class list
        @ODM = (@ODM, \%class);
   }

   return @ODM;
}

#####################################################################
# IOreadODMstanzasGlobal
# Purpose:
#     To read in the existing classes for an ODM and put them in the
#     global ODMs hash table.  If the ODMname exists in ODMs already,
#     no IO takes place.  Otherwise, the ODM is read in.
#
# Input:
#     ODMname - The name of the ODM class to read in
#
# Returns:
#     error - 0 completed successful
#             1 error
#     The ODMs global hash will have the new ODM stanzas for the ODM 
#     if the operation completed successfully.
sub IOreadODMstanzasGlobal
{
    my ($ODMname) = @_;
    my $error = 0;

    ::DEBUG_PRINT("IOreadODMstanzasGlobal: \n",
	$::DEBUG_IOreadODMstanzasGlobal);

    # See if we have a real ODM class
    my $exists = IOclassExists($ODMname);
    if ($exists)
    {
        if (!exists ($::ODM{$ODMname}))
        {
            ::DEBUG_PRINT("\tODM $ODMname does not exist in global yet\n", 
		$::DEBUG_IOreadODMstanzasGlobal);
    
            my @retVal = IOreadODMstanzas($ODMname);
            $::ODM{$ODMname} = (\@retVal);
        }
    }
    else
    {
        ::DEBUG_PRINT("\tODM $ODMname is not a real ODM class\n", 
		$::DEBUG_IOreadODMstanzasGlobal);
        $error = 1;
    }
    
    return $error;
} 

#####################################################################
# IOgetODMdefAsHash
# Purpose:
#     To read in an ODM class defintion and put it into memory as a 
#     hash table
#
# Input:
#     ODMname - The name of the ODM class to read in
#
# Returns:
#     The ODM class defintion stored as a hash table of field names -
#     field types were the field type is the key.
#     and error.
sub IOgetODMdefAsHash
{
    my ($ODMname) = @_;

    my %class = ();

    ::DEBUG_PRINT("IOgetODMdefAsHash: ODMname is $ODMname\n",
	$::DEBUG_IOgetODMdefAsHash);

    my $old = $/;
    undef $/;
 
    unless (open(ODM, "odmshow $ODMname |") )
    {
	::DEBUG_PRINT("\tERROR: cant odmshow $ODMname\n",
		$::DEBUG_IOgetODMdefAsHash);
      #  $class{"ErrorGettingTypes"}=1;
      #  return %class;
    }
    my $ODMstring = <ODM>;
    $/ = $old;
    close(ODM);

    ::DEBUG_PRINT("IOgetODMdefAsHash:\n$ODMstring", 
	$::DEBUG_IOgetODMdefAsHash) ;
    if ($ODMstring =~ /class.+?$ODMname.+?{(.+?)}/sg)
    {
        my $def = $1;
        # remove comments
        $def =~ s/\/\*.*?\*\///g;
        # remove any field lengths
        $def =~ s/\[.*?\]//g;
        # replace all spaces with a : to make the definition 
	#	easier to parse
        $def =~ s/\s+/:/g;
        ::DEBUG_PRINT("\tGOT STRUCTURE DEFINITION:\n$def\n", 
		$::DEBUG_IOgetODMdefAsHash);
        while ($def =~ /:(.+?):(.+?);/gs)
        {
            ::DEBUG_PRINT("\tFIELD:type=$1:value=$2\n", 
		$::DEBUG_IOgetODMdefAsHash);
            # put the field in the hash table and set its type
            $class{$2} = $1;
        }
    }else{
	::DEBUG_PRINT("\tClass name $ODMname did not match!\n",
		$::DEBUG_IOgetODMdefAsHash);
    }

    return %class;
}

#####################################################################
# IOgetODMdefAsList
# Purpose:
#     To read in an ODM class defintion and put it into memory as a
#     list
#
# Input:
#     ODMname - The name of the ODM class to read in
#
# Returns:
#     A list of fields in the ODM as they appear in the definition
sub IOgetODMdefAsList
{
    my ($ODMname) = @_;

    my @class = ();

    my $old = $/;
    undef $/;
    open(ODM, "odmshow $ODMname |");
    my $ODMstring = <ODM>;
    $/ = $old;
    close(ODM);

    ::DEBUG_PRINT("IOgetODMdefAsList:\n$ODMstring", 
	$::DEBUG_IOgetODMdefAsList) ;
    if ($ODMstring =~ /class.+?$ODMname.+?{(.+?)}/sg)
    {
        my $def = $1;
        # remove comments
        $def =~ s/\/\*.*?\*\///g;
        # remove any field lengths
        $def =~ s/\[.*?\]//g;
        # replace all spaces with a : to make the definition
	#	 easier to parse
        $def =~ s/\s+/:/g;
        ::DEBUG_PRINT("\tGOT STRUCTURE DEFINITION:\n$def\n", 
		$::DEBUG_IOgetODMdefAsList);

        while ($def =~ /:.+?:(.+?);/gs)
        {
            ::DEBUG_PRINT("\tFIELD:$1\n", $::DEBUG_IOgetODMdefAsList);
            # put the field in the hash table and set its value to ""
            @class = (@class, $1);
        }
    }

    return @class;
}

#####################################################################
# IOwriteODMstanzas
# Purpose:
#     Write the ODM's read in by IOreadODMstanzas back to the ODM 
#     database.
#
# Input:
#     ODMname - The name of the ODM class to read in
#     ODMclasses - This is a list of class structures to write.
#                  This parameter must be in the form returned by
#                  IOreadODMstanzas.
#     append - 1: append to existing entries in ODMname
#              0: delete all existing entries in ODMname, then add the
#                 entries in ODMclasses.
#
# Returns:
#    None.
#
# NOTES:	1. The system command garbled the \", so use tmp file.
#
sub IOwriteODMstanzas
{
    my ($ODMname, $ODMclasses, $append) = @_;

    my $class;
    my $key;
    my $ODMupdates = "";
    my @insertOrder = ();
    my %types = ();
    my $Msg="";

    ::DEBUG_PRINT("IOwriteODMstanzas:\n", $::DEBUG_IOwriteODMstanzas);

    # if we are not appending, then delete all the exiting
    # entries for the ODM specified by ODMname
    if (!$append)
    {
        ::DEBUG_PRINT("\tdeleting all odm entries for $ODMname\n", 
		$::DEBUG_IOwriteODMstanzas);
        # we are not appending, so delete all existing entries
        # in the $ODMname ODM
        open(ODM, "odmdelete -o $ODMname |");
	$Msg=<ODM>;
	::LOG_PRINT (1,"odmdelete -o $ODMname: $Msg","");
	close(ODM);
    }

    # get the insert order.  This is because the ODM fields
    # must be entered in the order that they appear after doing
    # an odmshow command.
    @insertOrder = IOgetODMdefAsList($ODMname);

    # get the types of the fields.  This is in case we are doing
    # a field conversion and need the type to determine wheter a
    # new value will have a "" around it or not
    %types = IOgetODMdefAsHash($ODMname);

    # now, loop through all the structures in memory and
    # create a big string to pass as input into odmadd
    foreach $class (@{$ODMclasses})
    {
       $ODMupdates = $ODMupdates . "\n$ODMname:\n";

       foreach $field (@insertOrder)
       {
           my $type = $types{$field};
           if (exists($class->{$field}))
           {
#	      if ($type eq "char")
#             {
#                   $ODMupdates = $ODMupdates . 
#			"\t$field = \\\"$class->{$field}\\\"\n";
#             }
#             else
#             {
                   $ODMupdates = $ODMupdates . 
			"\t$field = $class->{$field}\n";
#               }
               ::DEBUG_PRINT("\t$field exists in $ODMname\n", 
			$::DEBUG_IOwriteODMstanzas);
           }
#           else
#           {
 #              ::DEBUG_PRINT("\t$field does not exists in $ODMname\n", 
#			$::DEBUG_IOwriteODMstanzas );
 #              my $default = 0;
  #             if ($type eq "char")
  #             {
  #                 ::DEBUG_PRINT("\tdefaulting $field to a char\n", 
#			$::DEBUG_IOwriteODMstanzas);
 #                  $default = "\\\"\\\"";
  #             }
   #            $ODMupdates = $ODMupdates . "\t$field = $default\n";
    #       }
       }
    }

    $ODMupdates = $ODMupdates . "\n\n";

    ::DEBUG_PRINT("\t>>>IOwriteODMstanzas>>>\n", 
	$::DEBUG_IOwriteODMstanzas);
    ::DEBUG_PRINT("\tIOwriteODMstanzas: update:\n$ODMupdates", 
	$::DEBUG_IOwriteODMstanzas);

	open (OUT, "> $::TMP_FILE ") || die "cant open $::TMP_FILE for writing: $!";
	printf (OUT "%s",$ODMupdates);
	close (OUT);

    
    ::DEBUG_PRINT("\t<<<IOwriteODMstanzas<<<\n", 
	$::DEBUG_IOwriteODMstanzas);

    ::LOG_PRINT(1,"odmadding $ODMname\n");

    # add all the entries from $ODMclasses into the ODM database
      system("odmadd $::TMP_FILE ;rm $::TMP_FILE ");
}

#####################################################################
# IOwriteODMstanzasGlobal
# Purpose:
#     Write the ODM's stored in the ODMs global hash back to the ODM
#     database.
#
# Input:
#     none. 
#     Uses the ODMs global hash table.
#
# Returns:
#    None.
sub IOwriteODMstanzasGlobal
{
    foreach $key (keys %::ODM)
    {
        ::DEBUG_PRINT ("IOwriteODMstanzasGlobal: $key\n", 
		$::DEBUG_IOwriteODMstanzasGlobal);
        my $ODMstanzas = $::ODM{$key};
        IOwriteODMstanzas($key, \@{$ODMstanzas}, 0);
    }
}
#####################################################################
# ODMmanager ODM  I\O routines <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
#####################################################################

return TRUE;
### END cl_odmmanIO.pm ############################################### 
