#!/usr/bin/perl -w
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 1999,2005 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)68	1.10 src/43haes/usr/sbin/cluster/clconvert/cl_odmmanMANIP.pm.sh, hacmp.conversion, 52haes_r540, r5400622c 2/20/04 10:58:16
#
#####################################################################3
# cl_odmmanMANIP
######################################################################

package cl_odmmanMANIP;

#####################################################################
# ODMmanager ODM manipulaters >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 
#####################################################################
# MANIPASCIIsortStanzas
# Purpose:
#     Pass this routine into the PERL sort to sort a list of ODM 
#     classes by a particular field using an ascii sort.
#
# Input:
#     $a and $b - the standard sort routine parameters.
#     In this case $a and $b are hash tables of fields for an ODM.
#     The hash table is in the same format as those generated inside
#     IOreadODMstanzas and returned by IOgetODMdefAsList.
#
# Global Input:
#     $SORT_FIELD must be set to the name of the field you want to 
#     sort on
#
# Returns:
#     -1 if $a < $b
#      0 if $a = $b
#      1 if $a > $b
sub MANIPASCIIsortStanzas
{
    if ($a->{$::SORT_FIELD} lt $b->{$::SORT_FIELD})
    {
        return -1;
    }
    elsif ($a->{$::SORT_FIELD} eq $b->{$::SORT_FIELD})
    {
        return 0;
    }
    elsif ($a->{$::SORT_FIELD} gt $b->{$::SORT_FIELD})
    {
        return 1;
    }
}

#####################################################################
# MANIPNumericSortStanzas
# Purpose:
#     Pass this routine into the PERL sort to sort a list of ODM 
#     classes by a particular field using a numeric sort.
#
# Input:
#     $a and $b - the standard sort routine parameters.
#     In this case $a and $b are hash tables of fields for an ODM.
#     The hash table is in the same format as those generated inside
#     IOreadODMstanzas and returned by IOgetODMdefAsList.
#
# Global Input:
#     $SORT_FIELD must be set to the name of the field you want to 
#     sort on
#
# Returns:
#     -1 if $a < $b
#      0 if $a = $b
#      1 if $a > $b
sub MANIPNumericSortStanzas
{
    if ($a->{$::SORT_FIELD} < $b->{$::SORT_FIELD})
    {
        return -1;
    }
    elsif ($a->{$::SORT_FIELD} == $b->{$::SORT_FIELD})
    {
        return 0;
    }
    elsif ($a->{$::SORT_FIELD} > $b->{$::SORT_FIELD})
    {
        return 1;
    }
}

#####################################################################
# MANIPAlphaSortStanzas
# Purpose:
#     Pass this routine into the PERL sort to sort a list of ODM 
#     classes by a particular field using a alphabetic sort.
#
# Input:
#     $a and $b - the standard sort routine parameters.
#     In this case $a and $b are hash tables of fields for an ODM.
#     The hash table is in the same format as those generated inside
#     IOreadODMstanzas and returned by IOgetODMdefAsList.
#
# Global Input:
#     $SORT_FIELD must be set to the name of the field you want to
#     sort on
#
# Returns:
#     -1 if $a < $b
#      0 if $a = $b
#      1 if $a > $b
sub MANIPAlphaSortStanzas
{
    if ("\U$a->{$::SORT_FIELD}" lt "\U$b->{$::SORT_FIELD}")
    {
        return -1;
    }
    elsif ("\U$a->{$::SORT_FIELD}" eq "\U$b->{$::SORT_FIELD}")
    {
        return 0;
    }
    elsif ("\U$a->{$::SORT_FIELD}" gt "\U$b->{$::SORT_FIELD}")
    {
        return 1;
    }
}

#####################################################################
# MANIPstanzaExists
# Purpose:
#    To look in a list or ODM stanzas to see if a set of field="value"
#    pairs already exits.
#
# Input:
#    ODMstanzas - The list of stanzas to look through
#    fields - A hash table of field - value pairs to do the 
#             comparison on.  This table definition does not have
#	      to match the definitions in the list of ODMstanzas
#	      provided.  If the table does not match, defaults
#             are assumed for any missing field (i.e. "" for
#             strings, 0 for numbers.) 
#
# Return:
#    1 - stanza already exists
#    0 - stanza does not exist
sub MANIPstanzaExists
{
    my ($ODMname,$ODMstanzas, $fields) = @_;
    my $exists = 0;
    my $TmpValue;
    my $IsChar = 0;
    my $item;
    my %Types;

    ::DEBUG_PRINT("MANIPstanzaExists:\n",$::DEBUG_MANIPstanzaExists);

    if (!(defined $ODMname))
    {
	::DEBUG_PRINT("\tundefined ODMname\n",$::DEBUG_MANIPstanzaExists);
    }

    ::DEBUG_PRINT("\tfields to check:\n",$::DEBUG_MANIPstanzaExists);

    foreach $item (keys (%{$fields}))
    {
    	::DEBUG_PRINT("\t\t$item=$fields->{$item}\n",
		$::DEBUG_MANIPstanzaExists);
    }

    %Types = cl_odmmanIO::IOgetODMdefAsHash($ODMname);


    foreach $item (@{$ODMstanzas})
    { 
	::DEBUG_PRINT("\tNextStanza:\n",$::DEBUG_MANIPstanzaExists);
        my $fieldExists = 0;
        foreach $key (keys %{$item})
        { 
  	    $TmpValue = $item->{$key}; 
            
	    #########################
 	    # kill quotes if string #
	    #########################
	#    if (($Types{$key} eq "char") &&
	#        ($TmpValue =~ /^"(.*)"$/))
        #    {
	#	$TmpValue = $1;
	#    }

	    # since not all the fields will be in 
            # fields, do a check to see if it exists.
            # NOTE: fields that are not present in fields
            # are assumed to be the default "" or 0
            if (exists($fields->{$key}))
            {

 	        if ($TmpValue eq $fields->{$key})
                {
                    ::DEBUG_PRINT("\tfield $key is the same: $TmpValue\n", 
			$::DEBUG_MANIPstanzaExists);
                    $fieldExists = 1;
                }else{
                    ::DEBUG_PRINT("\tfield $key is different: $TmpValue,$fields->{$key}\n",
			 $::DEBUG_MANIPstanzaExists);
                    $fieldExists = 0;
                    last;
                }
            }else{
                ::DEBUG_PRINT("\tfield $key is not in fields\n",
			 $::DEBUG_MANIPstanzaExists);
 		#############################
		# if field is not in fields #
		# and value is default,     #
		# they match		    #
		#############################
#		if ( (($Types{$key} eq "char") && ($TmpValue eq "\"\"")) ||
#		     (($Types{$key} ne "char") && ($TmpValue == 0))   )
 #                {
 #             
 #                   ::DEBUG_PRINT("\tfield $key has a default value 
#'$TmpValue' thus is the same\n", $::DEBUG_MANIPstanzaExists);
 #                   $fieldExists = 1;
#
 #               }else{
 #
 #                  ::DEBUG_PRINT("\tfield $key is not a default 
#'$TmpValue' thus is different\n", $::DEBUG_MANIPstanzaExists);
 #                   $fieldExists = 0;
  #                  last;
   #             }# end if default

            }# end if exists
        }# end foreach
        if ($fieldExists)
        {
            ::DEBUG_PRINT("\tThe whole stanza exists\n",
		 $::DEBUG_MANIPstanzaExists);
	    ::LOG_PRINT(1,"WARNING: Stanza Exists at line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
            $exists = 1;
            last;
        }
	::DEBUG_PRINT("\n",$::DEBUG_MANIPstanzaExists);
    }
 
    ::DEBUG_PRINT("End MANIPstanzaExists\n",$::DEBUG_MANIPstanzaExists);

    return $exists;
}

#####################################################################
# MANIPcustom
# Purpose:
# Input:
# Returns:
sub MANIPcustom
{
    my ($ODMstring,$ODMnames) = @_;
    my %ODMdata = ();   
    my $done = 0;
    my $currentODM = "";
    my $nextODM = "";
    my @TmpStrings = ();
    my $string;
    my $CurStrings = ();
    my $key;
    my $ODM_name;
    my $GetFields = 0;
    my $NumOfEntries = 0;
    my $field;
    my $value;
    my @fields = ();
    my %fieldvalues= ();
    my $ODMstanzas;
    my %Types = ();
    my @DoneFields = ();
    my $GotIt = 0;
    my $WarnString = "";
    my $WarnHappened = 0;

    ::DEBUG_PRINT("MANIPcustom:\n",$::DEBUG_MANIPcustom);

    #########################################
    # delete all data for ODMnames from ODM #
    #########################################
    foreach $ODM_name (@{$ODMnames})
    {
	if (exists $::ODM{$ODM_name})
        {
	    delete $::ODM{$ODM_name};
        }
    }# end for each odm name

    ::DEBUG_PRINT("\tODMstring = '$ODMstring'\n",
	     $::DEBUG_MANIPcustom);

    #############################################
    # if any perl warnings/error messages are   #
    # tacked on the beginning of this string,   #
    # remove them and echo them to the log.     #
    #############################################
    # if you do match some optional spaces, a word, some space and colon,
    # then you have just the odms...else, process the initial stuff.
    $WarnHappened = 0;
    while ( !($ODMstring =~ /^\s*?(\w+)\s*?:\n/))
    {
	$WarnHappened = 1;
        # if we have no more characters other than white space, exit
        # loop because we did not return any stanzas... we just returened
        # warnings from perl.
        if ( $ODMstring =~ /^\s*$/ )
        {
           $ODMstring = "";
           last;
        }
	# now we expect some lines of stuff before seeing
	# space word space colon
	if ($ODMstring =~ /^(.*?)\n(.*)/s)
	{
	    $ODMstring = $2;
	    $WarnString .= $1."\n";
	}
    }

    if ($WarnHappened)
    {
	::LOG_PRINT(1,"Note: These messages were encountered when the ");
	::LOG_PRINT(1,"custom script was run:\n");
	::LOG_PRINT(1,"$WarnString \n");
    }


    ###################################
    # For each class, add data to ODM #
    ###################################
    
    # add /n to end so this works
    $ODMstring = $ODMstring. "\n";

    # while matching the format HACMPx:
    while ($ODMstring =~ /^\s*?(\w+)\s*?:\n(.+?)\n\n(.*)/s)
    {
 	$ODM_name  = $1;
	$class     = $2 . "\n";
	$ODMstring = $3;
	my %class = ();

	::DEBUG_PRINT("\tGot ODM $1\n",
	         $::DEBUG_MANIPcustom);
	
	##############
        # valid ODM? #
	##############
	if ( !( cl_odmmanIO::IOclassExists($ODM_name) ) )
	{
	    ::LOG_PRINT(1,"ERROR: ODM $ODM_name is not valid");
	    ::LOG_PRINT(1," resulting from custom script in ");
	    ::LOG_PRINT(1,"line $::LINE_NUMBER.\n");
	    return 1;
	}

	#########################
	# split up field/values #
	#########################
        %Types = cl_odmmanIO::IOgetODMdefAsHash($ODM_name);

	if ($class =~ /\\\n/)  # Field has continuation lines
	    { $class =~ s/\\\n//g; }

	@fields = split(/\n/,$class);

	if (exists $::ODM{$ODM_name})
	{
	    $NumOfEntries = $#{$::ODM{$ODM_name}};
	}else{
	    $NumOfEntries = -1;
	}
	$NumOfEntries++;

	::DEBUG_PRINT("\tnum of elements for $ODM_name is $NumOfEntries\n",
	         $::DEBUG_MANIPcustom);
	    

	foreach $field (@fields)
        {
	    chomp($field);

	    if ($field =~ /\s*(\S+?)\s*=\s*(.+)/)
	    {
	        ::DEBUG_PRINT("\tGot $1='$2'\n",
	             $::DEBUG_MANIPcustom);
		
 	        $field = $1;
	        $value = $2;
	
		@DoneFields = (@DoneFields, $field);
	
		if ($Types{$field} eq "char")
		{
		    if ( !($value =~ /^"(.*)"\s*?$/) )
		    {
			$value = "\"".$value."\"";
		    }
		    if ( $value =~ "\" \"")
                    {
			$value = "\"\"";
		    }
		    ::DEBUG_PRINT ("\tchar value is '$value'\n",
			$::DEBUG_MANIPcustom);
		}else{
		    if ($value =~/"/)
		    {

	::LOG_PRINT(1,"ERROR: Data returned from custom script has");
	::LOG_PRINT(1," quote in numeric data at line $::LINE_NUMBER.\n");
	return 1;

		    }
		}

	        $::ODM{$ODM_name}[$NumOfEntries]->{$field} = $value;   
 	    }else{

	::LOG_PRINT(1,"ERROR: Data returned from custom script in ");
	::LOG_PRINT(1,"wrong format at line $::LINE_NUMBER.\n");
	return 1;

	    }
	}# end for each field pair

	::DEBUG_PRINT("\tDone fields:\n",$::DEBUG_MANIPcustom);

	foreach $field (@DoneFields)
	{
	    ::DEBUG_PRINT("\t\t$field\n",$::DEBUG_MANIPcustom);
	}

	#####################
	# Fill out defaults #
	#####################
	foreach $key (keys (%Types))
	{	
	    $GotIt = 0;
	    foreach $field (@DoneFields)
	    {
		if ($field eq $key) {$GotIt = 1;}
	    }
	
	    if (!$GotIt)
	    {
		if ($Types{$key} eq "char")
		{
	            $::ODM{$ODM_name}[$NumOfEntries]->{$key} ="\"\"";   
		}else{
	            $::ODM{$ODM_name}[$NumOfEntries]->{$key} = 0;   
		}    
	    }
	} # end foreach type field
	@DoneFields = ();

    }# end while still getting stanzas

    ####################################
    # if this didnt work, wrong format #
    ##################################
    if ($ODMstring =~/\S/)  # matches non space
    {
	::LOG_PRINT(1,"ERROR: Data returned from custom script in");
        ::LOG_PRINT(1," wrong format at line $::LINE_NUMBER\n");
	return 1;
    }

    ::DEBUG_PRINT("End MANIPcustom\n",$::DEBUG_MANIPcustom);

    return 0;

}# end MANIPcustom
#####################################################################
# MANIPinsertStanzas
# Purpose:
#    To copy stanzas from one field - value pair to another.  This 
#    will do a best fit copy of stanzas from the OMDto the ODMfrom.
#    This means if the ODMfrom is not actually the same ODM as the 
#    ODMto, that the copy will still work.  Any field not specified
#    in the ODMfrom that is in the ODMto will be defaulted ("" for
#    char and 0 for numbers)
#    NOTE: now this function does supply quotes for char types,
#    but does not fill in the defaults (not necessary.)
#
# Input:
#    ODMname - The name of the ODM stanza to copy to
#    ODMto - A list of field-Value pairs to insert into.  This is a 
#	     list of ODM stanzas.
#    ODMfrom - The field-value pairs of the form ODM
#
# Returns:
#    ODMupdate - The destination ODM (ODMto) is updated to include 
#                the new stanza
sub MANIPinsertStanzas
{
    my ($ODMname, $ODMto, $ODMfrom) = @_;
    my %Types; 
    my %newField = ();
 
    %Types = cl_odmmanIO::IOgetODMdefAsHash($ODMname);
   
    ::DEBUG_PRINT("MANIPinsertStanzas:\n", $::DEBUG_MANIPinsertStanzas);

    # populate a new stanza (newField)
    foreach $key (keys (%Types))
    {
       # see if the field exists in the fromODM 
       if (exists ($ODMfrom->{$key}))
       {

           # it exists, so use that value
           ::DEBUG_PRINT("\tfield $key exists in source ODM def\n", 
		$::DEBUG_MANIPinsertStanzas); 
           $newField{$key} = $ODMfrom->{$key};
	   
	   # if char, needs quotes
	   if ($Types{$key} eq "char")
	   {
		$newField{$key}="\"".$newField{$key}."\"";
	   }
       }else{ # otherwise put default
	   if($Types{$key} eq "char")
	   {
		$newField{$key} = "\"\"";
	   }else{
		$newField{$key} = 0;
	   }
       }# end if exists
    } 
  
    # insert the new ODM stanza into the list 
    @{$ODMto} = (@{$ODMto}, \%newField); 
    
    return \@{$ODMto}; 
}#End MANIPinsertStanzas

#####################################################################
# MANIPinsertStanzasWithFields
# Purpose:
#    To copy stanzas from one field - value pair to another.  This 
#    will do a best fit copy of stanzas from the OMDto the ODMfrom.
#    This means if the ODMfrom is not actually the same ODM as the 
#    ODMto, that the copy will still work.  Any field not specified
#    in the ODMfrom that is in the ODMto will be defaulted ("" for
#    char and 0 for numbers)
#    NOTE: now this function does supply quotes for char types,
#    but does not fill in the defaults (not necessary.)
#
# Input:
#    ODMname - The name of the ODM stanza to copy to
#    ODMto - A list of field-Value pairs to insert into.  This is a 
#	     list of ODM stanzas.
#    ODMfrom - The field-value pairs of the form ODM
#
# Returns:
#    error - 1 if error 
#                
sub MANIPinsertStanzasWithFields
{
    my ($SourceODMname,$TargetODMname, $TargetODMStanzas, $SourceODMStanza, 
	$SourceFieldList, $TargetFieldList,
	$withvalues, $WithFieldValues, $unique, $auto, $autoFields ) = @_;
    my %Types = (); 
    my %newField = ();
    my $i=0;
    my $Sfield;
    my $Tfield;
    my $StanzaExists = 0;
    my $key;
    my $ValueCtr = 1;
    my $ValueString = "a";
    my $max;
    my $TmpValue;
    my $error;
    my $Ctr = 0;
    my $field;
    my %TargetTypes = ();

    %Types       = cl_odmmanIO::IOgetODMdefAsHash($SourceODMname);
    %TargetTypes = cl_odmmanIO::IOgetODMdefAsHash($TargetODMname);
   
    ::DEBUG_PRINT("MANIPinsertStanzasWithFields:\n", 
	$::DEBUG_MANIPinsertStanzasWithFields);
  
    ::DEBUG_PRINT("\tSource, Target ODM = $SourceODMname, $TargetODMname\n",
	$::DEBUG_MANIPinsertStanzasWithFields);

 
    #####################################
    # copy source field to target field #
    ##################################### 
    for ($i=0; $i <= $#{$SourceFieldList}; $i++)
    {
       $SField = $SourceFieldList->[$i];
       $TField = $TargetFieldList->[$i];

       # see if the field exists in the fromODM 
       if (exists ($SourceODMStanza->{$SField}))
       {

           # it exists, so use that value
           ::DEBUG_PRINT("\tfield $SField exists in source ODM def\n", 
		$::DEBUG_MANIPinsertStanzasWithFields); 
           $newField{$TField} = $SourceODMStanza->{$SField};
	   
           ::DEBUG_PRINT("\tnew field $TField = $newField{$TField}\n", 
		$::DEBUG_MANIPinsertStanzasWithFields); 
       }
    }# end for each Source field    
 
    #########################################
    # if with values, populate these fields #
    #########################################
    if ($withvalues)
    {
        foreach $TField (keys (%{$WithFieldValues}))
	{
	    $newField{$TField} = $WithFieldValues->{$TField};

            # if char, needs quotes
            if ($TargetTypes{$TField} eq "char")
            {
                $newField{$TField}="\"".$newField{$TField}."\"";
            }
	}	
    }# end if with values

    #########
    # auto? #
    #########
   
    if ($auto)
    {
	foreach $key (@{$autoFields})
	{
	    $max = MANIPfindMax (\@{$TargetODMStanzas}, $key,
		$TargetTypes{$key});

	    if ($TargetTypes{$key} eq "char")
	    {
		if ($max eq "\"\"")
		{
		    $ValueCtr = 1;
		}else{
		   #############################
		   # kill quotes, get base/num #
		   #############################
		   if ($max =~ /^"(.*)"$/)
		   {
			$TmpValue = $1;	
		   
			if ($TmpValue =~ /^(.*)(\d+)$/)
			{
			    $ValueString = $1;
			    $ValueCtr    = $2 +1;
			}else{
			    $ValueString = $TmpValue;
			    $ValueCtr    = 1;
			}

	            }
		    $newField{$key} = "\"".$ValueString.$ValueCtr."\"";
		}# end if empty string
	    }else{
		$newField{$key} = $max + 1; # numeric
	    }# end if type eq char
	    ::DEBUG_PRINT("\t$key = $newField{$key}\n",
		$::DEBUG_MANIPinsertStanzasWithFields);
	}# end foreach autofield
    } # end if auto

    ###########
    # unique? #
    ###########
    if ($unique)
    {
	$StanzaExists =
		MANIPstanzaExists($TargetODMname,
			\@{$TargetODMStanzas},\%newField);
        if ($StanzaExists) 
	{
             ::DEBUG_PRINT("\tStanzaExists\n",
		     $::DEBUG_MANIPinsertStanzasWithFields);
	     return \@{$TargetODMStanzas};
	}

    }# end if unique
    
    ####################
    # fill in defaults #
    ####################
    foreach $key (keys (%TargetTypes))
    {
	my $GotIt = 0;
	foreach $field (keys (%newField))
	{
	    if ($field eq $key) {$GotIt=1;}
	}

	if (!$GotIt)
	{
	    if ($TargetTypes{$key} eq "char")
	    {
		$newField{$key} = "\"\"";
	    }else{
		$newField{$key} = 0;
	    }
	}
    }


    ##################################################
    # if empty stanzas are produced don't write them #
    ##################################################
    $Ctr = 0;
	
    foreach $key (keys (%newField))
    {
	if ( ($TargetTypes{$key} eq "char") &&
	     ($newField{$key} ne "\"\""))
	    {$Ctr++;}

	if ( ($TargetTypes{$key} ne  "char") &&
	     ($newField{$key} != 0))
	    {$Ctr++;}
    }

    ########################################### 
    # insert the new ODM stanza into the list #
    ###########################################
    if ($Ctr > 0)
    {
	@{$TargetODMStanzas} = (@{$TargetODMStanzas}, \%newField); 
    }else{
	
	::LOG_PRINT(1,"WARNING: Copy as new command produced a stanza
with only defaults in line $::LINE_NUMBER.  This stanza was not inserted.\n",
		"$::LINE_NUMBER");
	
	::DEBUG_PRINT("\tWARNING: Copy as new command produced a stanza
with only defaults in line $::LINE_NUMBER.  This stanza was not inserted.\n",
	$::DEBUG_MANIPinsertStanzasWithFields);

	return 1;
    }
 
   ::DEBUG_PRINT("End MANIPinsertStanzasWithFields\n",
	$::DEBUG_MANIPinsertStanzasWithFields);

 #   return \@{$TargetODMStanzas}; 
    return 0; 


}#End MANIPinsertStanzasWithFields

#####################################################################
# MANIPcopyStanzasWithFields
# Purpose:
#    To copy stanzas from one field - value pair to another.  This 
#    will do a best fit copy of stanzas from the OMDto the ODMfrom.
#    This means if the ODMfrom is not actually the same ODM as the 
#    ODMto, that the copy will still work.  Any field not specified
#    in the ODMfrom that is in the ODMto will be defaulted ("" for
#    char and 0 for numbers)
# 
# Input:
#    Source, TargetODMname - The names of the ODMs to copy to,from
#    TargetODMStanzas - List of target stanzas
#    SourceODMStanza  - Hash of field value pairs for selected
#			source stanza
#    Source,TargetFieldList - Lists of the source, target fields
#    static           - 1 indicates that the StaticFieldValues
#			  can be used to further narrow the
#			  target stanza
#    StaticFieldValues - Hash of field value pairs to narrow the
#			 search for a target stanza if static=1
#    unique		- 1 indicates that if the resulting target 
# 			  stanza already exists, dont copy
#
# Returns:
#    ODMupdate - The destination ODM (ODMto) is updated to include 
#                the new stanza
sub MANIPcopyStanzasWithFields
{
    my ($SourceODMname,$TargetODMname, $TargetODMStanzas, $SourceODMStanza, 
	$SourceFieldList, $TargetFieldList, 
	$TargetFieldFields, $static, $StaticFieldValues,
	$unique ) = @_;
    my %TargetTypes = (); 
    my %SourceTypes = (); 
    my %newField = ();
    my $i=0;
    my $j=0;
    my $Sfield;
    my $Tfield;
    my $Svalue;
    my $Tvalue;
    my $StanzaExists = 0;
    my %TmpStanza = ();
    my $key;
    my $ValueCtr = 1;
    my $ValueString = "a";
    my $max;
    my $TmpValue;
    my $WhereSourceField;
    my $DoThisOne = 0;
    my $WhereTargetField;
    my @Tmp1;
    my @Tmp2;
    my $Ctr = 0;

    %SourceTypes = cl_odmmanIO::IOgetODMdefAsHash($SourceODMname);
    %TargetTypes = cl_odmmanIO::IOgetODMdefAsHash($TargetODMname);
   
    ::DEBUG_PRINT("MANIPcopyStanzasWithFields:\n", 
	$::DEBUG_MANIPcopyStanzasWithFields);
  
    ::DEBUG_PRINT("\tSource, Target ODM = $SourceODMname, $TargetODMname\n",
	$::DEBUG_MANIPcopyStanzasWithFields);

    ::DEBUG_PRINT("\tSource, Target field to match:\n",
	$::DEBUG_MANIPcopyStanzasWithFields);

    foreach $key (keys (%{$TargetFieldFields}))
    {
        ::DEBUG_PRINT("\t\t$key = $TargetFieldFields->{$key}\n",
	    $::DEBUG_MANIPcopyStanzasWithFields);
    }
 
    #######################################
    # for each target stanza that matches #
    # the criteria, copy source to target # 
    ####################################### 
    for ($i=0; $i <= $#{$TargetODMStanzas}; $i++)
    {
	::DEBUG_PRINT("\tTarget stanza $i\n",
		$::DEBUG_MANIPcopyStanzasWithFields);

	$Ctr = 0;  $DoThisOne = 0;
	foreach $WhereSourceField (keys (%{$TargetFieldFields}))
	{
	    $WhereTargetField = $TargetFieldFields->{$WhereSourceField};
	
	    ::DEBUG_PRINT("\t\tSource, Target fields:",
		$::DEBUG_MANIPcopyStanzasWithFields);

	    ::DEBUG_PRINT("  $WhereSourceField,$WhereTargetField\n",
		$::DEBUG_MANIPcopyStanzasWithFields);

	
	    ##############################
	    # target field fields match? #
	    ##############################

	    ###########################
	    # if both fields are char #
	    ###########################
	    if (($TargetTypes{$WhereTargetField} eq "char") &&
		($SourceTypes{$WhereSourceField} eq "char") )
	    {
		if ( $SourceODMStanza->{$WhereSourceField} eq
		     $TargetODMStanzas->[$i]->{$WhereTargetField} )
		{
	    	    ::DEBUG_PRINT("\t\tChar Source = Target field\n",
			$::DEBUG_MANIPcopyStanzasWithFields);

		     $DoThisOne++;
		}
	    
	    ##############################
	    # if both fields are numbers #
	    ##############################
	    }elsif (($TargetTypes{$WhereTargetField} ne "char") &&
		    ($SourceTypes{$WhereSourceField} ne "char") )
	    {
		if ( $SourceODMStanza->{$WhereSourceField} == 
		     $TargetODMStanzas->[$i]->{$WhereTargetField} )
		{
	    	    ::DEBUG_PRINT("\t\tNum Source = Target field\n",
			$::DEBUG_MANIPcopyStanzasWithFields);

		     $DoThisOne++;
		}
	    
	    #################################
	    # if one field is char, one not #
	    #################################
	    }else{
		$Tfield = $WhereTargetField;
		$Sfield = $WhereSourceField;
		$Tvalue = $TargetODMStanzas->[$i]->{$Tfield};
		$Svalue = $SourceODMStanza->{$Sfield};

		##################
		# target is char #
		##################
		if (($TargetTypes{$Tfield} eq "char")&&
		    ($Tvalue =~ /^"(.*)"$/ ) )
		{
		     $Tvalue = $1;
		     if ($Tvalue =~ /^(\d+)$/ )
		     {
		    	if ( $Svalue == $Tvalue)
			{
	    	    	    ::DEBUG_PRINT("\t\tMix Source = Target field\n",
				$::DEBUG_MANIPcopyStanzasWithFields);

		            $DoThisOne++;
		         }
		     }# end if digits
		}# end if target is char

		##################
		# source is char #
		##################
		if (($SourceTypes{$Sfield} eq "char")&&
		    ($Svalue =~ /^"(.*)"$/ ) )
		{
		     $Svalue = $1;
		     if ($Svalue =~ /^(\d+)$/ )
		     {
		    	if ( $Svalue == $Tvalue)
			{
	    	    	    ::DEBUG_PRINT("\t\tMix Source = Target field\n",
				$::DEBUG_MANIPcopyStanzasWithFields);

		            $DoThisOne++;
		         }
		     }# end if digits
		}# end if source is char


	    
	    }# end if type is char and mismatched

	    $Ctr++; 

	}#end foreach target field field

	##########################
        # if not match, continue #
	##########################
	if ($Ctr != $DoThisOne) 
	{
	    ::DEBUG_PRINT("\t\tNot all matched\n",
		$::DEBUG_MANIPcopyStanzasWithFields);
	
	    next;
	}

	###########
        # static? #
	###########
	if ($static)
	{
	     $Ctr = 0; $DoThisOne = 0;

	     foreach $TField (keys (%{$StaticFieldValues}) )
	     {

		##############################
		# static field values match? #
		##############################
		if ($TargetTypes{$TField} eq "char")
	        {	
		     if ( $TargetODMStanzas->[$i]->{$TField} eq
			  $StaticFieldValues->{$TField} )
		     {
			$DoThisOne = 1;
		     }
		}else{
		     if ( $TargetODMStanzas->[$i]->{$TField} == 
			  $StaticFieldValues->{$TField} )
		     {
			$DoThisOne = 1;
		     }
		}# end if type is char

		$Ctr++;
	    }# end foreach Static field value
	    
	}# end if static

	if ($Ctr != $DoThisOne) 
	{
	     next;
	}


	###########
	# unique? #
	###########
	if ($unique)
	{
	    #####################################
	    # Make fake target stanza to see if #
	    # the result already exists         #
	    #####################################
	    %TmpStanza = %{$TargetODMStanzas->[$i]};

	    for($j=0; $j<=$#{$SourceFieldList}; $j++)
	    {
		$SField = $SourceFieldList->[$j];
		$TField = $TargetFieldList->[$j];

		$TmpStanza{$TField}= 
			$SourceODMStanza->{$SField};
	    }# end for each source field

	    $StanzaExists =
		MANIPstanzaExists($TargetODMname,
			\@{$TargetODMStanzas},\%TmpStanza);

	    if ($StanzaExists)
	    {
		::DEBUG_PRINT("\tUnique: stanza that matches exists\n",
			$::DEBUG_MANIPcopyStanzasWithFields);
		next;
	    }

	} # end unique



	if ($Ctr != $DoThisOne) 
	{
	     next;
	}else{
	   ::DEBUG_PRINT("\t\tDo this one,ctr=$Ctr,Dothis=$DoThisOne\n",
			$::DEBUG_MANIPcopyStanzasWithFields);

	
	    ###############
	    # Do this one #
	    ###############
	    for ($j=0; $j<=$#{$SourceFieldList}; $j++)
	    {
       
		$SField = $SourceFieldList->[$j];
       		$TField = $TargetFieldList->[$j];


		::DEBUG_PRINT("\t\ttarget value is",
			$::DEBUG_MANIPcopyStanzasWithFields);

		::DEBUG_PRINT("$TargetODMStanzas->[$i]->{$TField}\n",
			$::DEBUG_MANIPcopyStanzasWithFields);

		::DEBUG_PRINT("\t\tsource value is",
			$::DEBUG_MANIPcopyStanzasWithFields);

		::DEBUG_PRINT("$SourceODMStanza->{$SField}\n",
			$::DEBUG_MANIPcopyStanzasWithFields);

		$TargetODMStanzas->[$i]->{$TField} = 
			$SourceODMStanza->{$SField};
	    }# end for each source field

	}# end if do this one


    }# end for each Target stanza
 

    ::DEBUG_PRINT("End MANIPCopyStanzasWithFields\n",
			$::DEBUG_MANIPcopyStanzasWithFields);


    return \@{$TargetODMStanzas}; 
}#End MANIPcopyStanzasWithFields

#####################################################################
# MANIPsortStanzas
# Purpose:
#    To sort the stanzas
# Input:
#    ODMname      - The name of the ODM stanza to process 
#    ODMstanzas   - set of odmstanzas to work on
#    Fields       - list of fields to sort on
#    HowSort      - list of how to sort:alpha,ASCII,numeric
#    DirSort      - list of direction to sort:ascending,descending
# Returns:
#    ODMupdate - The destination ODM (ODMto) is updated to include 
#                the new stanza
sub MANIPsortStanzas
{
    my $Tmp_a;
    my $Tmp_b;
    my $ShortFora;
    my $ShortForb;
    my $Field;
    my $FieldCtr = 0;

    ::DEBUG_PRINT("MANIPsortStanzas:\n",$::DEBUG_MANIPsortStanzas); 
  
    ::DEBUG_PRINT("\ta=$cl_odmmanPROCESS::a,b=$cl_odmmanPROCESS::b\n",
		$::DEBUG_MANIPsortStanzas); 
  
    $Tmp_a = $cl_odmmanPROCESS::b;
    $Tmp_b = $cl_odmmanPROCESS::a;
 
    ##########################################
    # until you run out of fields to sort on #
    ##########################################
    while ($FieldCtr < ($#::SORT_FIELDS + 1) )
    {
        
        ::DEBUG_PRINT("\tField ctr = $FieldCtr\n",
		$::DEBUG_MANIPsortStanzas); 

        if ($::DIR_SORT [ $FieldCtr ] =~ /ascending/i)
	{
	     $ShortFora = $cl_odmmanPROCESS::a;
	     $ShortForb = $cl_odmmanPROCESS::b;
             $Tmp_a =
	$::ODM{$::ODM_NAME}[$ShortFora]->{$::SORT_FIELDS [ $FieldCtr ]};
             $Tmp_b =
	$::ODM{$::ODM_NAME}[$ShortForb]->{$::SORT_FIELDS [ $FieldCtr ]};

	}else{
	      $ShortFora = $cl_odmmanPROCESS::a;
	      $ShortForb = $cl_odmmanPROCESS::b;
	      $Tmp_b =
	$::ODM{$::ODM_NAME}[$ShortFora]->{$::SORT_FIELDS [ $FieldCtr ]};
     	      $Tmp_a =
	$::ODM{$::ODM_NAME}[$ShortForb]->{$::SORT_FIELDS [ $FieldCtr ]};
	
	}

	#########
        # ASCII #
        #########
        
	if ( $::HOW_SORT [ $FieldCtr ] =~ /ascii/i )
        {
    
             ::DEBUG_PRINT("\t\tASCII\n",
		     $::DEBUG_MANIPsortStanzas); 

	    if     ( $Tmp_a lt $Tmp_b ) 
	    {
		return -1;
	    }elsif ( $Tmp_a eq $Tmp_b ) {
		$FieldCtr++;
	    }elsif ( $Tmp_a gt $Tmp_b) {
		return 1;
	    }
					
        #########
        # ALPHA #
        #########
        
        }elsif ( $::HOW_SORT [ $FieldCtr ] =~ /alpha/i) {
    
             ::DEBUG_PRINT("\t\talpha\n",
		     $::DEBUG_MANIPsortStanzas); 

	     if     ( "\U$Tmp_a" lt "\U$Tmp_b" )
    	     {
        	return -1;
    	     }elsif ( "\U$Tmp_a" eq "\U$Tmp_b" ) {
        	$FieldCtr++;
    	     }elsif ( "\U$Tmp_a" gt "\U$Tmp_b" ) {
        	return 1;
    	     }
	     
        ###########
        # NUMERIC #
        ###########
        
        }elsif ( $::HOW_SORT [ $FieldCtr ] =~ /numeric/i) {
    
             ::DEBUG_PRINT("\t\tnumeric\n",
		     $::DEBUG_MANIPsortStanzas); 


	     if ($Tmp_a =~ /^"(.*)"$/)
	     {
		$Tmp_a = $1;
	     }
	
	     if ($Tmp_b =~ /^"(.*)"$/)
	     {
		$Tmp_b = $1;
	     }

	     if     ( $Tmp_a < $Tmp_b )
    	     {
        	return -1;
    	     }elsif ( $Tmp_a == $Tmp_b) {
        	$FieldCtr++;
             }elsif ( $Tmp_a > $Tmp_b ) {
        	return 1;
	     }	
	}# end if how sort
     
    }# end while

    return 0;  # tie after all 

}# end MANIPsortStanzas
#####################################################################
# MANIPautoStanzas
# Purpose:
#    To auto generate unique values in the given field
# Input:
#    ODMname      - The name of the ODM stanza to process 
#    ODMstanzas   - set of odmstanzas to work on
#    start        - 1 indicating StartValue will be used
#    StartValue   - value to start at (increment from there)(opt.) 
#    Field        - field to perform auto on
#    all	  - 1 indicates to do it on all stanzas
#		  - 0 indicates to only do it to defaults
#			(0 for number, "" for char)
#
# Returns:
#    ODMupdate - The destination ODM (ODMto) is updated to include 
#                the new stanza
sub MANIPautoStanzas
{
    my ($ODMname, $start, $all, $ODMstanzas, 
	$Field, $StartValue) = @_;
    my %type = (); 
    my %newField = ();
    my $UpdateAny = 0; 
    my $Ctr = 0;
    my $IsChar = 0;
    my $IsDefault = 0;
    my $max;
    my $TmpValue = "a";
    my $TmpNum = 0;
    my $DoneFirstValue = 0;
    my $ValueCtr = 1;
    my $ValueString = "a";

    ::DEBUG_PRINT("MANIPautoStanzas:\n", $::DEBUG_MANIPautoStanzas);

    ::DEBUG_PRINT("\tODMname, start, all, Field, StartValue:\n",
	$::DEBUG_MANIPautoStanzas);
   
    ::DEBUG_PRINT("\t$ODMname, $start, $all, $Field, $StartValue:\n",
	$::DEBUG_MANIPautoStanzas);
   
    #####################
    # Get type of field #
    #####################
    %types = cl_odmmanIO::IOgetODMdefAsHash($ODMname);
    if ($types{$Field} eq "char") {$IsChar = 1;}

    ##########
    # start? #
    ##########
    if ($start)
    {
	if ($StartValue =~ /^(.*?)(\d+)$/)
	{
             ::DEBUG_PRINT("\t1,2 = $1,$2\n",
		$::DEBUG_MANIPautoStanzas);
   
	     $ValueString = $1;
	     $ValueCtr = $2;
             if ($ValueString eq "")
	     {
		$ValueString = "a";
	     }
	}else{
	     $ValueString = $StartValue;
	}
    }# end if start
        
	::DEBUG_PRINT("\tValue String,Ctr = $ValueString,$ValueCtr\n",
		$::DEBUG_MANIPautoStanzas);
   

    #######################
    # if not all, get max #
    #######################
    if ((!$all) && (!$start))
    {
    	$max = MANIPfindMax(\@{$ODMstanzas},$Field,$types{$Field});
	
        ::DEBUG_PRINT("\t\tmax = $max\n",
		$::DEBUG_MANIPautoStanzas);
   
        if ($IsChar)
        {
	    if ($max eq "")
	    {
	    	$ValueCtr = 1;
	    }else{
	    	######################################
	    	# kill quotes and look for num	 #
	    	# to get base.  update, put together #
	    	######################################
	        if ($max =~ /^"(.*)"$/)
	    	{
		    $TmpValue = $1;
                    ::DEBUG_PRINT("\t\tTmpValue = $TmpValue\n",
			$::DEBUG_MANIPautoStanzas);
   
		    if ($TmpValue =~ /^(.*)(\d+)$/)
		    {
			$ValueString = $1;
			$ValueCtr = $2 + 1;
		    }else{
		        $ValueString = $TmpValue;
			$ValueCtr = 1;
		    }
                    ::DEBUG_PRINT("\t\tValueString = $ValueString\n",
			$::DEBUG_MANIPautoStanzas);
   
                    ::DEBUG_PRINT("\t\tValueCtr = $ValueCtr\n",
			$::DEBUG_MANIPautoStanzas);
   
		}
            }# end if max eq ""
        
	}else{
	    if ($max == 0)
	    {
		$ValueCtr = 1;
	    }else{
		$ValueCtr = $max +1;
	    }
        }# end if is char
    }# end if not all

    ############################################
    # Foreach stanza, see if Field is default, #
    # if all or default update the value       #
    ############################################
    $Ctr = 0;
    foreach $stanza (@{$ODMstanzas})
    {
    	$IsDefault = 0;
    	if ( ( ( $IsChar) && ($stanza->{$Field} eq "\"\"")) ||
             ( (!$IsChar) && ($stanza->{$Field} == 0 ))  )
		{$IsDefault = 1;}

        ::DEBUG_PRINT("\t$Ctr: IsDefault = $IsDefault\n",
		$::DEBUG_MANIPautoStanzas);

	if ( ($IsDefault) || ($all) ) 
	{

             if ($IsChar)
	     {
		$TmpValue = "\"".$ValueString.$ValueCtr."\"";
	     }else{
	     	$TmpValue = $ValueCtr;
	     }

	    $ValueCtr++; 

            ::DEBUG_PRINT("\t\tTmpValue = $TmpValue\n",
		$::DEBUG_MANIPautoStanzas);
   
	    $ODMstanzas->[$Ctr]->{$Field} = $TmpValue;
	
        }

	$Ctr++;

    }# end foreach stanza



    ::DEBUG_PRINT("End MANIPautoStanzas\n",$::DEBUG_MANIPautoStanzas);

    return \@{$ODMstanzas}; 

}#End MANIPautoStanzas

#####################################################################
# MANIPupdateStanzas
# Purpose:
#    To update specified fields of specified stanzas 
# Input:
#    ODMname - The name of the ODM stanza to process 
#    ODMstanzas - set of odmstanzas to work on
#    where   - flag indicating FieldValues will be used
#    FieldValues - hash of field value pairs identifying stanza 
#    SetFields - hash of field value pairs to update 
#
# Returns:
#    ODMupdate - The destination ODM (ODMto) is updated to include 
#                the new stanza
sub MANIPupdateStanzas
{
    my ($ODMname, $where, $ODMstanzas, $FieldValues, $SetFields) = @_;
    my %type = (); 
    my %newField = ();
    my $UpdateAny = 0; 
    my $Ctr = 0;

    %types = cl_odmmanIO::IOgetODMdefAsHash($ODMname);
   
    ::DEBUG_PRINT("MANIPupdateStanzas:\n", $::DEBUG_MANIPupdateStanzas);

    ::DEBUG_PRINT("\t$ODMname and $where\n", $::DEBUG_MANIPupdateStanzas);
   

    ###########################
    # if not update whole odm #
    ###########################
    if ($where)
    {
	#######################
	# Get matching stanza #
	#######################
	foreach $stanza (@{$ODMstanzas})
        {
	    $Matched = 0;
	    foreach $key (keys (%{$FieldValues}) )
	    {
	
		#######################################
		# if type is char, the ODM will have  #
		# quotes around it		      #
		#######################################
		$TmpField = $stanza->{$key};
                if ($TmpField =~/^"/)
		{
  		     $TmpField = "\"".$FieldValues->{$key}."\"";
		}else{
		     $TmpField = $FieldValues->{$key};
		}

		##########
                # Match? #
		##########
	        if ($stanza->{$key} eq $TmpField)
	        {
		    $Matched++;
                    ::DEBUG_PRINT("\tmatched is $Matched\n", 
			$::DEBUG_MANIPupdateStanzas);
	        }else{
		    $Matched = 0;
                    ::DEBUG_PRINT("\tnot matched\n\n", 
			$::DEBUG_MANIPupdateStanzas);
		}
	    }#end foreach key

	    ####################################
    	    # if matched update correct fields #
	    ####################################
	    @Tmp = keys (%{$FieldValues});

	    if ($Matched == ($#Tmp+1)) #all matched
	    {
		foreach $key (keys (%{$SetFields}) )
		{
		    if ( ($stanza->{$key}) =~ /^"/)
		    {
			$TmpField = "\"".$SetFields->{$key}."\"";
		    }else{
			$TmpField = $SetFields->{$key};
		    }

		    $ODMstanzas->[$Ctr]->{$key} = $TmpField;
		}	

		$UpdateAny = 1;
	
	    }

	    $Ctr++;
	}# end foreach stanza

    }else{
	
	######################
	# update all stanzas #
	######################
	$Ctr = 0;
	foreach $stanza (@{$ODMstanzas})
	{
	    foreach $key (keys (%{$SetFields}) )
	    {
		#########################
		# if char, needs quotes #
		#########################    
		if ( ($stanza->{$key}) =~ /^"/)
		{
		     $TmpField = "\"".$SetFields->{$key}."\"";
		}else{
		     $TmpField = $SetFields->{$key};
		}

	 	$ODMstanzas->[$Ctr]->{$key} = $TmpField;
		
	    }
	    
	    $Ctr++;
	}
	$UpdateAny = 1; # in fact, all of them
    }#end if where
  
    if (!$UpdateAny)
    {
	::LOG_PRINT(1,"WARNING: Update statement did not result in any updates in line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
    }

    ::DEBUG_PRINT("End MANIPupdateStanzas\n",$::DEBUG_MANIPupdateStanzas);

    return \@{$ODMstanzas}; 

}#End MANIPupdateStanzas

#####################################################################
# MANIPdeleteStanzas
# Purpose:
#    To delete stanzas from the ODM possibly identified by a set of
#    field/value pairs
# Input:
#    ODMname - The name of the ODM stanza to process 
#    ODMstanzas - set of odmstanzas to work on
#    where   - flag indicating FieldValues will be used
#    FieldValues - hash of field value pairs identifying stanza 
# Returns:
#    ODMupdate - The destination ODMstanza is updated to include 
#                the new stanza
sub MANIPdeleteStanzas
{
    my ($ODMname, $ODMstanzas, $where, $FieldValues) = @_;
    my %type = (); 
    my %newField = ();
    my @ODMupdate = ();
    my $stanza;
    my $key;
    my $Matched = 0;
    my @Tmp;
    my $TmpField;
    my $DeletedAny = 0;

    ::DEBUG_PRINT("MANIPdeleteStanzas:\n", $::DEBUG_MANIPdeleteStanzas);

    ###########################
    # if not delete whole odm #
    ###########################
    if ($where)
    {
        ##############################
        # For each stanza that does  # 
        # not match FieldValues      #
        # write it to the ODMupdate  #
	##############################
	foreach $stanza (@{$ODMstanzas})
        {
	    $Matched = 0;
	    foreach $key (keys (%{$FieldValues}) )
	    {
	
		#######################################
		# if type is char, the ODM will have  #
		# quotes around it		      #
		#######################################
		$TmpField = $stanza->{$key};
                if ($TmpField =~/^"/)
		{
  		     $TmpField = "\"".$FieldValues->{$key}."\"";
		}else{
		     $TmpField = $FieldValues->{$key};
		}

	        if ($stanza->{$key} eq $TmpField)
	        {
		    $Matched++;
                    ::DEBUG_PRINT("\tmatched is $Matched\n", 
			$::DEBUG_MANIPdeleteStanzas);
	        }else{
		    $Matched = 0;
                    ::DEBUG_PRINT("\tnot matched\n\n", 
			$::DEBUG_MANIPdeleteStanzas);
		}
	    }#end foreach key

	    @Tmp = keys (%{$FieldValues});

	    if ($Matched < ($#Tmp+1)) # not all matched
	    {
		@ODMupdate = ( @ODMupdate, \%{$stanza} );
	    }else{  # deleted one
		$DeleteAny = 1;
	    }
	}# end foreach stanza

    }else{
	$DeleteAny = 1; # in fact, all of them
    }#end if where
  
    if (!$DeleteAny)
    {
	::LOG_PRINT(1,"WARNING: Delete statement did not result in any deletes in line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
    }

    ::DEBUG_PRINT("End MANIPdeleteStanzas:\n", $::DEBUG_MANIPdeleteStanzas);
    
    return \@ODMupdate; 
}# End MANIPdeleteStanzas

#####################################################################
# MANIPfindMax
# Purpose:
#   To search through a list of ODM stanzas and find the maximum 
#   value for a specified field.  This only works correctly for
#   numeric fields.
#
# Input:
#   ODMstanzas - The list of ODM stanzas to find the max of
#   field - The field to find the max of
#   Type  - "char" or not
#
# Returns:
#   max - The largest value at the field request from the list of 
#         ODMstanzas
sub MANIPfindMax
{
    my ($ODMstanzas, $field, $Type) = @_;
    my $max;
    my $exists = 0;

    ::DEBUG_PRINT("MANIPfindMax:\n",$::DEBUG_MANIPfindMax);

    ::DEBUG_PRINT("\ttype=$Type,field=$field\n",$::DEBUG_MANIPfindMax);

    if ($Type eq "char") {$max = "";}
    else                {$max = 0; }

    foreach $item (@{$ODMstanzas})
    {
	::DEBUG_PRINT("\t Stanza is:\n",
		$::DEBUG_MANIPfindMax);

	foreach $key (keys (%{$item}))
	{
	    ::DEBUG_PRINT("\t\t$key = $item->{$key}\n",
		$::DEBUG_MANIPfindMax);
	}

        # assume each stanza contains the same fields.
        # so on the first time in the loop, check to see if
        # the field parameter actually exists in the stanza
        if (!$exists)
        {
            if (exists($item->{$field}))
            {  
                $exists = 1;
	        ::DEBUG_PRINT("\texists, value = '$item->{$field}'\n",
		     $::DEBUG_MANIPfindMax);
        
            }else{
		::DEBUG_PRINT("\tfield does not exist\n",
			$::DEBUG_MANIPfindMax);
	
                # if it does not exist, then we have nothing to 
                # find the max of, so just exit.
                last; 
            }
        }
	
	::DEBUG_PRINT("\ttype, max = $Type,$max\n",
		$::DEBUG_MANIPfindMax);
	        
        
	if ($Type eq "char")
        {

            if ($item->{$field} gt $max)
            {
                $max = $item->{$field};
            }
	
	}else{
            
	    if ($item->{$field} > $max)
            {
                $max = $item->{$field};
            }
        } 
    }#end foreach
 
    ::DEBUG_PRINT("End MANIPfindMax\n",$::DEBUG_MANIPfindMax);

    return $max;
}# end MANPfindMax


#####################################################################
# ODMmanager ODM manipulaters <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
#####################################################################

return TRUE;

## END cl_odmmanMANIP.pm  #############################################
