#!/usr/bin/perl -w
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 1999,2005 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)69	1.9 src/43haes/usr/sbin/cluster/clconvert/cl_odmmanPROCESS.pm.sh, hacmp.conversion, 52haes_r540, r5400622c 2/20/04 10:58:21
#
#######################################################################
# cl_odmmanPROCESS.pm
#######################################################################

package cl_odmmanPROCESS;

#####################################################################
# ODMmanager command PROCESSOR >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 
#####################################################################
# PROCESSAuto
# Purpose:
#    To process the auto command read in by PARSEAuto
# Input:
#    ODMname - The ODM to insert into 
#    start   - 1 means use the included StartValue 
#              0 not 
#    all     - 1 means generate unique value for all stanzas
#              0 not 
#    Field   - field to generate unique value for 
#    StartValue - if all = 1, value to use to start 
# Returns:
#    0 - success
#    1 - error
sub PROCESSAuto
{
    my ($ODMname, $start, $all, $Field, $StartValue) = @_;
    my $error;
    my $item;
    my $max;
    my $ODMstanzas;
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my $UseList = 1;
    my $UseHash = 0;
    my $TmpValue;
    my %Types;

    ::DEBUG_PRINT ( "PROCESSAuto:\n" , $::DEBUG_PROCESSAuto);



   #####################
   # Valid ODM, field? #
   #####################
   if ( !( cl_odmmanIO::IOclassExists($ODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $ODMname does not exist in ");
	::LOG_PRINT(1,"line $::LINE_NUMBER\n");
	return 1;
    }

    if ( ::PARSEgetFieldValid($ODMname,$UseList,$Field) )
    	 {return 1;}

    if ($start)
    {  
         $TmpValue = "\"".$StartValue."\"";
	 
	 if ( ::PARSEgetValues($TmpValue) )
	 {return 1;} 
        
    	%Types = cl_odmmanIO::IOgetODMdefAsHash($ODMname);
    
	::DEBUG_PRINT("\ttype = $Types{$Field}\n",$::DEBUG_PROCESSAuto);
 

	if ( ($Types{$Field} ne "char") &&
	     ($StartValue =~/\D/) )
	{
	     ::LOG_PRINT(1,"Error:  Nonnumeric character(s) detected in ");
	     ::LOG_PRINT(1,"start value on line $::LINE_NUMBER\n");
	     return 1;
	}

    }
 
   ###########################
   # read in the ODM stanzas #
   ###########################
    if (cl_odmmanIO::IOreadODMstanzasGlobal($ODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $ODMname at line $::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    $ODMstanzas = $::ODM{$ODMname};

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSAuto)
    {
	print "before ODMstanzas:\n";
	print "ODM is $ODMname\n";
 
	::DEBUG_PRINT_ODMs($ODMname);

    }
    #### <<DEBUG

    ##############################################
    # insert into global $ODM from cl_odmmanMAIN #
    ##############################################
    $::ODM{$ODMname} = 
	cl_odmmanMANIP::MANIPautoStanzas($ODMname, $start, $all,
		\@{$ODMstanzas},$Field,$StartValue); 

    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ##### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSAuto)
    {
	print "After:\n";

	::DEBUG_PRINT_ODMs($ODMname);
    }
    ##### <<DEBUG

    ::DEBUG_PRINT("End PROCESSAuto\n\n",$::DEBUG_PROCESSAuto);
 
    return 0;

}# End PROCESSAuto

#####################################################################
# PROCESSCommit
# Purpose:
#    To process the commit command read in by PARSECommit
# Input:
#    None
# Returns:
#    0 - success
#    1 - error
sub PROCESSCommit
{
    my $ODMstanzas;
    my $ODMname;

    ::DEBUG_PRINT ( "PROCESSCommit:\n" , $::DEBUG_PROCESSCommit);

    ################################
    # If ODM was changed, do write #
    ################################
    if ($::ODMwasChanged)
    {
	::DEBUG_PRINT("\tWriting ODM's\n",$::DEBUG_PROCESSCommit);
	cl_odmmanIO::IOwriteODMstanzasGlobal;
    }else{
	::DEBUG_PRINT("\tNot Writing ODM's\n",$::DEBUG_PROCESSCommit);
    }

    ##################################
    # Now delete the ODM from memory #
    ##################################

    foreach $ODMname (keys (%::ODM))
    {
	delete $::ODM{$ODMname};
    }

    $::ODMwasChanged = 0;  # reset it

    ::DEBUG_PRINT("End PROCESSCommit\n\n",$::DEBUG_PROCESSCommit);

    return 0;

}# End PROCESSCommit

#####################################################################
# PROCESSCopyAsNew
# Purpose:
#    To process part of the copy command read in by PARSECopy
# Input:
#    SourceODMname,TargetODMname - The ODMs to copy from and to
#    SourceFields, TargetFields  - list of fields to copy fr/to
#    where - 1 means use WhereFieldValues to identify source
#		stanzas
#	     0 not
#    WhereFieldValues - hash of field/value pairs used if where =1
#    withvalues - 1 means use WithFieldValues to fill other
#			fields of the new stanza
#		  0 not
#    WithFieldValues - hash of field/value pairs used if withvalues
#		=1
#    auto	- 1 means for the field list in autoFields, create
#			unique values for these new stanzas
#                 0 dont bother
#    autoFields  - list of fields to make unique in the new
#		   stanzas is auto=1
#    unique     - 1 means if the new stanza you are trying to
#			copy to already exists, dont copy
#		 0 do it anyway 
# Returns:
#    0 - success
#    1 - error
sub PROCESSCopyAsNew
{
    my ($SourceODMname, $TargetODMname, $where, $withvalues, $auto,
	$unique, $SourceFields, $TargetFields, 
        $WhereFieldValues, $WithFieldValues, $autoFields) = @_;
    my $error;
    my $item;
    my $max;
    my $SourceODMstanzas;
    my $TargetODMstanzas;
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my $TmpValue;
    my $UseList = 1;
    my $UseHash = 0;
    my %Types;
    my $ValueCtr = 1;
    my $ValueString = "a";
    my $SourceStanza;
    my %FieldValues;
    my $i;
    my $DoThisStanza = 0;
    my $Ctr = 0;
    my @keylist;
    my $StanzaExists = 0;


    ::DEBUG_PRINT ( "PROCESSCopyAsNew:\n" , $::DEBUG_PROCESSCopyAsNew);
    
    ::DEBUG_PRINT ("\tSource, Target ODM's: $SourceODMname, $TargetODMname\n", 
	$::DEBUG_PROCESSCopyAsNew);
    
    ::DEBUG_PRINT ( "\tSource fields:\n" , $::DEBUG_PROCESSCopyAsNew);
    
    foreach $key (@{$SourceFields} )
    {
    	::DEBUG_PRINT ( "\t\t$key\n",
		$::DEBUG_PROCESSCopyAsNew );
    }

    ::DEBUG_PRINT ( "\tTarget fields:\n" , $::DEBUG_PROCESSCopyAsNew);
    
    foreach $key (@{$TargetFields} )
    {
    	::DEBUG_PRINT ( "\t\t$key\n",
		$::DEBUG_PROCESSCopyAsNew );
    }

    ::DEBUG_PRINT ( "\twhere, with, auto, unique\n" , 
	$::DEBUG_PROCESSCopyAsNew);
    
    ::DEBUG_PRINT ( "\t$where, $withvalues, $auto, $unique\n" , 
	$::DEBUG_PROCESSCopyAsNew);
    

    if ($where)
    {
        foreach $key (%{$WhereFields} )
        {
    	    ::DEBUG_PRINT ( "\twhere fields: $key=$WhereFields->{$key}\n",
		$::DEBUG_PROCESSCopyAsNew );
        }
    }

   if ($withvalues)
   {
        foreach $key (keys (%{$WithFieldValues}) )
        {
    	::DEBUG_PRINT ( "\twith fields: $key=$WithFieldValues->{$key}\n",
		$::DEBUG_PROCESSCopyAsNew );
        }
   }
   
   if ($auto)
   {
        foreach $key (@{$autoFields} )
        {
    	::DEBUG_PRINT ( "\tauto fields: $key\n",
		$::DEBUG_PROCESSCopyAsNew );
        }
   }

   ###############
   # Valid ODMs? #
   ###############
   if ( !( cl_odmmanIO::IOclassExists($SourceODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $SourceODMname does not exist ");
	::LOG_PRINT(1,"in line $::LINE_NUMBER\n");
	return 1;
    }

   if ( !( cl_odmmanIO::IOclassExists($TargetODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $TargetODMname does not exist in ");
	::LOG_PRINT(1,"line $::LINE_NUMBER\n");
	return 1;
    }

   #################
   # Valid Fields? #
   #################

    if ( ::PARSEgetFieldValid($SourceODMname,$UseList,$SourceFields) )
    	 {return 1;}

    if ( ::PARSEgetFieldValid($TargetODMname,$UseList,$TargetFields) )
    	 {return 1;}


    if ($auto)
    {
         if ( ::PARSEgetFieldValid($TargetODMname,$UseList,$autoFields) )
    	      {return 1;}
    } 
    if ($withvalues)
    {
         if ( ::PARSEgetFieldValid($TargetODMname,$UseHash,$WithFieldValues) )
    	      {return 1;}
    } 
    if ($where)
    {
         if ( ::PARSEgetFieldValid($SourceODMname,$UseHash,$WhereFieldValues) )
    	      {return 1;}
    } 

    ###########################################
    # if source and target odms are the same, #
    # unique must be used.                    #
    ###########################################
    if ( ($SourceODMname eq $TargetODMname) && (!$unique))
    {
	::LOG_PRINT(1,"ERROR: source = target ODM without using "); 
	::LOG_PRINT(1,"\"unique\" in line $::LINE_NUMBER\n");
	return 1;
    }
	
   ########################################
   # auto fields cant match Target fields #
   ########################################
   foreach $item (@{$autoFields})
   {
	foreach $key (@{$TargetFields})
 	{
	     if ($item eq $key)
	     {
		::LOG_PRINT(1,"ERROR: auto field matches target field ");
		::LOG_PRINT(1,"at line $::LINE_NUMBER\n","$::LINE_NUMBER");
	        return 1;
	     }
	}
   }

   ########################################
   # with fields cant match Target fields #
   ########################################
   foreach $item (keys (%{$WithFieldValues}))
   {
	foreach $key (@{$TargetFields})
 	{
	     if ($item eq $key)
	     {
		::LOG_PRINT(1,"ERROR: with field matches target field ");
		::LOG_PRINT(1,"at line $::LINE_NUMBER\n","$::LINE_NUMBER");
	        return 1;
	     }
	}
   }
 
   ###########################
   # read in the ODM stanzas #
   ###########################
    if (cl_odmmanIO::IOreadODMstanzasGlobal($SourceODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $SourceODMname at line 
$::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    if (cl_odmmanIO::IOreadODMstanzasGlobal($TargetODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $TargetODMname at line 
$::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    $SourceODMstanzas = $::ODM{$SourceODMname};
    $TargetODMstanzas = $::ODM{$TargetODMname};

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSCopyAsNew)
    {
	print "Source:\n";
	print "ODM is $SourceODMname\n";
 
	::DEBUG_PRINT_ODMs($SourceODMname);

    }
    #### <<DEBUG

    ########################################
    # Get the source fields, put in target #
    ########################################
    %Types = cl_odmmanIO::IOgetODMdefAsHash($SourceODMname); 
    $DoThisStanza = 0;
    @keylist = keys (%{$WhereFieldValues});

    foreach $SourceStanza (@{$SourceODMstanzas})
    {
	$DoThisStanza = 0;

        ##########
        # where? #
	##########
	if ($where)
	{
	     $Ctr = 0;
	     foreach $key (keys (%{$WhereFieldValues}) )
	     {
		 ###########################
		 # to compare, kill quotes #	
		 ###########################
		 if (($Types{$key} eq "char") && 
		    ($SourceStanza->{$key} =~ /^"(.*)"$/) ) 
		 {
		     $TmpValue = $1;
		 }else{
		     $TmpValue = $SourceStanza->{$key};
		 }
		
		 if ($WhereFieldValues->{$key} eq $TmpValue)
		 {
		     $Ctr++;
		 }		

	     }
	    
	     if ($Ctr == ($#keylist+1))
	     {
		 $DoThisStanza = 1;
	     }
        }else{
	     $DoThisStanza = 1;

	}# end if where	
   

	if($DoThisStanza)
        {
	    $error = 
		cl_odmmanMANIP::MANIPinsertStanzasWithFields(
		$SourceODMname, $TargetODMname,\@{$TargetODMstanzas},
		\%{$SourceStanza},\@{$SourceFields},
		\@{$TargetFields}, $withvalues, \%{$WithFieldValues},
		$unique,$auto, \@{$autoFields} );	
	}
    }# end foreach source stanza





    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ##### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSCopyAsNew)
    {
	print "After Target:\n";

	::DEBUG_PRINT_ODMs($TargetODMname);
    }
    ##### <<DEBUG

    ::DEBUG_PRINT("End PROCESSCopyAsNew\n\n",$::DEBUG_PROCESSCopyAsNew);

    return 0;

}# End PROCESSCopyAsNew

#####################################################################
# PROCESSCopyAsExisting
# Purpose:
#    To process part of the copy command read in by PARSECopy
# Input:
#    SourceODMname,TargetODMname - The ODMs to copy from and to
#    SourceFields, TargetFields  - list of fields to copy fr/to
#    SourceWhere - 1 means use SourceWhereFieldValues to identify 
#		     source stanzas
#	           0 not
#    SourceWhereFieldValues - hash of field/value pairs used if 
#		   SourceWhere =1
#    TargetWhereFieldFields - hash of source field/target field
#		     pairs 
#    unique     - 1 means if the new stanza you are trying to
#			copy to already exists, dont copy
#		 0 do it anyway 
#    static     - 1 means include the StaticFieldValues in the
#		    stanza you are writing
#    StaticFieldValues - hash of field/value pairs to copy if
#		static = 1
# NOTE:  the source where field values limits which source
#	 stanzas are used.  Also, the target where field
#	 fields says "for the field1=field2, if the value
#	 of the field1 in the source stanza equals the
#	 value of field2 in the target stanza, then this
#	 stanza is selected."    
# Returns:
#    0 - success
#    1 - error
sub PROCESSCopyAsExisting
{
    my ($SourceODMname, $TargetODMname, $SourceWhere, 
	$static, $unique, $SourceFields, $TargetFields,
        $SourceWhereFieldValues, $TargetWhereFieldFields,
	$StaticFieldValues ) = @_;
    my $error;
    my $item;
    my $max;
    my $SourceODMstanzas;
    my $TargetODMstanzas;
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my %SourceTypes = ();
    my %TargetTypes = ();
    my $TmpValue;
    my $UseList = 1;
    my $UseHash = 0;
    my $SourceType;
    my $TargetType;
    my $ValueCtr = 1;
    my $ValueString = "a";
    my $SourceStanza;
    my $TargetStanza;
    my %FieldValues;
    my $i;
    my $DoThisStanza = 0;
    my $Ctr = 0;
    my @keylist;
    my $StanzaExists = 0;
    my @TmpFieldList;
    my $SourceField;
    my $TargetField;
    my $SourceValue;
    my $TargetValue;

    ::DEBUG_PRINT ( "PROCESSCopyAsExisting:\n" , 
	$::DEBUG_PROCESSCopyAsExisting);
    
    ::DEBUG_PRINT ("\tSource, Target ODM's: $SourceODMname, $TargetODMname\n", 
	$::DEBUG_PROCESSCopyAsExisting);
   
    if ( $#{$SourceFields} != $#{$TargetFields} )
    {
	::LOG_PRINT(1,"ERROR:  Number of source and target fields are not 
equal at line $::LINE_NUMBER\n", "$::LINE_NUMBER");
	return 1;
    }
 
    ::DEBUG_PRINT ( "\tSource fields:\n" , $::DEBUG_PROCESSCopyAsExisting);
    
    foreach $key (@{$SourceFields} )
    {
    	::DEBUG_PRINT ( "\t\t$key\n",
		$::DEBUG_PROCESSCopyAsExisting );
    }

    ::DEBUG_PRINT ( "\tTarget fields:\n" , $::DEBUG_PROCESSCopyAsExisting);
    
    foreach $key (@{$TargetFields} )
    {
    	::DEBUG_PRINT ( "\t\t$key\n",
		$::DEBUG_PROCESSCopyAsExisting );
    }

    ::DEBUG_PRINT ( "\tSourceWhere, static, unique\n" , 
	$::DEBUG_PROCESSCopyAsExisting);
    
    ::DEBUG_PRINT ( "\t$SourceWhere, $static, $unique\n" , 
	$::DEBUG_PROCESSCopyAsExisting);
    

    if ($SourceWhere)
    {
        ::DEBUG_PRINT ( "\tSourceWhereFields:\n" , 
	    $::DEBUG_PROCESSCopyAsExisting);

        foreach $key (keys (%{$SourceWhereFields}) )
        {
    	    ::DEBUG_PRINT ( "\t\t$key=$SourceWhereFields->{$key}\n",
		$::DEBUG_PROCESSCopyAsExisting );
        }
    }

   ::DEBUG_PRINT ( "\tTargetWhereField Fields:\n" , 
	    $::DEBUG_PROCESSCopyAsExisting);

   foreach $key (keys (%{$TargetWhereFieldFields}) )
   {
    	    ::DEBUG_PRINT ( "\t\t$key=$TargetWhereFieldFields->{$key}\n",
		$::DEBUG_PROCESSCopyAsExisting );
   }


   if ($static)
   {
        ::DEBUG_PRINT ( "\tstatic Field values:\n" , 
	    $::DEBUG_PROCESSCopyAsExisting);

        foreach $key (keys (%{$StaticFieldValues}) )
        {
    	::DEBUG_PRINT ( "\t\t$key=$StaticFieldValues->{$key}\n",
		$::DEBUG_PROCESSCopyAsExisting );
        }
   }
   

   ###############
   # Valid ODMs? #
   ###############
   if ( !( cl_odmmanIO::IOclassExists($SourceODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $SourceODMname does not exist in 
line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
	return 1;
    }

   if ( !( cl_odmmanIO::IOclassExists($TargetODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $TargetODMname does not exist in 
line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
	return 1;
    }

   #################
   # Valid Fields? #
   #################

    if ( ::PARSEgetFieldValid($SourceODMname,$UseList,$SourceFields) )
    	 {return 1;}

    if ( ::PARSEgetFieldValid($TargetODMname,$UseList,$TargetFields) )
    	 {return 1;}

    if ($static)
    {
         if ( ::PARSEgetFieldValid($TargetODMname,$UseHash,$StaticFieldValues) )
    	      {return 1;}
    } 
 
    if ($SourceWhere)
    {
         if ( ::PARSEgetFieldValid($SourceODMname,$UseHash,
		$SourceWhereFieldValues) )
    	      {return 1;}
    } 

    if ( ::PARSEgetFieldValid($SourceODMname,$UseHash,
		$TargetWhereFieldFields) )
          {return 1;}
    @TmpFieldList = values(%{$TargetWhereFieldFields});
    if ( ::PARSEgetFieldValid($TargetODMname,$UseList,
		\@TmpFieldList) )
          {return 1;}



   ###########################
   # read in the ODM stanzas #
   ###########################
    if (cl_odmmanIO::IOreadODMstanzasGlobal($SourceODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $SourceODMname at line 
$::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    if (cl_odmmanIO::IOreadODMstanzasGlobal($TargetODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $TargetODMname at line 
$::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    $SourceODMstanzas = $::ODM{$SourceODMname};
    $TargetODMstanzas = $::ODM{$TargetODMname};

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSCopyAsExisting)
    {
	print "Source:\n";
	print "ODM is $SourceODMname\n";
 
	::DEBUG_PRINT_ODMs($SourceODMname);

    }
    #### <<DEBUG

    #################################################
    # for the TargetWhereFieldFields, if the source #
    # and target fields are not either both "char"  #
    # or both not "char" (number) then the char one #
    # must contain only numbers                     #
    #################################################
    %SourceTypes = cl_odmmanIO::IOgetODMdefAsHash($SourceODMname); 
    %TargetTypes = cl_odmmanIO::IOgetODMdefAsHash($TargetODMname); 
   
    foreach $key (keys (%{$TargetWhereFieldFields}))
    {
	$SourceField = $key;
	$TargetField = $TargetWhereFieldFields->{$key};

	$SourceType = $SourceTypes{$key};
	$TargetType = $TargetTypes{ $TargetWhereFieldFields->{$key} };

	::DEBUG_PRINT("\tsource field, type = ",
		$::DEBUG_PROCESSCopyAsExisting);

	::DEBUG_PRINT("$SourceField, $SourceType\n",
		$::DEBUG_PROCESSCopyAsExisting);

	::DEBUG_PRINT("\ttarget field, type = ",
		$::DEBUG_PROCESSCopyAsExisting);

	::DEBUG_PRINT("$TargetField, $TargetType\n",
		$::DEBUG_PROCESSCopyAsExisting);


	###########################
	# the source type is char #
	###########################
	if (($SourceType ne $TargetType) && ($SourceType eq "char"))
	{
	    foreach $SourceStanza (@{$SourceODMstanzas})
	    {
		if ( $SourceStanza->{$SourceField} =~ /^"(.*)"$/ )
		{
		     $SourceValue = $1;
		     if ($SourceValue =~ /^(\d+)/ )
		     {	
			$SourceValue = $1;
		     }else{
     ::LOG_PRINT(1,"ERROR: when comparing fields where the types do not
match, the character field (source here) must contain only numbers in line
$::LINE_NUMBER\n", "$::LINE_NUMBER");
		     return 1;

		     }
		}
	    } # end for each source stanza

	} # end if types not equal

	###########################
	# the target type is char #
	###########################
	if (($SourceType ne $TargetType) && ($TargetType eq "char"))
	{
	    foreach $TargetStanza (@{$TargetODMstanzas})
	    {
		if ( $TargetStanza->{$TargetField} =~ /^"(.*)"$/ )
		{
		     $TargetValue = $1;
		     if ($TargetValue =~ /^(\d+)$/ )
		     {
			$TargetValue = $1;
		     }else{
     ::LOG_PRINT(1,"ERROR: when comparing fields where the types do not
match, the character field (target here) must contain only numbers in line
$::LINE_NUMBER\n", "$::LINE_NUMBER");
		     return 1;

		     }
		}
	    } # end for each source stanza

	} # end if types not equal

    } # end foreach field fields

    ###############################
    # if source field is char and #
    # target field is num, source #
    # field must only have nums   #
    ###############################
    for ($i=0; $i<=$#{$SourceFields}; $i++)
    {
	if ( ($SourceTypes{ $SourceFields->[$i] } eq "char") &&
	     ($TargetTypes{ $TargetFields->[$i] } ne "char") )
	{
	    foreach $SourceStanza (@{$SourceODMstanzas})
	    {
		if ($SourceStanza->{$SourceFields->[$i]}
			=~ /^"(\d+)"$/)
		{
		    $key = $1;
		}else{
    ::DEBUG_PRINT("\tfield value is $SourceStanza->{$SourceFields->[$i]}\n",
	$::DEBUG_PROCESSCopyAsExisting);

    ::LOG_PRINT(1,"ERROR: To copy an alphanumeric field to a
numeric field, all values of the alphanumeric field must be
only numeric in line $::LINE_NUMBER\n","$::LINE_NUMBER");
		    return 1;
		}
	    }# end for each source stanza
	}

    }# end for each source field

    ########################################
    # Identify source stanzas, process the #
    # appropriate target stanzas           #
    ########################################
    $DoThisStanza = 0;
    @keylist = keys (%{$SourceWhereFieldValues});

    foreach $SourceStanza (@{$SourceODMstanzas})
    {
	$DoThisStanza = 0;

        #################
        # Source where? #
	#################
	if ($SourceWhere)
	{
	     $Ctr = 0;
	     foreach $key (keys (%{$SourceWhereFieldValues}) )
	     {
		 ###########################
		 # to compare, kill quotes #	
		 ###########################
		 if (($SourceTypes{$key} eq "char") && 
		    ($SourceStanza->{$key} =~ /^"(.*)"$/) ) 
		 {
		     $TmpValue = $1;
		 }else{
		     $TmpValue = $SourceStanza->{$key};
		 }
		
		 if ($SourceWhereFieldValues->{$key} eq $TmpValue)
		 {
		     $Ctr++;
		 }		

	     }
	    
	     if ($Ctr == ($#keylist+1))
	     {
		 $DoThisStanza = 1;
	     }
        }else{
	     $DoThisStanza = 1;

	}# end if Source where	
   

	if($DoThisStanza)
        {

	    $TargetODMstanzas = 
		cl_odmmanMANIP::MANIPcopyStanzasWithFields(
		$SourceODMname, $TargetODMname,\@{$TargetODMstanzas},
		\%{$SourceStanza},\@{$SourceFields},
		\@{$TargetFields}, \%{$TargetWhereFieldFields},
		$static, \%{$StaticFieldValues}, $unique );	
	}
    }# end foreach source stanza


    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ##### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSCopyAsExisting)
    {
	print "After Target:\n";

	::DEBUG_PRINT_ODMs($TargetODMname);
    }
    ##### <<DEBUG

    ::DEBUG_PRINT("End PROCESSCopyAsExisting\n\n",
	$::DEBUG_PROCESSCopyAsExisting);

    return 0;

}# End PROCESSCopyAsExisting

#####################################################################
# PROCESSCustom
# Purpose:
#    To process the custom command read in by PARSECustom
# Input:
#    ODMnames - List of ODMs to process 
#    Script  - Custom script to run
# Returns:
#    0 - success
#    1 - error
sub PROCESSCustom
{
    my ($ODMnames, $Script) = @_;
    my $error;
    my $item;
    my $max;
    my $ODMstanzas = "";
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my $UseList = 1;
    my $UseHash = 0;
    my $ODM_name;
    my @ODMstrings = ();
    my $TmpString;
    my $old;
    my @FieldOrder = ();
    my %Types = ();
    my $ScriptError = 0;

    my $UseTmpFile = 1; 
    
    ::DEBUG_PRINT ( "PROCESSCustom:\n" , $::DEBUG_PROCESSCustom);
 
   ###############
   # Valid ODMs? #
   ###############
   foreach $ODM_name (@{$ODMnames})
   {
  	if ( !( cl_odmmanIO::IOclassExists($ODM_name) ) )
   	{
	     ::LOG_PRINT(1,"ERROR: ODM name $ODM_name does not 
exist in line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
	     return 1;
    	}
    }
  

    
    ##################################################
    # if already in ODM process string, else read in #
    ##################################################
    foreach $ODM_name (@{$ODMnames})
    {

	if (exists $::ODM{$ODM_name})
	{
	    ::DEBUG_PRINT("\t$ODM_name already in ODM...\n",
		$::DEBUG_PROCESSCustom);

	    @FieldOrder = cl_odmmanIO::IOgetODMdefAsList($ODM_name);

            foreach $stanza (@{$::ODM{$ODM_name}})
            {
                $ODMstanzas = $ODMstanzas.$ODM_name.":\n";
	        foreach $key (@FieldOrder)
	        {
	            $ODMstanzas = $ODMstanzas."\t".$key." = ";
		        $ODMstanzas = $ODMstanzas.$stanza->{$key}."\n"; 
	        }
	        $ODMstanzas = $ODMstanzas."\n";
            }
    
	}else{
	    ::DEBUG_PRINT("\t$ODM_name not already in ODM...\n",
		$::DEBUG_PROCESSCustom);

            $old = $/;
            undef $/;

	    open(ODM,"odmget $ODM_name |");
	    @TmpString = <ODM>;
	    $/ = $old;
	    close(ODM);
	
	    $TmpString = join ("",@TmpString);
	    $TmpString = $TmpString . "\n";
	    $ODMstanzas = $ODMstanzas.$TmpString;   
	}# end if exists ODM name
   
    }# end for each ODM name

    
    ::DEBUG_PRINT("\tBefore, String is \n\n$ODMstanzas\n\n",
	$::DEBUG_PROCESSCustom);

    if ($UseTmpFile)
    {
	$old = $/;
	undef $/;
	open(ODM,"> $::TMP_FILE ");
	print ODM "$ODMstanzas";
	$/ = $old;
	close(ODM);
    }

    ###############
    # call script #
    ###############
    $old = $/;
    undef $/;
    ::DEBUG_PRINT("\tCalling custom script...\n",
	$::DEBUG_PROCESSCustom);

    ############################
    # script exists, executes? #
    ############################
    $Script = $ENV{CLPATH}."/scripts/custom/".$Script;
    ::LOG_PRINT (1,"Custom script name is $Script.\n");

    if (!(-x $Script))
    {
	::LOG_PRINT(1,"ERROR:  custom script $Script is not
executable in line $::LINE_NUMBER\n", "$::LINE_NUMBER");
	return 1;
    }

    if ($UseTmpFile)
    {

        if (!(open(ODM,"cat $::TMP_FILE | $Script 2>&1 |")))
        {
	    ::LOG_PRINT(1,"ERROR: Could not open $Script 
at line $::LINE_NUMBER\n","$::LINE_NUMBER");
	    return 1;
        }
    }else{	
 
        unless (open(ODM,"echo '$ODMstanzas' | $Script |"))
        {
	    ::LOG_PRINT(1,"ERROR: Could not open $Script 
at line $::LINE_NUMBER\n","$::LINE_NUMBER");
	    return 1;
        }

    }# end if use tmp file

    @ODMstrings = <ODM>;
    $ODMstanzas = join("",@ODMstrings);

    $/ = $old;
    close(ODM);
    $ScriptError = $?;

    if ($UseTmpFile)
    {
	system ("rm $::TMP_FILE ");
    }

    ::DEBUG_PRINT("\tDone with custom script...\n",
	$::DEBUG_PROCESSCustom);

    ::DEBUG_PRINT("\tAfter, String is \n\n$ODMstanzas\n\n",
	$::DEBUG_PROCESSCustom);

    ###################################################
    # if error in custom script, print error message. #
    ###################################################
    if ($ScriptError)
    {
	$error = 1;
	::LOG_PRINT(1,"Custom script returned an error:\n");
	::LOG_PRINT(1,"$ODMstanzas\n");

    }else{

    	##################################
    	# put these in the ODM structure #
    	##################################
        if ($ODMstanzas =~ /\S/)
        {
           ::DEBUG_PRINT("\tWe have charachters in the ODMstanzas\n\n",
              $::DEBUG_PROCESSCustom);
           $error = cl_odmmanMANIP::MANIPcustom($ODMstanzas,\@{$ODMnames});
        }
    }

    if ($error) {return 1;}

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSCustom)
    {

	print "After ODMstanzas:\n";

	foreach $ODM_name (@{$ODMnames})
	{

	    print "ODM is $ODM_name\n";
 
	   ::DEBUG_PRINT_ODMs($ODM_name);
	}
    }
    #### <<DEBUG

    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ::DEBUG_PRINT("End PROCESSCustom\n\n",$::DEBUG_PROCESSCustom);
 

    return 0;

}# End PROCESSCustom

#####################################################################
# PROCESSDelete
# Purpose:
#    To process the delete command read in by PARSEDelete
# Input:
#    ODMname - The ODM to delete from 
#    where   - Flag that indicates FieldValues are used
#    FieldValues - A hash table of field(key) and values that identify
#		   which stanza to delete (optional)
# Returns:
#    0 - success
#    1 - error
sub PROCESSDelete
{
    my ($ODMname, $where, $FieldValues) = @_;
    my $error;
    my $item;
    my $max;
    my $ODMstanzas;
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my $UseList = 1;
    my $UseHash = 0;

    ::DEBUG_PRINT ( "PROCESSDelete:\n" , $::DEBUG_PROCESSDelete);

    if ($where)
    {	
        if ( ::PARSEgetFieldValid($ODMname,$UseHash,$FieldValues) )
    	    {return 1;}

    	foreach $key (keys (%{$FieldValues}) )
    	{
    	    ::DEBUG_PRINT ( "\tfield values: $key, $FieldValues->{$key}\n",
		$::DEBUG_PROCESSDelete );
        }
    }
 
   ############
   # Validity #
   ############
   if ( !( cl_odmmanIO::IOclassExists($ODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $ODMname does not exist in ");
	::LOG_PRINT(1, "line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
	return 1;
    }


   ###########################
   # read in the ODM stanzas #
   ###########################
    if (cl_odmmanIO::IOreadODMstanzasGlobal($ODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $ODMname at line $::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    $ODMstanzas = $::ODM{$ODMname};

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSDelete)
    {
	print "before ODMstanzas:\n";
	print "ODM is $ODMname\n";
 
	::DEBUG_PRINT_ODMs($ODMname);

    }
    #### <<DEBUG

    ##############################################
    # delete from global $ODM from cl_odmmanMAIN #
    ##############################################
    $::ODM{$ODMname} = 
 	cl_odmmanMANIP::MANIPdeleteStanzas($ODMname, \@{$ODMstanzas}, 
 	$where, \%{$FieldValues});

    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ##### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSDelete)
    {
	print "After:\n";

	::DEBUG_PRINT_ODMs($ODMname);
    }
    ##### <<DEBUG

    ::DEBUG_PRINT("End PROCESSDelete\n\n",$::DEBUG_PROCESSDelete);
 

    return 0;

}# End PROCESSDelete

#####################################################################
# PROCESSInsert
# Purpose:
#    To process the insert command read in by PARSEInsert
# Input:
#    ODMname - The ODM to insert into 
#    unique - 1 means if this stanza already exists, don't add a new 
#		one.
#             0 add it whether it exists or not 
#    auto - 1 means we have some fields that will be automatically 
#		generated
#           0 means we are not auto generating any fields
#    fieldValues - A hash table of field values to insert.  Each key 
#		   in the hash table is the field, and each value
#		   is the value to insert in that field
#    autoFields -  If auto is 1, then this will contain the list of
#                  fields to automatically generate values for.  This
#		   only works for numeric fields.  PROCESSInsert
#		   will look for the current highest value and
#		   increment it by one, and put that value in the
#		   field.
# Returns:
#    0 - success
#    1 - error
sub PROCESSInsert
{
    my ($ODMname, $unique, $auto, $fieldValues, $autoFields) = @_;
    my $error;
    my $item;
    my $max;
    my $ODMstanzas;
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my $TmpValue;
    my $UseList = 1;
    my $UseHash = 0;
    my %Types;
    my $ValueCtr = 1;
    my $ValueString = "a";
 
    ::DEBUG_PRINT ( "PROCESSInsert:\n" , $::DEBUG_PROCESSInsert);

    foreach $key (keys (%{$fieldValues}) )
    {
    	::DEBUG_PRINT ( "\tfield values: $key, $fieldValues->{$key}\n",
		$::DEBUG_PROCESSInsert );
    }
 
    foreach $key (@{$autoFields} )
    {
 
    	::DEBUG_PRINT ( "\tauto fields: $key\n",
		$::DEBUG_PROCESSInsert );
    }
   #################
   # Valid fields? #
   #################
   if ( !( cl_odmmanIO::IOclassExists($ODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $ODMname does not exist"); 
	::LOG_PRINT(1,"in line $::LINE_NUMBER\n");
	return 1;
    }

    if ( ::PARSEgetFieldValid($ODMname,$UseHash,$fieldValues) )
    	 {return 1;}

    if ($auto)
    {
         if ( ::PARSEgetFieldValid($ODMname,$UseList,$autoFields) )
    	      {return 1;}
    } 
 
   ###########################
   # read in the ODM stanzas #
   ###########################
    if (cl_odmmanIO::IOreadODMstanzasGlobal($ODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $ODMname at line $::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    $ODMstanzas = $::ODM{$ODMname};

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSInsert)
    {
	print "before we insert, here is ODMstanzas:\n";
	print "ODM is $ODMname\n";
 
	::DEBUG_PRINT_ODMs($ODMname);

    }
    #### <<DEBUG

    ######################################
    # add quotes to char type fields in  #
    # fieldValues                        #
    ######################################
    %Types = cl_odmmanIO::IOgetODMdefAsHash($ODMname); 

    foreach $key (keys (%{$fieldValues}))
    {
 	if ($Types{$key} eq "char")
        {
		$fieldValues->{$key} = "\"".$fieldValues->{$key}."\"";
	} 
    }

    #########
    # auto? #
    #########
    if ($auto)
    {
        foreach $item (@{$autoFields})
        {

            $max = cl_odmmanMANIP::MANIPfindMax(\@{$ODMstanzas} ,$item,
		$Types{$item});
 	    if ($Types{$item} eq "char")
	    {
        	 if ($max eq "")
	         {
		     $ValueCtr = 1;
		 }else{
		     #####################################
		     # kill quotes and look for num      #
		     # to get base, update, put together #
		     #####################################
		     if ($max =~ /^"(.*)"$/)
	             {
		         $TmpValue = $1;

		         if ($TmpValue =~ /^(.*)(\d+)$/)
		         {
			    $ValueString = $1;
			    $ValueCtr    = $2 +1;
		         }else{
			    $ValueString = $TmpValue;
			    $ValueCtr = 1;
		          }	
		      }
	           $fieldValues->{$item} = "\"".$ValueString.$ValueCtr."\"";
		   }
	    }else{       

                 $fieldValues->{$item} = $max + 1;
 	     }
       }# end foreach
    }# end if auto

    ############
    # unique ? #
    ############
    if ($unique)
    {
        my $stanzaExists;
        $stanzaExists = 
		cl_odmmanMANIP::MANIPstanzaExists($ODMname,\@{$ODMstanzas}, 
			\%{$fieldValues});
        if ($stanzaExists)
        {
            ::DEBUG_PRINT("\tStanza already exists at line $::LINE_NUMBER\n", 
		$::DEBUG_PROCESSInsert);
            return 0;
        }
    }  

    ##############################################
    # insert into global $ODM from cl_odmmanMAIN #
    ##############################################
    $::ODM{$ODMname} = 
	cl_odmmanMANIP::MANIPinsertStanzas($ODMname, \@{$ODMstanzas}, 
	\%{$fieldValues});

    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ##### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSInsert)
    {
	print "After the insert we get:\n";

	::DEBUG_PRINT_ODMs($ODMname);
    }
    ##### <<DEBUG

    ::DEBUG_PRINT("End PROCESSInsert\n\n",$::DEBUG_PROCESSInsert);

    return 0;

}# End PROCESSInsert

#####################################################################
# PROCESSSort
# Purpose:
#    To process the sort command read in by PARSESort
# Input:
#    ODMname   - The ODM to process 
#    ByFields  - List of fields to sort on
#    HowSort   - List of how to sort where element can be:
#			alpha, ASCII, or numeric
#    DirSort   - List of direction for each sort:
#			ascending or descending 
# Returns:
#    0 - success
#    1 - error
sub PROCESSSort
{
    my ($ODMname, $ByFields, $HowSort, $DirSort) = @_;
    my $error;
    my $item;
    my $max;
    my $ODMstanzas;
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my $TmpValue;
    my $UseList = 1;
    my $UseHash = 0;
    my %Types;
    my $i;
    my $fld;
    my $stanza;

    ::DEBUG_PRINT ( "PROCESSSort:\n" , $::DEBUG_PROCESSSort);

    foreach $key (@{$ByFields} )
    {
    	::DEBUG_PRINT ( "\tBy fields: $key\n",
		$::DEBUG_PROCESSSort );
    }

    foreach $key (@{$HowSort} )
    {
    	::DEBUG_PRINT ( "\tHow Sort: $key\n",
		$::DEBUG_PROCESSSort );
    }

    foreach $key (@{$DirSort} )
    {
    	::DEBUG_PRINT ( "\tDirection to sort: $key\n",
		$::DEBUG_PROCESSSort );
    }


   ######################
   # Valid ODM, fields? #
   ######################
   if ( !( cl_odmmanIO::IOclassExists($ODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $ODMname does not exist in ");
	::LOG_PRINT(1,"line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
	return 1;
    }

    if ( ::PARSEgetFieldValid($ODMname,$UseList,$ByFields) )
    	 {return 1;}

   #######################
   # set the global vars #
   #######################
   @::SORT_FIELDS = @{$ByFields};
   @::HOW_SORT    = @{$HowSort};
   @::DIR_SORT    = @{$DirSort};
   $::ODM_NAME    = $ODMname;

   ::DEBUG_PRINT("\tfield = $::SORT_FIELDS[0]\n",
	$::DEBUG_PROCESSSort);
 
   ::DEBUG_PRINT("\thow = $::HOW_SORT[0]\n",
	$::DEBUG_PROCESSSort);
 
   ::DEBUG_PRINT("\tdir = $::DIR_SORT[0]\n",
	$::DEBUG_PROCESSSort);
 
   ###########################
   # read in the ODM stanzas #
   ###########################
    if (cl_odmmanIO::IOreadODMstanzasGlobal($ODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $ODMname at line $::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    $ODMstanzas = $::ODM{$ODMname};

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSSort)
    {
	print "before ODMstanzas:\n";
	print "ODM is $ODMname\n";
 
	::DEBUG_PRINT_ODMs($ODMname);

    }
    #### <<DEBUG
    
    ############################################
    # Check that char not compared in num sort #
    ############################################
    %Types = cl_odmmanIO::IOgetODMdefAsHash($ODMname);

    for ($i=0; $i <= $#{$ByFields}; $i++)
    {
        $TmpField = $ByFields->[$i];
	
    	if (($Types { $TmpField } eq "char") &&
	    ($HowSort->[$i] =~ /numeric/i) )
        {
	     foreach $stanza (@{$ODMstanzas})
	     {
		$TmpValue = $stanza->{$TmpField};
		
		if ($TmpValue =~/^"(.*)"$/)
		{
		    $TmpValue = $1;
		    if ($TmpValue =~ /\D/)
		    {
	::LOG_PRINT(1,"ERROR: Can not sort on char field unless ");
	::LOG_PRINT(1,"all values are numeric at line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
	return 1;

		    }
		}
	
	     }# end foreach stanza

        }# end if types eq char 

    } # end for each ByFields


    

    ###############################################
    # sort rank of global $ODM from cl_odmmanMAIN #
    ###############################################
    my @RankIn = (0..$#{$ODMstanzas});
    my @RankOut = ();

    ::DEBUG_PRINT("\tRank In:\n", $::DEBUG_PROCESSSort);
    foreach $item (@RankIn)
    {
        ::DEBUG_PRINT("\t\t$item\n", $::DEBUG_PROCESSSort);
    }
 
    @RankOut = sort cl_odmmanMANIP::MANIPsortStanzas @RankIn;
   
    ::DEBUG_PRINT("\tRank Out:\n", $::DEBUG_PROCESSSort);
    foreach $item (@RankOut)
    {
        ::DEBUG_PRINT("\t\t$item\n", $::DEBUG_PROCESSSort);
    }

    ##########################################
    # reorder global $ODM from cl_odmmanMAIN #
    ##########################################
    my @TmpODMstanzas = @{$ODMstanzas};

    for ($i = 0; $i <= ($#{$ODMstanzas}); $i++ )
    {
	$::ODM{$ODMname}[$i] = $TmpODMstanzas[$RankOut[$i]];
    }

    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ##### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSSort)
    {
	print "After:\n";

	::DEBUG_PRINT_ODMs($ODMname);
    }
    ##### <<DEBUG

    ::DEBUG_PRINT("End PROCESSSort\n\n",$::DEBUG_PROCESSSort);
 
    return 0;

}# End PROCESSSort

#####################################################################
# PROCESSUpdate
# Purpose:
#    To process the update command read in by PARSEUpdate
# Input:
#    ODMname - The ODM to insert into 
#    where - 1 means the FieldValues is used to identify the stanza 
#            0 not 
#    FieldValues - A hash table of field values to use to identify
#		   the stanza if where is 1.
#    SetFields -   A hash table of the fields whose value we set.
# Returns:
#    0 - success
#    1 - error
sub PROCESSUpdate
{
    my ($ODMname, $where, $FieldValues, $SetFields) = @_;
    my $error;
    my $item;
    my $max;
    my $ODMstanzas;
    my $key;
    my @FieldList = ();
    my $FieldOk = 0;
    my $TmpField;
    my $UseList = 1;
    my $UseHash = 0;

    ::DEBUG_PRINT ( "PROCESSUpdate:\n" , $::DEBUG_PROCESSUpdate);

    foreach $key (keys (%{$SetFields}) )
    {
    	::DEBUG_PRINT ( "\tfield values: $key, $SetFields->{$key}\n",
		$::DEBUG_PROCESSUpdate );
    }
 
    if ($where)
    { 
    	foreach $key (keys (%{$FieldValues}) )
    	{
    	    ::DEBUG_PRINT ( "\tfield values: $key, $FieldValues->{$key}\n",
		$::DEBUG_PROCESSUpdate );
    	}
     }

   ######################
   # Valid ODM, fields? #
   ######################
   if ( !( cl_odmmanIO::IOclassExists($ODMname) ) )
   {
	::LOG_PRINT(1,"ERROR: ODM name $ODMname does not exist in ");
	::LOG_PRINT(1,"line $::LINE_NUMBER\n",
		"$::LINE_NUMBER");
	return 1;
    }

    if ( ::PARSEgetFieldValid($ODMname,$UseHash,$SetFields) )
    	 {return 1;}

    if ($where)
    {
   	if ( ::PARSEgetFieldValid($ODMname,$UseHash,$FieldValues) )
    	 	{return 1;}
    }

   
   ###########################
   # read in the ODM stanzas #
   ###########################
    if (cl_odmmanIO::IOreadODMstanzasGlobal($ODMname))
    {
        ::LOG_PRINT(12,"ERROR: reading in $ODMname at line $::LINE_NUMBER\n",
		$::LINE_NUMBER);
        return 1;
    }

    $ODMstanzas = $::ODM{$ODMname};

    #### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSUpdate)
    {
	print "before ODMstanzas:\n";
	print "ODM is $ODMname\n";
 
	::DEBUG_PRINT_ODMs($ODMname);

    }
    #### <<DEBUG

    #########################################
    # update global $ODM from cl_odmmanMAIN #
    #########################################
    $::ODM{$ODMname} = 
	cl_odmmanMANIP::MANIPupdateStanzas($ODMname, $where, \@{$ODMstanzas}, 
	\%{$FieldValues}, \%{$SetFields});

    $::ODMwasChanged = 1;   # Indicates that we changed the ODM in MAIN


    ##### >>DEBUG
    if ($::DEBUG && $::DEBUG_PROCESSUpdate)
    {
	print "After:\n";

	::DEBUG_PRINT_ODMs($ODMname);
    }
    ##### <<DEBUG

    ::DEBUG_PRINT("End PROCESSUpdate\n\n",$::DEBUG_PROCESSUpdate);
 
    return 0;

}# End PROCESSUpdate
#####################################################################

#####################################################################
# ODMmanager PROCESSER <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#####################################################################

return TRUE;
### END cl_odmmanPROCESS.pm ###########################################
