#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_bookshelf.cgi.sh 1.33 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2003,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

###############################################################################
#
# Name: wsm_bookshelf.cgi
#
# Description: Prints a page of links to the HACMP documentation 
#
# Arguments: 
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
# Questions ? Comments ? Suggestions ? mailto:hafeedbk@us.ibm.com
#
###############################################################################

# @(#)45        1.33 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_bookshelf.cgi.sh, hacmp.utils, 52haes_r541, 0747C_hacmp541 1/11/08 11:26:55


#######################################################################
#
#  Function Name: find_file
#
#  Description: Determines if the specified file (see Input Parameters)
#               exists on the disk, and if so, creates a hyperlink to
#               that file. If the file does not exist, then a plain-text
#               message is displayed instead.
#
#  Input Parameters:
#
#        anchor         the URL/file to be linked to
#        anchor_text    the translated, descriptive string to display
#                       for this link/label.
#        item_complete  a Boolean indicator of whether or not this
#                       link is finished, and should be closed out
#                       (i.e. with "</li>").
#        anchor_type    a string indicating what type of document the
#                       current link is:  html, pdf, who, index
#        anchor_parent  an optional string, indicating the parent book
#                       for the current file. Used to make the output
#                       a little more verbose and helpful.
#
#  Output Parameters: None.
#
#  Return Values:
#          Success
#          A line for each error condition
#
#######################################################################
function find_file {
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    anchor=$1
    anchor_text=$2

    #
    # If there's anything specified in the third parameter
    # and it's not a false, the assume it's true.
    #
    item_complete=false
    if [[ (! -z $3) && ($3 != false) ]] then 
        item_complete=true
    fi

    typeset anchor_type=$4
    typeset anchor_parent=""
    if [[ -n "$5" ]]; then
        anchor_parent=$5
    else
        anchor_parent=$anchor_text
    fi

    filename="$DOCUMENT_ROOT../htdocs/en_US$anchor"
    if [[ -e $filename ]] then 

        typeset default_msg=''
        typeset msgnum=''
        if [[ $anchor_type == "html" ]]; then
            default_msg="View the HTML version of the '$anchor_text' document."
            msgnum=1036
        elif [[ $anchor_type == "pdf" ]]; then
            default_msg="View the PDF version of the '$anchor_parent' document."
            msgnum=1037
        elif [[ $anchor_type == "who" ]]; then
            default_msg="View the intended audience for the '$anchor_parent' document."
            msgnum=1039
        elif [[ $anchor_type == "index" ]]; then
            default_msg="View the index for the '$anchor_parent' document."
            msgnum=1038
        fi
        typeset help_msg=$(getmsg -s 1 cluster.cat "$msgnum" "$default_msg" "$anchor_parent")

        LINK="\
<li><font class='act_bookshelf act_bookshelf_link'
          title='$help_msg'
          onclick=\"JavaScript: location.href='$anchor';\"
          onMouseOver=\"JavaScript: this.className += ' highlight';\"
          onMouseOut=\"JavaScript: this.className = this.className.replace(/\s?highlight/g, '');\">$anchor_text</font>"

        if [[ $item_complete == true ]] then
            LINK="$LINK</li>"
        fi

        print "$LINK"
    else
        print "<li>$anchor_text is not installed."
    fi

} # End of "find_file()"



################################################################
################################################################
##
## Main starts here
##
################################################################
################################################################

PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Supports global/env specification
VERSION="1.33"

# set the platform
UNAME=$(uname -m)
case $UNAME in
    ppc* | i* )
        export PLATFORM="__LINUX__"
        ;;
    * )
        export PLATFORM="__AIX__"
        ;;
esac

export WSM_VERSION=$VERSION
export WSM_PROGNAME=$PROGNAME

# initialize the cgi library
CGI_BIN="."
BINDIR=/usr/es/sbin/cluster/wsm/cgi-bin
CONFDIR=/usr/es/sbin/cluster/wsm

# Run some access validations.  This script will also output the
# Content-type header so we don't have to do it here.
$BINDIR/wsm_auth.cgi
if (( $? != 0 )); then
    exit 0
fi


#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: there is no need to check for root ownership of the
#       WebSMIT configuration file here. It has already been
#       validated by the call to "wsm_auth.cgi", above.
#=============================================================
export SKIN_LOGO="/skins/DEFAULT_logo.gif"
CONF_FILE="$CONFDIR/wsm_smit.conf"
set -a
. $CONF_FILE >/dev/null
set +a

[[ $VERBOSE_LOGGING == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
typeset LOGFILE="/usr/es/sbin/cluster/wsm/logs/wsm_smit.log"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


TITLE=$(getmsg cluster.cat 972 "HACMP Documentation Bookshelf")

# print the html header and opening body tags
$CGI_BIN/wsm_html_lib body_start_noscript "$TITLE" act_bookshelf

print "<h1 class='act_bookshelf'>$TITLE</h1>"
print "<ul class='menu'>"

typeset CONCEPTS=$(getmsg cluster.cat 973 "Concepts and Facilities Guide")
typeset WHO_SHOULD=$(getmsg cluster.cat 974 "Who should use this guide?")
typeset INDEX=$(getmsg cluster.cat 975 "Index")
typeset PDF_VERSION=$(getmsg cluster.cat 976 "PDF Version")

#
# Create the Linux-only documentation links
#
if [[ "$PLATFORM" == "__LINUX__" ]]; then

    typeset BOOK_TITLE="HACMP for Linux: Installation and Administration Guide"
    find_file "/HAES/ha_linux/ha_linux.htm" "$BOOK_TITLE" false html
    print "<ul class='docOptions'>"
    find_file "/HAES/ha_linux.pdf" "$PDF_VERSION" true pdf "$BOOK_TITLE"
    find_file "/HAES/ha_linux/preface.html" "$WHO_SHOULD" true who "$BOOK_TITLE"
    find_file "/HAES/ha_linux/IX.html" "$INDEX" true index "$BOOK_TITLE"
    print "</ul></li>"

    print "<h3 class='act_bookshelf'>Related Books:</h3>"

fi # End of the Linux-only documentation


#
# Create the common documentation links
#
find_file "/HAES/ha_concepts/ha_concepts.htm" "$CONCEPTS" false html
print "<ul class='docOptions'>"
find_file "/HAES/ha_concepts.pdf" "$PDF_VERSION" true pdf "$CONCEPTS"
find_file "/HAES/ha_concepts/preface.html" "$WHO_SHOULD" true who "$CONCEPTS"
find_file "/HAES/ha_concepts/IX.html" "$INDEX" true index "$CONCEPTS"
print "</ul></li>"


typeset PLANNING=$(getmsg cluster.cat 1032 "Planning Guide")

find_file "/HAES/ha_plan/ha_plan.htm" "$PLANNING" false html
print "<ul class='docOptions'>"
find_file "/HAES/ha_plan.pdf" "$PDF_VERSION" true pdf "$PLANNING"
find_file "/HAES/ha_plan/preface.html" "$WHO_SHOULD" true who "$PLANNING"
find_file "/HAES/ha_plan/IX.html" "$INDEX" true index "$PLANNING"
print "</ul></li>"

typeset INSTALLATION=$(getmsg cluster.cat 1033 "Installation Guide")

find_file "/HAES/ha_install/ha_install.htm" "$INSTALLATION" false html
print "<ul class='docOptions'>"
find_file "/HAES/ha_install.pdf" "$PDF_VERSION" true pdf "$INSTALLATION"
find_file "/HAES/ha_install/preface.html" "$WHO_SHOULD" true who "$INSTALLATION"
find_file "/HAES/ha_install/IX.html" "$INDEX" true index "$INSTALLATION"
print "</ul></li>"

typeset ADMIN=$(getmsg cluster.cat 1034 "Administration Guide")

find_file "/HAES/ha_admin/ha_admin.htm" "$ADMIN" false html
print "<ul class='docOptions'>"
find_file "/HAES/ha_admin.pdf" "$PDF_VERSION" true pdf "$ADMIN"
find_file "/HAES/ha_admin/preface.html" "$WHO_SHOULD" true who "$ADMIN"
find_file "/HAES/ha_admin/IX.html" "$INDEX" true index "$ADMIN"
print "</ul></li>"

typeset TROUBLE=$(getmsg cluster.cat 1035 "Troubleshooting Guide")

find_file "/HAES/ha_troubleshoot/ha_troubleshoot.htm" "$TROUBLE" false html
print "<ul class='docOptions'>"
find_file "/HAES/ha_troubleshoot.pdf" "$PDF_VERSION" true pdf "$TROUBLE"
find_file "/HAES/ha_troubleshoot/preface.html" "$WHO_SHOULD" true who "$TROUBLE"
find_file "/HAES/ha_troubleshoot/IX.html" "$INDEX" true index "$TROUBLE"
print "</ul></li>"

typeset CLIENTS=$(getmsg cluster.cat 979 "Programming Client Applications")

find_file "/HAES/ha_clients/ha_clients.htm" "$CLIENTS" false html
print "<ul class='docOptions'>"
find_file "/HAES/ha_clients.pdf" "$PDF_VERSION" true pdf "$CLIENTS"
find_file "/HAES/ha_clients/preface.html" "$WHO_SHOULD" true who "$CLIENTS"
find_file "/HAES/ha_clients/IX.html" "$INDEX" true index "$CLIENTS"
print "</ul></li>"


#
# Create the AIX-only documentation links
#
if [[ "$PLATFORM" == "__AIX__" ]]; then
    # The Linux book has its own glossary, so only display this on AIX.

    typeset GLOSSARY=$(getmsg cluster.cat 1007 "Master Glossary")

    find_file "/HAES/ha_glossary/glossary.html" "$GLOSSARY" false html
    print "<ul class='docOptions'>"
    find_file "/HAES/ha_glossary.pdf" "$PDF_VERSION" true pdf "$GLOSSARY"
    print "</ul></li>"

fi # End of the AIX-only documentation

print "</ul>"


#==============================================================
# Add some links to the HACMP redbooks, and related material.
# The "for" loop, below, is organized in data triplets, with
# the first element being the display label, the second being
# the flyover help text, and the third being the site URL.
# This organization was done solely for the purpose of making
# this code easier to read and maintain!
#==============================================================
print "
<hr size=4/>
<h1 class='act_bookshelf'>HACMP Online</h1>
<ul class='menu'>"

typeset -i IDX=0
typeset label=
typeset desc=
typeset site=
for INFO in "HACMP on the Web"                                        \
                   "The primary HACMP web site."                      \
                   "http://www.ibm.com/systems/p/ha/"                 \
            "HACMP Success Stories"                                   \
                   "HACMP case studies."                              \
                   "http://www.ibm.com/software/success/cssdb.nsf/advancedsearchVW?SearchView&Query=(hacmp)+AND+[WebSiteProfileListTX]=eserverpseries&site=eserverpseries&frompage=ts&Start=1&Count=30&cty=en_us"                     \
            "HACMP Frequently Asked Questions"                        \
                   "Answers for frequently asked HACMP questions."    \
                   "http://www.ibm.com/systems/p/ha/faq.html"         \
            "HACMP/XD Frequently Asked Questions"                     \
                   "Answers for frequently asked HACMP/XD questions." \
                   "http://www.ibm.com/systems/p/ha/faq3.html"        \
            "HACMP Workshops"                                         \
                   "Workshops for HACMP."                             \
                   "http://www.redbooks.ibm.com/projects.nsf/WorkshopIndex/WorkshopIndex?SearchView&Query=HACMP&SearchMax=4999"                       \
            "[SEPARATOR]"                                             \
            "HACMP Resources"                                         \
                   "HACMP white papers and documentation."            \
                   "http://www.ibm.com/systems/p/ha/resources.html"   \
            "HACMP Documentation"                                     \
                   "HACMP documentation."                             \
                   "http://www.ibm.com/systems/p/library/hacmp_docs.html" \
            "HACMP Redbooks"                                          \
                   "All HACMP Redbook material."                      \
                   "http://www.redbooks.ibm.com/redbooks.nsf/Redbooks?SearchView&Query=HACMP&SearchOrder=4&SearchMax=4999"                            \
            "HACMP Redpapers"                                         \
                   "All HACMP Redpaper material."                     \
                   "http://www.redbooks.ibm.com/redbooks.nsf/Redpapers?SearchView&Query=HACMP&&SearchOrder=4SearchMax=4999"                           \
            "HACMP Technotes"                                         \
                   "All HACMP Technote material."                     \
                   "http://www.redbooks.ibm.com/redbooks.nsf/Tips?SearchView&Query=HACMP&SearchOrder=4&SearchMax=4999"                                \
            "HACMP ITSO Resources"                                    \
                   "All HACMP material developed by the IBM International Technical Support Organization."                                            \
                   "http://www.redbooks.ibm.com/cgi-bin/searchsite.cgi?Query=HACMP&SearchOrder=4&SearchMax=4999"
do
    if [[ $INFO == "[SEPARATOR]" ]]; then
        print "<hr size=1/>"
        IDX=0  # Reset the index
        label= desc= site=
    else
        (( IDX += 1 ))
        if (( IDX == 1 )); then label=$INFO; continue; fi
        if (( IDX == 2 )); then desc=$INFO; continue; fi
        site=$INFO

        if [[ -n $site && -n $label && -n $desc ]]; then
            print "\
<li><font class='act_bookshelf act_bookshelf_link'
          title=\"$desc\"
          onclick=\"JavaScript: location.href='$site';\"
          onMouseOver=\"JavaScript: this.className += ' highlight';\"
          onMouseOut=\"JavaScript: this.className = this.className.replace(/\s?highlight/g, '');\">$label</font>"
        fi

        IDX=0  # Reset the index
        label= desc= site=
    fi
done

print "</ul>"


# print the closing html
$CGI_BIN/wsm_html_lib body_end_noscript
