#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_clstat.cgi.sh 1.36.1.16 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2003,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)34        1.36.1.16 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_clstat.cgi.sh, hacmp.wsm, 52haes_r541, 0747C_hacmp541 1/11/08 11:32:14

###############################################################################
#
#  Name:  wsm_clstat.cgi
#
#  Description: Displays the state of the cluster in html suitable for
#		viewing in a web browser. Status is retrieved via the clstrmgr
#
#  Arguments:   None
#
#  Returns:     0 on success
#		1 on failure
#
#  Environment:
#
# Questions ? Comments ? Suggestions ? mailto:hafeedbk@us.ibm.com
#
###############################################################################

###############################################################################
# 
#  Name: log_error
#
#  Logs any errors to the server logfile
#
#  Arguments: 	none (refernces global errmsg)
#
#  Usage:        none
#
#  Returns:      none
#
#  Environment: 
#
###############################################################################
log_error()
{

if [[ $VERBOSE_LOGGING == "high" ]]; then
    # Log any errors
    {
        date +"[%b %d %Y, %T]"
        print -- "$PROGNAME:  error: $ERMSG "
        print "User IP = \"$REMOTE_ADDR\""
        print ""
    } >> $LOGFILE
fi

}

###############################################################################
# 
#  Name: print_address_info
#
#  Prints the address information for the node and network given in the
#  environment
#
#  Arguments: 	none
#
#  Usage:        print_address_info
#
#  Returns:      0         Success
#                1         Failure
#
#  Environment: 
#
###############################################################################
print_address_info()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    # Get key (IP addresses) from MIB
    addresses=$(echo "$ADDRESS_MIB_FUNC" | grep "$ADDRESS_IP\.$node_id\." | sort)
    print "<ul class=\"expanded\">"

    # Get the active Node for each IP address
    for address in $addresses
    do
        address_net_id=$(echo "$ADDRESS_MIB_FUNC" | 
                grep -w "$ADDRESS_NET.$node_id.$address =" | cut -f3 -d" ")

        if [[ "$address_net_id" == "$net_id" ]]
        then
            active_node=$(echo "$ADDRESS_MIB_FUNC" | 
                    grep -w "$ADDRESS_ACTIVE_NODE.$node_id.$address =" | 
                    cut -f3 -d" ")

            if [[ -n "$active_node" ]]
            then
                address_label=$(echo "$ADDRESS_MIB_FUNC" | 
                        grep -w "$ADDRESS_LABEL.$node_id.$address =" | 
                        cut -f2 -d\")

                address_state_raw=$(echo "$ADDRESS_MIB_FUNC" | 
                        grep -w "$ADDRESS_STATE.$node_id.$address =" | 
                        cut -f3 -d" ")

                case $address_state_raw in
                2) address_state="<span class=\"up\">$UP</span>" ;; 
                4) address_state="<span class=\"down\">$DOWN</span>" ;; 
                8) address_state="<span class=\"unknown\">$UNKNOWN</span>" ;;
                esac

                address_role_raw=$(echo "$ADDRESS_MIB_FUNC" | 
                    grep -w "$ADDRESS_ROLE.$node_id.$address =" | cut -f3 -d" ")

                case $address_role_raw in
                16 | 144) address_role="service" ;;
                32)       address_role="standby" ;;
                64)       address_role="boot" ;;
                128)      address_role="sharedService" ;;
                esac

                # for non-ip networks we do not print an ip address nor do we
                # offer other options for service interfaces
                if [[ "$PRINT_IP_ADDRESS" != "true"  ]]; then
                    address=""
                    address_role="nonip"
                fi

                print "<li><span class=\"bullet\"></span><div class=\"helpGrouper\">"
                print "<span class=\"nostatus\">$address</span> | $address_state"

                # adapters support different operations based on their type/function
                case "$address_role" in
                "nonip")   # non ip
                    print "<div class=\"helpTip\">"
                    print "<h3>$WHAT_WOULD_YOU_LIKE ${address_label}?</h3>"

                    #
                    # Link to show interface details
                    #
                    PANEL=cllsif.name.dialog
                    VALUE_LIST="_rawname:"
                    ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                        -s "value_list=$VALUE_LIST" \
                        -s "_rawname=${address_label}"
		
                    printf "%b" \
                        "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                        "panel_id=$PANEL\" TARGET=\"_blank\" " \
                        "title=\"$SHOW_DETAILS\" " \
                        "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

		    ((CLSTAT_ID=CLSTAT_ID+1))

                    print "</div>"
                    ;;
                "boot")   # boot
                    print "<div class=\"helpTip\">"
                    print "<h3>$WHAT_WOULD_YOU_LIKE_B ${address_label}?</h3>"

                    #
                    # Link to show interface details
                    #
                    PANEL=cllsif.name.dialog
                    VALUE_LIST="_rawname:"
                    ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                        -s "value_list=$VALUE_LIST" \
                        -s "_rawname=${address_label}"

                    printf "%b" \
                        "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                        "panel_id=$PANEL\" TARGET=\"_blank\" " \
                        "title=\"$SHOW_DETAILS\" " \
                        "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

		    ((CLSTAT_ID=CLSTAT_ID+1))

                    print "</div>"
                    ;;
                "service")   # service
                    print "<div class=\"helpTip\">"
                    print "<h3>$WHAT_WOULD_YOU_LIKE_S ${address_label}?</h3>"

                    print "<a href=\"javascript:gotoLink('$COMM_PATH','/cgi-bin/wsm_name_hdr.cgi?name=cl_swap_adapter')\" title=\"$MOVE_SERVICE\" onMouseOver=\"JavaScript: window.status=''; return true;\">$MOVE_SERVICE</a><br>"

                    #
                    # Link to show interface details
                    #
                    PANEL=cllsif.name.dialog
                    VALUE_LIST="_rawname:"
                    ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                        -s "value_list=$VALUE_LIST" \
                        -s "_rawname=${address_label}"

                    printf "%b" \
                        "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                        "panel_id=$PANEL\" TARGET=\"_blank\" " \
                        "title=\"$SHOW_DETAILS\" " \
                        "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

		    ((CLSTAT_ID=CLSTAT_ID+1))

                    print "</div>"
                    ;;
                esac

                print "</div>&nbsp;</li>"
            fi
        fi
    done

    print "</ul>"
}


###############################################################################
# 
#  Name: print_network_info
#
#  Prints the network information for the node given in the environment
#
#  Arguments: 	none
#
#  Usage:        print_network_info
#
#  Returns:      0         Success
#                1         Failure
#
#  Environment: 
#
###############################################################################
print_network_info()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    # Get network IDs
    network_ids=$(echo "$NETWORK_MIB_FUNC" | 
                  grep "$NETWORK_ID\.$node_id\." | cut -f3 -d" " | sort -u)

    # Get states for this networks on this node
    for net_id in $network_ids
    do 
        network_name=$(echo "$NETWORK_MIB_FUNC" | 
            grep -w "$NETWORK_NAME.$node_id.$net_id" | cut -f2 -d\")

        network_attribute=$(echo "$NETWORK_MIB_FUNC" | 
            grep -w "$NETWORK_ATTRIBUTE.$node_id.$net_id" | cut -f3 -d" ")

        network_state_raw=$(echo "$NETWORK_MIB_FUNC" | 
            grep -w "$NETWORK_STATE.$node_id.$net_id" | cut -f3 -d" ")

        case $network_state_raw in
        2) network_state="<span class=\"up\">$UP</span>" ;; 
        4) network_state="<span class=\"down\">$DOWN</span>" ;; 
        32) network_state="<span class=\"unstable\">$JOINING</span>" ;; 
        64) network_state="<span class=\"unstable\">$LEAVING</span>" ;;
        esac

        PRINT_IP_ADDRESS="true"

        # If serial type network, then don't attempt to print IP Address
        [[ $network_attribute == 4 ]] && PRINT_IP_ADDRESS="false"

        print "<li><div class='helpGrouper' onClick='JavaScript: twisty(this)' style='cursor:pointer'/>"
        print -- "$NETWORK <span class=\"nostatus\">$network_name</span> | $network_state"

        print "<div class=\"helpTip\">"
        print "<h3>$WHAT_WOULD_YOU_LIKE_N ${network_name}?</h3>"

        #
        # Link to show network details
        #
        PANEL=cm_change_show_a_networkdialog.
        VALUE_LIST="_rawname:"
        ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                              -s "value_list=$VALUE_LIST" \
                              -s "_rawname=${network_name}"

        printf "%b" \
            "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
            "panel_id=$PANEL&" \
            "cmd_to_classify=cm_change_show_a_network_in_the_hacmp_cluster_select" \
            "\" target=\"_blank\" " \
            "title=\"$SHOW_DETAILS\" " \
            "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

	((CLSTAT_ID=CLSTAT_ID+1))

        print "</div>"

        print "</div>&nbsp;"

        print_address_info
        print "</li>"

    done
}

###############################################################################
# 
#  Name: print_res_info
#
#  Prints the resourcess contained in a resource group
#
#  Arguments: 	none
#
#  Usage:        
#
#  Returns:      0         Success
#                1         Failure
#
#
#  Environment: 
#
###############################################################################
print_res_info()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    # get a list of resource group id's for the cluster
    RES_IDS=$(echo "$RES_MIB_FUNC" | grep "$RES_ID" | cut -f3 -d" " | sort -u)

    print "<ul class=\"expanded\">"

    print -- "$RES_IDS" | while read res_id
    do
        res_name=$(echo "$RES_MIB_FUNC" | 
            grep -w "$RES_NAME\.$rg\.$res_id =" | cut -f2 -d\")

        res_type_raw=$(echo "$RES_MIB_FUNC" | 
            grep -w "$RES_TYPE\.$rg\.$res_id =" | cut -f3 -d" ")

        case "$res_type_raw" in
        1000) res_type=$SERVICE_IP_LABEL ;;
        1001) res_type=$HTY_SERVICE ;;
        1002) res_type=$FILESYSTEM ;;
        1003) res_type=$VOLUME_GROUP ;;
        1004) res_type=$DISKS ;;
        1005) res_type=$AIX_CONNECTIONS ;;
        1006) res_type=$APPLICATION ;;
        1007) res_type=$CONCURRENT_VG ;;
        1008) res_type=$COMM_LINK ;;
        esac

        if [[ -n "$res_type" && -n "$res_name" ]]; then
            print "<li><span class=\"bullet\"></span>"
            print -- "$res_type: <span class=\"nostatus\">$res_name</span>"
            print "</li>"
        fi

    done

    print "</ul>"
}

###############################################################################
# 
#  Name: rg_per_node_info
#
#  Prints the resource groups contained in a node
#
#  Arguments: 	none
#
#  Usage:        print_node_info
#
#  Returns:      none
#
#  Environment: 
#
###############################################################################
rg_per_node_info()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    RG_STATE_MIB_FUNC=$(echo "$RG_STATE_MIB" | 
                        grep "$RG_STATE_BRANCH\..\..\.$node_id")
    RG_IDS=$(echo "$RG_STATE_MIB_FUNC" | 
             grep -w "$RG_STATE_BRANCH\.1" | cut -f3 -d" " | sort -u)

    # if there are no resource groups, $RES_IDS will be empty; in that case,
    # just return from the function
    if [[ -z $RG_IDS ]]; then
        return 0;
    fi

    for rg in $RG_IDS
    do
        rg_node_state_raw=$(echo "$RG_STATE_MIB_FUNC" | 
            grep "$RG_STATE_STATE\.$rg\.$rg_node" | cut -f3 -d" ")

        # if a group is down is isn't grouped with the node
        if [[ $rg_node_state_raw == 4 ]]; then
            continue;
        fi

        case $rg_node_state_raw in
        2) rg_node_state="<span class=\"up\">$ONLINE</span>" ;;
        4) rg_node_state="<span class=\"down\">$OFFLINE</span>" ;;
        8) rg_node_state="<span class=\"unknown\">$UNKNOWN</span>" ;;
        16) rg_node_state="<span class=\"unstable\">$ACQUIRING</span>" ;;
        32) rg_node_state="<span class=\"unstable\">$RELEASING</span>" ;;
        64) rg_node_state="<span class=\"down\">$ERROR</span>" ;;
        esac
        group_name=$(echo "$RG_MIB" | grep -w "$RG_NAME\.$rg" | cut -f2 -d\")

        RES_MIB_FUNC=$(echo "$RES_MIB" | grep "$RES_BRANCH\..\.$rg")

        print "<li><div class='helpGrouper' onClick='JavaScript: twisty(this)' style='cursor:pointer'/>"
        print "Resource Group: <span class=\"nostatus\">$group_name"
        print "</span> | $rg_node_state"

        print "<div class=\"helpTip\">"
        print "<h3>$WHAT_WOULD_YOU_LIKE ${group_name}?</h3>"

        #
        # Link to move RG
        #
        PANEL=cl_resgrp_move_node.select
        VALUE_LIST='group_name:'
        ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                              -s "value_list=$VALUE_LIST" \
                              -s "group_name=${group_name}"

        printf "%b" \
            "<a href=\"/cgi-bin/wsm_name_hdr.cgi?clstat_id=$CLSTAT_ID&" \
            "name=$PANEL\" target=\"_blank\" " \
            "title=\"$MOVE_GROUP\" " \
            "onMouseOver=\"JavaScript: window.status=''; return true;\">$MOVE_GROUP</a><br>\n"
        
	((CLSTAT_ID=CLSTAT_ID+1))

        #
        # Link to bring RG offline
        #
        PANEL=cl_resgrp_stop.select
        VALUE_LIST='group_name:'
        ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                              -s "value_list=$VALUE_LIST" \
                              -s "group_name=${group_name}"

        printf "%b" \
            "<a href=\"/cgi-bin/wsm_name_hdr.cgi?clstat_id=$CLSTAT_ID&" \
            "name=$PANEL\" target=\"_blank\" " \
            "title=\"$GROUP_OFFLINE\" " \
            "onMouseOver=\"JavaScript: window.status=''; return true;\">$GROUP_OFFLINE</a><br>\n"
        
	((CLSTAT_ID=CLSTAT_ID+1))

        #
        # Link to change RG
        #
        PANEL=cm_change_a_resource_group_dialog.
        VALUE_LIST='_rawname:'
        ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                              -s "value_list=$VALUE_LIST" \
                              -s "_rawname=${group_name}"

        printf "%b" \
               "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
               "panel_id=$PANEL&" \
               "cmd_to_classify=cm_change_resources_for_a_rg_select\" " \
               "target=\"_blank\" " \
               "title=\"$GROUP_CHANGE\" " \
               "onMouseOver=\"JavaScript: window.status=''; return true;\">$GROUP_CHANGE</a><br>\n"
        

	((CLSTAT_ID=CLSTAT_ID+1))

        #
        # Link to show RG details
        #
        PANEL=clshowres.dialog
        VALUE_LIST='_rawname:'
        ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                              -s "value_list=$VALUE_LIST" \
                              -s "_rawname=${group_name}"

        printf "%b" \
               "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
               "panel_id=$PANEL\" target=\"_blank\" " \
               "title=\"$SHOW_DETAILS\" " \
               "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"
        
	((CLSTAT_ID=CLSTAT_ID+1))

        print "</div>"

        print "</div>&nbsp;"

        print_res_info
        print "</li>"
    done

}

###############################################################################
# 
#  Name: print_node_info
#
#  Prints the node information for each node found in the MIB
#
#  Arguments: 	none
#
#  Usage:        print_node_info
#
#  Returns:      0         Success
#                1         Failure
#
#
#  Environment: 
#
###############################################################################
print_node_info()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    NODE_ID_COUNTER=0

    print "<h2>$ALL_NODES</h2>"
    print "<ul class=\"twistyRoot\">"

    while [[ $cluster_num_nodes != 0 ]]
    do
        # Get node information for each node
        node_id=$(echo "$NODE_MIB" |
            grep -w "$NODE_ID.$NODE_ID_COUNTER" | cut -f3 -d " ")

        (( NODE_ID_COUNTER += 1 ))

        # Node ids may not be contiguous
        if [[ -z "$node_id" ]]  
        then
            continue
        fi

        node_name=$(echo "$NODE_MIB" | grep -w "$NODE_NAME.$node_id" | cut -f2 -d\")
        node_state_raw=$(echo "$NODE_MIB" | 
            grep -w "$NODE_STATE.$node_id" | cut -f3 -d" ")

        case $node_state_raw in
        2) node_state="<span class=\"up\">$UP</span>" ;; 
        4) node_state="<span class=\"down\">$DOWN</span>" ;; 
        32) node_state="<span class=\"unstable\">$JOINING</span>" ;; 
        64) node_state="<span class=\"unstable\">$LEAVING</span>" ;;
        esac

        # COMM_PATH=$(ODMDIR=$DCD odmget -q "name = $node_name AND object = COMMUNICATION_PATH" HACMPnode | grep "value =" | cut -f2 -d\")
        COMM_PATH=$(ODMDIR=$DCD 
            /usr/es/sbin/cluster/utilities/cllsnode -pci $node_name |
            tail -1 | cut -f 2 -d":")
        FQDN=$(host "$COMM_PATH" | cut -f1 -d" ")
        if [[ $FQDN != "host:" ]]; then
            COMM_PATH="$FQDN"
        fi

        NETWORK_MIB_FUNC=$(echo "$NETWORK_MIB" | grep "$NETWORK_BRANCH\..\.$node_id")
        ADDRESS_MIB_FUNC=$(echo "$ADDRESS_MIB" | grep "$ADDRESS_BRANCH\..\.$node_id")

        print "<li><div class='helpGrouper' onClick='JavaScript: twisty(this)' style='cursor:pointer'/>"
        print "Node: <span class=\"nostatus\">$node_name</span> | $node_state"

        # for nodes, display a different set of options based on state
        case $node_state_raw in
        2)    # node is up
            print "<div class=\"helpTip\">"
            print "<h3>$WHAT_WOULD_YOU_LIKE ${node_name}?</h3>"

            # 
            # Link to stop cluster services
            #
            printf "%b" \
                "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?panel_id=clstop\" " \
                "target=\"_blank\" " \
                "title=\"$STOP_SERVICES\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$STOP_SERVICES</a><br>\n"

            #
            # Link to Open a smit session
            #
            print "<a href=\"javascript:gotoLink('$COMM_PATH','/cgi-bin/wsm_menu_opt.cgi?name=cm_hacmp_main_menu_dmn')\" title=\"$OPEN_SMIT\" onMouseOver=\"JavaScript: window.status=''; return true;\">$OPEN_SMIT</a><br>"

            # 
            # Link to show node details
            #
            PANEL=cllsnode.name.dialog
            VALUE_LIST='_rawname:'
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST"  \
                -s "_rawname=${node_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "panel_id=$PANEL\" target=\"_blank\" " \
                "title=\"$SHOW_DETAILS\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            print "</div>"

            ;;
        4)   # node is down
            print "<div class=\"helpTip\">"
            print "<h3>$WHAT_WOULD_YOU_LIKE ${node_name}?</h3>"

            # 
            # Link to start cluster services
            #
            printf "%b" "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?panel_id=clstart\" " \
                "target=\"_blank\" " \
                "title=\"$START_SERVICES\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$START_SERVICES</a><br>\n"

            #
            # Link to open a WebSMIT session to this node
            #
            printf "%b" "<a href=\"javascript:gotoLink('$COMM_PATH'," \
                    "'/cgi-bin/wsm_menu_opt.cgi?name=cm_hacmp_main_menu_dmn')\"" \
                    "title=\"$OPEN_SMIT\" " \
                    "onMouseOver=\"JavaScript: window.status=''; return true;\">$OPEN_SMIT</a><br>\n"

            #
            # Link to show node details
            #
            PANEL=cllsnode.name.dialog
            VALUE_LIST="_rawname:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL  \
            		-s "value_list=$VALUE_LIST"  \
            		-s "_rawname=${node_name}"
            printf "%b" \
                "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "panel_id=$PANEL\" target=\"_blank\" " \
                "title=\"$SHOW_DETAILS\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            print "</div>"
            ;;
        esac
        print "</div>&nbsp;"

        print "<ul class=\"expanded\">"
        print_network_info

        # check that the cluster has resource groups
        if [[ -n "$RGS" ]]; then
            rg_per_node_info
        fi

        print "</ul></li>"

        (( cluster_num_nodes -= 1 ))

    done

    print "</ul>"

}


###############################################################################
# 
#  Name: print_cluster_info
#
#  Prints the cluster information for the cluster found in the MIB of which
#  this node is a member.
#
#  Arguments: 	none
#
#  Usage:        print_cluster_info
#
#  Returns:      None
#
#  Environment: 
#
###############################################################################
print_cluster_info ()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    # retrieve cluster name and state
    cluster_name=$(echo "$CLUSTER_MIB" | grep "$CLUSTER_NAME\.0" | cut -f2 -d\")

    cluster_state_raw=$(echo "$CLUSTER_MIB" | 
        grep "$CLUSTER_STATE\.0" | cut -f3 -d" ")

    case $cluster_state_raw in
    2) cluster_state="<span class=\"up\">$UP</span>" ;; 
    4) cluster_state="<span class=\"down\">$DOWN</span>" ;;
    esac

    cluster_substate_raw=$(echo "$CLUSTER_MIB" | 
        grep "$CLUSTER_SUBSTATE\.0" | cut -f3 -d" ")

    case $cluster_substate_raw in
    8)   cluster_substate="<span class=\"unknown\">$UNKNOWN</span>" ;; 
    16)  cluster_substate="<span class=\"unstable\">$UNSTABLE</span>" ;; 
    32)  cluster_substate="<span class=\"up\">$STABLE</span>" ;; 
    64)  cluster_substate="<span class=\"down\">$ERROR</span>" ;; 
    128) cluster_substate="<span class=\"unstable\">$RECONFIG</span>" ;;
    esac

    cluster_num_nodes=$(echo "$CLUSTER_MIB" | 
        grep "$CLUSTER_NUM_NODES\.0" | cut -f3 -d" ")

    print "<div id=\"clusterHeading\">"
    print "<h1>Cluster: <span class=\"nostatus\">$cluster_name</span></h1>"
    print "State: $cluster_state | "
    print "SubState: $cluster_substate"

    print "<div class=\"helpTip\">"

    # 
    # Link to show cluster config
    #
    printf "%b" "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?" \
        "panel_id=cm_display_hacmp_config_menu_dmn\" " \
        "target=\"_blank\" " \
        "title=\"$SHOW_CONFIG\" " \
        "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_CONFIG</a><br>\n"
    #print "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?panel_id=cm_show_evs\" target=\"_blank\" title=\"$SHOW_EVSUM\" onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_EVSUM</a><br>"
    print "<a href=\"/cgi-bin/wsm_cldisp.cgi\" target=\"_blank\" title=\"$SHOW_APPS\" onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_APPS</a>"
    print "</div>"

    print "</div>"

}


###############################################################################
# 
#  Name: print_rg_info
#
#  Prints the resource group information for all groups
#
#  Arguments: 	none
#
#  Usage:        print_rg_info
#
#  Returns:      None
#                1         Failure
#
#  Environment:  
#
###############################################################################
print_rg_info ()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    # if no groups, print a space
    if [[ -z "$RGS" ]]
    then
        return 0
    fi

    # retrieve list of groups
    RG_IDS=$(echo "$RG_MIB" | grep "$RG_ID" | cut -f3 -d" " | sort -u)

    print "<h2>$ALL_GROUPS</h2>"
    print "<ul class=\"twistyRoot\">"

    # print all groups
    for rg in $RG_IDS
    do
        group_name=$(echo "$RG_MIB" | grep -w "$RG_NAME\.$rg" | cut -f2 -d\")

        #
        # For 5.2, group policy is changing to use all custom groups - update
        # this section to support that effort 
        #
        group_policy_raw=$(echo "$RG_MIB" | grep -w "$RG_POLICY\.$rg" | cut -f3 -d" ")
        case $group_policy_raw in
        1) group_policy="CASCADING" ;;
        2) group_policy="ROTATING" ;;
        3) group_policy="CONCURRENT" ;;
        4) group_policy=$(echo "$RG_MIB" | 
                grep -w "$RG_USERNAMED\.$rg" | cut -f2 -d\") 
            ;;
        5|0) group_policy="CUSTOM" ;;
        esac

        # mib contains state for this group on each node - make a hash (:-) of both
        RG_STATE_MIB_LOOP=$(echo "$RG_STATE_MIB" | grep "$RG_STATE_BRANCH\..\.$rg")
        RG_LOOP_NODES=$(echo "$RG_STATE_MIB_LOOP" | 
            grep -w "$RG_STATE_NODE\.$rg" | cut -f3 -d" ")
        rg_up=$(echo "$RG_STATE_MIB_LOOP" | 
            grep "$RG_STATE_STATE\.$rg" | cut -f3 -d" " | grep "2")

        print "<li><div class='helpGrouper' onClick='JavaScript: twisty(this)' style='cursor:pointer'/>"
        print -- "$RG_HEADER <span class=\"nostatus\">$group_name</span> |"
        print "Type: <span class=\"nostatus\">$group_policy</span>"

        if [[ -n $rg_up ]]; then
            print "<div class=\"helpTip\">"
            print "<h3>$WHAT_WOULD_YOU_LIKE ${group_name}?</h3>"

            #
            # Link to move rg
            #
            PANEL=cl_resgrp_move_node.select
            VALUE_LIST="group_name:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST" \
                -s "group_name=${group_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_name_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "name=$PANEL\" target=\"_blank\" " \
                "title=\"$MOVE_GROUP\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$MOVE_GROUP</a><br>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            #
            # Link to stop rg
            #
            PANEL=cl_resgrp_stop.select
            VALUE_LIST="group_name:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST" \
                -s "group_name=${group_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_name_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "name=$PANEL\" target=\"_blank\" " \
                "title=\"$GROUP_OFFLINE\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$GROUP_OFFLINE</a><br>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            #
            # Link to change rg
            #
            PANEL=cm_change_a_resource_group_dialog.
            VALUE_LIST="_rawname:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST" \
                -s "_rawname=${group_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "panel_id=$PANEL&" \
                "cmd_to_classify=cm_change_resources_for_a_rg_select&" \
                "\" target=\"_blank\" " \
                "title=\"$GROUP_CHANGE\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$GROUP_CHANGE</a><br>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            #
            # Link to show rg details 
            #
            PANEL=clshowres.dialog
            VALUE_LIST="_rawname:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST" \
                -s "_rawname=${group_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "panel_id=$PANEL\" target=\"_blank\" " \
                "title=\"$SHOW_DETAILS\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            print "</div>"


        else
            print "<div class=\"helpTip\">"
            print "<h3>$WHAT_WOULD_YOU_LIKE ${group_name}?</h3>"

            #
            # Link to start rg
            #
            PANEL=cl_resgrp_start.select
            VALUE_LIST="group_name:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST" \
                -s "group_name=${group_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_name_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "name=$PANEL\" target=\"_blank\" " \
                "title=\"$GROUP_ONLINE\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$GROUP_ONLINE</a><br>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            #
            # Link to change rg
            #
            PANEL=cm_change_a_resource_group_dialog.
            VALUE_LIST="_rawname:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST" \
                -s "_rawname=${group_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "panel_id=$PANEL&" \
                "cmd_to_classify=cm_change_resources_for_a_rg_select&" \
                "\" target=\"_blank\" " \
                "title=\"$GROUP_CHANGE\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$GROUP_CHANGE</a><br>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            #
            # Link to show rg details
            #
            PANEL=clshowres.dialog
            VALUE_LIST="_rawname:"
            ${BINDIR}/wsm_session -d clstat_${CLSTAT_ID}_$PANEL \
                -s "value_list=$VALUE_LIST" \
                -s "_rawname=${group_name}"

            printf "%b" \
                "<a href=\"/cgi-bin/wsm_cmd_hdr.cgi?clstat_id=$CLSTAT_ID&" \
                "panel_id=$PANEL\" target=\"_blank\" " \
                "title=\"$SHOW_DETAILS\" " \
                "onMouseOver=\"JavaScript: window.status=''; return true;\">$SHOW_DETAILS</a>\n"

	    ((CLSTAT_ID=CLSTAT_ID+1))

            print "</div>"
        fi  

        print "</div>&nbsp;"

        print "<ul class=\"expanded\">"
        # display node and state for all participating nodes
        for rg_node in $RG_LOOP_NODES
        do
            rg_node_name=$(echo "$NODE_MIB" | 
                grep -w "$NODE_NAME\.$rg_node" | cut -f2 -d\")
            rg_node_state_raw=$(echo "$RG_STATE_MIB_LOOP" | 
                grep "$RG_STATE_STATE\.$rg\.$rg_node" | cut -f3 -d" ")
            case $rg_node_state_raw in
            2)  rg_node_state="<span class=\"up\">$ONLINE</span>" ;;
            4)  rg_node_state="<span class=\"down\">$OFFLINE</span>" ;;
            8)  rg_node_state="<span class=\"unknown\">$UNKNOWN</span>" ;;
            16) rg_node_state="<span class=\"unstable\">$ACQUIRING</span>" ;;
            32) rg_node_state="<span class=\"unstable\">$RELEASING</span>" ;;
            64) rg_node_state="<span class=\"down\">$ERROR</span>" ;;
            esac

            print "<li><span class=\"bullet\"></span>"
            print -- "$LOCATION <span class=\"nostatus\">$rg_node_name"
            print "</span> | $rg_node_state</li>"
        done
        print "</ul></li>"
    done
    print "</ul>"

}

###############################################################################
#
# Name: body_start
#
# Description: add the html to start the body
#
# Arguments:    
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
#
print_script ()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    print "<link rel='stylesheet' type='text/css' href='/wsm_default.css' />"
    print "<link rel='stylesheet' type='text/css' href='/wsm_custom.css' />"
    $($CGI_BIN/wsm_display_css);

    print "<script type=\"text/javascript\">"
    print "<!-- Begin"

    # automatically refresh the page every thirty seconds
    print "timeOut = setInterval(\"window.location.reload()\", 240000);"

    # build the hyperlink to a node, assuming that all nodes have the same
    # webserver configurations then open the link in a new window
    print "function gotoLink(nodeip,doc) {"
    print "url = location.protocol+'//'+nodeip+':'+location.port+doc;"
    print "clstatChild = this.open(url,'clstatChild');"
    print "clstatChild.focus();"
    print "}"

    print "//  End -->"
    print "</script>"

    print "
<script src=\"/wsm_clstat.js\"></script>
"

}



###############################################################################
# Main starts here
###############################################################################
PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Supports global/env specification
VERSION="1.31"

export WSM_VERSION=$VERSION
export WSM_PROGNAME=$PROGNAME

# initialize the cgi library
BINDIR=/usr/es/sbin/cluster/wsm/cgi-bin
CONFDIR=/usr/es/sbin/cluster/wsm
CGI_BIN=$BINDIR
CONF_FILE="$CONFDIR/wsm_smit.conf"

# set the platform
UNAME=$(uname -m)
case $UNAME in
    ppc* | i* )
        export PLATFORM="__LINUX__"
        ;;
    * )
        export PLATFORM="__AIX__"
        ;;
esac

# Run some access validations.  This script will also output
# the Content-type header so we don't have to do it here.
$BINDIR/wsm_auth.cgi "cgi-bin/wsm_clstat.cgi"
if (( $? != 0 )); then
    exit 0
fi


if [[ -f $CONF_FILE ]]; then
    set -a
    # Permissions on this file have already been checked via wsm_auth.cgi
    . $CONF_FILE >/dev/null
    set +a
fi


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
typeset LOGFILE="/usr/es/sbin/cluster/wsm/logs/wsm_smit.log"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


# fetch message text from the catalog(s)
UP=$(getmsg cluster.cat 980 "UP")
DOWN=$(getmsg cluster.cat 981 "DOWN")
UNKNOWN=$(getmsg cluster.cat 982 "UNKNOWN")
ONLINE=$(getmsg cluster.cat 983 "ONLINE")
OFFLINE=$(getmsg cluster.cat 984 "OFFLINE")
ACQUIRING=$(getmsg cluster.cat 984 "ACQUIRING")
RELEASING=$(getmsg cluster.cat 984 "RELEASING")
ERROR=$(getmsg cluster.cat 984 "ERROR")
JOINING=$(getmsg cluster.cat 985 "JOINING")
LEAVING=$(getmsg cluster.cat 986 "LEAVING")
STATE=$(getmsg cluster.cat 987 "State:")
LOCATION=$(getmsg cluster.cat 987 "Location:")
NETWORK=$(getmsg cluster.cat 988 "Network:")
SERVICE_IP_LABEL=$(getmsg cluster.cat 625  "Service IP Label")
HTY_SERVICE=$(getmsg cluster.cat 696  "HTY Service Label")
FILESYSTEM=$(getmsg cluster.cat 628  "Filesystems")
VOLUME_GROUP=$(getmsg cluster.cat 624  "Volume groups")
DISKS=$(getmsg cluster.cat 620  "DIsks")
AIX_CONNECTIONS=$(getmsg cluster.cat 9999  "AIX Connection Services")
APPLICATION=$(getmsg cluster.cat 942  "Application Servers")
CONCURRENT_VG=$(getmsg cluster.cat 609  "Concurrent Volume groups")
COMM_LINK=$(getmsg -s 10 cluster.cat 320  "Communication Links")
UNSTABLE=$(getmsg cluster.cat 989 "UNSTABLE")
STABLE=$(getmsg cluster.cat 990 "STABLE")
RECONFIG=$(getmsg cluster.cat 991 "RECONFIG")
ALL_GROUPS=$(getmsg cluster.cat 992 "All Resource Groups")
RG_HEADER=$(getmsg cluster.cat 993 "Resource Group:")
WHAT_WOULD_YOU_LIKE=$(getmsg cluster.cat 995 "What would you like to do with")
WHAT_WOULD_YOU_LIKE_N=$(getmsg cluster.cat 996 "What would you like to do with network")
WHAT_WOULD_YOU_LIKE_B=$(getmsg cluster.cat 997 "What would you like to do with boot address")
SHOW_DETAILS=$(getmsg cluster.cat 998 "Show Details")
SHOW_CONFIG=$(getmsg cluster.cat 67 "Show Cluster Definition")
SHOW_APPS=$(getmsg cluster.cat 1008 "Show Cluster Applications")
START_SERVICES=$(getmsg cluster.cat 55 "Start Cluster Services")
STOP_SERVICES=$(getmsg cluster.cat 60 "Stop Cluster Services")
OPEN_SMIT=$(getmsg -s 40 cluster.cat 78 "Open a SMIT Session on a Node")
MOVE_GROUP=$(getmsg -s 42 cluster.cat 8 "Move a Resource Group to Another Node")
GROUP_CHANGE=$(getmsg -s 39 cluster.cat 13 "Change/Show All Resources and Attributes for a Resource Group")

# these need messages in the catalog
SHOW_EVSUM=$(getmsg cluster.cat 1009 "Show Summary of Cluster Events")
WHAT_WOULD_YOU_LIKE_S=$(getmsg cluster.cat 1010 "What would you like to do with service address")
MOVE_SERVICE=$(getmsg cluster.cat 1011 "Move it to a different interface")
GROUP_ONLINE=$(getmsg cluster.cat 1012 "Bring Resource Group On-line")
GROUP_OFFLINE=$(getmsg cluster.cat 1013 "Bring Resource Group Off-line")
ALL_NODES=$(getmsg cluster.cat 1014 "All Nodes")

# set up the WebSMIT environment variables
CLSTAT_CONF_FILE="../wsm_clstat.conf"
SKIN_LOGO="/skins/DEFAULT_logo.gif"
LOGFILE="../logs/wsm_clstat.log"

if [[ -f $CLSTAT_CONF_FILE ]]; then
    # all of the tooltip data is in the wsm_smit.conf
    export TOOLTIP_DATA=""
    set -a
    # Make sure the file being sourced is owned by root and only the owner has
    # write permissions.  This file is being sourced, so being writable by any
    # other person could allow them to run arbitrary commands as root.
    if [[ -n "$(find $CLSTAT_CONF_FILE ! -user root -o -perm -0020 -o -perm -0002)" ]]
    then
        print "Invalid permissions for $CLSTAT_CONF_FILE.  " \
              "File should only be writable by root." >>$LOGFILE
        exit 1
    fi
    . $CLSTAT_CONF_FILE >/dev/null
    set +a
fi

# print the WebSMIT header
$BINDIR/wsm_html_lib body_start_help_only "/wsm_clstat_hlp.html"

print_script

# set up some paths
export PATH=$(/usr/es/sbin/cluster/utilities/cl_get_path all)
HA_DIR="$(cl_get_path)"
DCD="/etc/${HA_DIR}/objrepos"
SCD="/usr/${HA_DIR}/sbin/cluster/etc/objrepos/stage"
ACD="/usr/${HA_DIR}/sbin/cluster/etc/objrepos/active"

#
# set up some global variables with SNMP branch info
#
# cluster
  CLUSTER_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.1"
  CLUSTER_NAME="$CLUSTER_BRANCH.2"
  CLUSTER_STATE="$CLUSTER_BRANCH.4"
  CLUSTER_SUBSTATE="$CLUSTER_BRANCH.8"
  CLUSTER_NUM_NODES="$CLUSTER_BRANCH.11"
# node
  NODE_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.2.1.1"
  NODE_ID="$NODE_BRANCH.1"
  NODE_STATE="$NODE_BRANCH.2"
  NODE_NAME="$NODE_BRANCH.4"
# network
  NETWORK_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.4.1.1"
  NETWORK_ID="$NETWORK_BRANCH.2"
  NETWORK_NAME="$NETWORK_BRANCH.3"
  NETWORK_ATTRIBUTE="$NETWORK_BRANCH.4"
  NETWORK_STATE="$NETWORK_BRANCH.5"
# address
  ADDRESS_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.3.1.1"
  ADDRESS_IP="$ADDRESS_BRANCH.2"
  ADDRESS_LABEL="$ADDRESS_BRANCH.3"
  ADDRESS_ROLE="$ADDRESS_BRANCH.4"
  ADDRESS_NET="$ADDRESS_BRANCH.5"
  ADDRESS_STATE="$ADDRESS_BRANCH.6"
  ADDRESS_ACTIVE_NODE="$ADDRESS_BRANCH.7"
# rg
  RG_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.11.1.1"
  RG_ID="$RG_BRANCH.1"
  RG_NAME="$RG_BRANCH.2"
  RG_POLICY="$RG_BRANCH.3"
  RG_USERNAMED="$RG_BRANCH.4"
# rg_state
  RG_STATE_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.11.3.1"
  RG_STATE_NODE="$RG_STATE_BRANCH.2"
  RG_STATE_STATE="$RG_STATE_BRANCH.3"
# res
  RES_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.11.2.1"
  RES_ID="$RES_BRANCH.2"
  RES_NAME="$RES_BRANCH.3"
  RES_TYPE="$RES_BRANCH.4"
# event
  EVENT_BRANCH="1.3.6.1.4.1.2.3.1.2.1.5.10.2.1"
  EVENT_TYPE="$EVENT_BRANCH.1"
  EVENT_NODE="$EVENT_BRANCH.2"
  EVENT_NETWORK="$EVENT_BRANCH.3"
  EVENT_TIME="$EVENT_BRANCH.4"
  EVENT_ID="$EVENT_BRANCH.5"
  EVENT_NODENAME="$EVENT_BRANCH.5"


# SNMPINFO = "/usr/sbin/snmpinfo -m dump -o /usr/es/sbin/cluster/hacmp.defs"
# is a "safe" command via clcomd
SNMPINFO="SNMPINFO"

RGINFO="/usr/es/sbin/cluster/utilities/clRGinfo"

# Ensure at least one resource group is defined in the DCD
RGS=$(ODMDIR=$DCD /usr/es/sbin/cluster/utilities/clgetgrp -c | 
	grep -v "#" | cut -f1 -d:)
	
FOUND=false

# Use local clstrmgr, if available
CLSTRMGR_ACTIVE=$(LANG=C lssrc -s clstrmgrES | grep "active")
if [[ -n "$CLSTRMGR_ACTIVE" ]];then
    # Save local node name.
    # LOCALNODENAME=$(ODMDIR=$ACD odmget HACMPcluster | fgrep -w nodename | cut -f2 -d\")
    LOCALNODENAME=$(/usr/es/sbin/cluster/utilities/get_local_nodename)

    [[ -z "$LOCALNODENAME" ]]  &&
    # LOCALNODENAME=$(ODMDIR=$DCD odmget HACMPcluster | fgrep -w nodename | cut -f2 -d\")
    LOCALNODENAME=$(ODMDIR=$DCD /usr/es/sbin/cluster/utilities/get_local_nodename)
    if [[ -z "$LOCALNODENAME" ]]; then

        ERMSG=$(getmsg scripts.cat 9619 "Unable to discover the name of the local node. Please check the cluster configuration.")
        print -- "$ERMSG"
        print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
        print "</body></html>"
        log_error
        exit 1
    fi
    ACTIVE_NODE=$LOCALNODENAME
    FOUND=true
else
    # Use any available clstrmgr.  Note the ACD may not exist, or be invalid, so use DCD
    # NODES=$(ODMDIR=$DCD odmget HACMPnode | fgrep name | awk -F \" {'print$2'} | sort -u)
    NODES=$(ODMDIR=$DCD /usr/es/sbin/cluster/utilities/cllsnode -c | grep -v "#node" | cut -f1 -d:)
    # Ensure at least one node is defined in the DCD
    if [[ -z "$NODES" ]]; then

        ERMSG=$(getmsg scripts.cat 9619 "Unable to discover the name of the local node. Please check the cluster configuration.")
        print -- "$ERMSG"
        print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
        print "</body></html>"
        log_error
        exit 1
    fi

    for node in $NODES
    do
        ACTIVE_NODE=$(ODMDIR=$DCD /usr/es/sbin/cluster/utilities/clgetactivenodes -n $node)
        RC="$?"
        if [[ $RC > 0 && $RC < 254 ]]
        then
            FOUND="true"
            break
        fi
    done
fi

# Exit if no running clstrmgr is found
$FOUND || {
    ERMSG=$(getmsg scripts.cat 9620 "Unable to determine any nodes with active cluster services.")
    print -- "$ERMSG"
    print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
    print "</body></htmL>"
    log_error
    exit 1
}

# Gather info from available clstrmgr.  Obtain only what
# is needed, so as not to exceed the environment limit
if [[ $PLATFORM == "__LINUX__" ]]; then
    GET_MIB_INFO="/usr/es/sbin/cluster/utilities/clsnmpinfo"
else
    GET_MIB_INFO="${BINDIR}/wsm_cmd_exec -m -n $ACTIVE_NODE -e"
fi

# Ensure the clstrmgr can return information before continuing
RETRY=5
CLUSTER_MIB=$($GET_MIB_INFO cluster)
RC=$?

if [[ $RC != 0 || -z $CLUSTER_MIB  ]]; then
    print "<BR>"

    ERMSG=$(getmsg scripts.cat 9670 "Waiting for the Cluster Manager Daemon (clstrmgrES) to stabilize...")
    print -- "$ERMSG"
    while [[ $RC != 0 && $RETRY > 0 ]]
    do
        sleep 3
        print "."
        RETRY=$(expr $RETRY - 1)
        CLUSTER_MIB=$($GET_MIB_INFO cluster)
        RC="$?"
    done
    print "\n"
    # if we make it through that loop and still have no info, quit
    if [[ $RC != 0 ||  -z $CLUSTER_MIB  ]]
    then
        print "<br>"

        ERMSG=$(getmsg cluster.cat 999 "Unable to communicate with the Cluster SMUX Peer Daemon")
        print -- "$ERMSG"
        print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
        print "</body></html>"
        log_error
        exit 1
    fi
fi

# Print title - basic cluster info
print_cluster_info

# get the rest of the needed info
NODE_MIB=$($GET_MIB_INFO node)
if [[ $? != 0 || -z $NODE_MIB ]]; then

    ERMSG=$(getmsg scripts.cat 9622 "Error obtaining node information via SNMP.  Exiting.")
    print -- "$ERMSG"
    print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
    print "</body></html>"
    log_error
    exit 1
fi

NETWORK_MIB=$($GET_MIB_INFO network)
if [[ $? != 0 || -z $NETWORK_MIB ]]; then

    ERMSG=$(getmsg scripts.cat 9622 "Error obtaining node information via SNMP.  Exiting.")
    print -- "$ERMSG"
    print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
    print "</body></html>"
    log_error
    exit 1
fi

ADDRESS_MIB=$($GET_MIB_INFO address)
if [[ $? != 0 || -z $ADDRESS_MIB ]]; then

    ERMSG=$(getmsg scripts.cat 9622 "Error obtaining node information via SNMP.  Exiting.")
    print -- "$ERMSG"
    print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
    print "</body></html>"
    log_error
    exit 1
fi

if [[ -n "$RGS" ]]; then
# get Resource Group info
    RG_MIB=$($GET_MIB_INFO resGroupTable)
    if [[ $? != 0 || -z $RG_MIB ]]; then
    
        ERMSG=$(getmsg scripts.cat 9622 "Error obtaining node information via SNMP.  Exiting.")
	print -- "$ERMSG"
        print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
	print "</body></html>"
	log_error
	exit 1
    fi
    
    RG_STATE_MIB=$($GET_MIB_INFO resGroupNodeTable)
    if [[ $? != 0 || -z $RG_STATE_MIB ]]; then
    
        ERMSG=$(getmsg scripts.cat 9622 "Error obtaining node information via SNMP.  Exiting.")
	print -- "$ERMSG"
        print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
	print "</body></html>"
	log_error
	exit 1
    fi
    
    RES_MIB=$($GET_MIB_INFO resTable)
    if [[ $? != 0 || -z $RES_MIB ]]; then
    
        ERMSG=$(getmsg scripts.cat 9622 "Error obtaining node information via SNMP.  Exiting.")
	print -- "$ERMSG"
        print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"
	print "</body></html>"
	log_error
	exit 1
    fi
fi

# Clear the session file; this page is like a menu page,
# in that all existing session data is invalid.
${BINDIR}/wsm_session -clear

# Create a new clstat ID and for storing needed values in the session file.
# Note: The extra "" in the date format are to prevent keyword expansion.
typeset -i CLSTAT_ID=$(date "+%H""%M""%S")


print "<div id=\"statusOutput\">"

#
# Print Topology Information
#
print_node_info

#
# Print full resource group info
#
print_rg_info


#
# print footer
#
cat << EOF;
<div class="formButtons">
<input type="button"
       value="Expand All"
       onclick="JavaScript: twistyAll('bodywrapper', 'expanded');"
       title="Click to expand all the available data sections."
/>

<input type="button"
       value="Collapse All"
       onclick="JavaScript: twistyAll('bodywrapper', 'collapsed');"
       title="Click to collapse all the available data sections."
/>

<input type="button"
       value="Refresh"
       onclick="JavaScript: document.location.reload();"
       title="Click to reload this page, and refresh its data."
/>
</div>

</div>
EOF

print "
<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
"

$CGI_BIN/wsm_html_lib body_end_noscript

exit 0
