#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_cmd_hdr.cgi.sh 1.102 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2003,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)19	1.102 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_cmd_hdr.cgi.sh, hacmp.utils, 52haes_r541, 0747C_hacmp541 1/11/08 11:40:39

###############################################################################
#
# Name:  wsm_cmd_hdr.cgi
#
# Description: 
#
#       Processes sm_cmd_hdr (smit odm) entries for the WebSMIT implementation
#       - cmd_hdr has 2 broad types of invocation:
#
#	    1) building a user dialog - dialogs have inputs, selectors, etc where
#	       the user specifies information which eventually passed to the
#	       command to execute
#
#	    2) command execution - once the user presses enter, this script is
#	       called again to actually execute the command
#
#	    There are two cmd_hdr fields that effect command execution:
#
#	        ask - If this is set, the user will be asked for confirmation
#	              before the command is run.
#
#	        ghost - The ghost setting causes a command to be run immediately,
#	                without the need for the user to press the "Execute"
#	                button.  If the 'ask' option is also set, then the user
#	                will still be asked for confirmation before the command is
#	                run, but that is all.
#
#
# Arguments: http string from the browser
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################

###############################################################################
#
# Name: log_write_dialog
#
# Description:  record user actions and system output to a logfile
#
# Arguments:    $ID - the id of the panel 
#               $TEXT - the title of the panel
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
function log_write_dialog 
{
    if [[ $VERBOSE_LOGGING == "high" ]]; then
        typeset TITLE=$(odmget -q "id = $ID" sm_cmd_hdr |
            awk -f ${CGI_BIN}/wsm_awkscr_noparam V=name)
        {
            date +"[%b %d %Y, %T]"
            print "Dialog screen selected:"
            print "      ID          = \"$ID\""
            print "      Title       = \"$TITLE\""
            print "      rawname     = \"$RAW_FIELD_VALUE\""
            print "      User IP     = \"$REMOTE_ADDR\""
            print ""
        } >>$LOGFILE
    fi
}

###############################################################################
#
# Name: get_cmd_hdr_name_ils
#
# Description: given an sm_cmd_hdr id, retrieve the name (accounting
#              for the message catalog entry) This will become the
#              title displayed at the top of the panel.
#
# Arguments:    id of the sm_cmd_hdr entry
#
# Returns:      text to use
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
function get_cmd_hdr_name_ils 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset ID=$1
  
    HDR_NAME_ILS_STANZA=$ID_STANZA

    # get the default text
    # NOTE: do not use 'print -- "$HDR_NAME_ILS_STANZA"' here.
    typeset TITLE=$(odmget -q "id = $ID" sm_cmd_hdr |
                    awk -f ${CGI_BIN}/wsm_awkscr_noparam V=name)

    # is there a message catalog specified ?
    typeset NAME_LOC="${HDR_NAME_ILS_STANZA#*name_msg_file = \"}"
    NAME_LOC="${NAME_LOC%%\"*}"

    if [[ -n $NAME_LOC ]]; then

        # get the message set
        typeset NAME_SET="${HDR_NAME_ILS_STANZA#*name_msg_set = }"
        NAME_SET="${NAME_SET%%[!0-9]*}"

        # get the message id
        typeset NAME_ID="${HDR_NAME_ILS_STANZA#*name_msg_id = }"
        NAME_ID="${NAME_ID%%[!0-9]*}"

        TITLE=$(getmsg -s "$NAME_SET" "$NAME_LOC" "$NAME_ID" "$TITLE")
    fi

    print -- "$TITLE"
} # End of "get_cmd_hdr_name_ils()"


###############################################################################
#
# Name: get_cmd_opt_name_ils
#
# Description: given an sm_cmd_opt ID and sequence number,
#              retrieve the name (accounting for the message
#              catalog entry). This will become the descriptive
#              label to the left of each data entry/display.
#
# Arguments:    id and sequence number of the sm_cmd_opt entry
#
# Returns:      text to use
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
function get_cmd_opt_name_ils 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    CMD_OPT_NAME_ILS_STANZA="$1"
  
    # get the default text
    typeset TITLE=$(echo "$CMD_OPT_NAME_ILS_STANZA" | 
                    awk -f ${CGI_BIN}/wsm_awkscr_noparam V=name)

    # is there a message catalog specified ?
    typeset NAME_LOC="${CMD_OPT_NAME_ILS_STANZA#*name_msg_file = \"}"
    NAME_LOC="${NAME_LOC%%\"*}"
    if [[ -n $NAME_LOC ]]; then

        # get the message set
        typeset NAME_SET="${CMD_OPT_NAME_ILS_STANZA#*name_msg_set = }"
        NAME_SET="${NAME_SET%%[!0-9]*}"

        # get the message id
        typeset NAME_ID="${CMD_OPT_NAME_ILS_STANZA#*name_msg_id = }"
        NAME_ID="${NAME_ID%%[!0-9]*}"

        # Getting rid of newlines eliminates some dspmsg errors
        TITLE=$(echo "$TITLE" | tr "\n" " ")

        TITLE=$(getmsg -s "$NAME_SET" "$NAME_LOC" "$NAME_ID" "$TITLE")
    fi

    print -- "$TITLE"
}


###############################################################################
#
# Name: body_start_exec
#
# Description: add the html to start the body
#
# Arguments:    
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
# Args:
#       $INVOKED_FROM
#
###############################################################################
function body_start_exec 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset HISTORY_BACK=2  # Default; assume command came from dialog via menu
    if [[ $INVOKED_FROM == 'name_selector' ]]; then
        HISTORY_BACK=3   # Command came from dialog via name select
    elif [[ $INVOKED_FROM == 'menu' ]]; then
        HISTORY_BACK=1   # Command came from menu selection
    fi

    $CGI_BIN/wsm_html_lib body_start_exec "$TITLE" "$HISTORY_BACK" act_smittree
} # End of "body_start_exec()"


#Store the values to the session
function proc_discovered_values
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    print -- "$*" | 
    {
        read NAMES
        read VALUES
    }
    NAMES="${NAMES#\#}"

    proc_new_valueset "$NAMES" "$VALUES"
}


###############################################################################
#
# Name: ring_entry
#
# Description:
#
#       This adds a ring selector field (as a drop box).
#
# Arguments:
#
#       KEY       - The sequence ID to pass in the CGI data
#       DISP_LIST - The 'disp_values' field value
#       OPT_VALS  - The 'aix_values' field value
#       REQUIRED  - The 'required' field value
#       VALUE_INDEX   - The 'value_index' field value
#       DISC      - The discovered value
#       TITLE     - Option text
#
# Returns:
#
#       None.
#
# Environment:
#
#       Executed by the http server daemon (user=nobody)
#
###############################################################################
function ring_entry 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset    KEY=$1          # this is the key returned with the selection
    typeset    DISP_LIST=$2    # comma seperated list of options (for display)
    typeset    OPT_VALS=$3     # comma seperated list of corresponding values
    typeset    REQUIRED=$4
    typeset -i VALUE_INDEX=$5
    typeset    DISC=$6
    typeset    TITLE=$7
    typeset    HELP_IND=''
    typeset    HELP_DIV=''
    typeset    HELP_ERR=0


    IT_BOLD="optional"	       # Italic (optional) is default
    [[ $REQUIRED == "B" ]] && IT_BOLD="required"

    #
    # Create an html "select" for this set of options
    # Watch out for deliberately blank entries, and honor
    # the SMIT panel designer's original layout.
    #
    if [[ -n $TITLE ]]; then

        typeset ORIGIFS="$IFS"
        IFS="
"
        typeset TMP_TITLE=
        for LINE in $TITLE; do
            [[ -n $TMP_TITLE ]] && TMP_TITLE="$TMP_TITLE<br>"
            TMP_TITLE="$TMP_TITLE$LINE"
        done
        IFS="$ORIG_IFS"
        [[ -n "$TMP_TITLE" ]] && TITLE="$TMP_TITLE"

        if [[ -z $HELP_MSG_LOC || -z $HELP_MSG_SET || -z $HELP_MSG_ID ]]; then
            # No help is available for this item
            print "
<tr>
<td class='$IT_BOLD'>
<label for='$KEY'>$TITLE</label>
</td>"
        else  # Help is available for this item
            print "
<tr>
<td class='$IT_BOLD'>
<span class='helpWrapper' onClick=\"JavaScript: displayHelpTip('wsm_help.cgi', 'cgi', '$HELP_MSG_LOC', '$HELP_MSG_SET', '$HELP_MSG_ID', '', '$TITLE');\">
<label class='helpWrapper' for='$KEY'>$TITLE</label>
<span class='helpIndicator'>(?)</span>
</span>
</td>"
        fi

        print "
<td>
<select name='$KEY' id='$KEY' class='act_smittree' valType='ring' valReq='$REQUIRED' onfocus=\"focus_help_loc='$HELP_MSG_LOC'; focus_help_set='$HELP_MSG_SET'; focus_help_id='$HELP_MSG_ID'\">"
    else
        print "
<!-- Spacer -->
<tr>
<td>&nbsp;</td>
<td>&nbsp;</td>
</tr>"
        return
    fi

    # The discovered value needs to be among either the disp_values or
    # aix_values.  If it's not, report an error, and make the discovered
    # value the only option for that field.  This is the behavior of ASCII
    # SMIT, albeit undocumented.
    if [[ -n "$DISC" ]] ; then
        if [[ "$DISP_LIST" != @(?(*,)$DISC?(,*)) &&
              "$OPT_VALS"  != @(?(*,)$DISC?(,*))  ]]
        then
            print "
            <script>
            <!-- Begin
            alert('An internal WebSMIT error has occurred.  See the log file for more information.');
            // End -->
            </script>
            "
            {
                print "1800-083 Data error: could not find matching ring"
                print "field value for corresponding Command_to_Discover"
                print "value: \"$DISC\""
                print "The current actual value will be used as the"
                print "default value for the following dialog.  Use"
                print "local problem reporting procedures."
            } >>$LOGFILE

            DISP_LIST="$DISC"
            OPT_VALS="$DISC"
        fi
    fi

    #
    # Process the ring's disp_values and aix_values, as provided by DISP_LIST
    # and OPT_VALS.  This loop compares against both disp_values and aix_values
    # simultaneously and allows for empty values in the disp/aix_value lists.
    #

    # If no aix values, use display values, and vice-versa.
    typeset    DISP_REM="${DISP_LIST:-$OPT_VALS}"  # Remaining disp_values
    typeset    AIX_REM="${OPT_VALS:-$DISP_LIST}"   # Remaining aix_values
    typeset    DISP_CUR                            # Current disp_values value
    typeset    AIX_CUR                             # Current aix_values value

    typeset -i fin=0
    typeset -i i=0

    while ! ((fin))
    do  
        # If there are any comma delimiters left, strip off the next field to
        # process.
        if [[ $DISP_REM == @(*,*) ]] ; then 
            DISP_CUR=${DISP_REM%%,*}  # next disp_values value
            DISP_REM=${DISP_REM#*,}   # trim remaining disp_values
            AIX_CUR=${AIX_REM%%,*}    # next aix_values value
            AIX_REM=${AIX_REM#*,}     # trim remaining aix_values
        else 
            # This is the last field
            DISP_CUR="$DISP_REM"
            AIX_CUR="$AIX_REM"
            fin=1
        fi

        # If the discovered value matches the disp_value or the aix_value, then
        # this is the default selection.  The discovered value takes precedence
        # over the value_index.
        typeset SELECTED=""
        if [[ -n "$DISC" && 
            ( "$DISC" == "$DISP_CUR" || "$DISC" == "$AIX_CUR" ) ]]
        then
            SELECTED="selected"
        elif [[ -z "$DISC" ]]; then
            if (( $i == $VALUE_INDEX )); then
                SELECTED="selected"
            fi
        fi
        print "<option value=\"$AIX_CUR\" $SELECTED>$DISP_CUR</option>"
        (( i += 1 ))
    done

    print "</select>"

} # End of "ring_entry()"



###############################################################################
#
# Name: text_entry
#
# Description:
#       This adds a text, numeric or hex input field 
#
#
# Arguments:
#
#       KEY   - The SEQ number associated with the option
#       NUM_TXT - The type of data ('num', 'hex', 'txt', 'RO')
#       REQUIRED - The 'required' field value
#       DISC     - The discovered value
#       TEXT     - The option text
#       LIST_ID  - The option ID
#       LIST_SEQ - The option SEQ number
#
# Returns:
#
#       none
#
# Environment:
#
#       Executed by the http server daemon (user=nobody)
#
###############################################################################
function text_entry 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset KEY=$1           # this is the key returned with any text
    typeset NUM_TXT=$2       # type - num, hex or txt
    typeset REQUIRED=$3      # required field ?
    typeset DISC=$4          # is there a corresponding command to discover
    typeset TEXT=$5          # text for the field
    typeset LIST_ID=$6       # list function
    typeset LIST_SEQ=$7
    typeset HELP_IND=''
    typeset HELP_DIV=''
    typeset HELP_ERR=0
    typeset INPUT_TYPE=text

    IT_BOLD="optional"	     # Italic (optional) is default
    [[ $REQUIRED == "B" ]] && IT_BOLD="required"

    #
    # if the input supports a list option, build the command to list
    #
    if [[ "$OPTYPE" == "l"  ]]; then
        # build list button
        LIST_BUTTON="<input class='act_smittree act_smittree_button' type='button' value='  List  ' name='foo' onclick=\"openList('$LIST_ID','$LIST_SEQ','$FORM_ID_COUNTER', '$RAW_FIELD_NAME', '$RAW_FIELD_VALUE'); return false;\" title='Display the list of available selections.'>"
        LIST_JS=" list_id='$LIST_ID'; list_seq='$LIST_SEQ'; list_key='$FORM_ID_COUNTER'; list_raw_field_name='$RAW_FIELD_NAME'; list_raw_field='$RAW_FIELD_VALUE';"

    else
        LIST_BUTTON=""
        LIST_JS=" list_id=''; list_seq=''; list_key=''; list_raw_field_name=''; list_raw_field='';"
    fi  # done building list button

    #
    # get maximum entry length
    #
    LENGTH="${STANZAS[$index]#*entry_size = }"
    LENGTH="${LENGTH%%[!0-9]*}"

    #
    # Begin an HTML table and add the Java to display help.
    # Watch of for deliberately blank entries, and honor
    # the SMIT panel designer's original layout.
    #
    if [[ -n $TEXT ]]; then
        typeset HELP_AVAILABLE=true
        typeset COLSPAN=1
        if [[ -z $DISC && $NUM_TXT == 'RO' && -z $LIST_BUTTON ]] ; then
            COLSPAN=2
            HELP_AVAILABLE=
        elif [[ -z $HELP_MSG_LOC || -z $HELP_MSG_SET || -z $HELP_MSG_ID ]]; then
            HELP_AVAILABLE=
        fi

        typeset HELPCODE=""
        if [[ -n $HELP_AVAILABLE ]]; then
            typeset ORIGIFS="$IFS"
            IFS="
"
            typeset ARG_TEXT=
            for LINE in $TEXT; do
                [[ -n $ARG_TEXT ]] && ARG_TEXT="$ARG_TEXT<br>"
                ARG_TEXT="$ARG_TEXT$LINE"
            done
            IFS="$ORIG_IFS"
            [[ -n "$ARG_TEXT" ]] && TEXT="$ARG_TEXT"

            HELPCODE="<span class='helpWrapper' onClick=\"JavaScript: displayHelpTip('wsm_help.cgi', 'cgi', '$HELP_MSG_LOC', '$HELP_MSG_SET', '$HELP_MSG_ID', '', '$TEXT');\">"
        fi

        print "
<tr>
<td class='$IT_BOLD' COLSPAN=$COLSPAN>
$HELPCODE"

        if [[ -n $HELP_AVAILABLE ]]; then
            print "\
<label class='helpWrapper' for='$FORM_ID_COUNTER'>$TEXT</label>
<span class='helpIndicator'>(?)</span>
</span>"
        else
            print "<label for='$FORM_ID_COUNTER'>$TEXT</label>"
        fi

        print "</td>"

        # Accommodate "spacer" entries
        if [[ -n $DISC || $NUM_TXT != 'RO' || -n $LIST_BUTTON ]] ; then
           print "<td style='white-space: nowrap;'>"
        fi
    else
        print "
<!-- Spacer -->
<tr>
<td>&nbsp;</td>
<td>&nbsp;</td>
</tr>"
        (( FORM_ID_COUNTER += 1 ))
        return
    fi

    #
    # Setup some of the attributes of this entry item
    #
    typeset -i MAX_TEXT_FIELD_SIZE=40
    typeset -i MAX_NUM_FIELD_SIZE=9

    VALTYPE="valType=\"$NUM_TXT\""

    # Determine the size of the input box.
    if (( LENGTH == 0 || LENGTH > MAX_TEXT_FIELD_SIZE )) ; then
        SIZE=$MAX_TEXT_FIELD_SIZE
    else
        SIZE=$LENGTH
    fi

    # A zero entry_size means allow the maximum.  Non-zero means that is the
    # max size of the allowed input.
    typeset MAX_LENGTH_STR=""
    if (( LENGTH > 0 )) ; then
        MAX_LENGTH_STR="maxlength=\"$LENGTH\""
    fi

    READONLY=""
    case "$NUM_TXT" in
    RO)
        READONLY="readonly"
        VALTYPE=""
        ;;
    num|hex)
        ((SIZE>MAX_NUM_FIELD_SIZE)) && SIZE=$MAX_NUM_FIELD_SIZE
        ;;
    esac

    DISCVALUE=""
    if [[ -n "$DISC" ]]; then
        DISCVALUE="VALUE=\"$DISC\""
    fi

    # Accommodate "spacer" entries
    if [[ -z $DISC && $NUM_TXT == 'RO' && -z $LIST_BUTTON ]] ; then
        INPUT_TYPE=hidden
        [[ -z $TEXT ]] &&
            print "<br>"
    fi

    print "\
<input class='act_smittree' type=$INPUT_TYPE name=\"$KEY\" id=\"$FORM_ID_COUNTER\" $VALTYPE valLength=\"$LENGTH\" valReq=\"$REQUIRED\" $MAX_LENGTH_STR size=$SIZE $DISCVALUE onfocus=\"focus_help_loc='$HELP_MSG_LOC'; focus_help_set='$HELP_MSG_SET'; focus_help_id='$HELP_MSG_ID';$LIST_JS\" $READONLY>
$LIST_BUTTON
</td>
</tr>"

    (( FORM_ID_COUNTER += 1 ))
}

# global option id index used by build form(), text_entry(), etc
typeset -i index

###############################################################################
#
# Name: build_form
#
# Description:
#
#       This function builds the HTML form from the cmd_hdr entry.  Each form
#       item corresponds to a cmd_opt stanza.  Duplicate SEQ numbers and
#       multiple option IDs are handled.
#
#       If we are building the form and the GHOST option is set, then it should
#       be the case that the ASK option is also set (otherwise, the command
#       would be executed without building the form).  In this case, the form
#       is still built, but the ASK popup is automatically presented.
#
# Arguments:
#
#       BASENAME: the ID of the panel being constructed/handled
#
# Returns:
#
#       None
#
# Environment:
#
#       Executed by the http server daemon (user=nobody)
#
###############################################################################
function build_form 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset BASENAME=$1

    # get the text string to use as a title
    MAIN_TITLE=$(get_cmd_hdr_name_ils $ID)

    # print the form title
    print "<h1>$MAIN_TITLE</h1>"

    #
    # Start a form
    #

    #
    # Add the asking code (wsm_should_ask) if the 'ask' stanza field is set.
    # If the 'onsubmit' code returns false, the form will not be submitted.
    # Then prepareForm() will validate the fields and make any necessary
    # adjustments.  Ghost screens with the 'ask' setting are handled at the end
    # of this function.
    #
    ONSUBMIT="return prepareForm();"
    if [[ "$GHOST" != "y" && "$SHOULD_ASK" == "y" ]] ; then
        ONSUBMIT="if (wsm_should_ask()) { $ONSUBMIT } else { return false; }"
    fi
    print "\
<form id='cmdForm'
      name='cmdForm'
      method='post'
      action=\"/cgi-bin/wsm_cmd_hdr.cgi?panel_id=$BASENAME\"
      onsubmit=\"JavaScript: $ONSUBMIT\">"

    # pass the name
    print "<input type=hidden NAME=\"panel_id\" value=\"$ID\">"

    # determine if this dialog came from a name select - for navigation
    if [[ -n $FROM_NAME_SELECT ]]; then
        print "<input type=\"hidden\" name=\"fromNameSelect\" value=\"true\">"
        FROM_MENU="true"
    fi

    # pass the caller type
    print "<input type=\"hidden\" name=\"from\" value=\"$INVOKED_FROM\">"

    # this is the rest of the header for the form
    print "<table border=0>"

    # Get the list of options for this dialog - each of these translates
    # (possibly) to an input entry on the dialog. The option_id field can
    # specify multiple options seperated by commas.
    typeset OPT_ID="${ID_STANZA#*option_id = \"}"
    OPT_ID="${OPT_ID%%\"*}"
    OPT_LIST=$(echo "$OPT_ID" | sed s/","/" "/g)

    # Get the location of the help message
    MAIN_HELP_MSG_LOC="${ID_STANZA#*help_msg_loc = \"}"
    MAIN_HELP_MSG_LOC="${MAIN_HELP_MSG_LOC%%\"*}"

    # help_msg_id is either a number which points to aix help text or an
    # encoding of "msg set, msg id"
    MAIN_HELP_MSG_ID="${ID_STANZA#*help_msg_id = \"}"
    MAIN_HELP_MSG_ID="${MAIN_HELP_MSG_ID%%\"*}"

    # Split into two values
    [[ -n $MAIN_HELP_MSG_ID ]] && {
        MAIN_HELP_MSG_SET=${MAIN_HELP_MSG_ID%,*}
        MAIN_HELP_MSG_ID=${MAIN_HELP_MSG_ID#*,}
    }

    # Each field needs a unique id. We'll use a counter.  List pop-up
    # windows will use this to determine which form field get the
    # chosen value.
    FORM_ID_COUNTER=0

    #
    # The stanzas in sm_cmd_opt will not necessarily be output in sequence
    # and there may be more than one stanza with the same sequence number.
    # The following loop handles both of these problems.
    # 
    # Sort the list of IDs and make sure they are uniq in our list.  For
    # each unique ID, use a while loop to process the stanza(s) for that ID.
    # We use "sm_cmd_opt:" as the marker of a new stanza.
    # 
    set -A STANZAS
    index=-1
    set -A seq_counter

    # Process all opts for this panel
    for OPT_ID in $OPT_LIST
    do
        for seq_num in $(odmget -q "id = $OPT_ID" sm_cmd_opt |
                         grep 'id_seq_num ='                 |
                         cut -f2 -d\"                        |
                         sort -u)
        do
            # When reading the ODM information, use read -r to preserve 
            # escape sequences.
            odmget -q "id = $OPT_ID AND id_seq_num = $seq_num" sm_cmd_opt | \
            while read -r line; do
                # Skip blank lines, and move to next array index at the start of
                # a new stanza.
                if [[ "$line" == *([[:space:]]) ]] ; then continue ; fi
                if [[ "$line" == @(sm_cmd_opt:) ]] ; then
                    if (( (index += 1) >= 0 )) ; then
                        # If this is the start of a new stanza, prefix the
                        # opt_id so that, if there are multiple opt_id's for
                        # the cmd_hdr, we know down the line which opt_id this
                        # stanza came from (so, for example, we get the correct
                        # cmd_to_list).
                        STANZAS[$index]="OPT_ID = \"$OPT_ID\""
                    fi
                fi
                STANZAS[$index]="${STANZAS[$index]}
$line"
            done
        done
    done      # all opt ids for this entry

    # Process the array of stanzas.
    index=-1
    while (( index < ${#STANZAS[@]}-1 ))
    do
        # Increment first, so we don't have to keep track of every
        # 'continue' or 'break'.
        ((index += 1))

        # Extract the current stanza's seq_num.
        SEQ="${STANZAS[$index]#*id_seq_num = \"}"
        SEQ="${SEQ%%\"*}"

        # Keep track of which sequence numbers have already been processed.
        # If there are multiple stanzas with the same sequence number, we
        # need to qualify the sequence number to distinguish it from the
        # other(s) when the command_to_exec is being built.
        if [[ -n ${seq_counter[$SEQ]} ]]; then
            # Assign to the array index first, then turn SEQ into a
            # non-number.
            (( seq_counter[$SEQ] += 1 ))  
            SEQ=${SEQ}_${seq_counter[$SEQ]}
        else
            (( seq_counter[$SEQ] = 1 ))
        fi

        # Extract the current stanza's seq_num.
        OPT_ID="${STANZAS[$index]#*OPT_ID = \"}"
        OPT_ID="${OPT_ID%%\"*}"

        #
        # Extract some values from the stanza.  Use shell operations to do
        # so when possible to avoid pipes to external programs.
        #

        # Get the text for the entry
        TEXT=$(get_cmd_opt_name_ils "${STANZAS[$index]}")

        # Get the location of the help message
        HELP_MSG_LOC="${STANZAS[$index]#*help_msg_loc = \"}"
        HELP_MSG_LOC="${HELP_MSG_LOC%%\"*}"

        # help_msg_id is either a number which points to aix help text or an
        # encoding of "msg set, msg id"
        HELP_MSG_ID="${STANZAS[$index]#*help_msg_id = \"}"
        HELP_MSG_ID="${HELP_MSG_ID%%\"*}"

        # Split into two values
        [[ -n $HELP_MSG_ID ]] && {
            HELP_MSG_SET=${HELP_MSG_ID%,*}
            HELP_MSG_ID=${HELP_MSG_ID#*,}
        } 

        # The KEY ends up being the name of the form field associated with
        # this cmd_opt.  Its name is directly reflective of the cmd_opt's
        # sequence no.
        #
        KEY="seq__${SEQ}"

        # The "required" field determines if its bold or italic on the page
        REQ="${STANZAS[$index]#*required = \"}"
        REQ="${REQ%%\"*}"

        #
        # required field:
        # Defines if a command field must be sent to the cmd_to_exec command defined 
        # in the associated dialog header object. The value of required is a string. 
        # If the object is part of a selector screen, the required field should 
        # normally be set to "" (empty string). If the object is part of a dialog 
        # screen, valid values are: 
        # "" or "n" - No; the option is added to the command string in the 
        # cmd_to_exec command only if the user changes the initially-displayed 
        # value. This is the default case. 
        # "y" - Yes; the value of the prefix field and the value of the entry field 
        # are always sent to the cmd_to_exec command. 
        # "+" - The value of the prefix field and the value of the entry field are 
        # always sent to the cmd_to_exec command. The entry field must contain at 
        # least one non-blank character. SMIT will not allow the user to run the 
        # task until this condition is satisfied. 
        # "?" - Except when empty; the value of the prefix field and the value of 
        # the entry field are sent to the cmd_to_exec field unless the entry field 
        # is empty. 
        case $REQ in
        "y") 
            REQ="Y"
            ;;
        "+") 
            REQ="B"
            ;;
        "?") 
            REQ="E"
            ;;
        *) 
            REQ="N"
            ;;
        esac

        # If there is a discovered field name, retrieve its value from the
        # session.  This value was not necessarily generated by
        # cmd_to_discover.  This value also serves as the "default" value
        # during data validation.  Proper use of quotes in the call to
        # wsm_session will allow for multi-word values and/or field names.
        DISC=""
        DISC_FIELD="${STANZAS[$index]#*disc_field_name = \"}"
        DISC_FIELD="${DISC_FIELD%%\"*}"
        if [[ -n $DISC_FIELD ]] ; then
            DISC="$(${CGI_BIN}/wsm_session -d valueset -g "$DISC_FIELD")"
        fi

        # the op_type defines the option type
        OPTYPE="${STANZAS[$index]#*op_type = \"}"
        OPTYPE="${OPTYPE%%\"*}"

        # If we have a poorly written stanza, where there optype is 'l',
        # meaning there is a cmd_to_list, but the actual cmd_to_list is empty,
        # then reset the optype.
        if [[ $OPTYPE == 'l' ]] ; then
            LIST_CMD="${STANZAS[$index]#*cmd_to_list = \"}"
            LIST_CMD="${LIST_CMD%%\"*}"

            if [[ -z $LIST_CMD ]] ; then
                OPTYPE=''
            fi
        fi


        # The type of value required by the entry field. The value of
        # entry_type is a string. Valid values are: 
        #
        # "" or "n" - No entry; the current value cannot be modified via direct 
        #       type-in. The field is informational only. 
        # "t" - Text entry; alphanumeric input can be entered. 
        # "#" - Numeric entry; only the numeric characters 0 through 9 can be
        #       entered. A - (minus sign) or + (plus sign) can be entered as 
        #       the first character. 
        # "x" - Hex entry; hexadecimal input only can be entered. 
        # "f" - File entry; a file name should be entered. 
        # "r" - Raw text entry; alphanumeric input can be entered. Leading and 
        #       trailing spaces are considered significant and are not stripped
        #       off the field. 

        # what type of entry (text, ring, etc) is this?  Null string defaults
        # to "n", or no entry.
        TYPE="${STANZAS[$index]#*entry_type = \"}"
        TYPE="${TYPE%%\"*}"
        TYPE="${TYPE:-n}"

        # ring - get the list of values displayed
        RING="${STANZAS[$index]#*disp_values = \"}"
        RING="${RING%%\"*}"

        case $OPTYPE in
        r )
            # The value index determines which is the "default" entry
            VAL_IDX="${STANZAS[$index]#*value_index = }"
            VAL_IDX="${VAL_IDX%%[!0-9]*}"

            # Each value in the ring corresponds to a different key
            VALS="${STANZAS[$index]#*aix_values = \"}"
            VALS="${VALS%%\"*}"

            ring_entry "$KEY" "$RING" "$VALS" "$REQ" "$VAL_IDX" "$DISC" "$TEXT"
            ;;
        * )
            # If there is no discovered value, then see if there is a
            # disp_values value.  It's not documented as such, but ASCII SMIT
            # uses that value for non-ring fields when there is either no
            # field_to_discover or it has no value.
            if [[ -z "$DISC" ]] ; then
                # Each value in the ring corresponds to a different key
                VALS="${STANZAS[$index]#*aix_values = \"}"
                VALS="${VALS%%\"*}"

                DISC="${RING:-$VALS}"
            fi

            # Switch on field type; call the function to build the field
            case $TYPE in
            "#" )
                # This is a numeric entry
                text_entry "$KEY" "num" "$REQ" "$DISC" "$TEXT" "$OPT_ID" "$SEQ"
                ;;
            x )
                # This is a hex entry
                text_entry "$KEY" "hex" "$REQ" "$DISC" "$TEXT" "$OPT_ID" "$SEQ"
                ;;
            t | f )
                # This is a text entry
                text_entry "$KEY" "txt" "$REQ" "$DISC" "$TEXT" "$OPT_ID" "$SEQ"
                ;;
            r )
                # Raw entry type
                text_entry "$KEY" "raw" "$REQ" "$DISC" "$TEXT" "$OPT_ID" "$SEQ"
                ;;
            * )
                # This is a read only entry.  The form value will not be
                # modifiable and the value returned will be verified
                # against the "default" value in the valueset.
                text_entry "$KEY" "RO" "$REQ" "$DISC" "$TEXT" "$OPT_ID" "$SEQ"
                ;;
            esac
        esac      # switch on op_type
    done      # all id_seq for this opt id

    # complete the (html) table
    print "</table>\n"

    ${BINDIR}/wsm_cmd_exec -r
    if [[ $? == 1 ]] ;  then

        # tells wsm_cmd_hdr.cgi that this data should be executed
	print "<input type='hidden' name='Execute.x' value='dummy'>"
        print "<div class='formButtons'><input class='act_smittree act_smittree_button' type='submit' value='Execute' title='Click to perform the operation with the current data.'></div>"
    else
	{
	    print ""
	    print "wsm_cmd_hdr.cgi: User has read only access"
	} >>$LOGFILE
    fi

    print "\
</form>

<div id='cmdKey'>
<h3>Key:</h3>
<div><span class='required'>Required</span><br>
<span class='optional'>Optional</span><br>
</div>
</div>"

    # If this is a ghost dialog and the 'ask' stanza field is "y", then
    # automatcally ask once the form is built.  The form will be visible in the
    # background, but the user will not be able to act on it.  'Ok' will submit
    # the form, and 'Cancel' will go back.
    if [[ "$GHOST" == "y" && "$SHOULD_ASK" == "y" ]] ; then
        print "
<script>
<!-- Begin
if (wsm_should_ask()) {
    document.cmdForm.submit();
} else {
    wsm_back();
}
// End -->
</script>
"
    fi
}


###############################################################################
#
# Name: exec_the_cmd
#
# Description:
#
#       This function executes the command built by the first invocation of
#       this script.  It is called when the user presses the Execute button or
#       during the first invocation for ghost dialogs that do not require
#       confirmation ('ask').
#
# Arguments:
#
#       None
#
# Returns:
#
#       None
#
# Environment:
#
#       Executed by the http server daemon (user=nobody)
#
###############################################################################
function exec_the_cmd 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    # do the actual command execution
    COMMAND="${WORKDIR}/$ID.COMMAND.$$"

    print "<div class=\"outputFrame\">Output from command to execute follows below:</div>"
    print "<pre class=\"output\">"

    # Run the command.  Print the output as it appears.  This is allows the
    # user to know that something is actually happening, especially for
    # long-running commands.
    #
    # NOTE: it is *important* that IFS be set to an empty string below! Doing
    #       this ensures that the "read" function, below, preserves the data
    #       *exactly*. Without this, for example, all leading whitespace will
    #       be removed, rendering tabular data unreadable.
    (
        ${BINDIR}/wsm_cmd_exec -f "$COMMAND" -i "$ID" -t cmd_to_exec 2>&1

        # We can't just use $?  after the pipe, because that will give us the
        # return code of the command after the pipe.  We care about the return
        # code of the piped command.
        print "\nWSM_RC=$?"

    ) | while IFS= read LINE ; do

        # Skip the WebSMIT "BEGIN" line
        if [[ $LINE == @(<!-- IBM WebSMIT Command Output Begin -->) ]]; then
            continue
        fi

        # Grab the return code of the command execution.  This sets RC.
        if [[ $LINE == @(WSM_RC=*) ]] ; then
            eval ${LINE#WSM_}
            continue
        fi

        # Grab and print the line
        CMD_OUTPUT="$CMD_OUTPUT
$LINE"
        print -- "$LINE"
    done

    print "</pre>"

    # 'cat' the generated script
    #
    # NOTE: this *must* be done via wsm_cmd_exec, because the file being
    #       'cat'ted is accessible only by root (for security). Therefore,
    #       the setuid capabilities of wsm_cmd_exec are needed in order
    #       to do *anything* with the file.
    print "<div class=\"outputFrame\">The command to execute was:</div>"
    print "<pre class=\"output\">"
    ${BINDIR}/wsm_cmd_exec -C -f "$COMMAND"
    print "</pre>"

    # The return code may have been negative, in which case $? will be
    # 255+(negative value returned).  Because of this, 128 is the maximum
    # returnable value before we are not able to distinguish between a
    # positive and negative return value.  This little formula gives us the
    # negative return value when above 128.
    RC=$(( (RC>128)*-256+RC ))

    # wsm_build_cmd returns -2 if there was a data validation error.
    if (( RC == -2 )) ; then
        typeset MSG=$(getmsg wsm_smit.cat 80 "There was a problem with your request.  Please try again.")
        print "<i>$MSG</i>"
        return 1
    fi

    # Print the result
    print "<div class=\"outputFrame\">Command completed with return code: $RC</div>"

    # Hit the user over the head with an error message if the command fails
    if [[ $RC != 0 ]]; then
        print "<script type=\"text/javascript\">"
        print "alert('Command failed! Return code was: $RC');"
        print "</script>"
    fi


    # log the executed command strings
    {
        print "\
  Output from command to execute follows below:
==== start ====
$CMD_OUTPUT
==== stop ====
  Command completed with return code $RC
"
    } >>$LOGFILE

    #=========================================================
    # If this invocation was to start the cluster simulator,
    # *and* that startup was successful, then change the
    # standard HACMP logo to the cluster simulator logo.
    #=========================================================
    if [[ $BASENAME == "clsimstart" ]]; then
        if (( RC == 0 )); then
            print "
<script type='text/javascript'>
var logo = parent.parent.HeaderFrame.document.getElementById('HACMP_LOGO');
logo.src = '/wsm_logo_ha_simulator.gif';
</script>
"
        fi

    #=========================================================
    # If this invocation was to stop the cluster simulator,
    # *and* that stoppage was successful, then restore the
    # standard HACMP logo.
    #=========================================================
    elif [[ $BASENAME == "clsimstop" ]]; then
        if (( RC == 0 )); then
            print "
<script type='text/javascript'>
var logo = parent.parent.HeaderFrame.document.getElementById('HACMP_LOGO');
logo.src = '/wsm_logo_ha.gif';
</script>
"
        fi
    fi

} # End of "exec_the_cmd()"



####################
# Main starts here #
####################

#
# parse some cgi input, execute the command, and display the results
#
PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Support global/env specification
VERSION="1.55"

export WSM_PROGNAME=$PROGNAME
export WSM_VERSION=$VERSION

WORKDIR=/tmp
BINDIR=/usr/es/sbin/cluster/wsm/cgi-bin
CONFDIR=/usr/es/sbin/cluster/wsm
CGI_BIN=$BINDIR

# Run some access validations.  This script will also output the
# Content-type header so we don't have to do it here.
$BINDIR/wsm_auth.cgi
if (( $? != 0 )); then
    exit 0
fi

# smit menus are in
export ODMDIR=/usr/lib/objrepos


#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: there is no need to check for root ownership of the
#       WebSMIT configuration file here. It has already been
#       validated by the call to "wsm_auth.cgi", above.
#=============================================================
CONF_FILE="$CONFDIR/wsm_smit.conf"
LOGFILE="$CONFDIR/logs/wsm_smit.log"
export SCRIPTFILE="../logs/wsm_smit.script"
export SKIN_LOGO="/skins/DEFAULT_logo.gif"
set -a
. $CONF_FILE >/dev/null 2>&1
set +a

[[ $VERBOSE_LOGGING == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
typeset LOGFILE="/usr/es/sbin/cluster/wsm/logs/wsm_smit.log"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


# initialize the cgi library
TYPE="j"
EXECUTE="FALSE"


#===================================================================
# Check for any message catalog data from the originating/parent
# menu item. If there is any, it needs to be retrieved here, before
# the following call to "wsm_cgiread" overwrites QUERY_STRING with
# the posted form data.
#===================================================================
typeset PARENT_HELP_MSG_LOC=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_loc)
typeset PARENT_HELP_MSG_SET=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_set)
if [[ $PARENT_HELP_MSG_SET == *,* ]]; then
    PARENT_HELP_MSG_SET=${PARENT_HELP_MSG_SET%%,*}
fi
typeset PARENT_HELP_MSG_ID=$(${CGI_BIN}/wsm_cgiparse  -v parent_help_msg_id)
if [[ $PARENT_HELP_MSG_ID == *,* ]]; then
    PARENT_HELP_MSG_ID=${PARENT_HELP_MSG_ID##*,}
fi


#=============================================================================
# Read in the CGI form data so that we can parse it later with wsm_cgiparse.
# We overwrite the QUERY_STRING environment variable with the output of
# wsm_cgiread.  wsm_cgiread will output either the original QUERY_STRING
# value or the data that was read from stdin during a POST request.
#=============================================================================
export QUERY_STRING=$(${CGI_BIN}/wsm_cgiread)


# get some basic info so that we know whether we're building a form or 
# exec'ing a command
BASENAME=$(${CGI_BIN}/wsm_cgiparse -v panel_id)

# Was this panel reached via a fastpath?
FASTPATH=$(${CGI_BIN}/wsm_cgiparse -v fastpath)

# What type of panel was this script invoked from?
# This determines how to handle the "F3" key!
INVOKED_FROM=$(${CGI_BIN}/wsm_cgiparse -v from)

# Get the ID of the referring panel, used to run the appropriate
# cmd_to_classify.
CMD_TO_CLASSIFY_ID=$(${CGI_BIN}/wsm_cgiparse -v cmd_to_classify)

# See if this is a generated link
CLSTAT_ID=$(${CGI_BIN}/wsm_cgiparse -v clstat_id)
TTIP_ID=$(${CGI_BIN}/wsm_cgiparse -v tt_id)

# We've arrived here via a generated link.  Extract the faked selector
# field value, if any, from the session.  Otherwise, it comes from CGI info.
if [[ -n $CLSTAT_ID ]] ; then

    # Process the ':'-delimited values stored to the session by the status modules.
    VALUE_LIST=$(${CGI_BIN}/wsm_session \
            -d clstat_id_${CLSTAT_ID}_${BASENAME} -g value_list)
    IFS=:
    for VAR in $VALUE_LIST ; do
        unset IFS
        VAL=$(${CGI_BIN}/wsm_session -d clstat_id_${CLSTAT_ID}_${BASENAME} -g $VAR)
        if [[ -n $VAL ]] ; then
            ${CGI_BIN}/wsm_session -d valueset -s "$VAR"="$VAL"
            printf "%b" \
                "${CGI_BIN}/wsm_session -d valueset " \
                "-s \"$VAR\"=\"$VAL\"\n" >>$LOGFILE
        fi
    done

elif [[ -n $CMD_TO_CLASSIFY_ID ]] ; then

    # If we reached this panel via a fastpath, then clear the session.  A new
    # SMIT path is being started, so there should be nothing in the session
    # left over from previous panels.  Do this here, before we add anything to
    # the session.
    if [[ $FASTPATH == true ]] ; then
        ${CGI_BIN}/wsm_session -clear
    fi

    # If we're coming from a name select, the cmd_to_classify ID *MUST* be
    # present, otherwise there wouldn't be a field value.  Validate and save
    # incoming raw values in the session file.  If there is no raw field value,
    # don't overwrite the potentially already present value in the session.
    RAW_FIELD_NAME=$(${CGI_BIN}/wsm_cgiparse -v raw_field_name)
    RAW_FIELD_NAME=${RAW_FIELD_NAME:=_rawname}
    RAW_FIELD_VALUE=$(${CGI_BIN}/wsm_cgiparse -v "$RAW_FIELD_NAME")

    #
    # Validate the user's value
    #
    typeset STANZA=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr);
    OPT_ID="${STANZA#*option_id = \"}"
    OPT_ID="${OPT_ID%%\"*}"

    # Extract from the ODM the stanza the value will be validated against.
    STANZA="$(odmget -q "id = $OPT_ID AND id_seq_num = 0" sm_cmd_opt)"

    # If the link was generated by a tooltip from the navigation frame, then we
    # need to verify the values that were provided in the CGI request.  The
    # argument will be validated with a call to 'validate_arg' in the next
    # block, so at this point, we need to add the appropriate values *TO* the
    # session.
    if [[ -n $TTIP_ID ]] ; then
        proc_tooltip_values $BASENAME $OPT_ID
    fi


    # Validate the value supplied by the client.  We'll only be coming from a
    # name_hdr, so the cmd_opt ID will always be '0'.  If validation fails,
    # report the error and quit.
    if ! validate_arg "$STANZA" "0" "$RAW_FIELD_VALUE" "$CMD_TO_CLASSIFY_ID"
    then
        # report error to user and exit
        body_start_exec

        print "<div class=\"errorBox\">"
        typeset MSG=$(getmsg wsm_smit.cat 80 "There was a problem with your request.  Please try again.")
        print "<i>$MSG</i>"
        print "<div class='formButtons'><input class='act_smittree act_smittree_button' TYPE='button' value='Back' onclick='history.go(-1)'></div>"
        print "</div>"

        $CGI_BIN/wsm_html_lib body_end_exec "act_smittree act_smittree_link"
        exit
    else
        # The argument was valid.  Save it to the session.
        if [[ -n "$RAW_FIELD_VALUE" ]] ; then
            ${CGI_BIN}/wsm_session -d valueset -s "$RAW_FIELD_NAME"="$RAW_FIELD_VALUE"
            print -n "wsm_name_hdr.cgi:">>$LOGFILE
            printf "%s" "${CGI_BIN}/wsm_session -d valueset " \
                        "-s \"$RAW_FIELD_NAME\"=\"$RAW_FIELD_VALUE\"" >>$LOGFILE
        fi
    fi
else
    # If we reached this panel via a fastpath, then clear the session.  A new
    # SMIT path is being started, so there should be nothing in the session
    # left over from previous panels.  Do this here, before we add anything to
    # the session.
    if [[ $FASTPATH == true ]] ; then
        ${CGI_BIN}/wsm_session -clear
    fi
fi

# If we have a cmd_to_classify ID, get the command.
if [[ -n $CMD_TO_CLASSIFY_ID ]]; then

    # value passed is the panel id, harvest value from that panel
    CMD_TEXT=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr |
        awk -f ${CGI_BIN}/wsm_awkscr_noparam V=cmd_to_classify)

    # harvest the cooked field name, using the default if not defined
    COOKED_FIELD_NAME=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr |
                awk -f ${CGI_BIN}/wsm_awkscr_noparam V=cooked_field_name)
    [[ -z $COOKED_FIELD_NAME ]] && COOKED_FIELD_NAME=_cookedname

    # if the type (of the caller) is "c", then the cmd to classify will be
    # used to generate the panel to go to (when combined with the base name)
    typeset C2C_STANZA=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr)
    CMD_TO_CLASSIFY_TYPE="${C2C_STANZA#*	type = \"}" # NOTE: must contain a TAB char.
    CMD_TO_CLASSIFY_TYPE="${CMD_TO_CLASSIFY_TYPE%%\"*}"
    [[ -z $CMD_TO_CLASSIFY_TYPE ]] && CMD_TO_CLASSIFY_TYPE="j"
fi

# If there was a command to classify, execute it
if [[ -n $CMD_TEXT ]]; then

        CMD_FILE="${WORKDIR}/${CMD_TO_CLASSIFY_ID}.CLASSIFY_COMMAND.$$"
        CLASSIFY_OUT=$(${BINDIR}/wsm_cmd_exec -f $CMD_FILE -i $CMD_TO_CLASSIFY_ID -t cmd_to_classify)
        RC=$?

        if (( RC == 0 )); then
            {
	            print "\
Output from Command_to_Classify follows:

$CLASSIFY_OUT
"
            } >>$LOGFILE

        else

            # log the failure
            {
	            print "\
1800-081 Error exit status ($RC) returned by
Command_to_Classify; the output is:

$CLASSIFY_OUT
"
            } >>$LOGFILE
    
            # report error to user and exit
            body_start_exec

            print "<div class=\"errorBox\">"
            getmsg wsm_smit.cat 9999 "1800-081 An error occurred:"
            print "\
<pre>
$CLASSIFY_OUT
</pre>
<div class='formButtons'><input class='act_smittree act_smittree_button' TYPE='button' value='Back' onclick='history.go(-1)'></div>
</div>"
            $CGI_BIN/wsm_html_lib body_end_exec "act_smittree act_smittree_link"
            exit
        fi
fi

# Process multiple values from the classify command, placing them in the
# session.  Loop through and match fields to values in the classify command
# output.
if [[ -n "$CLASSIFY_OUT" ]]
then
    proc_new_valueset "$COOKED_FIELD_NAME" "$CLASSIFY_OUT"
else
    # if there was no output from the classify command, seek the value from
    # the session file
    if [[ -n $COOKED_FIELD_NAME ]] ; then
        CLASSIFY_OUT="$(${CGI_BIN}/wsm_session -d valueset -g "$COOKED_FIELD_NAME")"
    fi
fi

# By default, the panel id is just the basename - it can be modified below
ID=$BASENAME

if [[ $CMD_TO_CLASSIFY_TYPE == "r" ]]; then
    ID=${BASENAME}${RAW_FIELD_VALUE}
else 
    # if the type of the caller was "c", build the id of our panel from the
    # concatenation of the id and the output from the command to classify
    if [[ $CMD_TO_CLASSIFY_TYPE == "c" ]]; then
        ID=${BASENAME}${CLASSIFY_OUT%%:*}
    fi
fi

# check that the id passes the filters
$CGI_BIN/wsm_filter "$ID"
if [[ $? != 0 ]]; then
    exit 0
fi


ID_STANZA=$(odmget -q "id = $ID" sm_cmd_hdr)

# the command and the postfix can both use the special _rawname keyword.
DISC_CMD="${ID_STANZA#*cmd_to_discover = \"}"
DISC_CMD="${DISC_CMD%%\"*}"

if [[ -n $DISC_CMD ]]; then

    # execute the command using a temporary work file
    CMD_TO_DISC="${WORKDIR}/$ID.CMD_TO_DISCOVER.$$"
    DISC_DATA=$(${BINDIR}/wsm_cmd_exec -f $CMD_TO_DISC -i $ID -t cmd_to_discover)
    RC=$?

    # log the command output
    if (( RC == 0 )); then
        {
            print "
Command_to_Discover output follows:
$DISC_DATA"
        } >>$LOGFILE


        # Process the output of the cmd_to_discover.  Put the resulting
        # values in the session, making for easier retrieval later.
        proc_discovered_values "$DISC_DATA"

    else
        # log the error if the command has failed
        {
            date +"[%b %d %Y, %T]"
            print "\
1800-081 Error exit status ($RC) returned by
Command_to_Discover; the output is:
$DISC_DATA
(The Command_to_Discover is:
$DISC_CMD
.)"
        } >>$LOGFILE
    fi
fi


#=============================================================
# If this script was invoked from a context-sensitive, popup
# menu, then check the values in QUERY_STRING and see if any
# of them match the discovery values for this menu/panel. If
# so, the value is the label of the item that the user right-
# clicked over, and should be stored in the session data.
#=============================================================
if [[ -n $TTIP_ID ]] ; then
    typeset ORIGIFS="$IFS"
    IFS="\&"
    for PARM in $QUERY_STRING; do
        typeset VAR=${PARM%%=*}
        typeset VAL=${PARM##*=}

        #=======================================================
        # Make sure this parameter is in the DISC_DATA output,
        # just so we know that it's a valid/known variable.
        #=======================================================
        if [[ -n $VAR && $DISC_DATA == *?(\#|:)${VAR}?(:)* ]]; then
            if [[ $VAL == "ALLNODES" ]]; then # Special case, for composite
                VAL=
                typeset NODE=

                typeset TMPODM=/etc/objrepos
                get_simulator_mode
                typeset -i SIMMODE=$?
                (( SIMMODE == 1 )) && TMPODM=$(get_odmdir)

                ODMDIR=$TMPODM cllsnode | grep '^NODE ' | \
                while IFS= read -r NODE; do
                    NODE=${NODE##* }
                    NODE=${NODE%:*}
                    [[ -n $NODE ]] && VAL="$VAL,$NODE"
                done
                VAL=${VAL##,}
            fi

            #==============================================================
            # Override this variable in the current valueset. Doing this
            # will result in the *expected* item, from the menu that the
            # user clicked on, being populated in the upcoming data entry
            # form. For example, the name of a node, or resource group.
            #==============================================================
            if [[ -n $VAL ]]; then
                if (( ${#VAL} < 256 )); then
                    ${CGI_BIN}/wsm_session -d valueset -s "$VAR"="$VAL"
                fi
            fi
        fi
    done
    IFS="$ORIGIFS"
fi


TITLE=$(get_cmd_hdr_name_ils $ID)

# test to make sure we were passed a valid panel id
if [[ -z $ID_STANZA ]]; then
    # print and error and exit
    body_start_exec

    print "\
<div class=\"errorBox\">
Error: There are currently no SMIT screen entries defined for this FastPath in the ODM.
This FastPath may require installation of additional software before it can be accessed.
<div class='formButtons'><input class='act_smittree act_smittree_button' type='button' value='Back' onclick='history.go(-1)'></div>
</div>"

    $CGI_BIN/wsm_html_lib body_end_exec "act_smittree act_smittree_link"
    exit
fi

# the ghost setting will tell us if we're building a form or running a command
GHOST="${ID_STANZA#*ghost = \"}"
export GHOST="${GHOST%%\"*}"

# if we came from a dialog, there will be an Execute.x in the querystring
SHOULD_EXECUTE=$(${CGI_BIN}/wsm_cgiparse Execute.x)
[[ -n $SHOULD_EXECUTE ]] && EXECUTE="TRUE"

# determines if we prompt user before executing command
SHOULD_ASK="${ID_STANZA#*	ask = \"}"  # NOTE: "ask" needs a preceding TAB char.
SHOULD_ASK="${SHOULD_ASK%%\"*}"
[[ -z $SHOULD_ASK ]] && SHOULD_ASK="n"

# Execute now if:
#       The user pressed the "Execute" button (obviously)
#       This is a ghost dialog with no 'ask' prompt
#
# If this is a ghost screen with an 'ask' prompt, the form still gets built,
# but with javascript code interjected to do the asking and submit the form if
# the user responds in the affirmative.
# 
# If this screen has 'ask' set but it is not a ghost screen, then we build the
# form and the prompt will be attached to the "Execute" button.
#
if [[ $EXECUTE == "TRUE" || 
    ( "$GHOST" == "y" && "$SHOULD_ASK" == "n" ) ]]; then
    # execute the command
    body_start_exec
    exec_the_cmd
    $CGI_BIN/wsm_html_lib body_end_exec "act_smittree act_smittree_link"
    # log_write_exec "$CMD_TEXT" "$CMD_OUTPUT" $RC >/dev/null
else

    #============================================================
    # If there was no locally (i.e. in the current ODM stanza)
    # message catalog data provided for this panel, but there
    # *was* some message catalog data passed in from the parent
    # panel, then use the parent information.
    #============================================================
    if [[ -z $MAIN_HELP_MSG_LOC     || \
          -z $MAIN_HELP_MSG_SET     || \
          -z $MAIN_HELP_MSG_ID      || \
          $MAIN_HELP_MSG_SET == "0" || \
          $MAIN_HELP_MSG_ID  == "0" ]]
    then
        if [[ -n $PARENT_HELP_MSG_LOC && \
              -n $PARENT_HELP_MSG_SET && \
              -n $PARENT_HELP_MSG_ID ]]
        then
            MAIN_HELP_MSG_LOC=$PARENT_HELP_MSG_LOC
            MAIN_HELP_MSG_SET=$PARENT_HELP_MSG_SET
            MAIN_HELP_MSG_ID=$PARENT_HELP_MSG_ID
        fi
    fi

    # we're building a form
    typeset HISTORY_BACK=2
    if [[ $INVOKED_FROM == 'menu' ]]; then
        HISTORY_BACK=1  # Command came from menu opt
    fi
    $CGI_BIN/wsm_html_lib body_start_cmd_hdr   \
                          "$TITLE"             \
                          "$HISTORY_BACK"      \
                          act_smittree         \
                          "$MAIN_HELP_MSG_LOC" \
                          "$MAIN_HELP_MSG_SET" \
                          "$MAIN_HELP_MSG_ID"

    #
    # Construct and display the menu
    #
    if [[ $VERBOSE_LOGGING == "high" ]]; then
        time build_form $BASENAME
        # Label the output from "time", above, in the output log...
        print "DEBUG (PERF): wsm_cmd_hdr.cgi: build_form($BASENAME)" 1>&2
    else
        build_form $BASENAME
    fi

    $CGI_BIN/wsm_html_lib body_end_cmd_hdr     \
                          SHOW_COMMAND         \
                          "$ID"                \
                          "$MAIN_HELP_MSG_LOC" \
                          "$MAIN_HELP_MSG_SET" \
                          "$MAIN_HELP_MSG_ID"  \
                          "$MAIN_TITLE"        \
                          "act_smittree act_smittree_link"
    log_write_dialog
fi

exit
