#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_cmd_list.cgi.sh 1.45 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2003,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)01	1.45 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_cmd_list.cgi.sh, hacmp.wsm, 52haes_r541, 0747C_hacmp541 1/11/08 11:44:01


###############################################################################
#
# Name: format_list_for_html
#
# Description: Convert white space to displayable values, honoring
#              column widths when such widths can be detected.
#
# Arguments:   The pre-generated list to be formatted.
#
# Returns:     The formatted list is returned via STDOUT.
#
# Environment: Executed by the http server daemon (user=nobody)
#
###############################################################################
function format_list_for_html
{
    typeset LIST="$1"

    #================================================
    # Check for tab characters. If there are tabs,
    # then this list probably has multiple columns.
    #================================================
    if [[ $LIST != ${LIST//	/\&nbsp\;} ]]; then
        typeset DEFAULT_IFS="$IFS"

        LIST=${LIST//		/	} # Replace multiple tabs with one

        #=============================================
        # Determine the maximum width of each column
        #=============================================
        typeset LINE=
        print -- "$LIST" | while IFS= read -r LINE; do
            typeset -i COL=1
            IFS="	"  # Set to a tab only (i.e. no other whitespace)
            for COLUMN in $LINE; do
                [[ -z ${WIDTHS[$COL]} ]] && WIDTHS[$COL]=0

                # Trim whitespace, as needed
                if (( COL > 1 )); then  # Preserve line indentation
                    while [[ $COLUMN == \ * ]]; do COLUMN=${COLUMN#* }; done
                fi
                while [[ $COLUMN == *\  ]]; do COLUMN=${COLUMN% *}; done

                if [[ -n $COLUMN ]]; then
                   (( ${#COLUMN} > ${WIDTHS[$COL]} )) && WIDTHS[$COL]=${#COLUMN}
                fi
                (( COL += 1 ))
            done
            IFS="$DEFAULT_IFS"
        done

        #================================================
        # As needed, pad each value in the list to make
        # it match the maximum width of its column
        #================================================
        typeset COLUMN_SPACING=4
        typeset LINE=
        typeset NEWLIST=
        print -- "$LIST" | while IFS= read -r LINE; do
            [[ -n $NEWLIST ]] && NEWLIST="$NEWLIST
"
            typeset -i DIFF=0
            typeset -i COL=1
            IFS="	"  # Set to a tab only (i.e. no other whitespace)
            for COLUMN in $LINE; do
                # Trim whitespace, as needed
                if (( COL > 1 )); then  # Preserve line indentation
                    while [[ $COLUMN == \ * ]]; do COLUMN=${COLUMN#* }; done
                fi
                while [[ $COLUMN == *\  ]]; do COLUMN=${COLUMN% *}; done

                NEWLIST="$NEWLIST$COLUMN"
                (( DIFF = ${WIDTHS[$COL]} - ${#COLUMN} + COLUMN_SPACING ))
                if (( COL < ${#WIDTHS[*]} )); then
                    for (( I=0; I<DIFF; I+=1 )); do
                        NEWLIST="$NEWLIST "
                    done
                    (( COL += 1 ))
                fi
            done
            IFS="$DEFAULT_IFS"
        done
        [[ -n $NEWLIST ]] && LIST="$NEWLIST"
    fi

    if [[ $LIST != ${LIST// /\&nbsp\;} ]]; then
        LIST="${LIST// /\&nbsp\;}"   # Replace spaces
    fi

    print -- "$LIST"
} # End of "format_list_for_html()"


###############################################################################
#
# Name: get_cmd_opt_name_ils
#
# Description: given an sm_cmd_opt id and sequence number, retrieve the name
#              (accounting for the message catalog entry)
#
# Arguments:    id and sequence number of the sm_cmd_opt entry
#
# Returns:      text to use
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
get_cmd_opt_name_ils ()
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset ID=$1
    typeset SEQ=$2

    typeset STANZA=$(odmget -q "id = $ID AND id_seq_num = $SEQ" sm_cmd_opt)

    # get the default text
    # NOTE: do not use 'print "$STANZA"' here. See defect 597503.
    typeset TITLE=$(odmget -q "id = $ID AND id_seq_num = $SEQ" sm_cmd_opt |
                    awk -f ${CGI_BIN}/wsm_awkscr_noparam V=name)

    # is there a message catalog specified ?
    typeset NAME_LOC="${STANZA#*name_msg_file = \"}"
    NAME_LOC="${NAME_LOC%%\"*}"

    if [[ -n $NAME_LOC ]]; then

        # get the message set
        typeset NAME_SET="${STANZA#*name_msg_set = }"
        NAME_SET="${NAME_SET%%[!0-9]*}"

        # get the message ID
        typeset NAME_ID="${STANZA#*name_msg_id = }"
        NAME_ID="${NAME_ID%%[!0-9]*}"

        TITLE=$(getmsg -s "$NAME_SET" "$NAME_LOC" "$NAME_ID" "$TITLE")
    fi

    print -- "$TITLE"
} # End of "get_cmd_opt_name_ils()"


###############################################################################
#
# Name: wsm_cmd_list.cgi
#
# Description: manages the popup list mechanism (pf4) on smit dialogs
#
# Arguments:
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################

PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Support global/env specification

# initialize
WORKDIR="/tmp"
BINDIR=/usr/es/sbin/cluster/wsm/cgi-bin
CONFDIR=/usr/es/sbin/cluster/wsm
export ODMDIR="/usr/lib/objrepos"
CGI_BIN="."

# Run some access validations.  This script will also output the
# Content-type header so we don't have to do it here.
$BINDIR/wsm_auth.cgi
if (( $? != 0 )); then
    exit 0
fi

ID=0
ID_SEQ_NUM=0
FIELD_KEY=0
TITLE=""
LIST=""
RC=0

#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: there is no need to check for root ownership of the
#       WebSMIT configuration file here. It has already been
#       validated by the call to "wsm_auth.cgi", above.
#=============================================================
export SKIN_LOGO="/skins/DEFAULT_logo.gif"
LOGFILE='/usr/es/sbin/cluster/wsm/logs/wsm_smit.log'
CONF_FILE="${CONFDIR}/wsm_smit.conf"
set -a
. $CONF_FILE >/dev/null 2>&1
set +a

[[ $VERBOSE_LOGGING == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
typeset LOGFILE="/usr/es/sbin/cluster/wsm/logs/wsm_smit.log"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


# Get the name of the menu we are building.
# Note: for some reason this cgi is called a second time after the user
# presses the OK button. In the second invocation the paramters are not
# set - the only side effect of this second invocation is some messages in 
# the error_log (odmget fails because the id is not set), but we will avoid
# that by exiting the script if the id is not set. Note that this is done
# before the call to wsm_auth since it spits the "content" header.

# if the id is not set, we are all done
ID=$(${CGI_BIN}/wsm_cgiparse -v id)
[[ -z $ID ]] && exit 0


# parse the rest of the params from the cgi input
ID_SEQ_NUM=$(${CGI_BIN}/wsm_cgiparse -v id_seq_num)
FIELD_KEY=$(${CGI_BIN}/wsm_cgiparse -v key)
LIST_COMMAND="${WORKDIR}/${ID}.${ID_SEQ_NUM}.LIST_COMMAND.$$"

typeset STANZA=$(odmget -q "id = $ID AND id_seq_num = $ID_SEQ_NUM" sm_cmd_opt)

typeset FIELDS="${STANZA#*cmd_to_list_mode = \"}"
FIELDS="${FIELDS%%\"*}"

typeset MULTI="${STANZA#*multi_select = \"}"
MULTI="${MULTI%%\"*}"

typeset ENTRY_TYPE="${STANZA#*entry_type = \"}"
ENTRY_TYPE="${ENTRY_TYPE%%\"*}"

[[ -z $ENTRY_TYPE ]] && ENTRY_TYPE="n"


# cmd_opt multi select field.
# Defines if the user can make multiple selections from a list of valid 
# values produced by the command in the cmd_to_list field of the dialog 
# object. The value of multi_select is a string. Valid values are: 
# "" - No; a user can select only one value from a list. This is the default 
# case. 
# "," - Yes; a user can select multiple items from the list. When the 
# command is built, a comma is inserted between each item. 
# "y" - Yes; a user can select multiple values from the list. When the 
# command is built, the option prefix is inserted once before the string of 
# selected items. 
# "m" - Yes; a user can select multiple items from the list. When the 
# command is built, the option prefix is inserted before each selected item. 
#
# for the purposes of the list command, it is really just a yes or no
case "$MULTI" in
    "," ) 
	SEP=$MULTI
	MULTI=y
	;;
    "y" | "m" ) 
	SEP=" "
	MULTI=y
	;;
     * )
	MULTI=n
	;;
esac


# ils lookup of name
TITLE=$(get_cmd_opt_name_ils $ID $ID_SEQ_NUM)

# execute the list command to get the list of items
LIST=$($BINDIR/wsm_cmd_exec -f "$LIST_COMMAND" -i $ID -s $ID_SEQ_NUM -t cmd_to_list)
RC=$?
LIST=$(format_list_for_html "$LIST")

# start the html
$CGI_BIN/wsm_html_lib body_start_popup "List Box" act_smittree

# if the command to list failed, we are all done
if [[ $RC != 0 || -z "$LIST" ]]; then

    if [[ -z "$LIST" ]]; then
    	LIST=$($BINDIR/wsm_cmd_exec -f "$LIST_COMMAND" -i $ID -s $ID_SEQ_NUM -t cmd_to_list 2>&1)
    fi
    if [[ -z "$LIST" ]]; then
	LIST="$(getmsg wsm_smit.cat 9999 "Error: There were no items of this type.")"
    else
        LIST=${LIST//\&nbsp\;/ }
        LIST="<XMP>$LIST</XMP>"
    fi
    
    print "<pre class=\"listError\">"
    print -- "$LIST" | while read i
    do
        print -- "$i"
    done
    print "</pre>"
    
    print "<input type='submit' class='act_smittree act_smittree_button' value='Close' onclick='window.close();'>"
else

    typeset LINES=$(echo "$TITLE" | wc -l)
    if (( LINES > 1 ))
    then
        typeset HEAD=$(echo "$TITLE" | head -1) # Handle multi-line name headers
        TITLE="${TITLE##$HEAD}"
        if [[ -n $TITLE ]]; then
            TITLE=$(echo "$TITLE" | sed 's/\\//g') # Remove line cont. chars
            TITLE="<h2>$HEAD</h2><h3>$TITLE</h3>"
        fi
    else
        TITLE="<h2>$TITLE</h2>"
    fi

    # build the menu
    LIST_CNT=0
    if [[ "$MULTI" == "y" ]]; then    # list is a multiselect

        print -- "
$TITLE

<form id='listForm' class='act_smittree'>

<ul>"

	print -- "$LIST" | while read LINE
        do
            #==================================================================
            # When setting the item value, which is what will be extracted
            # and displayed in the input box when the user makes a selection,
            # go ahead and restore the real spaces (i.e. versus the HTML
            # non-breaking spaces), and trim any leading/trailing whitespace.
            #==================================================================
            typeset TMP_LINE=${LINE//\&nbsp\;/ }
            while [[ $TMP_LINE == [[:space:]]* ]]; do TMP_LINE=${TMP_LINE## }; done
            while [[ $TMP_LINE == *[[:space:]] ]]; do TMP_LINE=${TMP_LINE%% }; done
            typeset ITEM_VAL
            if [[ "$FIELDS" == @(1|2) ]]; then
                if [[ $FIELDS == '1' ]]; then
                    ITEM_VAL="${TMP_LINE%% *}"
                else
                    ITEM_VAL="${TMP_LINE##* }"
                fi
            else
                ITEM_VAL=$TMP_LINE
            fi
            print "\
<li><input type='checkbox'
           name='list'
           value=\"$ITEM_VAL\"
           id=\"$ITEM_VAL\"
           class='act_smittree'
           onmouseover=\"JavaScript: this.style.cursor='pointer';\"
           onmouseout=\"JavaScript: this.style.cursor='default';\"
           onclick=\"JavaScript:
var re = new RegExp('\s?$ITEM_VAL', 'g');
sequence = sequence.replace(re, '');
if (this.checked==true) {
    sequence += ' $ITEM_VAL';
}\">
<label for=\"$ITEM_VAL\"
       class='act_smittree'
       onmouseover=\"JavaScript: this.style.cursor='pointer';\"
       onmouseout=\"JavaScript: this.style.cursor='default';\"
       onclick=\"JavaScript:
var obj = document.getElementById('$ITEM_VAL');
if (obj) obj.onclick;
\">$LINE</label></li>"

            # Add the value to the session
            ${BINDIR}/wsm_session -d "${ID}__${ID_SEQ_NUM}_list" -s "item_$LIST_CNT"="$ITEM_VAL"
            (( LIST_CNT += 1 ))
	done

        # Add the total number of list items to the session for later
        # processing.
        ${BINDIR}/wsm_session -d "${ID}__${ID_SEQ_NUM}_list" -s "CNT"="$LIST_CNT"
	print "</ul>"

        print "
<div class='formButtons'>
<input type='button'
       value='   Ok   '
       class='act_smittree act_smittree_button'
       onClick=\"JavaScript: if (passMulti('$FIELD_KEY','$SEP', sequence)) { window.close(); }\">
<input type='button'
       value='   Cancel   '
       class='act_smittree act_smittree_button'
       onClick='JavaScript: pf3();'>
</div>

</form>"
	
    else  # list is single select

        print -- "\
$TITLE

<form id='listForm'>
<select name='list' STYLE='font-family: courier;' class='act_smittree'>"
	
	print -- "$LIST" | while read LINE
        do
            #==================================================================
            # When setting the item value, which is what will be extracted
            # and displayed in the input box when the user makes a selection,
            # go ahead and restore the real spaces (i.e. versus the HTML
            # non-breaking spaces), and trim any leading/trailing whitespace.
            #==================================================================
            typeset TMP_LINE=${LINE//\&nbsp\;/ }
            while [[ $TMP_LINE == [[:space:]]* ]]; do TMP_LINE=${TMP_LINE## }; done
            while [[ $TMP_LINE == *[[:space:]] ]]; do TMP_LINE=${TMP_LINE%% }; done
            typeset ITEM_VAL
            if [[ "$FIELDS" == @(1|2) ]]; then
                if [[ $FIELDS == '1' ]]; then
                    ITEM_VAL="${TMP_LINE%% *}"
                else
                    ITEM_VAL="${TMP_LINE##* }"
                fi
            else
                ITEM_VAL=$TMP_LINE
            fi
            print "<option value=\"$ITEM_VAL\" STYLE=\"font-family: courier;\">$LINE</option>"

            # Add the value to the session
            ${BINDIR}/wsm_session -d "${ID}__${ID_SEQ_NUM}_list" -s "item_$LIST_CNT"="$ITEM_VAL"
            (( LIST_CNT += 1 ))
        done

        # Add the total number of list items to the session for later
        # processing.
        ${BINDIR}/wsm_session -d "${ID}__${ID_SEQ_NUM}_list" -s "CNT"="$LIST_CNT"

	print "</select>"
	
        print "
<div class='formButtons'>
<input type='button'
       value='   Ok   '
       class='act_smittree act_smittree_button'
       onClick=\"JavaScript: if (passSingle('$FIELD_KEY')) { window.close(); }\">
<input type='button'
       value='   Cancel   '
       class='act_smittree act_smittree_button'
       onClick='JavaScript: pf3();'>
</div>

</form>"
    fi
fi

print "</div>

<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>

</body>
</html>"

# log the command
if [[ $VERBOSE_LOGGING == "high" ]]; then
    print "Command To List was:" >>$LOGFILE
    odmget -q "id = $ID AND id_seq_num = $ID_SEQ_NUM" sm_cmd_opt | 
	awk -f ${CGI_BIN}/wsm_awkscr_noparam V=cmd_to_list >>$LOGFILE


    {
    	print "Output from Command To List was:"
    	print "[$LIST]"
    	print "Exit status from Command To List was: $RC"
	date +"[%b %d %Y, %T]"
	print "List popup selected:"
	print "	Option id   = \"$ID\""
	print "	Seq Num     = \"$ID_SEQ_NUM\""
	print "	User IP     = \"$REMOTE_ADDR\""
	print ""
    } >>$LOGFILE
fi
