#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_customize.cgi.sh 1.14 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2007,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 


#################################################################
#################################################################
##
## wsm_customize.cgi
##
##     Constructs the WebSMIT customization GUI.
##
## EXTERNAL DEPENDENCIES
##
##	wsm.js
##	wsm_custom.js
##
#################################################################
#################################################################



##############################################################################
##############################################################################
##
## Function: create_colors_list
##
## Description:
##
##       Creates the colors selection list inside of a table cell.
##
## Inputs:
##
##       n/a
##
## Requisite Environment:
##
##       COLORS   A global array of color definitions.
##
## Return value:
##
##       n/a
##
##############################################################################
##############################################################################
function create_colors_list
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    print "
<td halign='left' valign='middle' class='customize'>
<select id='color_names'
        name='color_names'
        class='customize'
        style='border-style: ridge; border-width: medium'
        size=20
        onclick=\"JavaScript: wsm_color_select(this, wsm_info);\">"

    typeset NCMSG=$(getmsg -s "$MAINSET" \
                              "$WSMCAT"  \
                              20         \
                              "New Color");

    typeset MOMSG_TMPL=$(getmsg -s "$MAINSET" \
                                   "$WSMCAT"  \
                                   10         \
                                   "Click to select \"COLOR_NAME\" and view it in the \"$NCMSG\" sample area."                \
                                   "COLOR_NAME")

    typeset -i IDX=0
    while (( IDX < ${#COLORS[*]} )); do
        typeset LINE=${COLORS[$IDX]}

        typeset COLOR_NAME=${LINE##*\|}
        COLOR_NAME=${COLOR_NAME## }
        COLOR_NAME=${COLOR_NAME%% }

        typeset COLOR_VALUE=${LINE%%\|*}
        COLOR_VALUE=${COLOR_VALUE## }
        COLOR_VALUE=${COLOR_VALUE%% }

        if [[ -n $COLOR_NAME && -n $COLOR_VALUE ]]; then

            #=======================================
            # Substitute the color name for the
            # placeholder in the message template.
            #=======================================
            typeset HALF1=${MOMSG_TMPL%COLOR_NAME*}
            typeset HALF2=${MOMSG_TMPL#*COLOR_NAME}
            typeset MOMSG="$HALF1$COLOR_NAME$HALF2"

            print "\
<option class='customize' id='$COLOR_VALUE' value='$COLOR_NAME'
        title='$MOMSG'
> $COLOR_NAME </option>"
        fi

        (( IDX += 1 ))
    done

    print "
</select>
</td>
"
} # End of "create_colors_list()"


##############################################################################
##############################################################################
##
## Function: create_samples_area
##
## Description:
##
##       Creates the color samples display objects. They are editable,
##       so customers can try their own text/numbers/symbols in them,
##       to see how they might look with the new colors.
##
## Inputs:
##
##       DEFAULT_BG_COLOR    The default background color for WebSMIT.
##
##       DEFAULT_FG_COLOR    The default foreground color for WebSMIT.
##
## Requisite Environment:
##
##       n/a
##
## Return value:
##
##       n/a
##
##############################################################################
##############################################################################
function create_samples_area
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset DEFAULT_BG_COLOR=$1
    typeset DEFAULT_FG_COLOR=$2

    typeset -i ROWS=3
    typeset -i COLS=10

    typeset ORIGMSG=$(getmsg -s "$MAINSET" \
                                "$WSMCAT"  \
                                20         \
                                "These are the original colors for the selected object.")
    typeset ORIGLBL=$(getmsg -s "$MAINSET" \
                                "$WSMCAT"  \
                                21         \
                                "--Sample--\n Original\n  Colors")

    typeset CURRMSG=$(getmsg -s "$MAINSET" \
                                "$WSMCAT"  \
                                22         \
                                "These are the current color selections for the selected customizable object.")
    typeset CURRLBL=$(getmsg -s "$MAINSET" \
                                "$WSMCAT"  \
                                23         \
                                "--Sample--\n Current\n  Colors")

    typeset NEWMSG=$(getmsg -s "$MAINSET" \
                               "$WSMCAT"  \
                               24         \
                               "These are the new, stored color selections for the selected customizable object.")
    typeset NEWLBL=$(getmsg -s "$MAINSET" \
                               "$WSMCAT"  \
                               25         \
                               "--Sample--\n   New\n  Colors")

    print "
<td halign='left' valign='middle'>
<textarea id='original_sample'
          name='original_sample'
          class='customize'
          style='border-style: inset; font-weight: bold;'
          cols=$COLS
          rows=$ROWS
          title=\"$ORIGMSG\"
          style='background-color:$DEFAULT_BG_COLOR; color:$DEFAULT_FG_COLOR; overflow:hidden; font-weight:bolder; font-size:larger'>
$ORIGLBL
</textarea><br>

<textarea id='current_sample'
          name='current_sample'
          class='customize'
          style='border-style: inset; font-weight: bold;'
          cols=$COLS
          rows=$ROWS
          title=\"$CURRMSG\"
          style='background-color:$DEFAULT_BG_COLOR; color:$DEFAULT_FG_COLOR; overflow:hidden; font-weight:bolder; font-size:larger'>
$CURRLBL</textarea><br>

<textarea id='new_sample'
          name='new_sample'
          class='customize'
          style='border-style: inset; font-weight: bold;'
          cols=$COLS
          rows=$ROWS
          title=\"$NEWMSG\"
          style='background-color:$DEFAULT_BG_COLOR; color:$DEFAULT_FG_COLOR; overflow:hidden; font-weight:bolder; font-size:larger'>
$NEWLBL</textarea><br>
</td>
"
} # End of "create_samples_area()"


##############################################################################
##############################################################################
##
## Function: create_objects_list
##
## Description:
##
##       Creates the customizable objects list in a table cell.
##
## Inputs:
##
##       DEFAULT_BG_COLOR    The default background color for WebSMIT.
##
##       DEFAULT_FG_COLOR    The default foreground color for WebSMIT.
##
## Requisite Environment:
##
##            WSM_OBJECTS    A global array containing all the colorizable
##                           objects, along with their associated information.
##
## Return value:
##
##       n/a
##
##############################################################################
##############################################################################
function create_objects_list
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset DEFAULT_BG_COLOR=$1
    typeset DEFAULT_FG_COLOR=$2

    typeset OBJMSG_TMPL=$(getmsg -s "$MAINSET" \
                                    "$WSMCAT"  \
                                    30         \
                                    "Click to view/edit the current color settings for \"WSMOBJ\".")

    print "
<td align='left' valign='middle'>
<select id='wsm_objects'
        name='wsm_objects'
        class='act_smittree customize'
        style='border-style: ridge; border-width: medium'
        size=20
        onclick='JavaScript: wsm_object_select(this, wsm_info);'>"

    typeset -i IDX=0
    while (( IDX < ${#WSM_OBJECTS[*]} )); do
        typeset OBJECT_DATA=${WSM_OBJECTS[$IDX]}

        typeset OBJECT=${OBJECT_DATA%%\|*}
        if [[ -n $OBJECT ]]; then

            typeset OBJSTR=${OBJECT##*\&nbsp\;}

            if [[ $OBJECT != $MAJOR_SEPARATOR &&
                  $OBJECT != $MINOR_SEPARATOR ]]
            then
                typeset OBJMSG_TMPL=$(getmsg -s "$MAINSET" \
                                                "$WSMCAT"  \
                                                30         \
                                                "Click to view/edit the current color settings for \"WSMOBJ\".")
            fi

            #=======================================
            # Substitute the object name for the
            # placeholder in the message template.
            #=======================================
            typeset HALF1=${OBJMSG_TMPL%WSMOBJ*}
            typeset HALF2=${OBJMSG_TMPL#*WSMOBJ}
            typeset OBJMSG="$HALF1$OBJSTR$HALF2"

            if [[ $OBJECT != $MAJOR_SEPARATOR &&
                  $OBJECT != $MINOR_SEPARATOR ]]
            then
                print "
<option value='$OBJECT'
        title='$OBJMSG'
        class='act_smittree customize'
        style='{background-color: $DEFAULT_BG_COLOR; color: $DEFAULT_FG_COLOR}' > $OBJECT </option>"
            else
                print "
<option value='$OBJECT'
        class='act_smittree customize'
        style='{background-color: $DEFAULT_BG_COLOR; color: $DEFAULT_FG_COLOR}' > $OBJECT </option>"
            fi

            (( IDX += 1 ))
        fi
    done

    print "
</select>
</td>
"
} # End of "create_objects_list()"


##############################################################################
##############################################################################
##
## Function: create_button_bar
##
## Description:
##
##       Creates a row of buttons, to allow the user to control
##       this GUI.
##
## Inputs:
##
##       n/a
##
## Requisite Environment:
##
##       n/a
##
## Return value:
##
##       n/a
##
##############################################################################
##############################################################################
function create_button_bar
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset RESET_LBL=$(getmsg -s "$MAINSET" "$WSMCAT" 100 "Reset")
    typeset RESET_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 101 \
                 "Reset any current selections/changes in this tool.")
    typeset RESET_CONFIRM_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 102 \
                 "Are you sure you want to discard the following changes?")
    typeset RESET_NO_CUST_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 121 \
                 "No new/unsaved color selections have been made yet.\\\\\\\nThere is nothing to reset.")

    typeset DEFAULTS_LBL=$(getmsg -s "$MAINSET" "$WSMCAT" 103 "Defaults")
    typeset DEFAULTS_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 104 \
                 "Restore the default WebSMIT color set.")
    typeset DEFAULTS_CONFIRM_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 122 \
                 "Are you sure you want to discard the following saved customizations?")
    typeset DEFAULTS_RELOAD_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 105 \
                 "The default WebSMIT colors have been restored.\\\\\\\n\\\\\\\nWould you like to reload WebSMIT now to see the new color settings?")
    typeset DEFAULTS_ERROR_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 106 \
                 "ERROR: unable to restore the default WebSMIT colors (could not delete the WebSMIT colors cookie:  WEBSMIT:COLORS)\\\\\\\nThe custom color set is still in use.")
    typeset DEFAULTS_NO_CUST_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 120 \
                 "No customizations have been saved yet.\\\\\\\nThe defaults are already being displayed.")

    typeset DISPLAY_LBL=$(getmsg -s "$MAINSET" "$WSMCAT" 107 "Display")
    typeset DISPLAY_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 108 \
                 "Show the current set of unsaved changes.")
    typeset DISPLAY_CURRENT_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 109 \
                 "Current customizations")
    typeset DISPLAY_NONE_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 110 \
                 "No new/unsaved changes have been made yet.\\\\\\\nThere is nothing to display.")

    typeset PREVIEW_LBL=$(getmsg -s "$MAINSET" "$WSMCAT" 111 "Preview")
    typeset PREVIEW_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 112 \
                 "Temporarily apply the new color selections to the selected customizable object.")
    typeset PREVIEW_NO_SEL_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 125 \
                 "A customizable object must be selected first.")

    typeset SAVE_LBL=$(getmsg -s "$MAINSET" "$WSMCAT" 113 "Save")
    typeset SAVE_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 114 \
                 "Save the current color selections (for this browser only).")
    typeset SAVE_ASK_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 115 \
                 "Save the following color customizations?")
    typeset SAVE_RELOAD_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 116 \
                 "The color customizations have been stored in this browser, and an effort has been made to apply them. However, it might be necessary to reload WebSMIT to fully pick up the new settings.\\\\\\\n\\\\\\\nWould you like to reload WebSMIT now?")
    typeset SAVE_NO_CHANGES_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 117 \
                 "No changes have been made yet.\\\\\\\nThere is nothing to save.")

    typeset RESTORE_LBL=$(getmsg -s "$MAINSET" "$WSMCAT" 123 "Restore")
    typeset RESTORE_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 124 \
                 "Temporarily apply the original color selections to the selected customizable object.")
    typeset RESTORE_NO_SEL_MSG=$(getmsg -s "$MAINSET" "$WSMCAT" 125 \
                 "A customizable object must be selected first.")

    print "
<hr size=4 class='customize'>
&nbsp;
<input type='button'
       id='reset'
       name='reset'
       value=' $RESET_LBL '
       class='act_smittree act_smittree_button customize'
       title='$RESET_MSG'
       onclick=\"JavaScript:
var cMsg = new String();
for (var c in changes) {
    var colorObj = getElementById(changes[c]);
    cMsg += c + ' ==> ' + colorObj.value + '\\\n';
}
if (cMsg.length > 0) {
    if (confirm('$RESET_CONFIRM_MSG\\\n\\\n' + cMsg + '\\\n')) {

        //====================================================
        // Apply defaults, then apply saved colors (if any).
        //====================================================
        var colors = get_cookie('WEBSMIT:COLORS');
        for (var c in changes) {

            var wsm_data = wsm_info[c].split('|');
            var level         = wsm_data[0];
            var color_type    = wsm_data[1].toUpperCase();
            var association   = wsm_data[2];
            var nodes         = wsm_data[3];
            var classes       = wsm_data[4];
            var ancestor      = wsm_data[5];
            var default_color = wsm_data[6];

            //================================================
            // If this object had a previously saved color,
            // restore it. If not, then restore the default.
            //================================================
            if (colors) {
                var allCust = colors.split('|');
                for (var a in allCust) {
                    var cust = allCust[a].split('@');
                    if (cust[0] == c) {
                        default_color = cust[1];
                        break;
                    }
                }
            }

            //========================================================
            // Erase the change entry, without changing array size.
            // Not usually safe to change an array that is currently
            // being iterated through! So don't use 'delete()' here!
            //========================================================
            changes[c] = '';

            //----------------------------------------------------
            // Apply the new color, just in case the user didn't
            //----------------------------------------------------
            if (previewed) { // Only need this if the color was previewed
                preview_color(default_color,
                              c,
                              wsm_info);
            }

        } // End of the changes loop

        // Reset this optimization flag
        previewed = false;

        //====================
        // Reset everything.
        //====================
        changes = new Array();   // Erase the keys
        document.forms[0].reset;
    }

} else {
    alert('$RESET_NO_CUST_MSG');
}
\"/>

<input type='button'
       id='defaults'
       name='defaults'
       value=' $DEFAULTS_LBL '
       class='act_smittree act_smittree_button customize'
       title='$DEFAULTS_MSG'
       onclick=\"JavaScript:
var colors = get_cookie('WEBSMIT:COLORS');
if (colors) {
    //==============================================================
    // Add the contents of the cookie to the confirmation message.
    //==============================================================
    var allCust = colors.split('|');
    var cMsg = new String();
    for (var c in allCust) {
        if (allCust[c].length > 0) {
            var cust = allCust[c].split('@');
            var colorObj = getElementById(cust[1]);
            if (colorObj) {
                cMsg += cust[0] + ' ==> ' + colorObj.value + '\\\n';
            }
        }
    }

    //===================================================
    // If the user confirms that they meant to hit this
    // button, then go ahead and delete the cookie.
    //===================================================
    if (confirm('$DEFAULTS_CONFIRM_MSG\\\n\\\n' + cMsg + '\\\n')) {

        //=======================================================
        // Before deleting the colors cookie, compile a list of
        // all objects that are known to have been customized.
        //=======================================================
        var items = new Array();
        for (var c in changes) {
            var item = changes[c];
            items[item] = item;
        }
        var allCust = colors.split('|');
        for (var a in allCust) {
            if (allCust[a].length > 0) {
                var cust = allCust[a].split('@');
                var obj = cust[0];
                items[obj] = obj;
            }
        }

        //==================================
        // Go ahead and remove the cookie.
        //==================================
        delete_cookie('WEBSMIT:COLORS', '/');
        colors = get_cookie('WEBSMIT:COLORS');

        if (colors) {
            alert('$DEFAULTS_ERROR_MSG');
        } else {

            //===================================================
            // In an effort to not force the customer to reload
            // WebSMIT, attempt to restore the default colors.
            //===================================================
            for (var c in items) {
                var item = items[c];

                var wsm_data = wsm_info[item].split('|');
                var level          = wsm_data[0];
                var color_type     = wsm_data[1].toUpperCase();
                var association    = wsm_data[2];
                var nodes          = wsm_data[3];
                var classes        = wsm_data[4];
                var ancestor       = wsm_data[5];
                var original_color = wsm_data[6];

                //----------------------------------------------------
                // Apply the new color, just in case the user didn't
                //----------------------------------------------------
                var default_color = '$DEFAULT_FG_COLOR';
                if (color_type != 'FG') {
                    default_color = '$DEFAULT_BG_COLOR';
                }

                preview_color(default_color,
                              item,
                              wsm_info);
            }

            // NOTE: this reload may no longer be needed. However, it *might*
            //       prove beneficial in the future. Just in case, it is being
            //       commented out, but left in the code for easy restoration.
            //if (confirm('$DEFAULTS_RELOAD_MSG')) {
            //    top.location.reload();
            //}
        }
    }

} else {

    //============================================================
    // If control arrives here, that means that no saved colors
    // were found; the only reason to be clicking this button!
    // However, to be friendly to user, we check here to see if
    // perhaps they made some color selections that they haven't
    // saved yet, and simply want to discard them. That's what
    // the Reset button is for, so if that is the case, we click
    // the Reset button for them! Aren't we nice? :-)
    //============================================================

    // NOTE: this loop will only be entered if color selections
    //       have been made. Why a loop? ~sigh~ This silliness
    //       is needed because, currently, JavaScript does not
    //       reliably report the length of associative arrays.
    var click_reset = false;
    for (var c in changes) {
        click_reset = true;
        break;
    }

    if (click_reset) {
        var reset_btn = getElementById('reset');
        if (reset_btn) { reset_btn.click(); }
    } else {
        alert('$DEFAULTS_NO_CUST_MSG');
    }
}
\"/>


&nbsp;&nbsp;&nbsp;
<input type='button'
       id='display'
       name='display'
       value=' $DISPLAY_LBL '
       class='act_smittree act_smittree_button customize'
       title='$DISPLAY_MSG'
       onclick=\"JavaScript:
var cMsg = '$DISPLAY_CURRENT_MSG:\\\n\\\n';
var count = 0;
for (var c in changes) {
    var colorObj = getElementById(changes[c]);
    cMsg += c + ' ==> ' + colorObj.value + '\\\n';
    count++;
}
if (count > 0) {
    alert(cMsg);
} else {
    alert('$DISPLAY_NONE_MSG');
}
\"/>

<input type='button'
       id='preview'
       name='preview'
       value=' $PREVIEW_LBL '
       class='act_smittree act_smittree_button customize'
       title='$PREVIEW_MSG'
       onclick=\"JavaScript:
var wiobj = window.document.forms[0].wsm_objects;
if (wiobj.selectedIndex >= 0) {
    var selected_obj = wiobj.options[wiobj.selectedIndex].value;

    // Remove leading whitespace/nbsp, and trailing whitespace
    selected_obj = selected_obj.replace(/^[\\s\\xA0]+/g, '').replace(/\\s+\$/g, '');

    if (changes[selected_obj]) {
        preview_color(window.document.forms[0].color_names,
                      window.document.forms[0].wsm_objects,
                      wsm_info);
    }
} else {
    alert('$PREVIEW_NO_SEL_MSG');
}
\"/>

<input type='button'
       id='restore'
       name='restore'
       value=' $RESTORE_LBL '
       class='act_smittree act_smittree_button customize'
       title='$RESTORE_MSG'
       onclick=\"JavaScript:
//===================================================================
// Apply the colors from the 'Original' sample to the 'New' sample.
// This should behave *exactly* as if the colors were clicked on
// individually by the user.
//===================================================================
var colorObj = window.document.forms[0].color_names;
var wiobj = window.document.forms[0].wsm_objects;
if (wiobj.selectedIndex >= 0) {
    var selected_obj = wiobj.options[wiobj.selectedIndex].value;
    var orig_obj_index = wiobj.selectedIndex;
    for (var o=0; o<2; o++) {
        // Remove leading whitespace/nbsp, and trailing whitespace
        selected_obj = selected_obj.replace(/^[\s\xA0]+/g, '').replace(/\s+$/g, '');

        if (wsm_info[selected_obj]) {
            var wsm_data    = wsm_info[selected_obj].split('|');
            var color_type  = wsm_data[1].toUpperCase();
            var association = wsm_data[2];

            var color = window.document.forms[0].original_sample.style.backgroundColor;
            if (color_type != 'BG') {
                color = window.document.forms[0].original_sample.style.color;
            }
            color = convert_rgb_to_hex(color);
            for (var i=0; i<colorObj.length; i++) {
                if (colorObj.options[i].id == color) {
                    colorObj.selectedIndex = i;
                    wsm_color_select(colorObj, wsm_info)
                    break;
                }
            }

            selected_obj = association; // Reset to the association (if any)
            for (var i=0; i<wiobj.length; i++) {
                var object = wiobj.options[i].value;
                object = object.replace(/^[\s\xA0]+/g, '').replace(/\s+$/g, '');
                if (object == selected_obj) {
                    wiobj.selectedIndex = i;
                    break;
                }
            }
        }
    }

    wiobj.selectedIndex = orig_obj_index;
    window.document.forms[0].preview.click();
} else {
    alert('$RESTORE_NO_SEL_MSG');
}
\"/>


&nbsp;&nbsp;&nbsp;
<input type='button'
       id='save'
       name='save'
       value=' $SAVE_LBL '
       class='act_smittree act_smittree_button customize'
       title='$SAVE_MSG'
       onclick=\"JavaScript:
var cMsg = new String();
var css_str = '|';
for (var c in changes) {
    var colorObj = getElementById(changes[c]);
    css_str += c + '\@' + colorObj.id + '|';
    cMsg += c + ' ==> ' + colorObj.value + '\\\n';
}
if (css_str != '|') {

    if (confirm('$SAVE_ASK_MSG\\\n\\\n' + cMsg + '\\\n\\\n')) {

        var currCookie = get_cookie('WEBSMIT:COLORS');

        //====================================================
        // Append all previously saved values that have not
        // also just been modified (i.e. they were changed).
        //====================================================
        if (currCookie) {
            var curr = currCookie.split('|');
            for (var c in curr) {
                if (curr[c].length > 0) {
                    var parts = curr[c].split('\@');
                    if (parts) {
                        if (css_str.indexOf('|' + parts[0] + '\@') < 0) {
                            css_str += curr[c] + '|';
                        }
                    }
                }
            }
        }

        //========================================
        // Save the color settings in a cookie
        //========================================
        set_cookie('WEBSMIT:COLORS',
                   css_str,
                   'Fri, 31 Dec 2099 23:59:59 UTC', '/');

        //======================================================
        // Call preview_color() for each change, taking care
        // to honor their layer/level priority order. In other
        // words, the more global changes have to be done
        // first, followed by the more specific changes.
        //======================================================
        for (var level=1; level<=3; level++) {
            for (var c in changes) {
                var wsm_data = wsm_info[c].split('|');
                var c_level  = wsm_data[0];
                if (c_level == level) {
                    var colorObj = getElementById(changes[c]);
                    var new_color = colorObj.id;

                    //----------------------------------------------------
                    // Apply the new color, just in case the user didn't
                    //----------------------------------------------------
                    preview_color(new_color,
                                  c,
                                  wsm_info);
                }
            }
        }

        changes = new Array();

        // NOTE: this reload may no longer be needed. However, it *might*
        //       prove beneficial in the future. Just in case, it is being
        //       commented out, but left in the code for easy restoration.
        //if (confirm('$SAVE_RELOAD_MSG')) {
        //    top.location.reload();
        //}
    }

} else {
    alert('$SAVE_NO_CHANGES_MSG');
}
\"/>
"
} # End of "create_button_bar()"



##############################################################################
##############################################################################
##############################################################################
##
## MAIN
##
##############################################################################
##############################################################################
##############################################################################

PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Supports global/env specification


#========================================
# Set up some helpful, global variables
#========================================
WSMDIR="/usr/es/sbin/cluster/wsm"
BINDIR="$WSMDIR/cgi-bin"
LOGFILE="$WSMDIR/logs/wsm_smit.log"
WSMCAT="wsm_smit.cat"
MAINSET=2
COLORSET=3


#=============================================================
# Run some access validations.  This script will also output
# the "Content-type" header so we don't have to do it here.
#=============================================================
$BINDIR/wsm_auth.cgi
if (( $? != 0 )); then
    exit 0
fi


#=========================
# Clear the session file
#=========================
$BINDIR/wsm_session -clear


#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: there is no need to check for root ownership of the
#       WebSMIT configuration file here. It has already been
#       validated by the call to "wsm_auth.cgi", above.
#=============================================================
CONF_FILE="$WSMDIR/wsm_smit.conf"
set -a
. $CONF_FILE >/dev/null
set +a

[[ $VERBOSE_LOGGING == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
typeset LOGFILE="/usr/es/sbin/cluster/wsm/logs/wsm_smit.log"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


#======================================
# Load the list of customizable items
#======================================
set -A WSM_OBJECTS
export DEFAULT_BG_COLOR='#FFFFFF'
export DEFAULT_FG_COLOR='#000000'
typeset MAJOR_SEPARATOR='=======================================';
typeset MINOR_SEPARATOR='- - - - - - - - - - - - - - - - - - - - - - - - - - -'
typeset -i LIDX=0
typeset LINE=
while IFS= read LINE; do
    if [[ -n $LINE && ${LINE## } != \#* ]]; then

        if [[ $LINE == *MAJOR?SEPARATOR* ]]; then
            LINE=$MAJOR_SEPARATOR
        elif [[ $LINE == *MINOR?SEPARATOR* ]]; then
            LINE=$MINOR_SEPARATOR
        fi

        #=================================================================
        # "WSM_OBJECTS" is the ordered list of display items. This list
        # keeps everything displayed in the same order specified in the
        # data definitions file.
        #=================================================================
        typeset INDENT=""
        typeset -i I;
        for (( I=0; I<${#LINE}; I+=1 )); do
            [[ ${LINE:$I:1} != [[:space:]] ]] && break
            INDENT="$INDENT&nbsp;"
        done
        LINE="${INDENT}${LINE:$I}"

        WSM_OBJECTS[$LIDX]="$LINE"
        (( LIDX += 1 ))
    fi
done < "$BINDIR/wsm_customize.dat"


#=====================================================
# Load the table of currently recognized HTML colors
#=====================================================
set -A COLORS   # A list of color names and values, pipe delimited
unset LINE
typeset -i CIDX=0
while read LINE; do
    LINE=${LINE## }
    LINE=${LINE%% }

    [[ $LINE == \#* ]] && continue

    typeset COLOR_NAME=
    typeset COLOR_VALUE=
    typeset COLOR_ID=
    for VAL in $LINE; do
        if [[ -z $COLOR_NAME ]]; then
            COLOR_NAME=$VAL
        elif [[ -z $COLOR_VALUE ]]; then
            COLOR_VALUE=$VAL
        else
            COLOR_ID=$VAL
            if [[ -n $COLOR_ID ]]; then
                COLOR_NAME=$(getmsg -s "$COLORSET" \
                                       "$WSMCAT"   \
                                       "$COLOR_ID" \
                                       "$COLOR_NAME")
            fi
        fi
    done

    if [[ -n $COLOR_NAME && -n $COLOR_VALUE ]]; then
        COLORS[$CIDX]="$COLOR_VALUE|$COLOR_NAME"
        (( CIDX += 1 ))
    fi
done < "$BINDIR/wsm_customize.colors"


#=================
# Start the page
#=================
typeset TITLE=$(getmsg -s "$MAINSET" \
                          "$WSMCAT"  \
                          1          \
                          "Customization")

#
# Print the HTML header
#
$BINDIR/wsm_html_lib body_start_menu \
                     "$TITLE"        \
                     "wsm_smit.cat"  \
                     "2"             \
                     "1"             \
                     "act_smittree customize"

print "
<form name='wsm_customize' class='act_smittree customize'>
"


#=======================================
# Load the object info into JavaScript
#=======================================
print "<script language='JavaScript'>"
typeset -i IDX=0
while (( IDX < ${#WSM_OBJECTS[*]} )); do
    typeset OBJECT=${WSM_OBJECTS[$IDX]}
    (( IDX += 1 ))

    [[ -z $OBJECT ]] && continue
    [[ $OBJECT == $MAJOR_SEPARATOR ]] && continue
    [[ $OBJECT == $MINOR_SEPARATOR ]] && continue

    OBJECT=${OBJECT##*\&nbsp\;}
    OBJECT=${OBJECT## }
    OBJECT=${OBJECT%% }

    typeset NAME=${OBJECT%%\|*}		OBJECT=${OBJECT#*\|} # 1st field
    typeset MSGID=${OBJECT%%\|*}	OBJECT=${OBJECT#*\|} # 2nd field
    if [[ -n $MSGID ]]; then
        NAME=$(getmsg -s "$MAINSET" \
                         "$WSMCAT"  \
                         "$MSGID"   \
                         "$NAME")
    fi
    typeset LEVEL=${OBJECT%%\|*}        OBJECT=${OBJECT#*\|} # 3rd field
    typeset COLOR_TYPE=${OBJECT%%\|*}   OBJECT=${OBJECT#*\|} # 4th field
    OBJECT=${OBJECT#*\|}                # Skip the fifth field
    typeset ASSOCIATION=${OBJECT%%\|*}  OBJECT=${OBJECT#*\|} # 6th field
    typeset ALL_NODES=${OBJECT%%\|*}    OBJECT=${OBJECT#*\|} # 7th field
    typeset ALL_CLASSES=${OBJECT%%\|*}  OBJECT=${OBJECT#*\|} # 8th field
    typeset PARENT=${OBJECT%%\|*}       OBJECT=${OBJECT#*\|} # 9th field

    ORIGIFS="$IFS"
    IFS=','

    typeset NODES=
    for NODE in $ALL_NODES; do
        NODE=${NODE## }
        NODE=${NODE%% }
        NODES="$NODES $NODE"
    done
    NODES=${NODES## }

    typeset CLASSES=
    for CLASS in $ALL_CLASSES; do
        CLASS=${CLASS## }
        CLASS=${CLASS%% }
        CLASSES="$CLASSES $CLASS"
    done
    CLASSES=${CLASSES## }

    IFS="$ORIGIFS"

    print "wsm_info['$NAME']=\"$LEVEL|$COLOR_TYPE|$ASSOCIATION|$NODES|$CLASSES|$PARENT\";"
done


#=========================================================
# Append the original, detected colors for each object,
# defaulting to the WebSMIT defaults where no associated
# object/area is provided.
#=========================================================
print "
for (var obj in wsm_info) {
    var wsm_data = wsm_info[obj].split('|');
    var level         = wsm_data[0];
    var color_type    = wsm_data[1].toUpperCase();
    var association   = wsm_data[2];
    var nodes         = wsm_data[3];
    var classes       = wsm_data[4];

    //------------------------------------------------
    // Determine the starting color for this object:
    //    1) check for a cookie value
    //    2) attempt to query the object directly
    //    3) just use the default
    //------------------------------------------------
    var orig_primary = gcc_cookie(obj);  // Get Current Color - Cookie
    // NOTE: temporarily commented out to improve the page load
    //       performance. This might be restored in the future,
    //       if the performance is improved.
    //if (! orig_primary || orig_primary.length == 0) {
    //    orig_primary = gcc_actual(color_type,
    //                              classes,
    //                              'a font body',
    //                              nodes);
    //}
    if (! orig_primary || orig_primary.length == 0) {
        orig_primary = (color_type == 'FG' ? default_fg_color
                                           : default_bg_color);
    }
    wsm_info[obj] += '|' + orig_primary;

    var complement = '';
    if (association.indexOf('#') == 0) {
        complement = association;
    } else {
        if (changes[association]) {
            complement = changes[association];
        } else {
            if (wsm_info[association]) {
                var wsm_assoc_data = wsm_info[association].split('|');
                var complement = gcc_cookie(association);
                // NOTE: temporarily commented out to improve the page load
                //       performance. This might be restored in the future,
                //       if the performance is improved.
                //if (! complement || complement.length == 0) {
                //    complement = gcc_actual(wsm_assoc_data[1].toUpperCase(),
                //                            wsm_assoc_data[4],
                //                            'a font body',
                //                            wsm_assoc_data[3]);
                //}
                if (! complement || complement.length == 0) {
                    complement = (color_type == 'FG' ? default_bg_color
                                                     : default_fg_color);
                }
            }
        }
    }

    if (! complement || complement.length == 0) {
        complement = (color_type == 'FG' ? default_fg_color
                                         : default_bg_color);
    }

    wsm_info[obj] += '|' + complement;
}
</script>"


###########################
###########################
##
## Construct the display
##
###########################
###########################
print "
<table id='selection_table'
       border=0
       class='act_smittree customize'
       cellspacing=8>
<tr class='customize'>"

create_colors_list

create_samples_area $DEFAULT_BG_COLOR \
                    $DEFAULT_FG_COLOR

create_objects_list $DEFAULT_BG_COLOR \
                    $DEFAULT_FG_COLOR

print "</tr>
</table>

<!-- Apply the formatting; the strict DTD won't allow it to be embedded. -->
<script type='text/javascript'>
    var colors_obj = window.document.forms[0].color_names;
    for (var i=0; i<colors_obj.length; i++) {
        colors_obj.options[i].style.fontWeight = 'bold';
        colors_obj.options[i].style.backgroundColor = colors_obj.options[i].id;
    }
</script>
"

create_button_bar

#
# Print the HTML footer
#
print "</form>\n\n"
$BINDIR/wsm_html_lib body_end_menu \
                     customize     \
                     wsm_smit.cat  \
                     2             \
                     2             \
                     "$TITLE"      \
                     "act_smittree act_smittree_link customize"

exit 0
