#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_help.cgi.sh 1.37 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2003,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)20	1.37 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_help.cgi.sh, hacmp.wsm, 52haes_r541, 0747C_hacmp541 1/11/08 11:54:55

###############################################################################
#
# Name: wsm_help.cgi
#
# Description: manages a popup window which displays help text for a smit panel
#
# Arguments: 
#
#       $1 - mode: 'text' to just print the message, else HTML output
#       $2 - smit panel id
#       $3 - the help msg set (for dspmsg)
#       $4 - the help msg id  (for dspmsg)
#       $5 - (optional) 'nodefault' to return an empty value for the default
#            message  
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################


PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Supports global/env specification

###############################################################################
#
# Name:
#
# Description: add the html to start the body
#
# Arguments:
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
#
# where to look for the catalog(s)
#
# somehow this needs to account for NLSPATH
export CATDIR=/usr/lib/nls/msg/C
ERROR="false"

# initialize
CGI_BIN="."
BINDIR=/usr/es/sbin/cluster/wsm/cgi-bin
CONFDIR=/usr/es/sbin/cluster/wsm


#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: it is necessary to make sure the file being sourced
#       is owned by root and only the owner has write
#       permissions. Being writable by any other person
#       could allow them to run arbitrary commands as root
#       (a.k.a. constitutes a security hole).
#=============================================================
CONF_FILE="$CONFDIR/wsm_smit.conf"
LOGFILE='/usr/es/sbin/cluster/wsm/logs/wsm_smit.log'
if [[ -f $CONF_FILE ]]; then

    if [[ -n "$(find $CONF_FILE ! -user root -o -perm -0020 -o -perm -0002)" ]]
    then
        print "Invalid permissions for $CONF_FILE.  " \
              "File should only be writable by root." >>$LOGFILE
        exit 1
    fi

    set -a
    . $CONF_FILE >/dev/null
    set +a
fi

[[ $VERBOSE_LOGGING == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
typeset LOGFILE="/usr/es/sbin/cluster/wsm/logs/wsm_smit.log"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


# initialize the default smit message
if [[ $5 == 'nodefault' ]] ; then
    TEXT=''
else
    TEXT=$(getmsg -s 99 smit.cat 151 "1800-071 No help available for this topic.")
fi

# wsm_help.cgi can also be run from the command line; check for arguments
MODE=$1
if [ -z "$MODE" -o \
     -z "$HTTP_USER_AGENT" ]
then
    if [ "$MODE" != "${MODE%.cat}" ]; then
        MODE=text
    fi
fi

if [[ "$MODE" == "text" ]]; then
    HELP_MSG_LOC=$2

    # Make sure these have values, or dspcat will dump hords of text.
    HELP_MSG_SET=${3:-9999}
    HELP_MSG_ID=${4:-9999}
else
    # parse input from incoming string
    HELP_MSG_LOC=$(${CGI_BIN}/wsm_cgiparse -v help_msg_loc)
    HELP_MSG_SET=$(${CGI_BIN}/wsm_cgiparse -v help_msg_set)
    HELP_MSG_ID=$(${CGI_BIN}/wsm_cgiparse -v help_msg_id)
    HELP_MSG_TOPIC=$(${CGI_BIN}/wsm_cgiparse -v help_msg_topic)
    EXTRA_MSG=$(${CGI_BIN}/wsm_cgiparse -v extra_msg)
fi


# There are 2 different styles used for specifying help (in the odm):
# 1) "fully qualified" - entry will have a message catalog name in the 
#   help_msg_loc field and a set and id specifier in the help_msg_id field.
#   The caller will break the set and id into seperate fields.
#   In this mode we will use dspcat to retrieve the text from the catalog
#   and format it into html for display.
# 2) "aix/smit help" - here the help_msg_loc field will be blank and the 
#   help_msg_id specifies a number to use to lookup html help text.
#   In this mode we will simply print the (system provided) html to screen.
#
if [[ -z $HELP_MSG_LOC ]]   # if no location specified
then    
    if [[ -n $HELP_MSG_SET ]]
    then
        # we have been called with the aix/smit help style where the set is
        # an integer index into a file which points to some html.
        # The lookup goes like this:
        # look in /usr/share/man/info/en_US/doc_registry/smithelp.ref
        # any entry will be in the form:
        # <name of html file> <key>
        # where the html file has a fully qualified path name and
        # the key is of form "SMHelp_aix_NNN" where NNN is the help id
        # stored in odm
        REGISTRY=/usr/share/man/info/en_US/doc_registry/smithelp.ref
        KEY=SMHelp_aix_$HELP_MSG_SET
        HELP_HTML=$(fgrep -w $KEY $REGISTRY | cut -f1 -d" " | head -1)
        if [[ -n $HELP_HTML ]]
        then
            # TEXT=$(cat $HELP_HTML |
            TEXT=$(awk ' BEGIN { IN=0 }
$1 == "<A" && $2 ~ start { IN=1 }
IN == 1 { print }
IN == 1 && $1 == "<HR>" { IN = 0 }'  start=$KEY $HELP_HTML)
        else
            ERROR=true
        fi
    fi

else
    # we have been called with the name of a message catalog - fetch 
    # the message 
    # For some reason, providing any default text to dspmsg was causing dspmsg
    # to *always* return that default text, but only when invoked through a
    # CGI script (worked from the command-line). I didn't have time to debug
    # this properly, unfortunately, so I just put in a workaround for now of
    # removing the default text, so dspcat can have a shot at retrieving the
    # desired message.
    CAT_TEXT=$(/usr/bin/dspmsg -s $HELP_MSG_SET $HELP_MSG_LOC $HELP_MSG_ID 2>/dev/null)
    RC=$?
    if [[ -z "$CAT_TEXT" || $RC != 0 ]]; then

        #====================================================================
        # The call to dspmsg with system defaults (NLSPATH, LANG) did not
        # return any text. Try an explicit path to the default message loc,
        # using dspcat instead of dspmsg (if possible).
        #====================================================================
        for LPATH in "/usr/lib/nls/msg/$LANG"    \
                     "/usr/es/lib/nls/msg/$LANG" \
                     "/usr/lib/nls/msg/en_US"    \
                     "/usr/es/lib/nls/msg/en_US" \
                     "/usr/lib/nls/msg/C"        \
                     "/usr/es/lib/nls/msg/C"
        do
            #=============================================================
            # NOTE: disallow a trailing slash, since that means "$LANG"
            #       is not set, which would result in a bogus directory.
            #=============================================================
            if [[ $LPATH != */ && -d $LPATH ]]; then
                CATDIR=$LPATH
                break
            fi
        done
        if [[ -f /usr/bin/dspcat && -d $CATDIR ]]; then
            CAT_TEXT=$(/usr/bin/dspcat $CATDIR/$HELP_MSG_LOC $HELP_MSG_SET $HELP_MSG_ID 2>/tmp/.wsmhelp.$$)
            RC=$?
        fi

        if [[ -n "$CAT_TEXT" && $RC == 0 ]]
        then
            TEXT=$CAT_TEXT
        else
            ERRTEXT=$(cat /tmp/.wsmhelp.$$)
            if [ -n "$HTTP_USER_AGENT" ]; then
                TEXT="<P>$TEXT</P><FONT COLOR=RED><B>$ERRTEXT</B></FONT>"
            else
                TEXT="$TEXT\n$ERRTEXT"
            fi
            ERROR=true
        fi

        rm -f /tmp/.wsmhelp.$$
    else 
        TEXT=$CAT_TEXT
    fi

fi


# convert non-displayable chars to HTML-compatible markup
if [ -n "$HTTP_USER_AGENT" ]; then
    TEXT=$(echo "${TEXT}" | sed -e "s/^[ \t]*$/<p>/g" )
fi

if [[ "$MODE" == "text" ]]; then
    if [[ -n $EXTRA_MSG ]]; then
        EXTRA_MSG=$(echo "$EXTRA_MSG" | sed "s|</*[a-zA-Z]*>||g")
        print -- "$EXTRA_MSG\n\n"
    fi
    print -- "$TEXT"
    exit $RC
fi



##
## NOTE: If this script was invoked in plain-text mode (i.e. such as from
##       the command-line), then the help text, if any, has already been
##       displayed, and the script exited from. In other words, execution
##       does not get here unless we *want* to display HTML.
##

# print html headers
$CGI_BIN/wsm_html_lib body_start_popup "Help" act_smittree

if [[ -n $HELP_MSG_TOPIC && $HELP_MSG_TOPIC != 'undefined' ]]; then
    print "<h2>$HELP_MSG_TOPIC</h2>"
fi

print "<div class='helpText act_smittree'>"

# WebSMIT adds its own horizontal line. Avoid having two of them!
TEXT=${TEXT%% }
TEXT=${TEXT%%\<HR\>}

# print the text
[[ -n $EXTRA_MSG ]] && print -- "$EXTRA_MSG<br><br>\n\n"
print -- "$TEXT"

# build the close button
CLOSE=$(getmsg cluster.cat 1000 "Close")
print "<div class='formButtons'><input type='submit' value=\"$CLOSE\" class='act_smittree act_smittree_button' onclick='javascript:window.close()'></div>"

print "</div>
</div>

<script language='JavaScript'>
if (docbody) { docbody.style.visibility='visible'; }
</script>
</body>
</html>"

if [[ $VERBOSE_LOGGING == "high" && $ERROR == "true" ]]; then
    # Log any errors
    {
        date +"[%b %d %Y, %T]"
        print "Error: could not find help text."
        print "mode was \"$MODE\""
        print "help_msg_loc was \"$HELP_MSG_LOC\""
        print "help_msg_set was \"$HELP_MSG_SET\""
        print "help_msg_id was \"$HELP_MSG_ID\""
        print "help_msg_topic was \"$HELP_MSG_TOPIC\""
        print "User IP = \"$REMOTE_ADDR\""
        print ""
    } >>$LOGFILE
fi


