#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_login.cgi.sh 1.18 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2004,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

###############################################################################
#
# Name: wsm_login.cgi 
# Description: Prints the WebSMIT login page
#
# Arguments: 
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
# Questions ? Comments ? Suggestions ? mailto:hafeedbk@us.ibm.com
#
###############################################################################


# @(#)41        1.18 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_login.cgi.sh, hacmp, 52haes_r541, 0747C_hacmp541 1/11/08 12:06:00
#
# Main starts here
#


PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Supports global/env specification
VERSION="1.18"

export WSM_VERSION=$VERSION
export WSM_PROGNAME=$PROGNAME

# initialize the cgi library
CGI_BIN="."
CONFDIR=/usr/es/sbin/cluster/wsm

export LOGFILE='/usr/es/sbin/cluster/wsm/logs/wsm_smit.log'


#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: it is necessary to make sure the file being sourced
#       is owned by root and only the owner has write
#       permissions. Being writable by any other person
#       could allow them to run arbitrary commands as root
#       (a.k.a. constitutes a security hole).
#=============================================================
CONF_FILE="$CONFDIR/wsm_smit.conf"
export SKIN_LOGO="/skins/DEFAULT_logo.gif"
if [[ -f $CONF_FILE ]]; then

    if [[ -n "$(find $CONF_FILE ! -user root -o -perm -0020 -o -perm -0002)" ]]
    then
        print "Invalid permissions for $CONF_FILE.  " \
              "File should only be writable by root." >>$LOGFILE
        exit 1
    fi

    set -a
    . $CONF_FILE >/dev/null
    set +a
fi

[[ $VERBOSE_LOGGING == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


SITE=$(${CGI_BIN}/wsm_cgiparse -v site)

# print the html header and opening body tags
$CGI_BIN/wsm_html_lib body_start_noscript "Login"

#=========================
# Get localized messages
#=========================
typeset HACMP_LOGO_HELP=$(getmsg -s 1 wsm_smit.cat 9999 "Click to go to the HACMP home page.")
typeset IBM_LOGO_HELP=$(getmsg -s 1 wsm_smit.cat 9999 "Click to go to the IBM home page.")
typeset USERNAME=$(getmsg -s 1 wsm_smit.cat 9999 "Username:")
HOSTNAME=$(hostname)
typeset USERNAME_HELP=$(getmsg -s 1 wsm_smit.cat 9999 "Enter a login ID that is valid on host $HOSTNAME.")
typeset PASSWORD=$(getmsg -s 1 wsm_smit.cat 9999 "Password:")
typeset PASSWORD_HELP=$(getmsg -s 1 wsm_smit.cat 9999 "Enter the password for the specified login ID.")
typeset LOGIN=$(getmsg -s 1 wsm_smit.cat 9999 "Login")
typeset LOGIN_HELP=$(getmsg -s 1 wsm_smit.cat 9999 "Click to initiate the login process with WebSMIT.")

cat <<END
<form method=post action="/cgi-bin/wsm_auth.cgi?login&site=$SITE">

    <table id="headerBar" width='100%' border=0 cellspacing=0 style="position: absolute; top: 0px;">
    <tr>
    <td align='left' valign='middle' width='33%'>
       <a class="logo"
          href="http://www.ibm.com"
          target="_blank"
          title="$IBM_LOGO_HELP"
          onMouseOver="JavaScript: window.status=''; return true;"
       >
           <img alt="IBM Logo" src="/wsm_logo_ibm.gif"/>
       </a>
    </td>

    <td align='center' valign='middle' width='34%'>
       <a class="logo"
          href="http://www.ibm.com/systems/p/ha/"
          target="_blank"
          title="$HACMP_LOGO_HELP"
          onMouseOver="JavaScript: window.status=''; return true;"
       >
           <img alt="HACMP Logo" src="/wsm_logo_ha.gif">
       </a>
    </td>

    <td align='right' valign='middle' width='33%'>
       <a class="logo"
          href="http://www.ibm.com"
          target="_blank"
          title="$IBM_LOGO_HELP"
          onMouseOver="JavaScript: window.status=''; return true;"
       >
           <img alt="IBM Logo" src="/wsm_logo_ibm.gif">
       </a>
    </td>
    </tr>
    </table>

    <br><br><br>
    <div id="loginForm">
        <div title="$USERNAME_HELP"
             onMouseOver="JavaScript: window.status=''; return true;">
            <label for="user">$USERNAME</label>
            <input id="user" type=text size=20 name="user">
        </div>

        <div title="$PASSWORD_HELP"
             onMouseOver="JavaScript: window.status=''; return true;">
            <label for="passwd">$PASSWORD</label>
            <input id="passwd" type=password size=20 name="passwd">
        </div>

        <div class="formButtons">
            <input type=submit
                   class="act_smittree_button"
                   value="$LOGIN"
                   title="$LOGIN_HELP"
                   onMouseOver="JavaScript: window.status=''; return true;">
        </div>
    </div>
</form>

<script type="text/javascript">
    <!-- put the cursor on the login form -->
    if(document.forms[0]) {
        for(x=0;x<=document.forms[0].elements.length;x++) {
            if(document.forms[0].elements[x].type!="hidden") {
                document.forms[0].elements[x].focus();
                break;
            }
        }
    }
</script>

END

$CGI_BIN/wsm_html_lib body_end_noscript
