# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_menu_opt.awk 1.8 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2007 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

#==================================================================
# get_value()
#
# Returns a normalized value from on ODM line for a SMIT panel.
# The value (everything on the right-hand side of the equal sign)
# is extracted from line, and cleaned up by stripping off leading
# and trailing whitespace and doublequotes.
#==================================================================
function get_value(str) {
    idx = index(str, "=")
    str = substr(str, ++idx)
    gsub(/(^[ \t]+|[ \t]+$)/, "", str)
    gsub(/(^"|"$)/, "", str)
    return str
} # End of function "get_value()"

function format_for_mo(str) {
    gsub(/'/, "\\'", str)
    return str
} # End of function "format_for_mo()"

BEGIN {
    FS="\n"  # Each field appears on its own line
    RS=""    # Each record is separated by a blank line
    OFS=""   # Eliminate the output field separator
    errors=0
    record_count=0 # Used so we can tell if anything got displayed!

    #============================
    # Load the allow/deny lists
    #============================
    allow_list = ""
    deny_list = ""
    if (allow_file) {
        cmd="cat \"" allow_file "\" | grep -v \"^#\""
        cmd|getline allow_list
        close(cmd)
    }
    if (deny_file) {
        cmd="cat \"" deny_file "\" | grep -v \"^#\""
        cmd|getline deny_list
        close(cmd)
    }

    #================================================================
    # The next highest menu in the hierarchy (where user came from)
    # is displayed in the browser title and on the menu as well
    #================================================================
    print "<h1 class='act_smittree'>" title "</h1>"
    print "<div id=\"menuBox\" class=\"act_smittree\">"
    print "<ul class=\"menu act_smittree\">"

} # End of the "BEGIN" clause


{ # Start of the main body
    if (/^sm_menu_opt:/) {

        if (VERBOSE_LOGGING == "high") {
            print "wsm_menu_opt.awk: Processing the following record:\n" $0 "\n" | "cat 1>&2"
        }

        #=================================================
        # Initialize the SMIT panel data field variables
        #=================================================
        id_seq_num = ""      # A numeric ID that indicates display order
        text = ""            # A string describing the function for this record
        mo_text = ""         # A string to use for mouseOver text, only
        text_msg_file = ""   # A message catalog, for translating "text"
        text_msg_set = ""    # A set number, used for translating "text"
        text_msg_id = ""     # A ID number, used for translating "text"
        next_id = ""         # The ID of the next entity to go to if this one
                             # is clicked on.
        next_type = ""       # The type of "next_id": null (information-only),
                             # dialog(d), name selector(n), or another menu(m).
        help_msg_loc = ""    # The location of the help message
        help_msg_id_com = "" # The combined set/ID for the help message

        #===========================================
        # Loop through each line of this record,
        # extracting the values initialized above.
        #===========================================
        x=1
        while ( x < NF ) { # Loop thru each line of this record
            if ($x ~ /id_seq_num *= */) {
                id_seq_num = get_value($x)
            } else if ($x ~ /text *= */) {
                if (text == "") {
                    text = get_value($x)
                } else {
                    print "ERROR: sm_menu_opt ID $ID and id_seq_num " id_seq_num " has multiple entries in the $ODM object repository!"
                    errors++
                }

            } else if ($x ~ /text_msg_file *= */) {
                text_msg_file = get_value($x)

            } else if ($x ~ /text_msg_set *= */) {
                text_msg_set = get_value($x)

            } else if ($x ~ /text_msg_id *= */) {
                text_msg_id = get_value($x)

            } else if ($x ~ /next_id *= */) {
                next_id = get_value($x)

            } else if ($x ~ /next_type *= */) {
                next_type = get_value($x)

            } else if ($x ~ /help_msg_loc *= */) {
                help_msg_loc = get_value($x)

            } else if ($x ~ /help_msg_id *= */) {
                help_msg_id_com = get_value($x)
            }
            x++
        } # End of the line loop for this record

        #===============================================
        # Extract the help message information, if any
        #===============================================
        help_msg_set = ""
        help_msg_id = ""
        split(help_msg_id_com, help_msg_ary, ",")
        help_msg_set = help_msg_ary[1]
        help_msg_id = help_msg_ary[2]

        if (VERBOSE_LOGGING == "high") {
            print "wsm_menu_opt.awk: id_seq_num: " id_seq_num | "cat 1>&2"
            print "wsm_menu_opt.awk: current text: " text | "cat 1>&2"
            print "wsm_menu_opt.awk: text_msg_file: " text_msg_file | "cat 1>&2"
            print "wsm_menu_opt.awk: text_msg_set: " text_msg_set | "cat 1>&2"
            print "wsm_menu_opt.awk: text_msg_id: " text_msg_id | "cat 1>&2"
            print "wsm_menu_opt.awk: next_id: " next_id | "cat 1>&2"
            print "wsm_menu_opt.awk: next_type: " next_type | "cat 1>&2"
            print "wsm_menu_opt.awk: help_msg_loc: " help_msg_loc | "cat 1>&2"
            print "wsm_menu_opt.awk: help_msg_id (original data): " help_msg_id_com | "cat 1>&2"
            print "wsm_menu_opt.awk:     help_msg_set: " help_msg_set | "cat 1>&2"
            print "wsm_menu_opt.awk:     help_msg_id: " help_msg_id | "cat 1>&2"
        }

        #==============================================================
        # If a message catalog was defined for this record, construct
        # an appropriate dspmsg command string, then invoke it to get
        # the translated string for this record.
        #==============================================================
        if (text_msg_file != "") {
            dspmsg_cmd = "dspmsg"
            if (text_msg_set != "") {
                dspmsg_cmd = dspmsg_cmd " -s " text_msg_set
            }
            dspmsg_cmd = dspmsg_cmd " " text_msg_file " " text_msg_id " " "\"" text "\""
            text_trans = ""
            dspmsg_cmd|getline text_trans
            close(dspmsg_cmd)

            if (text_trans != "" && text_trans != "\t") {
                text = text_trans
            }
        }

        mo_text = format_for_mo(text)

        if (VERBOSE_LOGGING == "high") {
            print "wsm_menu_opt.awk: translated text: " text | "cat 1>&2"
        }

        #================================================
        # Only display a link for the current record
        # if a valid label was found for it. In other
        # words, if there is no text, skip this record.
        #================================================
        if (text != "") {

            #=========================================================
            # Check the deny/allow files, in that order (deny takes
            # precedence over allow, if both contain the same entry).
            #=========================================================
            if (next_id != "") {

                if (deny_file && index(deny_list, next_id) > 0) {
                    if (VERBOSE_LOGGING == "high") {
                        print "\nwsm_menu_opt.awk: Access to the \"" text "\" item has been explicitly denied." | "cat 1>&2"
                    }
                    next
                }

                if (allow_file && index(allow_list, next_id) == 0) {
                    if (VERBOSE_LOGGING == "high") {
                        print "\nwsm_menu_opt.awk: Access to the \"" text "\" item has explicitly *not* been allowed." | "cat 1>&2"
                    }
                    next
                }

            #=============================================================
            # If there is no pointer to another SMIT panel defined for
            # this record, then it is an information-only entry. Display
            # the text here, then continue to the next record.
            #=============================================================
            } else {
                print "<li class=\"info\">" text "</li>"
                record_count++
                next
            }


            # All the validation tests have been passed, and
            # a new SMIT menu item is about to be displayed.
            record_count++

            #=======================================
            # Retrieve the translated help message
            #=======================================
            dspmsg_cmd = "dspmsg -s 1 wsm_smit.cat 9999 \"Get help for the '" text "' operation.\" \"" text "\""
            flyover_handler = ""
            dspmsg_cmd|getline flyover_handler
            flyover_handler = "title=\"" flyover_handler "\""
            close(dspmsg_cmd)

            #================================================================
            # Help for each listed item comes in two forms, the standard
            # message loc/set/id form, or the AIX HTMLized form. The latter
            # is indicated by the presence of *only* a message ID, but with
            # that ID *not* containing a comma.
            #================================================================
            have_help = 0
            if ( (help_msg_loc != "" && help_msg_set != "" && help_msg_id != "") || (help_msg_id_com != "" && help_msg_id_com !~ ",") ) {
                have_help = 1
            }

            ## NAME SELECT
            ## ===========
            if (next_type == "n") {

                #=======================================
                # Retrieve the translated help message
                #=======================================
                dspmsg_cmd = "dspmsg -s 1 wsm_smit.cat 9999 \"Proceed to the '" text "' panel.\" \"" text "\""
                dspmsg_cmd = dspmsg_cmd " " text_msg_file " " text_msg_id " " "\"" text "\""
                text_trans = ""
                dspmsg_cmd|getline text_trans
                close(dspmsg_cmd)

                print "<li class='act_smittree'>\n<font ",
                             "class='act_smittree act_smittree_link'\n",
                             "      title=\"" text_trans "\"\n",
                             "      onclick=\"JavaScript: parent.document.getElementById('ActSMIT_IFRAME').src = location.protocol + '//' + location.host + '/cgi-bin/wsm_name_hdr.cgi?name=", next_id, "&parent_help_msg_loc=", help_msg_loc, "&parent_help_msg_set=", help_msg_set, "&parent_help_msg_id=", help_msg_id, "';\"\n",
                             "      onMouseOver=\"JavaScript: this.className += ' highlight'; window.status=''; return true;\"\n",
                             "      onMouseOut=\"JavaScript: this.className = this.className.replace(/\\s?highlight/g, '');\"\n",
                          ">", text, "</font>"

                if (have_help) {
                    print "<span class='helpIndicator' ",
                             flyover_handler,
                          "onClick=\"JavaScript: displayHelpTip('wsm_help.cgi', 'cgi', '", help_msg_loc, "', '", help_msg_set, "', '", help_msg_id, "', '', '", text, "');\"",
                          "> (?) </span></li>"
                }

            ## COMMAND
            ## =======
            } else if (next_type == "d") {

                #=======================================
                # Retrieve the translated help message
                #=======================================
                dspmsg_cmd = "dspmsg -s 1 wsm_smit.cat 9999 \"Proceed to the '" text "' panel.\" \"" text "\""
                dspmsg_cmd = dspmsg_cmd " " text_msg_file " " text_msg_id " " "\"" text "\""
                text_trans = ""
                dspmsg_cmd|getline text_trans
                close(dspmsg_cmd)

                print "<li class='act_smittree'>\n<font ",
                             "class='act_smittree act_smittree_link'\n",
                             "      title=\"" text_trans "\"\n",
                             "      onclick=\"JavaScript: parent.document.getElementById('ActSMIT_IFRAME').src = location.protocol + '//' + location.host + '/cgi-bin/wsm_cmd_hdr.cgi?panel_id=", next_id, "&parent_help_msg_loc=", help_msg_loc, "&parent_help_msg_set=", help_msg_set, "&parent_help_msg_id=", help_msg_id, "';\"\n",
                             "      onMouseOver=\"JavaScript: this.className += ' highlight'; window.status=''; return true;\"\n",
                             "      onMouseOut=\"JavaScript: this.className = this.className.replace(/\\s?highlight/g, '');\"\n",
                          ">", text, "</font>"

                if (have_help) {
                    print "<span class='helpIndicator' ",
                             flyover_handler,
                          "onClick=\"JavaScript: displayHelpTip('wsm_help.cgi', 'cgi', '", help_msg_loc, "', '", help_msg_set, "', '", help_msg_id, "', '', '", text, "');\"",
                          "> (?) </span></li>"
                }

            ## MENU
            ## ====
            } else {

                #=======================================
                # Retrieve the translated help message
                #=======================================
                dspmsg_cmd = "dspmsg -s 1 wsm_smit.cat 9999 \"Proceed to the '" text "' menu.\" \"" text "\""
                dspmsg_cmd = dspmsg_cmd " " text_msg_file " " text_msg_id " " "\"" text "\""
                text_trans = ""
                dspmsg_cmd|getline text_trans
                close(dspmsg_cmd)

                print "<li class='act_smittree'>\n<font ",
                             "class='act_smittree act_smittree_link'\n",
                             "      title=\"" text_trans "\"\n",
                             "      onclick=\"JavaScript: parent.document.getElementById('ActSMIT_IFRAME').src = location.protocol + '//' + location.host + '/cgi-bin/wsm_menu_opt.cgi?name=", next_id, "';\"\n",
                             "      onMouseOver=\"JavaScript: this.className += ' highlight'; window.status=''; return true;\"\n",
                             "      onMouseOut=\"JavaScript: this.className = this.className.replace(/\\s?highlight/g, '');\"\n",
                          ">", text, "</font>"

                if (have_help) {
                    print "<span class='helpIndicator' ",
                             flyover_handler,
                          "onClick=\"JavaScript: displayHelpTip('wsm_help.cgi', 'cgi', '", help_msg_loc, "', '", help_msg_set, "', '", help_msg_id, "', '', '", text, "');\"",
                          "> (?) </span></li>"
                }
            }

        } else { # Display the intended spacer
            print "<BR>"
        }
    }
} # End of the main body


END {
    if (record_count == 0) {
        print "<div class='errorBox'>"
        print "Error: There are currently no SMIT screen entries defined for this FastPath in the ODM."
        print "This FastPath may require installation of additional software before it can be accessed."
        print "<div class='formButtons'><input type='button' value='Back' onclick='history.go(-1)'></div>"
        print "</div>\n"
    } else {
        print "</ul></div>"
    }

    exit errors
} # End of the "END" clause
