#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_menu_opt.cgi.sh 1.58 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2003,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)21	1.58 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_menu_opt.cgi.sh, hacmp.utils, 52haes_r541, 0747C_hacmp541 1/11/08 12:08:01

###############################################################################
#
# Name: wsm_menu_opt
#
# Description: Processes sm_menu_opt (basic smit menu odm) entries into html
#
# Arguments: name (id) of the menu to be processed.
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################




###############################################################################
#
# Name: log_write
#
# Description:  record user actions and system output to a logfile
#
# Arguments:    name (id) of the smit panel
#
# Returns:      none
#
# Environment:  Executed by the http server daemon (user=nobody)
#       Refers to global $REMOTE_ADDR
#
###############################################################################
function log_write 
{
    typeset ID=$1

    #
    # find the title of the current panel - no need for ILS here as it goes
    # to the log file
    #
    typeset STANZA=$(odmget -q "next_id = $ID" $ODM)
    typeset TITLE="${STANZA#* text = \"}"
    TITLE="${TITLE%%\"*}"
    if [[ $VERBOSE_LOGGING == "high" ]]; then
        {
            date +"[%b %d %Y, %T]"
            print "Menu screen selected:"
            print "  FastPath    = \"$ID\""
            print "  Title       = \"$TITLE\""
            print "  User IP     = \"$REMOTE_ADDR\""
            print ""
        } >>$LOGFILE
    fi
} # End of "log_write()"



###############################################################################
#
# Name: get_panel_info
#
# Description: gets the panel title and help msg info
#
# Arguments:    $NAME
#
# Returns:      None
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
function get_panel_info 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset ID=$1

    #
    # Get the default text for the menu opt that referred us here
    #
    REFERRER=$(odmget -q "next_id = $ID AND alias != y" sm_menu_opt | 
               grep "[[:space:]]id =" | cut -f2 -d\" | tail -1)
  
    #
    # Get the referer stanza
    #
    REFERRER_STANZA=$(odmget -q "id = $REFERRER AND next_id = $ID" sm_menu_opt)
  
    #
    # This is the name of the referer
    #
    # NOTE: do not use 'print "$REFERRER_STANZA"' here. See defect 597503.
    TITLE=$(odmget -q "id = $REFERRER AND next_id = $ID" sm_menu_opt |
            awk -f ${CGI_BIN}/wsm_awkscr_noparam V=text)

    #
    # Is there a message catalog specified?
    #
    typeset NAME_LOC="${REFERRER_STANZA#*text_msg_file = \"}"
    NAME_LOC="${NAME_LOC%%\"*}"

    if [[ -n $NAME_LOC ]]; then
        # get the message set
        typeset NAME_SET="${REFERRER_STANZA#*text_msg_set = }"
        NAME_SET="${NAME_SET%%[!0-9]*}"

        # get the message ID
        typeset NAME_ID="${REFERRER_STANZA#*text_msg_id = }"
        NAME_ID="${NAME_ID%%[!0-9]*}"

        TITLE=$(getmsg -s "$NAME_SET" "$NAME_LOC" "$NAME_ID" "$TITLE")
    fi

    #
    # is there help text specified ?
    #
    R_HELP_MSG_LOC="${REFERRER_STANZA#*help_msg_loc = \"}"
    R_HELP_MSG_LOC="${R_HELP_MSG_LOC%%\"*}"

    #
    # help_msg_id is either a number which points to aix help text or an
    # encoding of "msg set, msg id"
    #
    R_HELP_MSG_ID="${REFERRER_STANZA#*help_msg_id = \"}"
    R_HELP_MSG_ID="${R_HELP_MSG_ID%%\"*}"

    #
    # split into two values
    #
    if [[ -n $R_HELP_MSG_ID ]]
    then
        R_HELP_MSG_SET="${R_HELP_MSG_ID%%,*}"
        R_HELP_MSG_ID="${R_HELP_MSG_ID##*,}"
    fi
} # End of "get_panel_info()"



###############################################################################
#
# Name: make_menu
#
# Description: make a top level menu - lookup all the options in this
#   particular menu and build html links to them
#
# Arguments:    name (id) of the menu
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
function make_menu 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    ID=$1
    ODM=sm_menu_opt

    typeset HELP_IND=''
    typeset HELP_DIV=''
    typeset HELP_ERR=0

    if [ -z "$ALLOW_FILE" -o \
         ! -r "$ALLOW_FILE" ]
    then
        ALLOW_FILE="" # Make sure that awk doesn't try to process this, below
    fi

    if [ -z "$DENY_FILE" -o \
         ! -r "$DENY_FILE" ]
    then
        DENY_FILE="" # Make sure that awk doesn't try to process this, below
    fi

    #=========================================================
    # Process the ODM data for the current SMIT panel ($ID).
    # Perform all necessary translations, and convert the
    # the information into a web page.
    #=========================================================
    print -- "$MENU_STANZAS" | awk            \
        -v title="$TITLE"                     \
        -v allow_file="$ALLOW_FILE"           \
        -v deny_file="$DENY_FILE"             \
        -v VERBOSE_LOGGING="$VERBOSE_LOGGING" \
        -f "$BINDIR/wsm_menu_opt.awk"

    #=============================================================
    # Append the "Customize WebSMIT" panel, if the "Extended
    # Configuration" panel was just displayed. This is a
    # WebSMIT-only panel (i.e. it doesn't appear in ASCII SMIT).
    #=============================================================
    if [[ $ID == "cm_extended_config_menu_dmn" ]]
    then
        typeset CUSTOMIZE=$(getmsg -s 1 wsm_smit.cat 9999 "Customize WebSMIT")
        typeset CUSTOMIZE_HELP=$(getmsg -s 1 wsm_smit.cat 9999 "Launch the WebSMIT customization tool.")
        typeset CUSTOMIZE_HELP_HELP=$(getmsg -s 1 wsm_smit.cat 9999 "Get help for the 'Customize WebSMIT' operation." "Customize WebSMIT")
        print "
<div id='menuBox' class='act_smittree'>
<ul class='menu act_smittree'>
<li class='act_smittree'>
<font title=\"$CUSTOMIZE_HELP\"
      class='act_smittree act_smittree_link'
      onclick=\"JavaScript: parent.document.getElementById('ActSMIT_IFRAME').src = location.protocol + '//' + location.host + '/cgi-bin/wsm_customize.cgi';\"
      onMouseOver=\"JavaScript: this.className += ' highlight'; window.status=''; return true;\"
      onMouseOut=\"JavaScript: this.className = this.className.replace(/\\s?highlight/g, '');\"
>$CUSTOMIZE</font>

<span class='helpIndicator' title=\"$CUSTOMIZE_HELP_HELP\" onClick=\"JavaScript: displayHelpTip('wsm_help.cgi', 'cgi', 'wsm_smit.cat', '2', '2', '', 'Customize WebSMIT');\"> (?) </span></li>
</ul></div>
"
    fi

} # End of "make_menu()"



##########################
##########################
##
## Main starts here
##
##########################
##########################
PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ $VERBOSE_LOGGING == "high" ]] && set -x # Supports global/env specification
VERSION="1.32"

export WSM_VERSION=$VERSION
export WSM_PROGNAME=$PROGNAME

#
# Set the ODMDIR for reading SMIT stanzas
#
export ODMDIR=/usr/lib/objrepos

#
# Initialize
#
BINDIR=/usr/es/sbin/cluster/wsm/cgi-bin
CONFDIR=/usr/es/sbin/cluster/wsm
CGI_BIN=$BINDIR
CONF_FILE="$CONFDIR/wsm_smit.conf"
COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
export SKIN_LOGO="/skins/DEFAULT_logo.gif"
export LOGFILE='/usr/es/sbin/cluster/wsm/logs/wsm_smit.log'

#
# Run some access validations.  This script will also output
# the Content-type header so we don't have to do it here.
#
$BINDIR/wsm_auth.cgi
if (( $? != 0 )); then
    exit 0
fi

#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: there is no need to check for root ownership of the
#       WebSMIT configuration file here. It has already been
#       validated by the call to "wsm_auth.cgi", above.
#=============================================================
set -a
. $CONF_FILE >/dev/null
set +a

[[ $VERBOSE_LOGGING == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


#
# Clear the session file
#
$BINDIR/wsm_session -clear

#
# Get the name of the menu we are building
#
NAME=$(${CGI_BIN}/wsm_cgiparse -v name)


#==========================================================================
# In order to support the "hacmp" fastpath string, in the same manner as
# the command-line does (i.e. "smitty hacmp"), when it is detected, the
# appropriate internal name is used instead. Do *NOT* reset the variable
# to this internal name! User's don't like to see their input changed!
#==========================================================================
export ALLOW_LIST=""
export DENY_LIST=""
FILTER_OBJ=$NAME
if [[ $NAME == "hacmp" || $NAME == "HACMP" ]]; then
    FILTER_OBJ="cm_hacmp_main_menu_dmn"
fi
if [[ $VERBOSE_LOGGING == "high" ]]; then
    time . $CGI_BIN/wsm_filter "$FILTER_OBJ"
    # Label the output from "time", above, in the output log...
    print "DEBUG (PERF): wsm_menu_opt.cgi: wsm_filter \"$FILTER_OBJ\"" 1>&2
else
    . $CGI_BIN/wsm_filter "$FILTER_OBJ"
fi
if [[ $? != 0 ]]; then
    exit 0
fi

#==========================================================================
# In order to support the "hacmp" fastpath string, in the same manner as
# the command-line does (i.e. "smitty hacmp"), when it is detected, the
# appropriate internal name is used instead. Do *NOT* reset the variable
# to this internal name! User's don't like to see their input changed!
#==========================================================================
if [[ $NAME == "hacmp" || $NAME == "HACMP" ]]; then
    MENU_STANZAS=$(odmget -q "id = cm_hacmp_main_menu_dmn" sm_menu_opt)
else
    MENU_STANZAS=$(odmget -q "id = $NAME" sm_menu_opt)
fi

#
# Determine what type of ID we have been passed
#
if [[ -z $MENU_STANZAS ]]; then
    #
    # The panel is not a menu. Determine if we were
    # passed an ID for a different type of menu...
    #
    PANEL_IS_NAME=$(odmget -q "id = $NAME" sm_name_hdr)

    if [[ -z $PANEL_IS_NAME ]]; then
        #
        # The panel is not a name select. Determine
        # if we were passed an ID for a cmd_hdr...
        #
        PANEL_IS_CMD=$(odmget -q "id = $NAME" sm_cmd_hdr | grep 'has_name_select = "n*"')
        if [[ -n $PANEL_IS_CMD ]]; then
            #
            # The panel is a cmd hdr
            #
            print "Location: /cgi-bin/wsm_cmd_hdr.cgi?panel_id=$NAME&from=menu"
            print ""
            exit 0;
        fi
    else
        #
        # The panel is a name select
        #
        print "Location: /cgi-bin/wsm_name_hdr.cgi?name=$NAME&from=menu"
        print ""
        exit 0;
    fi
fi



#
# Make sure that all odmget calls refer only to menus
#
ODM=sm_menu_opt


#==========================================================================
# In order to support the "hacmp" fastpath string, in the same manner as
# the command-line does (i.e. "smitty hacmp"), when it is detected, the
# appropriate internal name is used instead. Do *NOT* reset the variable
# to this internal name! User's don't like to see their input changed!
#==========================================================================
if [[ $NAME == "hacmp" || $NAME == "HACMP" ]]; then
    get_panel_info cm_hacmp_main_menu_dmn
else
    get_panel_info $NAME
fi

#
# Print the HTML header
#
$CGI_BIN/wsm_html_lib body_start_menu   \
                      "$TITLE"          \
                      "$R_HELP_MSG_LOC" \
                      "$R_HELP_MSG_SET" \
                      "$R_HELP_MSG_ID"  \
                      act_smittree

#
# Construct and display the menu
#
if [[ $VERBOSE_LOGGING == "high" ]]; then
    time make_menu $NAME
    # Label the output from "time", above, in the output log...
    print "DEBUG (PERF): wsm_menu_opt.cgi: make_menu($NAME)" 1>&2
else
    make_menu $NAME
fi

#
# Print the HTML footer
#
$CGI_BIN/wsm_html_lib body_end_menu     \
                      "$ID"             \
                      "$R_HELP_MSG_LOC" \
                      "$R_HELP_MSG_SET" \
                      "$R_HELP_MSG_ID"  \
                      "$TITLE"          \
                      "act_smittree act_smittree_link"

log_write $NAME
