#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_name_hdr.cgi.sh 1.86 
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2003,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)22       1.61 src/43haes/usr/sbin/cluster/wsm/cgi-bin/wsm_name_hdr.cgi.sh, hacmp.wsm, 52haes_r541 7/18/06 07:29:13


###############################################################################
#
# Name: wsm_name_hdr.cgi
#
# Description: processes sm_name_hdr (smit odm) entries which are "name
#              selects". These are the popup menus from which the user
#              makes one or more selection - that selection is passed to
#              the downstream panel (command to execute, a dialog, or even
#              another name select).
#
# Arguments:
#
# Returns:     0 - success
#              1 - failure
#
# Environment: Executed by the http server daemon (user=nobody)
#
###############################################################################

###############################################################################
#
# Name: get_cmd_opt_name_ils
#
# Description: given an sm_cmd_opt id and sequence number, retrieve the name
#              (accounting for the message catalog entry)
#
# Arguments:   1: the ID to use
#              2: the pre-retrieved ODM stanza for the specified ID:
#                 odmget -q "id = $ID" sm_cmd_opt
#
# Returns:     text to use
#
# Environment: Executed by the http server daemon (user=nobody).
#
###############################################################################
function get_cmd_opt_name_ils 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset ID="$1"
    typeset OPT_STANZA="$2"

    if [[ -z $OPT_STANZA ]]
    then
        OPT_STANZA=$(odmget -q "id = $ID" sm_cmd_opt)
    fi

    # get the default text
    # NOTE: do not use 'print "$OPT_STANZA"' here.
    typeset TITLE=$(odmget -q "id = $ID" sm_cmd_opt |
                    awk -f ${CGI_BIN}/wsm_awkscr_noparam V=name)

    # is there a message catalog specified ?
    typeset NAME_LOC="${OPT_STANZA#*name_msg_file = \"}"
    NAME_LOC="${NAME_LOC%%\"*}"
    if [[ -n $NAME_LOC ]]; then

        # get the message set
        typeset NAME_SET="${OPT_STANZA#*name_msg_set = }"
        NAME_SET="${NAME_SET%%[!0-9]*}"

        # get the message id
        typeset NAME_ID="${OPT_STANZA#*name_msg_id = }"
        NAME_ID="${NAME_ID%%[!0-9]*}"

        TITLE=$(getmsg -s "$NAME_SET" "$NAME_LOC" "$NAME_ID" "$TITLE")
    fi

    print -- "$TITLE"
} # End of "get_cmd_opt_name_ils()"


###############################################################################
#
# Name: text_entry
#
# Description:
#       This adds a text, numeric or hex input field
#
#
# Arguments:
#
#       KEY   - The SEQ number associated with the option
#       NUM_TXT - The type of data ('num', 'hex', 'txt', 'RO')
#       REQUIRED - The 'required' field value
#       DISC     - The discovered value
#       TEXT     - The option text
#       LIST_ID  - The option ID
#       LIST_SEQ - The option SEQ number
#       OPT_STANZA - The SMIT stanza from the ODM (optional)
#
# Returns:
#
#       none
#
# Environment:
#
#       Executed by the http server daemon (user=nobody)
#
###############################################################################
function text_entry 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset KEY=$1            # this is the key returned with any text
    typeset NUM_TXT=$2        # type - num, hex or txt
    typeset REQUIRED=$3       # required field ?
    typeset DISC=$4           # is there a corresponding command to discover
    typeset TEXT=$5           # text for the field
    typeset LIST_ID=$6        # list function
    typeset LIST_SEQ=$7
    typeset OPT_STANZA=$8     # optional SMIT stanza from the ODM

    typeset HELP_IND=''
    typeset HELP_DIV=''
    typeset HELP_ERR=0

    typeset IT_BOLD="optional"  # Italic (optional) is default
    [[ $REQUIRED == "B" ]] && IT_BOLD="required"

    #
    # if the input supports a list option, build the command to list
    #
    if [[ "$OPTYPE" == "l"  ]]; then

        # build list button
        LIST_BUTTON="<input class='act_smittree act_smittree_button' type='button' value='  List  ' name='foo' onclick=\"JavaScript: openList('$LIST_ID','$LIST_SEQ', 0 , '$RAW_FIELD_NAME', '$RAW_FIELD_VALUE'); return false;\" title='Display the list of available selections.'>"
        LIST_JS=" list_id='$LIST_ID'; list_seq='$LIST_SEQ'; list_key='$FORM_ID_COUNTER'; list_raw_field_name='$RAW_FIELD_NAME'; list_raw_field='$RAW_FIELD_VALUE';"

    else
        LIST_BUTTON=""
        LIST_JS=" list_id=''; list_seq=''; list_key=''; list_raw_field_name='';list_raw_field='';"
    fi  # done building list button

    #
    # get maximum entry length
    #
    LENGTH=$ENTRY_SIZE

    #
    # Begin an HTML table and add the Java to display help.
    # Watch out for deliberately blank entries, and honor
    # the SMIT panel designer's original layout.
    #
    if [ -n "$TEXT" ]; then

        if [[ -z $OPT_STANZA ]]; then
            OPT_STANZA=$(odmget -q "id = $OPT_ID" sm_cmd_opt 2>/dev/null);
        fi

        typeset HELP_MSG_LOC="${OPT_STANZA#*help_msg_loc = \"}"
        HELP_MSG_LOC="${HELP_MSG_LOC%%\"*}"

        typeset HELP_MSG_ID="${OPT_STANZA#*help_msg_id = \"}"
        HELP_MSG_ID="${HELP_MSG_ID%%\"*}"

        #
        # split into two values
        #
        typeset HELP_MSG_SET
        if [[ -n $HELP_MSG_ID ]]
        then
            HELP_MSG_SET="${HELP_MSG_ID%%,*}"
            HELP_MSG_ID="${HELP_MSG_ID##*,}"
        fi

        #====================================================================
        # If necessary, make special allowances for multi-line name headers
        #====================================================================
        typeset LINES=$(echo "$TEXT" | wc -l)
        typeset MAIN="$TEXT"
        typeset HEAD="$TEXT"
        if (( LINES > 1 ))
        then
            HEAD=$(echo "$TEXT" | head -1)
            TEXT="${TEXT##$HEAD}"
            if [[ -n $TEXT ]]; then
                TEXT=$(echo "$TEXT" | sed 's/\\//g')
                MAIN="$HEAD<br>$TEXT"
            fi
        fi

        print "\
<center>
<table>
<tr>"
        if [[ -z $HELP_MSG_LOC || -z $HELP_MSG_SET || -z $HELP_MSG_ID ]]; then
            print "\
<td class='$IT_BOLD' align='center' valign='middle'>
<label for='0'>$MAIN</label>
</td>

<td class='$IT_BOLD' align='left' valign='middle'>
</td>"
        else  # Help is available
            print "\
<td class='$IT_BOLD' align='center' valign='middle'>
<span class='helpWrapper' onClick=\"JavaScript: displayHelpTip('wsm_help.cgi', 'cgi', '$HELP_MSG_LOC', '$HELP_MSG_SET', '$HELP_MSG_ID', '', '$HEAD');\">
<label class='helpWrapper' for='0'>$MAIN</label>
<span class='helpIndicator'>(?)</span>
</span>
</td>"
        fi

        print "
<td align='left' valign='middle'>"
    else
        print "
<!-- Spacer -->
<tr>
<td>&nbsp;</td>
<td>&nbsp;</td>
</tr>
</table>
</center>"
        return
    fi

    #
    # Setup some of the attributes of this entry item
    #
    typeset -i MAX_TEXT_FIELD_SIZE=40
    typeset -i MAX_NUM_FIELD_SIZE=9

    VALTYPE="valType=\"$NUM_TXT\""

    # Determine the size of the input box.
    if (( LENGTH == 0 || LENGTH > MAX_TEXT_FIELD_SIZE )) ; then
        SIZE=$MAX_TEXT_FIELD_SIZE
    else
        SIZE=$LENGTH
    fi

    # A zero entry_size means allow the maximum.  Non-zero means that is the
    # max size of the allowed input.
    typeset MAX_LENGTH_STR=""
    if (( LENGTH > 0 )) ; then
        MAX_LENGTH_STR="maxlength=\"$LENGTH\""
    fi

    READONLY=""
    case "$NUM_TXT" in
    RO)
        READONLY="readonly"
        VALTYPE=""
        ;;
    num|hex)
        ((SIZE>MAX_NUM_FIELD_SIZE)) && SIZE=$MAX_NUM_FIELD_SIZE
        ;;
    esac

    DISCVALUE=""
    if [ -n "$DISC" ]; then
        DISCVALUE="VALUE=\"$DISC\""
    fi

    print "&nbsp;<input class='act_smittree' type=text checked=1 name=\"$KEY\" id='0' $VALTYPE valLength='$LENGTH' valReq=\"$REQUIRED\" $MAX_LENGTH_STR size=$SIZE $DISCVALUE onfocus=\"focus_help_loc='$HELP_MSG_LOC'; focus_help_set='$HELP_MSG_SET'; focus_help_id='$HELP_MSG_ID';$LIST_JS\" $READONLY>
$LIST_BUTTON"

    # close the table
    print "\
</td>
</tr>
</table>
</center>"

} # End of "text_entry()"


###############################################################################
#
# Name: get_name_hdr_name_ils
#
# Description:  given an sm_name_hdr id, retrieve the name (accounting for
#               the message catalog entry)
#
# Arguments:    ID of the entry
#               the pre-retrieved ODM entry for the ID (optional)
#
# Returns:      text to use
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
function get_name_hdr_name_ils 
{
    [[ $VERBOSE_LOGGING == "high" ]] && set -x

    typeset ID="$1"
    typeset ID_STANZA="$2"

    if [[ -z $ID_STANZA ]]
    then
        ID_STANZA=$(odmget -q "id = $ID" sm_name_hdr)
    fi

    # get the default text
    # NOTE: do not use 'print "$ID_STANZA"' here.
    typeset TITLE=$(odmget -q "id = $ID" sm_name_hdr |
                    awk -f ${CGI_BIN}/wsm_awkscr_noparam V=name)

    # is there a message catalog specified ?
    NAME_LOC="${ID_STANZA#*name_msg_file = \"}"
    NAME_LOC="${NAME_LOC%%\"*}"

    if [[ -n $NAME_LOC ]]; then

        # get the message set
        typeset NAME_SET="${ID_STANZA#*name_msg_set = }"
        NAME_SET="${NAME_SET%%[!0-9]*}"

        # get the message id
        typeset NAME_ID="${ID_STANZA#*name_msg_id = }"
        NAME_ID="${NAME_ID%%[!0-9]*}"

        TITLE=$(getmsg -s "$NAME_SET" "$NAME_LOC" "$NAME_ID" "$TITLE")
    fi

    print -- "$TITLE"
} # End of "get_name_hdr_name_ils()"


###############################################################################
#
# Name: log_write
#
# Description:  record user actions and system output to a logfile
#
# Arguments:    Name (id) of the panel
#		Title (text)
#		Next id (where to go to next)
#
# Returns:      0 - success
#               1 - failure
#
# Environment:  Executed by the http server daemon (user=nobody)
#
###############################################################################
function log_write 
{
    ID=$1
    TITLE=$2
    NEXT_ID=$3

    if [[ $VERBOSE_LOGGING == "high" ]]; then
        {
            date +"[%b %d %Y, %T]"
            print "Selector screen selected:"
            print "	FastPath    = \"$ID\""
            print "	Title       = \"$TITLE\""
            print "	next_id     = \"$NEXT_ID\""
            print "	User IP     = \"$REMOTE_ADDR\""
            print ""
        } >>$LOGFILE
    fi
}



####################
# Main starts here #
####################

typeset PROGNAME=$(basename ${0})
export PS4='${PROGNAME}[${LINENO}] '
[[ "$VERBOSE_LOGGING" == "high" ]] && set -x # Supports global/env specification
VERSION="1.42"

export WSM_PROGNAME=$PROGNAME
export WSM_VERSION=$VERSION

# set the ODMDIR for reading smit stanzas
export ODMDIR=/usr/lib/objrepos

BINDIR=/usr/es/sbin/cluster/wsm/cgi-bin
CONFDIR=/usr/es/sbin/cluster/wsm
CGI_BIN=$BINDIR
CLASSIFY_OUT=""
TYPE="j"        # default
CMD_TEXT=""
LIST_CMD_STR=""

# Run some access validations.  This script will also output the
# Content-type header so we don't have to do it here.
$BINDIR/wsm_auth.cgi
if (( $? != 0 )); then
    exit 0
fi


# Set up the WebSMIT environment variables
WORKDIR=/tmp
export SKIN_LOGO="/skins/DEFAULT_logo.gif"

#=============================================================
# Set up the WebSMIT environment
# ------------------------------
# NOTE: there is no need to check for root ownership of the
#       WebSMIT configuration file here. It has already been
#       validated by the call to "wsm_auth.cgi", above.
#=============================================================
CONF_FILE="$CONFDIR/wsm_smit.conf"
LOGFILE='/usr/es/sbin/cluster/wsm/logs/wsm_smit.log'
set -a
. $CONF_FILE >/dev/null
set +a

[[ "$VERBOSE_LOGGING" == "high" ]] && set -x


#=========================================================================
# Load the common WebSMIT functions.
#
# NOTE: make sure the file being sourced is owned by root and only the
#       owner has write permissions.  Being writable by any other person
#       could allow them to run arbitrary commands as root.
#=========================================================================
typeset COMMON_LIB="/usr/es/sbin/cluster/wsm/cgi-bin/wsm_common"
typeset LOGFILE="/usr/es/sbin/cluster/wsm/logs/wsm_smit.log"
if [[ -n "$(find $COMMON_LIB ! -user root -o -perm -0020 -o -perm -0002)" ]]
then
    print "ERROR: invalid permissions for $COMMON_LIB. " \
          "This file must only be writable by root." >>$LOGFILE 2>&1
    exit 1
else
    . $COMMON_LIB >>$LOGFILE 2>&1
    set_wsm_locale >>$LOGFILE 2>&1
fi


#===================================================================
# Check for any message catalog data from the originating/parent
# menu item. If there is any, it needs to be retrieved here, before
# the following call to "wsm_cgiread" overwrites QUERY_STRING with
# the posted form data.
#===================================================================
typeset PARENT_HELP_MSG_LOC=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_loc)
typeset PARENT_HELP_MSG_SET=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_set)
if [[ $PARENT_HELP_MSG_SET == *,* ]]; then
    PARENT_HELP_MSG_SET=${PARENT_HELP_MSG_SET%%,*}
fi
typeset PARENT_HELP_MSG_ID=$(${CGI_BIN}/wsm_cgiparse  -v parent_help_msg_id)
if [[ $PARENT_HELP_MSG_ID == *,* ]]; then
    PARENT_HELP_MSG_ID=${PARENT_HELP_MSG_ID##*,}
fi


# Read in the CGI form data so that we can parse it later with wsm_cgiparse.
# We overwrite the QUERY_STRING environment variable with the output of
# wsm_cgiread.  wsm_cgiread will output either the original QUERY_STRING
# value or the data that was read from stdin during a POST request.
export QUERY_STRING=$(${CGI_BIN}/wsm_cgiread)


#=========================================================================
# Loop through each consecutive call to this script, "wsm_name_hdr.cgi",
# processing the chain of ghost name selects (if any) until a definition
# is encounter that requires something to be displayed to the user. At
# that point, the loop will exit, and the appropriate display processing
# will be performed, either by this script, or by "wsm_cmd_hdr.cgi".
#=========================================================================
typeset CONSTRUCT=
while [[ -z $CONSTRUCT ]]; do

    #============================================
    # Get the name of the menu we are building.
    # If we're coming from wsm_name_hdr.cgi,
    # the basename will be in "panel_id".
    #============================================
    BASENAME=$(${CGI_BIN}/wsm_cgiparse -v panel_id)
    if [[ -z $BASENAME ]]; then
        # if we came from wsm_menu_opt, the basename will be in "name"
        BASENAME=$(${CGI_BIN}/wsm_cgiparse -v name)
    fi

    # Was this panel reached via a fastpath?
    FASTPATH=$(${CGI_BIN}/wsm_cgiparse -v fastpath)


    #=========================================================================
    # cmd_to_classify
    # ---------------
    # Always pass the referring ID. The "_rawvalue" supplied by the user is
    # used as input to the cmd_to_classify, so it can't be run until we have
    # the user's input.  What we do with that output depends on the type of
    # the referring panel.
    #=========================================================================
    CMD_TO_CLASSIFY_ID=$(${CGI_BIN}/wsm_cgiparse -v cmd_to_classify)


    #====================================================
    # Lookup any cmd_opts.
    # NOTE: this is required for validate_arg(), below.
    #====================================================
    STANZA=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr 2>/dev/null)
    OPT_ID="${STANZA#*option_id = \"}"
    OPT_ID="${OPT_ID%%\"*}"

    # See if this link was generated by a status module
    CLSTAT_ID=$(${CGI_BIN}/wsm_cgiparse -v clstat_id)
    TTIP_ID=$(${CGI_BIN}/wsm_cgiparse -v tt_id)

    #==========================================================================
    # We've arrived here via a generated link. Extract the faked selector field
    # value, if any, from the session. Otherwise, it comes from CGI info.
    #==========================================================================
    if [[ -n $CLSTAT_ID ]]; then

        #============================================
        # Process the colon-delimited values stored
        # to the session by the status modules.
        #============================================
        VALUE_LIST=$(${CGI_BIN}/wsm_session \
                                   -d clstat_id_${CLSTAT_ID}_${BASENAME} \
                                   -g value_list)
        IFS=:
        for VAR in $VALUE_LIST; do
            unset IFS
            VAL=$(${CGI_BIN}/wsm_session -d clstat_id_${CLSTAT_ID}_${BASENAME} -g $VAR)
            if [[ -n $VAL ]]; then
                ${CGI_BIN}/wsm_session -d valueset -s "$VAR"="$VAL"
                printf "%b" \
                    "${CGI_BIN}/wsm_session -d valueset " \
                    "-s \"$VAR\"=\"$VAL\"\n" >>$LOGFILE
            fi
        done

    elif [[ -n $CMD_TO_CLASSIFY_ID ]]; then

        #===================================================================
        # If we reached this panel via a fastpath, then clear the session.
        # A new SMIT path is being started, so there should be nothing in
        # the session left over from previous panels. Do this here, before
        # we add anything to the session.
        #===================================================================
        if [[ $FASTPATH == true ]]; then
            ${CGI_BIN}/wsm_session -clear
        fi

        #======================================================================
        # If we're coming from a name select, the cmd_to_classify ID *MUST* be
        # present. We'll need the raw field data. Validate and save incoming
        # raw values in the session file. If there is no raw field value, don't
        # overwrite the potentially already present value in the session.
        #======================================================================
        RAW_FIELD_NAME=$(${CGI_BIN}/wsm_cgiparse -v raw_field_name)
        RAW_FIELD_NAME=${RAW_FIELD_NAME:=_rawname}
        RAW_FIELD_VALUE=$(${CGI_BIN}/wsm_cgiparse -v "$RAW_FIELD_NAME")

        #======================================================================
        # Extract from the ODM the stanza the value will be validated against
        #======================================================================
        typeset V_STANZA="$(odmget -q "id =$OPT_ID AND id_seq_num = 0" sm_cmd_opt)"

        #====================================================================
        # If the link was generated by a tooltip from the navigation frame,
        # then we need to verify the values that were provided in the CGI
        # request. The argument will be validated with a call to
        # 'validate_arg' in the next block, so at this point, we need to
        # add the appropriate values *TO* the session.
        #====================================================================
        if [[ -n $TTIP_ID ]]; then
            proc_tooltip_values $BASENAME $OPT_ID
        fi


        #===========================================================
        # Validate the value supplied by the client. We'll only be
        # coming from a name_hdr, so the cmd_opt ID will always be
        # '0'. If the validation fails, report the error and quit.
        #===========================================================
        if ! validate_arg "$V_STANZA" "0" "$RAW_FIELD_VALUE" "$CMD_TO_CLASSIFY_ID"
        then
            # report error to user and exit
            TITLE=$(get_name_hdr_name_ils $BASENAME)
            typeset MSG=$(getmsg wsm_smit.cat 80 "There was a problem with your request.  Please try again.")
            $CGI_BIN/wsm_html_lib body_start_name "$TITLE"
            print "\
<div class='errorBox'>
<i>$MSG</i>
<div class='formButtons'>
<input class='act_smittree act_smittree_button' TYPE='button' value='Back' onclick='history.go(-1)'>
</div>
</div>"
            $CGI_BIN/wsm_html_lib body_end_name "$ID" "$TITLE" "act_smittree act_smittree_link"
            return 0;
        else
            # The argument was valid. Save it to the session.
            if [[ -n "$RAW_FIELD_VALUE" ]]; then
                $CGI_BIN/wsm_session -d valueset -s "$RAW_FIELD_NAME"="$RAW_FIELD_VALUE"
                print -n "wsm_name_hdr.cgi:">>$LOGFILE
                print "${CGI_BIN%/}/wsm_session -d valueset -s \"$RAW_FIELD_NAME\"=\"$RAW_FIELD_VALUE\"" >>$LOGFILE
            fi
        fi


        #===================================================================
        # Value passed is the panel id; harvest the classify command text.
        # NOTE: do not use 'print "$C2C_STANZA"' here.
        #===================================================================
        typeset C2C_STANZA=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr)
        CMD_TEXT=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr |
                   awk -f ${CGI_BIN}/wsm_awkscr_noparam V=cmd_to_classify)

        #===================================================================
        # Harvest the cooked field name, using the default if not defined.
        # NOTE: do not use 'print "$C2C_STANZA"' here.
        #===================================================================
        COOKED_FIELD_NAME=$(odmget -q "id = $CMD_TO_CLASSIFY_ID" sm_name_hdr |
                   awk -f ${CGI_BIN}/wsm_awkscr_noparam V=cooked_field_name)
        [[ -z $COOKED_FIELD_NAME ]] && COOKED_FIELD_NAME=_cookedname

        #=======================================================
        # If the type (of the caller) is "c", then the command
        # to classify will be used to generate the next panels
        # ID, by combining it with the base name.
        #=======================================================
        CMD_TO_CLASSIFY_TYPE="${C2C_STANZA#*	type = \"}" # NOTE: must contain a TAB char.
        CMD_TO_CLASSIFY_TYPE="${CMD_TO_CLASSIFY_TYPE%%\"*}"
        [[ -z $CMD_TO_CLASSIFY_TYPE ]] && CMD_TO_CLASSIFY_TYPE="j"

    else
        #===================================================================
        # If we reached this panel via a fastpath, then clear the session.
        # A new SMIT path is being started, so there should be nothing in
        # the session left over from previous panels. Do this here, before
        # adding anything new to the session.
        #===================================================================
        if [[ $FASTPATH == true ]]; then
            ${CGI_BIN}/wsm_session -clear
        fi
    fi

    #================================================
    # If there is a command to classify, execute it
    #================================================
    if [[ -n $CMD_TEXT ]]; then

        CMD_FILE="${WORKDIR}/${CMD_TO_CLASSIFY_ID}.CLASSIFY_COMMAND.$$"
        CLASSIFY_OUT=$(${BINDIR}/wsm_cmd_exec -f $CMD_FILE -i $CMD_TO_CLASSIFY_ID -t cmd_to_classify 2>&1)
        RC=$?
    
        if (( RC == 0 )); then
            {
                print "
Output from Command_to_Classify follows:

$CLASSIFY_OUT
"
            } >>$LOGFILE
        else  # log the failure
            {
                print "
1800-081 Error exit status ($RC) returned by
Command_to_Classify; the output is:

$CLASSIFY_OUT
"
            } >>$LOGFILE

            #============================================
            # Report this error to the user, then exit.
            #============================================
            TITLE=$(get_name_hdr_name_ils $BASENAME)
            $CGI_BIN/wsm_html_lib body_start_name "$TITLE"
            print "\
<div class='errorBox'>
1800-081 An error occurred:
<pre>
$CLASSIFY_OUT
</pre>
<div class='formButtons'><input class='act_smittree act_smittree_button' TYPE='button' value='Back' onclick='history.go(-1)'></div>
</div>"
            $CGI_BIN/wsm_html_lib body_end_name "$ID" "$TITLE" "act_smittree act_smittree_link"
            return 0;
        fi
    fi

    #====================================================================
    # Process the values from the classify command, placing them in the
    # session. Loop through and match fields to values in the classify
    # command output.
    #====================================================================
    proc_new_valueset "$COOKED_FIELD_NAME" "$CLASSIFY_OUT"


    #==========================================================================
    # By default, the panel id is just the basename - it can be modified below
    #==========================================================================
    ID=$BASENAME

    if [[ $CMD_TO_CLASSIFY_TYPE == "r" ]]; then
        ID=${BASENAME}${RAW_FIELD_VALUE}

    #========================================================================
    # If the type of the caller was "c", build the ID of our panel from the
    # concatenation of the ID and the output from the command to classify.
    # If multiple values were generated, use the first.
    #========================================================================
    elif [[ $CMD_TO_CLASSIFY_TYPE == "c" ]]; then
        #
        # If the block above did not generate output from the classify
        # command, use the _previous_ panel's command to classify
        #
        [[ -z $CLASSIFY_OUT ]] &&
        CLASSIFY_OUT=$(${CGI_BIN}/wsm_cgiparse -v wsm_classify_out)

        ID=${BASENAME}${CLASSIFY_OUT%%:*}
    fi


    #=========================================
    # Check the panel id against the filters
    #=========================================
    $CGI_BIN/wsm_filter "$ID"
    if [[ $? != 0 ]]; then
        exit 0
    fi

    ID_STANZA=$(odmget -q "id = $ID" sm_name_hdr)

    # Get the text to display
    TITLE=$(get_name_hdr_name_ils "$ID" "$ID_STANZA")

    # the option_id points to a cmd_opt which may have more options
    OPT_ID="${ID_STANZA#*option_id = \"}"
    OPT_ID="${OPT_ID%%\"*}"

    # if the name_hdr did not have a title, seek one out from the cmd_opt
    if [[ -z "$TITLE" && -n $OPT_ID ]]; then
        TITLE=$(odmget -q "id = $OPT_ID" sm_cmd_opt | 
    	awk -f ${CGI_BIN}/wsm_awkscr_noparam V=name)
        # add ILS lookup of title (from cmd_opt) here
    fi

    # the id of the next panel to logically proceed to after this panel
    NEXT_ID="${ID_STANZA#*next_id = \"}"
    NEXT_ID="${NEXT_ID%%\"*}"

    # the type of this name-hdr
    TYPE="${ID_STANZA#*	type = \"}"  # NOTE: must be a TAB character before the "type" string!!
    TYPE="${TYPE%%\"*}"
    TYPE=${TYPE:-j}  # default

    # the next type - the type of the panel to go to  - can be another name 
    # selector or a dialog
    NTYPE="${ID_STANZA#*next_type = \"}"
    NTYPE="${NTYPE%%\"*}"
    NTYPE=${NTYPE:-d}  # "d" is the default

    # for name selects, the ghost setting comes from name_hdr
    GHOST="${ID_STANZA#*ghost = \"}"
    GHOST="${GHOST%%\"*}"
    GHOST=${GHOST:-n}  # n is default

    # the cmd_to_list
    LIST_CMD_STR=$(get_stanza_field $OPT_ID cmd_to_list)

    if [[ $GHOST == "y" && -z $LIST_CMD_STR ]]; then

        #================================================================
        # Retrieve any message catalog data, used for retrieving a help
        # message, that may have been passed in from the originating,
        # parent panel. Be sure to pass it on to the next CGI script
        # that gets called from here, as needed!
        #================================================================
        typeset PARENT_HELP_MSG_LOC=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_loc)
        typeset PARENT_HELP_MSG_SET=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_set)
        if [[ $PARENT_HELP_MSG_SET == *,* ]]; then
            PARENT_HELP_MSG_SET=${PARENT_HELP_MSG_SET%%,*}
        fi
        typeset PARENT_HELP_MSG_ID=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_id)
        if [[ $PARENT_HELP_MSG_ID == *,* ]]; then
            PARENT_HELP_MSG_ID=${PARENT_HELP_MSG_ID##*,}
        fi
        export QUERY_STRING="panel_id=$NEXT_ID&cmd_to_classify=$ID&parent_help_msg_loc=$PARENT_HELP_MSG_LOC&parent_help_msg_set=$PARENT_HELP_MSG_SET&parent_help_msg_id=$PARENT_HELP_MSG_ID"

        if [[ $NTYPE == "d" ]]; then
            print -- "$QUERY_STRING" | ./wsm_cmd_hdr.cgi
            exit 0
        fi
    else
        CONSTRUCT=true
    fi
done


#
# Construct the name select dialog
#

# get the raw field name; the default is "_rawname"
typeset RAW_FIELD_NAME="${ID_STANZA#*raw_field_name = \"}"
RAW_FIELD_NAME="${RAW_FIELD_NAME%%\"*}"
[[ -z $RAW_FIELD_NAME ]] && RAW_FIELD_NAME="_rawname"


#
# cmd_opt multi select field.
#
# Defines if the user can make multiple selections from a list of valid
# values produced by the command in the cmd_to_list field of the dialog
# object. The value of multi_select is a string. Valid values are:
# "" - No; a user can select only one value from a list. This is the default
# case.
# "," - Yes; a user can select multiple items from the list. When the
# command is built, a comma is inserted between each item.
# "y" - Yes; a user can select multiple values from the list. When the
# command is built, the option prefix is inserted once before the string of
# selected items.
# "m" - Yes; a user can select multiple items from the list. When the
# command is built, the option prefix is inserted before each selected item.
#
INPUT_TYPE=checkbox 	# default
SEP=" "
CHECKED=""              # list elements unchecked by default
typeset OPT_STANZA=$(odmget -q "id = $OPT_ID" sm_cmd_opt 2>/dev/null);
typeset MULTI="${OPT_STANZA#*multi_select = \"}"
MULTI="${MULTI%%\"*}"
case "$MULTI" in
    "," )
        MULTI=c
        SEP=","
        ;;
    "y" | "m" )		# y and m are functionally equivalent
        MULTI=y
        ;;
     * )
        MULTI=n
    	INPUT_TYPE=radio # only one selection allowed
        CHECKED=checked  # the first radio option will be selected by default.
        ;;
esac

#
# Retrieve additional fields for this selector
#
typeset ENTRY_TYPE="${OPT_STANZA#*entry_type = \"}"
ENTRY_TYPE="${ENTRY_TYPE%%\"*}"
[[ -z "$ENTRY_TYPE" ]] && ENTRY_TYPE="n"

typeset REQ="${OPT_STANZA#*required = \"}"
REQ="${REQ%%\"*}"

typeset OPTYPE="${OPT_STANZA#*op_type = \"}"
OPTYPE="${OPTYPE%%\"*}"

typeset ENTRY_SIZE="${OPT_STANZA#*entry_size = }"
ENTRY_SIZE="${ENTRY_SIZE%%[!0-9]*}"

TEXT=$(get_cmd_opt_name_ils "$OPT_ID" "$OPT_STANZA")

# 
# If this selector does not support editing and we found a list command, run it
# If this selector is editable, there will be a list button for the user ...
# 
if [[ ( $ENTRY_TYPE == "n" || $GHOST == "y" ) && -n $LIST_CMD_STR ]]; then
    LIST=$(get_list $ID $OPT_ID)
    ERR=$?
    if [[ $ERR != 0 ]]; then
        # report error to user and exit
        $CGI_BIN/wsm_html_lib body_start_name "$TITLE"
        print "<div class=\"errorBox\">"
        print "1800-050 An error occurred:"
        print "<pre>"
        print -- "$LIST"
        print "</pre>"
        print "<div class='formButtons'><input class='act_smittree act_smittree_button' TYPE='button' value='Back' onclick='history.go(-1)'></div>"
        print "</div>"
        $CGI_BIN/wsm_html_lib body_end_name "$ID" "$TITLE" "act_smittree act_smittree_link"
        log_write "$ID" "$TITLE" "$NEXT_ID"
        return $ERR
    fi

    #=================================================================
    # Sometimes get_list returns no list, but provides a reason why
    # via STDERR. That function deposits any STDERR in a file, which
    # we load here. It's important that the file name/path used here
    # exactly matches that used in the get_list().
    #=================================================================
    typeset LIST_CMD_FILE="${WORKDIR}/${ID}.default.LIST_COMMAND.$$"
    GET_LIST_STDERR=
    if [[ -e "$LIST_CMD_FILE.stderr" ]]; then
        GET_LIST_STDERR=$(cat "$LIST_CMD_FILE.stderr")
        rm -f "$LIST_CMD_FILE.stderr"
    fi

fi # if non-editable and cmd str exists

#============================================================
# If there was no locally (i.e. in the current ODM stanza)
# message catalog data provided for this panel, but there
# *was* some message catalog data passed in from the parent
# panel, then use the parent information.
#============================================================
if [[ -z $MAIN_HELP_MSG_LOC     || \
      -z $MAIN_HELP_MSG_SET     || \
      -z $MAIN_HELP_MSG_ID      || \
      $MAIN_HELP_MSG_SET == "0" || \
      $MAIN_HELP_MSG_ID  == "0" ]]
then
    if [[ -n $PARENT_HELP_MSG_LOC && \
          -n $PARENT_HELP_MSG_SET && \
          -n $PARENT_HELP_MSG_ID ]]
    then
        MAIN_HELP_MSG_LOC=$PARENT_HELP_MSG_LOC
        MAIN_HELP_MSG_SET=$PARENT_HELP_MSG_SET
        MAIN_HELP_MSG_ID=$PARENT_HELP_MSG_ID
    fi
fi


#
# Begin the html
#
$CGI_BIN/wsm_html_lib body_start_name      \
                      "$TITLE"             \
                      act_smittree         \
                      "$MAIN_HELP_MSG_LOC" \
                      "$MAIN_HELP_MSG_SET" \
                      "$MAIN_HELP_MSG_ID"

print "<h1>$TITLE</h1>"

# What we do next depends on the next thing to go to 
if [[ $NTYPE == "d" ]]; then
    # this name selector leads to a dialogue - build a form and present the
    # selections to the user
    NEXT_SCRIPT="/cgi-bin/wsm_cmd_hdr.cgi?from=name_selector"
else
    # passSelect() will combine multiple selection, separating them with SEP.
    NEXT_SCRIPT="/cgi-bin/wsm_name_hdr.cgi"
fi

#================================================================
# Retrieve any message catalog data, used for retrieving a help
# message, that may have been passed in from the originating,
# parent panel. Be sure to pass it on to the next CGI script
# that gets called from here, as needed!
#================================================================
typeset PARENT_HELP_MSG_LOC=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_loc)
typeset PARENT_HELP_MSG_SET=$(${CGI_BIN}/wsm_cgiparse -v parent_help_msg_set)
if [[ $PARENT_HELP_MSG_SET == *,* ]]; then
    PARENT_HELP_MSG_SET=${PARENT_HELP_MSG_SET%%,*}
fi
typeset PARENT_HELP_MSG_ID=$(${CGI_BIN}/wsm_cgiparse  -v parent_help_msg_id)
if [[ $PARENT_HELP_MSG_ID == *,* ]]; then
    PARENT_HELP_MSG_ID=${PARENT_HELP_MSG_ID##*,}
fi
if [[ $NEXT_SCRIPT != *\?* ]]; then
    NEXT_SCRIPT="$NEXT_SCRIPT?"
else
    NEXT_SCRIPT="$NEXT_SCRIPT&"
fi
NEXT_SCRIPT="${NEXT_SCRIPT}parent_help_msg_loc=$PARENT_HELP_MSG_LOC&parent_help_msg_set=$PARENT_HELP_MSG_SET&parent_help_msg_id=$PARENT_HELP_MSG_ID"

if [[ $ENTRY_TYPE == 'n' || $GHOST == 'y' ]]; then
    ONSUBMIT="return passSelect('$RAW_FIELD_NAME', '$SEP')"
else
    ONSUBMIT="return prepareForm();"
fi

# We always replace the browser's current history location because otherwise
# the back button will ask the user to reload the page, resubmit form data,
# etc...
print "<form method=post action=\"$NEXT_SCRIPT\" onSubmit=\"$ONSUBMIT\">"

#
# Store some critical data in hidden variables, so
# that they will be passed to the next page load.
#
print "<input type=hidden NAME='panel_id' value=\"$NEXT_ID\">
<input type='hidden' name='raw_field_name' value=\"$RAW_FIELD_NAME\">
<input type='hidden' name='parent_help_msg_loc' value=\"$PARENT_HELP_MSG_LOC\">
<input type='hidden' name='parent_help_msg_set' value=\"$PARENT_HELP_MSG_SET\">
<input type='hidden' name='parent_help_msg_id' value=\"$PARENT_HELP_MSG_ID\">"

#
# Prevent this from geting duplicated in radio button "name" fields.
#
if [[ "$INPUT_TYPE" != "radio" ]]; then
    print "<input type=\"hidden\" name=\"$RAW_FIELD_NAME\" value=\"\">"
fi

#
# Give the next panel our ID so it can run the command to classify based on
# user input.
#
print "<input type=\"hidden\" name=\"cmd_to_classify\" value=\"$ID\">"


# add a table with all the options for this selector
# if it is a multi select list, generate a unique key for each (possible) 
# selection - this key will be decoded in cmd hdr/build cmd
typeset -i multi_index=0
[[ $MULTI != "n" ]] && RAW_FIELD_NAME="wsm_multi_opt_"$multi_index

# the cmd_to_list_mode tells us which field from the output to use
typeset MODE="${OPT_STANZA#*cmd_to_list_mode = \"}"
MODE="${MODE%%\"*}"

#=============================================================
# Set a variable to indicate if this is a type of panel that
# we can do something extra with, in terms of displaying
# useful/helpful additional information to the user.
#=============================================================
typeset RESOURCE_TYPE='default'
if [[ $OPT_ID == *_snap* ]]; then
    RESOURCE_TYPE='snapshot'
elif [[ $OPT_ID == *@(resource_group|resgrp)* && \
        $OPT_ID != *select_node* ]]
then
    RESOURCE_TYPE='resource_group'
fi


################################################
################################################
##
## Load the appropriate functions module
##
################################################
################################################
if [[ -r "$CGI_BIN/mod.$RESOURCE_TYPE" ]]; then
    . "$CGI_BIN/mod.$RESOURCE_TYPE"
fi


#
# If this is non-editable, parse output of list command into a list of options
#
LIST_CNT=0
if [[ $ENTRY_TYPE == "n" || $GHOST == "y" ]]; then

    print "<div id=\"nameBox\">"
    print "<table border=0 cellspacing=4>"

    typeset PRINTED_OPTIONS=""
    typeset -i COUNT=0
    print -- "$LIST" | while read STRING
    do
        if [[ $MODE == @(1|2) ]]; then
            # strip most of the formatting to make cutting the fields easier
            RAW_STRING=$(echo "$STRING" | tr "\t" " " | tr -s " " | cut -f${MODE} -d" ")
        else
            RAW_STRING=$STRING
        fi

        #=====================================================
        # Some list commands output an informational header.
        # These lines have '#' as the first char. If this is
        # such a line, format it differently than a link.
        #=====================================================
        if [[ $STRING == \#* ]]; then
            STRING="${STRING###}"
            STRING="${STRING## }"
            if [[ -n $STRING ]]
            then
                if [[ $RESOURCE_TYPE == 'default' ]]
                then
                    # Replace tabs with eight spaces
                    typeset DISPLAY_STRING="${STRING//	/        }"

                    # Account for the removed leading "# " if columns are
                    # in use, and these are column headers being displayed.
                    # Also account for the space consumed by the leading
                    # bullets (i.e. radio buttons, checkboxes) that will
                    # be displayed in front of the following selections.
                    if [[ $DISPLAY_STRING == *[[:space:]][[:space:]][[:space:]]* ]]
                    then
                        typeset FIRST_HDR=${DISPLAY_STRING%%[[:space:]]*}
                        DISPLAY_STRING=${DISPLAY_STRING//$FIRST_HDR/$FIRST_HDR&nbsp;&nbsp;}
                    fi

                    # Replace all remaining spaces with "nbsp;" characters
                    DISPLAY_STRING="&nbsp;&nbsp;&nbsp;${DISPLAY_STRING// /&nbsp\;}"

                    print "
<tr class='nameListHeader'>
<td class='nameListHeader'>
$DISPLAY_STRING
</td>
</tr>"
                fi
            fi
            PRINTED_OPTIONS=true
            continue;
        elif [[ -z $STRING ]]; then
            print "<tr><td>&nbsp;</td></tr>"  # An intentional spacer
            continue;
        fi

        (( COUNT += 1 ))
        display_bullet "$RAW_FIELD_NAME" \
                       "$RAW_STRING"     \
                       "$INPUT_TYPE"     \
                       "$CHECKED"        \
                       "$STRING"         \
                       "$TITLE"          \
                       "$COUNT"
        PRINTED_OPTIONS=true

        #=================================================================
        # If this is a radio-button list, and we've just checked the
        # first entry in the list, then blank out the CHECKED attribute.
        # This attribute is already empty if it's a checkbox.
        #=================================================================
        if [[ "$INPUT_TYPE" == "radio" && "$LIST_CNT" == "0" ]]; then
            CHECKED=""        # All remaining options will be unchecked.
        fi

        # Add the value to the session
        ${BINDIR}/wsm_session -d "${OPT_ID}__0_list" -s "item_$LIST_CNT"="$RAW_STRING"
        (( LIST_CNT += 1 ))
        PRINTED_OPTIONS=true

    done # End of the STRING loop


    #========================================================================
    # If we went through the loop and printed only headers, inform the user
    #========================================================================
    if [[ -n $LIST_CMD_STR && $PRINTED_OPTIONS != "true" ]]
    then
        typeset LIST_ERR_MSG="Error: There were no items of this type."
        [[ -n $GET_LIST_STDERR ]] && LIST_ERR_MSG="<XMP>$GET_LIST_STDERR</XMP>"
        print "
<div class='errorBox'>
$LIST_ERR_MSG
<div class='formButtons'>
<input class='act_smittree act_smittree_button' TYPE='button' value='Back' onclick='history.go(-1)'>
</div>
</div>"

        $CGI_BIN/wsm_html_lib body_end_name "$ID" "$TITLE" "act_smittree act_smittree_link"
        log_write "$ID" "$TITLE" "$NEXT_ID"
        return 0;
    fi

    #==========================================================================
    # Add the total number of list items to the session for later processing.
    #==========================================================================
    ${BINDIR}/wsm_session -d "${OPT_ID}__0_list" -s "CNT"="$LIST_CNT"

    print "</table>"

else
    #==============================================================
    # This is an editable input - draw a text input entry
    # ---------------------------------------------------
    # Although the cmd_opt may have a disc_field_name, ASCII SMIT
    # does not present any value, so we fake out DISC here ...
    #==============================================================
    text_entry "$RAW_FIELD_NAME" \
               "txt"             \
               "$REQ"            \
               " "               \
               "$TEXT"           \
               "$OPT_ID"         \
               "0"               \
               "$OPT_STANZA"
fi


#===========================================================================
# Add a button to send the selection.
# -----------------------------------
# If we had a list of some sort, and that list was a radio box, but it
# had nothing listed, then show the Back button. In all other cases show
# the Continue button. Checkboxes allow multiple selections, including
# none, although other restrictions may dictate that at least one checkbox
# entry is required.
#===========================================================================
if [[ "$ENTRY_TYPE" == "n" && $LIST_CNT == 0 && "$INPUT_TYPE" == "radio" ]]
then
    typeset BACK=$(getmsg wsm_smit.cat 47 "Back")
    typeset BACK_HELP=$(getmsg wsm_smit.cat 9999 "Click to return to the previous page.")
    print "<div class='formButtons'><input class='act_smittree act_smittree_button' TYPE='button' value=\"$BACK\" title=\"$BACK_HELP\" onclick='history.go(-1)'></div>"
else
    typeset CONTINUE=$(getmsg wsm_smit.cat 9999 "Continue")
    typeset CONTINUE_HELP=$(getmsg wsm_smit.cat 9999 "Click to continue with the current selection.")
    print "<div class='formButtons'><input class='act_smittree act_smittree_button' type='submit' value=\"$CONTINUE\" title=\"$CONTINUE_HELP\"></div>"
fi


#============================
# All done; finish the HTML
#============================
print "</div>
</form>"

$CGI_BIN/wsm_html_lib body_end_name                    \
                      "$ID"                            \
                      "$TITLE"                         \
                      "act_smittree act_smittree_link" \
                      "$MAIN_HELP_MSG_LOC"             \
                      "$MAIN_HELP_MSG_SET"             \
                      "$MAIN_HELP_MSG_ID"

log_write "$ID" "$TITLE" "$NEXT_ID"
exit 0;
