/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2006               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* @(#)23       1.2 src/43haes/usr/sbin/cluster/wsm/docroot/bril.js, hacmp.wsm, 52haes_r540, r5400622c 5/2/06 08:53:25 */


/*
 * BRIL: BRowser Independent Layer
 *
 * Description:
 *
 *      NOTE: This should be included in each frame that needs it to avoid
 *            scoping issues when these functions are called.
 *
 *      There are many ways in which Mozilla and IE differ.  This object
 *      aspires to blur the distinction and provide safe haven for code that
 *      gets caught in the cross-fire. :)
 *
 *      There should only ever be one of these objects, so its methods are
 *      declared as part of the object itself, instead of its prototype.
 *
 *      The BRowser Indepentence Type (BRIT) is 
 *
 *
 */
if (is_undefined('BRIL'))
{
    BRIL = {
        isIE : false,
        isMoz : false
    };

    /* Determine the browser.  If not IE, assume Mozilla.  NOTE: This
     * assumption will likely run rampant throughout this class. */
    if (window.ActiveXObject) { BRIL.isIE = true; }
    else { BRIL.isMoz = true; }


    /* Method: newXMLHttp
     *
     * Description:
     *
     *      Return the AJAX object used for making requests to the server in
     *      the background.
     *      
     */
    BRIL.newXMLHttp = function() {

        var oXML;
        if (this.isIE) { 
            try {
                return new ActiveXObject("Msxml2.XMLHTTP");
            } 
            catch (e) {
                try {
                    return new ActiveXObject("Microsoft.XMLHTTP");
                }
                catch (e2) {
                    return false;
                }
            }
        }
        else { return new XMLHttpRequest(); }

    };


    /* Utility functions */
    isIE = function() { return BRIL.isIE; };
    isMoz = function() { return BRIL.isMoz; };

    
    /* 
     * 
     * More-than-utility functions
     *
     *
     */


    /* Function: getInnerWinWidth / getInnerWinHeight
     *
     * Description:
     *
     *      Gets the width of the window's visible area.  Not including
     *      scrollbars.
     *
     * Arguments:
     *
     *      w - The window whose size we're after.  If not supplied, then the
     *          current window is assumed.
     *
     */
    BRIL.getInnerWinWidth = function(w) {
        w = w ? w : window;  // optional argument
        if (this.isMoz) return w.innerWidth;
        else            return w.document.body.clientWidth;
    };
    BRIL.getInnerWinHeight = function(w) {
        w = w ? w : window;  // optional argument
        if (this.isMoz) return w.innerHeight;
        else            return w.document.body.clientHeight;
    };

    /* Function: getInnerWidth / getInnerHeight
     *
     * Description:
     *
     *      Gets the width of an object's visible area.  Not including
     *      scrollbars.
     *
     * Arguments:
     *
     *      o - The object whose size we're after.  If not supplied, 0 is
     *          returned.
     *
     */
    BRIL.getInnerWidth = function(o) {
        if ( !o ) { return 0; }
        if (this.isMoz) return o.innerWidth;
        else            return o.clientWidth;
    };
    BRIL.getInnerHeight = function(o) {
        if ( !o ) { return 0; }
        if (this.isMoz) return o.innerHeight;
        else            return o.clientHeight;
    };

    /* Function: evt
     *
     * Get the event object for the current event.
     *
     * Moz - this isn't necessary
     * IE  - the event object is not passed to the function, but is held in a
     *       global variable.  If that is null, then we might want the event
     *       for the tooltip, which is at the top level.
     */
    BRIL.evt = function(e) {
        if (this.isIE) {
            if (event) return event;
            else return top.event;
        }
        return e;
    };

    // Stop event processing now, preventing parent elements from processing
    // the event.
    BRIL.stopEventBubble = function(e) {
        e = this.evt(e);
        if (this.isIE) e.returnValue = false;
        else           e.preventDefault();
    };

    /* Get the target DOM node of an event (i.e. the element that the event
     * handler was associated with) */
    BRIL.etarget = function(e) {
        e = this.evt(e);
        var t = isIE() ? e.srcElement : e.currentTarget;
        return t;
    };


}

// ++++++++++ END bril.js ++++++++++ //
