/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2006               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* @(#)24       1.4 src/43haes/usr/sbin/cluster/wsm/docroot/common.js, hacmp.wsm, 52haes_r540, r5400622c 5/26/06 12:44:16 */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * File: common.js
 *
 * Description:
 *
 *      This file contains code that is common to both the navigation and
 *      activity frames.
 *
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */




var ACTIVITY   = 'activity';
var NAVIGATION = 'navigation';
var GLOBAL     = 'global';

/* Being able to reference the top-level item without qualifying it will make a
 * huge difference if/when the real location changes. */
var MODULES = top.MODULES;
var TabIdMap = top.TabIdMap;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Utility functions
 */


/**************************************************************************
 * Function: frameType
 *
 * Description:
 *
 *      Utility function to determine which frame we are in at any given time.
 *
 */
function frameType()
{
    return (self.name.match(/[Aa]ctivity/) ?  ACTIVITY : NAVIGATION);
}


/**************************************************************************
 * Function: fixModuleBodyHeight
 *
 * Description:
 *
 *      If the contents of a module become larger than the viewable area, we
 *      do not want window-level scrollbars to appear; they scroll the tabs out
 *      of view.  Instead, the div is set to scroll its content.  We want the
 *      bottom of the div to expand to the bottom of the window so as much of
 *      the content is visible as posible.  This doesn't happen automatically.
 */
function fixActiveBodySize()
{
    var m = top.getActiveModule(frameType());
    var goodmeasure = 10;


    // If there is no active module.... this shouldn't happen.
    if ( typeof m == 'undefined' ) {
        throw "Could not find active " + frameType() + " module!";
    }

    var e = m.mBodyElem();

    if (isIE()) {
        // Calculate the new height (less 10 for the status bar)
        var w = BRIL.getInnerWinWidth() - goodmeasure;

        // Set the new height
        e.style.width = w + 'px';
    }

    // Calculate the new height (less 10 for the status bar)
    var h = BRIL.getInnerWinHeight() - e.offsetTop - goodmeasure;

    // Set the new height
    e.style.height = h + 'px';
}


/**************************************************************************
 * Function: getChildrenByTagName
 *
 * Description:
 *
 *      Like document.getElementsByTagName, but only immediate descendants.
 *
 */
function getChildrenByTagName(element, type) {
    if ( (typeof element) == 'undefined' || element === null)
    {
        top.error("getChildrenByTagName: 'element' is undefined or null");
        return undefined;
    }

    var children = new Array();
    for (var i = 0; i < element.childNodes.length; i++) {
        var n = element.childNodes[i];

        if (n.nodeName == type) {
            children.push(n);
        }
    }
    return children;
}

/**************************************************************************
 * Function: getChildrenByClass
 *
 * Description:
 *
 *      Get all children of 'element' whose class-style is 'className'.
 *
 */
function getChildrenByClass(element, className) {
    if ( (typeof element) == 'undefined' || element === null)
    {
        top.error("getChildrenByTagName: 'element' is undefined or null");
        return undefined;
    }

    var children = new Array();
    for (var i = 0; i < element.childNodes.length; i++) {
        var n = element.childNodes[i];

        if (n.className && n.className == className) {
            children.push(n);
        }
    }
    return children;
}



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Handlers
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

window.onload = function() {
    for (mName in top.MODULES) {
        var module = top.MODULES[mName];
        if (module) {
            if (module.window == window)
            {
                module.onload();
            }
        }
    }
};

/**************************************************************************
 * Event Handler: onresize
 *
 * Description:
 *
 *      Actions to take when the window/frame is resized.
 */

window.onresize = function() {
    fixActiveBodySize();
};

/**************************************************************************
 * Event Handler: onunload
 *
 * Description: Actions to take when the browser leaves the document
 *
 */
window.onunload = function() {

    /* Remove handlers and modules for this window from the top-level handler
     * list.  They will no longer exist, but if the references to them remain
     * in the list, then there will be errors. 
     *
     * This is really all for the benefit of a frame reload.  There shouldn't
     * really be a need for a user to reload a frame (as opposed to the entire
     * page), but we don't want there to be "can't execute unloaded script"
     * errors if/when they do.
     */
    top.HandlerChain.unload(this);

    for (mId in top.TabIdMap) {
        if (top.TabIdMap[mId].window == window) {
            delete top.TabIdMap[mId];
        }
    }
    for (mName in top.MODULES) {
        var module = top.MODULES[mName];
        if (module) {
            if (module.window == window)
            {
                module.cleanup();
                delete top.MODULES[mName];
            }
        }
    }

};

/**************************************************************************
 * Event Handler: onerror
 *
 * Description: Log all JavaScript errors
 *
 */
window.onerror = function(msg, url, line) {
    top.logger('** ERROR: ' + msg + "\n\tSource: " + url + ":" + line);
    return false;  // Allow default browser error processing
};



// ++++++++++ END common.js ++++++++++ //
