/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/* 52haes_r541 src/43haes/usr/sbin/cluster/wsm/modules/navigation/ModuleStatus.js 1.12 */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* COPYRIGHT International Business Machines Corp. 2006,2007              */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* @(#)39       1.12 src/43haes/usr/sbin/cluster/wsm/modules/navigation/ModuleStatus.js, hacmp.wsm, 52haes_r541, 0736B_hacmp541 8/31/07 16:22:10 */


if ( (typeof ModuleStatus) != 'function' || isIE() ) {

    function ModuleStatus(name, type, title) {
        this.base = Module;
        this.base(name, type, title);

        this.path = '';             /* dir path to the module */
        this.treeRootID = '';       /* root of the twisty tree */
        this.selected = undefined;  /* currently selected item */

        this.statusRef = undefined;
        this.drawnStatus = undefined;

        this.waiting_cursor = 'progress';

    }

    ModuleStatus.prototype = new Module();
    ModuleStatus.prototype.constructor = ModuleStatus;

    /***********************************************************************
     * Method: init
     *
     * Description: Things to do after the page is done loading
     *
     */
    ModuleStatus.prototype.onload = function() {
        this.working_on();
    };


    /***********************************************************************
     * Method: cleanup
     *
     * Description: Things to be done when the module or window is going away,
     *              like when the frame is reloaded or the window is navigated
     *              elsewhere.
     *
     */
    ModuleStatus.prototype.cleanup = function() {
        /* 
         * clear timers
         * abort requests
         */
        if (this.updateTimerId) window.clearTimeout(this.updateTimerId);
        if (this.oStatusUpdater) {
            this.oStatusUpdater.abort();
        }
    };

    ModuleStatus.prototype.mActivate = function() {

        /* Call the parent class's method so we can just add to it here. */
        Module.prototype.mActivate.call(this);

        /* Update our status tree */
        this.updateStatus();
    };

    /***********************************************************************
     * Method: updateStatus
     *
     * Description:
     *
     *      Update the displayed status based on new data recieved from the
     *      server.
     *
     */
    ModuleStatus.prototype.updateStatus = function(statusXML) {

        /* If we were passed a status XML, then assume it's an update and store
         * a reference to it. */
        if (undefined == statusXML) { statusXML = this.statusRef; }
        else                        { this.statusRef = statusXML; }

        /* If this module is not active, don't draw the status */
        if ( ! this.mActive() ) { return; }

        /* If the status we have has already been "drawn", don't do it again.
         * Otherwise, draw this one and track it as being drawn. */
        if (this.drawnStatus == this.statusRef) {
            return;
        }
        else {
            this.drawnStatus = this.statusRef;
        }

        top.debug(this.name + ": updateStatus drawing the tree...");

        this.working_on();

        /* Get the existing top of the tree */
        var currTree = document.getElementById(this.treeRootID);
        var aCurrTreeLists = getChildrenByTagName(currTree, 'UL');

        /* Apply the XSLT transformation to the incoming XML. */
        var newTree;
        if (isMoz()) {
            newTree = this.transform.transformToFragment(statusXML,
                                                         document);
        }
        else {
            /* Create a node and assign the response to its innerHTML;  the
             * poor-man's method for creating a DOM tree.  We're receiving
             * html, not XML, so we can't just use responseXML. */
            newTree = document.createElement('div');
            newTree.innerHTML = statusXML.transformNode(this.transform);
        }

        top.debug("Got top level element from transformation.  Type: "
                  + newTree.nodeName + ", id: " +
                  + newTree.id
                 );

        /* There should only be one "twisty_tree" div */
        var ch = getChildrenByTagName(newTree, 'DIV');
        if ( !ch.length || ch[0].id != this.treeRootID ) {
            top.debug("*** ERROR: Invalid tree root ID received (" 
                      + (ch.length ? ch[0].id : 'null') + ")");
            return;
        }

        /* There should only be one <ul> under the "twisty_tree" div.
         * Use that to merge with the current. */
        var aNewSubLists = getChildrenByTagName(ch[0], 'UL');

        /* merge the trees */
        if ( !aCurrTreeLists.length ) {
            /* Initially, the current tree will be empty.  In that
             * case, just take the whole blob and we're done. */
            aCurrTreeLists[0] = aNewSubLists[0].cloneNode(true);
            currTree.appendChild(aCurrTreeLists[0]);
        }
        else {
            /* Otherwise, merge them recursively... */
            this.mergeStatusTree(aCurrTreeLists[0], aNewSubLists[0]);
        }

        /* Update the status icons, as needed... */
        this.bubbleStatus(aCurrTreeLists[0]);

        this.working_off();
    };



    /***********************************************************************
     * Method: mergeStatusTree
     *
     * Description:
     *
     *      Remove deleted items, add new items, and update changed items.
     *      Uses a recursive algorithm to do so...
     *
     *      Assumptions:
     *
     *          - An <li> (list item) will ONLY contain one line item (twisty,
     *            status, and link), and an optional sub-list (ul).
     *          - A <ul> will ONLY contain <li>'s
     *
     * Arguments:
     *
     *      orig  - the current <ul>
     *      fresh - the updated <ul>
     *
     */
    ModuleStatus.prototype.mergeStatusTree = function(current, fresh) {

        /* Disable debugging for this function, due to the exorbidant number of
         * messages it produces. */
        var origDebug = top.DEBUG;
        top.DEBUG = false;

        var i;

        /* Sanity check to ensure we're dealing with UL elements */
        if (current.nodeName != 'UL' || fresh.nodeName != 'UL') {
            top.error("mergeStatusTree: got non-UL arguments! current.nodeName: " 
                      + current.nodeName + ", fresh.nodeName: " + fresh.nodeName);
            return;
        }


        /*     
         * merge
         *     for all <li>'s
         *         delete removed <li>'s
         *         add    new     <li>'s
         *         for common <li>'s
         *             delete removed <ul>'s
         *             add new        <ul>'s
         *             merge common   <ul>'s
         *
         */

        /* Delete missing items first */
        var aCurrSubItems = getChildrenByTagName(current, 'LI');
        for (i in aCurrSubItems) {
            var currItem = aCurrSubItems[i];
            if ( !this.ulHasItem(fresh, currItem) ) {
                top.debug("mergeStatusTree: REMOVING item '"
                          + this.liName(currItem) + "'");
                current.removeChild(currItem);
            }
        }

        /* for all <li>'s */
        var newChild;
        var aNewSubItems = getChildrenByTagName(fresh, 'LI');
        for (i in aNewSubItems) {

            var newItem = aNewSubItems[i];
            top.debug("mergeStatusTree: checking new item '"
                      + this.liName(newItem) + "'");

            /* If it's not in the current tree, add it */
            currItem = this.ulHasItem(current, newItem);
            if ( undefined === currItem ) {
                top.debug("mergeStatusTree: ADDING item '"
                          + this.liName(newItem) + "'");
                newChild = newItem.cloneNode(true); /* deep copy */
                current.appendChild(newChild);
            }
            else { /* merge */

                var currItemName = this.liName(currItem);
                top.debug("mergeStatusTree: MERGING item '" + currItemName + "'");
                
                /* 
                 * first child is the twisty.  Ignore it.
                 */


                /* 
                 * the status icon
                 */
                var currStatusIcon = this.getStatusImage(currItem);
                var newStatusIcon  = this.getStatusImage(newItem);

                /* update the status icon and label, if needed */
                if (currStatusIcon && currStatusIcon.src != newStatusIcon.src)
                {
                    currStatusIcon.src   = newStatusIcon.src;
                    currStatusIcon.title = newStatusIcon.title;
                    if (newStatusIcon.id) {
                        var label_id = newStatusIcon.id.replace(/__image/, '__label');
                        var label_obj = document.getElementById(label_id);
                        if (label_obj) {
                            if ( newStatusIcon.title.indexOf('ONLINE') >= 0 ||
                                 newStatusIcon.title.indexOf('STABLE') >= 0 ||
                                 newStatusIcon.title.indexOf('UP')     >= 0 )
                            {
                                label_obj.className = label_obj.name;
                            } else { // Not fully functional; gray it out!
                                label_obj.className = 'inactive_status';
                            }
                        }
                    }
                }

                /* If both have a sub-list, merge them.  There should only be
                 * one sub-list per line-item. */
                var aCurrSubLists = getChildrenByTagName(currItem, 'UL');
                var aNewSubLists = getChildrenByTagName(newItem, 'UL');
                if (aCurrSubLists.length) {
                    if (aNewSubLists.length) {
                        /* They both have the sub-list.  Make a recursive call
                         * to merge them. */
                        top.debug("mergeStatusTree: MERGING sub-lists for '"
                                  + currItemName + "'");
                        this.mergeStatusTree(aCurrSubLists[0], aNewSubLists[0]);
                    }
                    else {
                        /* The sub-list is not in the update.  Remove it. */
                        top.debug("mergeStatusTree: removing missing sublist for item '"
                                  + currItemName + "'");
                        currItem.removeChild(aCurrSubLists[0]);
                    }
                }
                else {
                    /* There is a new sub-list that needs to be added. */
                    if (aNewSubLists.length) {
                        top.debug("mergeStatusTree: Copying new sub-list for '"
                                  + currItemName + "'");
                        newChild = aNewSubLists[0].cloneNode(true);
                        currItem.appendChild(newChild);

                        /* The default for a list is 'collapsed'.  If the item
                         * is added to an item that is 'expanded', set the list
                         * to the same so it will be visible. */
                        var aCurrItemIcons = getChildrenByTagName(currItem, 'IMG');
                        if ( aCurrItemIcons[0].src == IMAGES.minus.src ) {
                            newChild.className = 'expanded';
                        }
                    }
                }
            }
        }
        /* Restore debugging */
        top.DEBUG = origDebug;
    };

    /***********************************************************************
     * Method: bubbleStatus
     *
     * Description:
     *
     *      Bubble-up errors to the status of problem items so the problem can
     *      be reflected even when it is collapsed.
     *
     */
    ModuleStatus.prototype.bubbleStatus = function(ulNode, parentItem) {

        var origDebug = top.DEBUG;
        top.DEBUG = false;

        /* for each <li>
         *
         *   if is_composite(li)
         *     (alert, mixed) = bubble(sub-list)
         *
         *   if this == mixed
         *     mixed = true
         *
         *   if this == error
         *     alert = true
         *
         *   return { 'mixed' : mixed, 'alert' : alert }
         *
         */
        top.debug("In bubbleStatus...");

        /* Utility function telling if a line-item is a composite */
        var is_composite = function(li) {
            var n;
            if (li.nodeName != 'LI') {
                top.error("liName: received non-LI (" + li.nodeName + ")");
                return false;
            }
            for (n in li.childNodes) {
                if (li.childNodes[n].nodeName == 'FONT') {
                    return li.childNodes[n].getAttribute('composite') == '1';
                }
            }
            return false;
        };

        var getItemAttribute = function(li, attr) {
            var aA = getChildrenByTagName(li, 'FONT');
            if ( !aA.length ) { return null; }
            return aA[0].getAttribute(attr);
        };

        var error = false;      /* There are one or more errors */
        var mixed = false;      /* The status of subitems is mixed */

        var prev_status = undefined; /* tracking to see if mixed == true */

        /* Parent item is Concurrent Resource Group? */
        var parentIsCRG = false;
        if (parentItem) {
            if ('resource_group' == getItemAttribute(parentItem, 'component_type') 
                && 'concurrent' == getItemAttribute(parentItem, 'type_name'))
            {
                parentIsCRG = true;
            }
        }

        /* Get all line-items and process each.  Composite sub-items will be
         * sent to this function recursively. */
        var aItems = getChildrenByTagName(ulNode, 'LI');
        for (var i in aItems) {
            var li = aItems[i];        /* The current item */
            var subError  = false;     /* Sub-item/child in error state */
            var common    = undefined; /* The stati are all this img src */
            var isCRG     = false;     /* Concurrent Resource Group? */

            top.debug("bubbleStatus: Processing line-item: '" + this.liName(li) + "'");

            /* Is the current item a concurrent resource group? */
            if ('resource_group' == getItemAttribute(li, 'component_type') 
                && 'concurrent' == getItemAttribute(li, 'type_name'))
            {
                isCRG = true;
            }

            /* The status image will tell us the status */
            var stat = this.getStatusImage(li);

            /* Send the sublist to this function and process the results. */
            var aSubULList = getChildrenByTagName(li, 'UL');
            if (aSubULList.length) {
                var oRes = this.bubbleStatus(aSubULList[0], li);

                /* Change these only if they have values */
                if (oRes.mixed) mixed = true;
                if (oRes.error) subError = error = true;
                if (oRes.common) common = oRes.common;

                top.debug("bubbleStatus: item '" + this.liName(li)
                          + "'  (composite: "+ is_composite(li)
                          + ", common: " + common
                          + ", mixed: " + mixed
                          + ", error: " + error);
            }
            else {  /* leaf */

                if (/error/.test(stat.src)) { error = true; }

                if ( !/none/.test(stat.src) ) {

                    if (parentIsCRG)
                    {
                        /* If this is a composite RG, then it's concurrent. */
                        if ('node' == getItemAttribute(li, 'component_type')) {
                            if (prev_status != undefined && stat.src != prev_status) {
                                mixed = true;
                            }
                            else {
                                prev_status = stat.src;
                            }
                        }
                    }
                    else {

                        /* If this is not the first item, and its status image is
                         * different than the previous one's, then we have a mixed
                         * status. */
                        if (prev_status != undefined && stat.src != prev_status) {
                            /* The sub-items have a mixed status.  We only want to
                             * return a mixed status for the current item if it is
                             * a composite.  If the current item is not a
                             * composite, then it should have a status of its own.
                             */
                            if (is_composite(li)) mixed = true;
                        }
                        else {
                            prev_status = stat.src;
                        }
                    }
                }
            }

            /* If this is a composite, update its status accordingly */
            if (is_composite(li) || isCRG) {

                var c = isCRG ? 'concurrent' : 'composite';

                /* Set the image of the current item based on what we found
                 * out...  If all sub-items had the same status, use that.
                 * Otherwise, set it to mixed.  Then set the blinking if
                 * needed. */
                if (common) {
                    stat.src = common.src;
                    stat.title = common.title;
                }
                else if (mixed) {
                    top.debug(c + " w/ non-common: setting mixed!");
                    stat.src = IMAGES.status_mixed.src;
                    stat.title = "Online on Some Nodes";
                }
            }

            /* Make it blink if there is an error below, unless we already have
             * an error image for this item... */
            if (subError && aSubULList.length) {
                if ( !/error/.test(stat.src) ) {
                    stat.src = stat.src.replace(/(status_.*)(\.gif)/, '$1_alert$2');
                }
            }
            top.debug("bubbleStatus: stat.src is now '" + stat.src + "'");
        }

        /* Restore orig debugging */
        top.DEBUG = origDebug;

        /* Return the n-tuple object */
        return {
            'mixed' : mixed,
            'error' : error,
            'common' : mixed ? undefined : stat
        };

    };



    /***********************************************************************
     ***********************************************************************
     *
     * UTILITY FUNCTIONS
     *
     */


    /*
     * Utility function to get a line-item's status image
     *
     */
    ModuleStatus.prototype.getStatusImage = function(li) {
        var img = (getChildrenByClass(li, 'status_image'))[0];
        return img;
    };

    /* Function: liName
     *
     * Description: Utility function to get the name of the line-item,
     *              represented by an <li>.
     *
     */
    ModuleStatus.prototype.liName = function(li) {
        var n;
        if (li.nodeName != 'LI') {
            top.error("liName: received non-LI (" + li.nodeName + ")");
            return null;
        }
        for (n in li.childNodes) {
            if (li.childNodes[n].nodeName == 'FONT') {
                return li.childNodes[n].innerHTML;
            }
        }
        return null;
    };

    /* Function: ulHasItem
     *
     * Description: Utility function to determine whether or not a <ul>
     *              list contains the line-item 'item' (an <li>).
     *
     */
    ModuleStatus.prototype.ulHasItem = function(ul, item) {
        if (ul.nodeName != 'UL') {
            top.error("ulHasItem: received non-UL (" + ul.nodeName + ")");
            return null;
        }
        var c;
        var aItems = getChildrenByTagName(ul, 'LI');
        var itemName = this.liName(item);
        for (c in aItems) {
            if (this.liName(aItems[c]) == itemName) {
                return aItems[c];
            }
        }
        return undefined;
    };




} /* ModuleStatus */



// ++++++++++ END modules/navigation/ModuleStatus.js ++++++++++ //
