<?xml version="1.0"?>
<!-- @(#)89       1.5 src/43haes/usr/sbin/cluster/wsm/modules/navigation/hw/status.xsl, hacmp.wsm, 52haes_r541, 0736B_hacmp541 8/31/07 16:09:30 -->

<xsl:stylesheet
  xmlns:statemap="Translation of status codes"
  xmlns:imgmap="Translation of generic status to filenames"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xalan="http://xml.apache.org/xalan"
  exclude-result-prefixes="statemap imgmap"
  version="1.0">

  <!-- exclude-result-prefixes: prevents unwanted namespace declarations -->

  <xsl:include href="../status_common.xsl" />

  <!-- This language parameter can be set on the command line, with english as
  <!-  the default. -->
  <xsl:param name="language" select="'en'" />

  <!-- Declare what kind of output we are producing: HTML with indentation.
  <!-  -->
  <xsl:output method="html" indent="yes" />

  <!-- The name of this module -->
  <xsl:variable name="module_name" select="'Hardware'" />

  <!-- Create a variable to more easily access the state table and to reduce
  <!-  the calls to the document function to one. -->
  <xsl:variable name="statetbl"
    select="document('../../status_table.xml')/statetable" />

  <!-- Create a variable to more easily access the image table and to reduce
  <!-  the calls to the document function. -->
  <xsl:variable name="imgtbl"
    select="document('../status_images.xml')/imagetable" />

  <!-- Where our images are located on the server. -->
  <xsl:variable name="imageroot" select="$imgtbl/root" />


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-
  <!-  Template: main (sorta)
  <!-
  <!-  Description:
  <!-
  <!-       This is the starting point of the processing, since it matches the
  <!-       root of the document.
  <!-
  <!-  -->
  <xsl:template match="/">
    <div class="twisty_tree" id="HardwareNavTree">

      <ul class="twistyRoot">

        <xsl:apply-templates select="status/cluster" />

      </ul>
    </div>
  </xsl:template>




  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: cluster
  <!-
  <!-  -->
  <xsl:template match="cluster">
    <li>
      <xsl:call-template name="lineitem">
          <xsl:with-param name="tab_name" select="'NnN'" />
      </xsl:call-template>

      <ul class="collapsed">
        <xsl:choose>
          <xsl:when test="../sites/site">
            <xsl:apply-templates select="../sites/site" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="../nodes" />
          </xsl:otherwise>
        </xsl:choose>
      </ul>
    </li>
  </xsl:template>




  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: site
  <!-
  <!-  -->
  <xsl:template match="site">
    <li>
      <xsl:call-template name="lineitem">
          <xsl:with-param name="tab_name" select="'NnN'" />
      </xsl:call-template>

      <ul class="collapsed">
        <xsl:for-each select="nodes/node">
          <xsl:apply-templates select="/status/nodes/node[@id=current()/@id]" />
        </xsl:for-each>
      </ul>
    </li>
  </xsl:template>



  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: nodes
  <!-
  <!-  -->
  <xsl:template match="nodes">
    <li>
      <xsl:call-template name="composite">
          <xsl:with-param name="tab_name" select="'NnN'" />
      </xsl:call-template>

      <ul class="collapsed">
        <xsl:apply-templates select="./node" />
      </ul>
    </li>
  </xsl:template>


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: node
  <!-
  <!-  -->
  <xsl:template match="node">
    <xsl:variable name="id" select="@id" />
    <li>
      <xsl:call-template name="lineitem">
          <xsl:with-param name="tab_name" select="'NnN'" />
      </xsl:call-template>

      <ul class="collapsed">
        <!-- Save this for the RG view -->
        <!--
        <xsl:apply-templates select="/clusterstate/resource_groups">
          <xsl:with-param name="node_id" select="$id" />
        </xsl:apply-templates>
        -->

        <xsl:apply-templates
          select="/status/resource_groups/resource_group[nodes/node/@id=$id]">
          <xsl:with-param name="node_id" select="$id" />
        </xsl:apply-templates>

        <xsl:apply-templates
          select="/status/networks/network[nodes/node/Id = $id]" >
          <xsl:with-param name="node_id" select="$id" />
        </xsl:apply-templates>

      </ul>
    </li>
  </xsl:template>


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: resource_group
  <!-
  <!-  -->
  <xsl:template match="resource_group">
      <xsl:param name="node_id" />

      <xsl:variable name="state"
                    select="$statetbl/resource_group/state[@code=current()/nodes/node[@id=$node_id]/NodeState]" />

      <li>
          <xsl:call-template name="lineitem">
              <xsl:with-param name="state"    select="$state"   />
              <xsl:with-param name="leaf"     select="true()"   />
              <xsl:with-param name="tab_name" select="'NnN'"    />
              <xsl:with-param name="node_id"  select="$node_id" />
          </xsl:call-template>
      </li>
  </xsl:template>


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: network
  <!-
  <!-  -->
  <xsl:template match="network">
      <xsl:param name="node_id" />

      <xsl:variable name="state"
                    select="$statetbl/network/state[@code=current()/nodes/node[@id=$node_id]/State]" />

      <li>
          <xsl:call-template name="lineitem">
              <xsl:with-param name="state"    select="$state"   />
              <xsl:with-param name="leaf"     select="true()"   />
              <xsl:with-param name="tab_name" select="'NnN'"    />
              <xsl:with-param name="node_id"  select="$node_id" />
          </xsl:call-template>
      </li>
  </xsl:template>


</xsl:stylesheet>

<!-- vim:set sw=2: -->
