<?xml version="1.0"?>
<!-- @(#)87       1.20 src/43haes/usr/sbin/cluster/wsm/modules/navigation/status_common.xsl, hacmp.wsm, 52haes_r541, 0747B_hacmp541 11/13/07 16:56:03 -->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:statemap="Translation of status codes"
  xmlns:imgmap="Translation of generic status to filenames"
  exclude-result-prefixes="statemap imgmap"
  version="1.0">

  <!-- exclude-result-prefixes: prevents unwanted namespace declarations -->

  <xsl:variable name="typestbl"
                select="document('../types_table.xml')/typestable" />


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-
  <!-  Template: lineitem
  <!-
  <!-  Description:
  <!-
  <!-       Generate HTML for a navigational line-item.  That consists of a
  <!-       twisty, a status icon, and the text for the item.  This template
  <!-       cannot know the structure of the final hierarchy, so sub-items will
  <!-       be handled by the caller of this template.
  <!-
  <!-
  <!- -->
  <xsl:template name="lineitem">

    <!-- Parameters -->
    
    <!-- The statetable state node for the current item.  The default bases it
    <!-  on the tagname of the current item (like 'cluster' or 'node').  This
    <!-  will allow us to provide the proper image and text based on the state
    <!-  code for the type of item we're looking at. -->
    <xsl:param name="state"
               select="$statetbl/*[name()=name(current())]/state[@code=current()/State]" />

    <!-- If this item does not have sub-items, then use a bullet instead of a
    <!-  twisty. -->
    <xsl:param name="leaf" select="false()" />

    <!-- The name/text of the item.  This defaults to the value of the 'Name'
    <!-  sub-node, but the caller may need to override it. -->
    <xsl:param name="name" select="Name" />

    <!-- The ID number of the node that this lineitem is being
    <!-  displayed under (if any; only applies to N&N tab) -->
    <xsl:param name="node_id" />

    <!-- The text to prepend to the text for the item, if any -->
    <xsl:param name="prefix" />

    <!-- The resource type code.  Ignored for all others. -->
    <xsl:param name="type_code" />

    <!-- The resource type code.  Ignored for all others. -->
    <xsl:param name="type_name" />

    <!-- The tab from which this template was invoked: "N&N" or "RGs" -->
    <xsl:param name="tab_name" />

    <!-- The type of item (component_type).  This should be passed in if it is
    <!-  anything other than the tag name (the name of the xpath node). -->
    <xsl:param name="type" select="name(.)" />


    <!-- Get the status string based on our language -->
    <xsl:variable name="state_str">
        <xsl:choose>
            <xsl:when test="$type='resource'">
                <xsl:choose>

                    <!-- ##  Service Label  ## -->
                    <xsl:when test="$type_code=1000">

                        <xsl:variable name="svc_label_name" select="Name"/>

                        <xsl:variable name="svc_label_state">
                            <xsl:for-each select="../../../../addresses/address[Label=$svc_label_name and NodeId=ActiveNode]">
                                <xsl:value-of select="State"/>
                            </xsl:for-each>
                        </xsl:variable>

                        <!-- If a valid state can be detected for this service
                        <!-  label, then use its string. Otherwise, retrieve
                        <!-  the "unknown state" string and use it instead. -->
                        <xsl:choose>
                            <xsl:when test="string($statetbl/service_label/state[@code=$svc_label_state]/string[@lang=$language])">
                                <xsl:value-of select="$statetbl/service_label/state[@code=$svc_label_state]/string[@lang=$language]" />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$statetbl/generic/state[@name='unknown']/string[@lang=$language]" />
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>

                    <xsl:otherwise>
                        <!-- If a valid state node was received from the
                        <!-  caller, use its string. Otherwise, retrieve
                        <!-  the "unknown state" string and use it.  -->
                        <xsl:choose>
                            <xsl:when test="string($state)">
                                <xsl:value-of select="$state/string[@lang=$language]" />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$statetbl/generic/state[@name='unknown']/string[@lang=$language]" />
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:otherwise>

                </xsl:choose>
            </xsl:when>

            <xsl:otherwise>
                <!-- If a valid state node was received from the
                <!-  caller, use its string. Otherwise, retrieve
                <!-  the "unknown state" string and use it.  -->
                <xsl:choose>
                    <xsl:when test="string($state)">
                        <xsl:value-of select="$state/string[@lang=$language]" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$statetbl/generic/state[@name='unknown']/string[@lang=$language]" />
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <!-- Get the type string -->
    <xsl:variable name="type_str" select="$typestbl/component[@type=$type]/string[@lang=$language]" />

    <!-- Get the image based on the image name in the state node.
    <!-
    <!-  NOTE: to add images for other resources:
    <!-         1) create the icons (16x16 GIFs), one for each supported state
    <!-         2) in the "docroot/modules/navigation/status_images.xml" file,
    <!-            add a new entry for each GIF. Select a name attribute for
    <!-            entry that is comprised of the resource type name and the
    <!-            state string.
    <!-         3) in the "docroot/modules/status_table.xml" file, add a new
    <!-            child element named after the resource. See "<service_label>"
    <!-            for an example.
    <!-         4) add an entry to the "image_file" variable code, below, for
    <!-            the new type_code (these are defined in "types_table.xml").
    <!- -->
    <xsl:variable name="image_file">
        <xsl:choose>
            <xsl:when test="$type='resource'">
                <xsl:choose>

                    <!-- ##  Service Label  ## -->
                    <xsl:when test="$type_code=1000">

                        <xsl:variable name="svc_label_name" select="Name"/>

                        <xsl:variable name="svc_label_state">
                            <xsl:for-each select="../../../../addresses/address[Label=$svc_label_name and NodeId=ActiveNode]">
                                <xsl:value-of select="State"/>
                            </xsl:for-each>
                        </xsl:variable>

                        <xsl:value-of select="$imgtbl/image[@name=$statetbl/service_label/state[@code=$svc_label_state]/image]" />
                    </xsl:when>

<!--
<!-  DEV NOTE: the following "when" tests can/should all be replaced with
<!-            a single "otherwise" containing the following code:
<!-
<!-            xsl:value-of select="$imgtbl/image[@name='$type_str']"
<!-
<!-            The reason it is not like that *now* is that we simply did
<!-            not have time to provide an icon for each resource type
<!-            defined in "types_table.xml" in the 5.4.1 release! So for
<!-            now, the resource types that we do *not* have an icon for
<!-            will get displayed as the "status_up" image. :-/
<!- -->
                    <!-- ##  File System  ## -->
                    <xsl:when test="$type_code=1002">
                        <xsl:value-of select="$imgtbl/image[@name='file_system']" />
                    </xsl:when>

                    <!-- ##  Volume Group  ## -->
                    <xsl:when test="$type_code=1003">
                        <xsl:value-of select="$imgtbl/image[@name='volume_group']" />
                    </xsl:when>

                    <!-- ##  Application Server  ## -->
                    <xsl:when test="$type_code=1006">
                        <xsl:value-of select="$imgtbl/image[@name='resource_application']" />
                    </xsl:when>

                    <xsl:otherwise>
                        <xsl:variable name="temp_img" select="$imgtbl/image[@name=$state/image]" />
                        <xsl:choose>
                            <xsl:when test="$state_str='none'">
                                <xsl:value-of select="$imgtbl/image[@name='up']" />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$imgtbl/image[@name=$state/image]" />
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:otherwise>
                <xsl:value-of select="$imgtbl/image[@name=$state/image]" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="unknown_img" select="'status_unknown.gif'" />

    <!-- The text for the item -->
    <xsl:variable name="text">
        <!-- Since the "up" image is narrower than the other images,
        <!-  a spacer needs to be inserted to keep the descriptive
        <!-  labels vertically aligned. NOTE: can't use "&nbsp;" in
        <!-  XSL; have to use "&#xA0;" instead. -->
        <xsl:if test="contains($image_file, 'status_up.gif')">
            <xsl:text>&#xA0;&#xA0;</xsl:text>
        </xsl:if>

        <xsl:text> </xsl:text>
        <xsl:choose>
            <xsl:when test="$prefix">
                <xsl:value-of select="concat($prefix, ': ', $name)" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$name" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <!-- Determine the twisty/leaf image -->
    <xsl:variable name="twisty_img">
        <xsl:choose>
            <xsl:when test="$leaf">
                <xsl:value-of select="concat($imageroot, 'topic.gif')"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="concat($imageroot, 'plus.gif')"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <!-- 
    <!-  the twisty
    <!- -->
    <xsl:choose>
        <xsl:when test="$leaf">
            <!-- Use a little bit of indentation to make things look cleaner -->
            <span class="indenter"></span>
        </xsl:when>
        <xsl:otherwise>
            <img src="{$twisty_img}" class="collapsed" onclick="twisty(this)" />
        </xsl:otherwise>
    </xsl:choose>

    <!-- 
    <!-  the status icon
    <!- -->
    <img class="status_image"
         onmousedown="top.MODULES['{$module_name}'].handleClick(this,event);">

        <!-- Assign an appropriate ID, whose root *exactly* matches the
        <!-  root text of the ID assigned to the associated text label,
        <!-  defined below. The only difference between these two IDs
        <!-  is their ending: "__image" versus "__label". This similarity
        <!-  is exploited to provide visual changes of the labels to reflect
        <!-  status updates. NOTE: the IDs are a bit convoluted, but that
        <!-  is necessary to ensure their uniqueness (*required*). -->
        <xsl:attribute name="id">
            <xsl:if test="$tab_name='RGs'">nav_swtree</xsl:if>
            <xsl:if test="$tab_name='NnN'">nav_hwtree</xsl:if>
            <xsl:if test="$name">
                <xsl:text>__</xsl:text>
                <xsl:value-of select="$name"/>
            </xsl:if>
            <xsl:if test="./Id">
                <xsl:text>__</xsl:text>
                <xsl:value-of select="./Id"/>
            </xsl:if>
            <xsl:if test="$node_id">
                <xsl:text>__</xsl:text>
                <xsl:value-of select="$node_id"/>
            </xsl:if>
            <xsl:if test="../../Name">
                <xsl:text>__</xsl:text>
                <xsl:value-of select="../../Name"/>
            </xsl:if>
            <xsl:text>__image</xsl:text>
        </xsl:attribute>

        <xsl:attribute name="src">
            <xsl:value-of select="$imageroot"/>

            <xsl:choose>
                <xsl:when test="string($image_file)">
                    <xsl:value-of select="$image_file" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$unknown_img" />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>

        <xsl:attribute name="title">
            <xsl:value-of select="$state_str" />
        </xsl:attribute>

        <!-- The following attributes are used to support -->
        <!-- the custom context menu system.              -->
        <xsl:attribute name="component_type">
            <xsl:choose>
                <xsl:when test="$type='resource'">
                    <xsl:value-of select="$type_name"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$type" />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>

        <xsl:attribute name="component_name">
            <xsl:value-of select="$name" />
        </xsl:attribute>
    </img>

    <!-- 
    <!-  the text/label
    <!- -->
    <font onmousedown="top.MODULES['{$module_name}'].handleClick(this,event);"
          onmouseover="JavaScript: this.className += ' highlight'; window.status=''; return true;"
          onmouseout="JavaScript: this.className = this.className.replace(/\s?highlight/, ''); return true;">

      <!-- Assign an appropriate ID, whose root *exactly* matches the
      <!-  root text of the ID assigned to the associated status icon,
      <!-  defined above. The only difference between these two IDs
      <!-  is their ending: "__label" versus "__image". This similarity
      <!-  is exploited to provide visual changes of the labels to reflect
      <!-  status updates. NOTE: the IDs are a bit convoluted, but that
      <!-  is necessary to ensure their uniqueness (*required*). -->
      <xsl:attribute name="id">
          <xsl:if test="$tab_name='RGs'">nav_swtree</xsl:if>
          <xsl:if test="$tab_name='NnN'">nav_hwtree</xsl:if>
          <xsl:if test="$name">
              <xsl:text>__</xsl:text>
              <xsl:value-of select="$name"/>
          </xsl:if>
          <xsl:if test="./Id">
              <xsl:text>__</xsl:text>
              <xsl:value-of select="./Id"/>
          </xsl:if>
          <xsl:if test="$node_id">
              <xsl:text>__</xsl:text>
              <xsl:value-of select="$node_id"/>
          </xsl:if>
          <xsl:if test="../../Name">
              <xsl:text>__</xsl:text>
              <xsl:value-of select="../../Name"/>
          </xsl:if>
          <xsl:text>__label</xsl:text>
      </xsl:attribute>

      <!-- Store the active class assignment in the "name" attribute. -->
      <!-- This is then used to restore the correct display class     -->
      <!-- when a formerly inactive resource comes back on-line.      -->
      <xsl:attribute name="name">
          <xsl:if test="$tab_name='RGs'">nav_swtree</xsl:if>
          <xsl:if test="$tab_name='NnN'">nav_hwtree</xsl:if>

          <xsl:text> </xsl:text>
          <xsl:value-of select="$type"/>

          <xsl:text> </xsl:text>
          <xsl:if test="$type='resource'">
              <xsl:value-of select="$type_name"/>
          </xsl:if>
      </xsl:attribute>

      <!-- Assign an appropriate class, based upon -->
      <!-- the tab that invoked this template.     -->
      <xsl:attribute name="class">
          <xsl:choose>
              <xsl:when test="$state_str='DOWN' or $state_str='OFFLINE' or contains($state_str, 'not available')">
                  <xsl:text>inactive_status</xsl:text>
              </xsl:when>

              <xsl:otherwise>
                  <xsl:if test="$tab_name='RGs'">nav_swtree</xsl:if>
                  <xsl:if test="$tab_name='NnN'">nav_hwtree</xsl:if>

                  <xsl:text> </xsl:text>
                  <xsl:value-of select="$type"/>

                  <xsl:text> </xsl:text>
                  <xsl:if test="$type='resource'">
                      <xsl:value-of select="$type_name"/>
                  </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>

      <!-- We need a separate 'name' attribute for JUST the name, in case we
      <!-  want to add extra text to the title and/or displayed text for
      <!-  clarity or some other reason. -->
      <xsl:attribute name="component_name">
          <xsl:value-of select="$name" />
      </xsl:attribute>

      <xsl:attribute name="composite">0</xsl:attribute>

      <xsl:attribute name="component_type">
          <xsl:value-of select="$type" />
      </xsl:attribute>

      <xsl:attribute name="type_code">
          <xsl:value-of select="$type_code" />
      </xsl:attribute>

      <xsl:attribute name="type_name">
          <xsl:value-of select="$type_name" />
      </xsl:attribute>

      <!-- Establish some flyover help -->
      <xsl:attribute name="title">
          <xsl:choose>
              <xsl:when test="$type_str">
                  <xsl:choose>
                      <xsl:when test="$prefix">
                          <xsl:value-of select="concat($prefix, ': ', $name)" />
                      </xsl:when>

                      <xsl:otherwise>
                          <xsl:value-of select="concat($type_str, ': ', $text)" />
                      </xsl:otherwise>
                  </xsl:choose>
              </xsl:when>

              <xsl:otherwise>
                  <xsl:variable name="title_str">
                      <xsl:if test="$type='resource'">
                          <xsl:value-of select="$typestbl/resource[@code=$type_code]/fullstring[@lang=$language]"/>
                          <xsl:if test="$name">
                              <xsl:text>: </xsl:text>
                              <xsl:value-of select="$name"/>
                          </xsl:if>
                      </xsl:if>
                  </xsl:variable>

                  <xsl:choose>
                      <xsl:when test="$title_str">
                          <xsl:value-of select="$title_str" />
                      </xsl:when>
                      <xsl:otherwise>
                          <xsl:value-of select="$text" />
                      </xsl:otherwise>
                  </xsl:choose>
              </xsl:otherwise>
          </xsl:choose>
      </xsl:attribute>

      <!-- Display the label for the user -->
      <xsl:value-of select="$text"/>

    </font>

  </xsl:template>
  <!-- ##  End of the "lineitem" template  ## -->


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-
  <!-  Template: composite
  <!-
  <!-  Description:
  <!-
  <!-       Generate the HTML for a composite line-item.  This is similar to
  <!-       the 'lineitem' template, but there is no status in the XML for this,
  <!-       because it is not a cluster object, and there is no 'name' for the
  <!-       line item (for the same reason), so the text/name is the type (i.e.
  <!-       the tag name of the XML item).
  <!-
  <!-
  <!-  -->
  <xsl:template name="composite">

      <!-- The tab from which this template was invoked: "N&N" or "RGs" -->
      <xsl:param name="tab_name" />

      <!-- Get the image based on the image name in the state node -->
      <xsl:variable name="state_str"  select="'UNKNOWN'" />
      <xsl:variable name="image_file" select="$imgtbl/image[@name='unknown']" />

      <!-- the twisty -->
      <img src="{concat($imageroot, 'plus.gif')}"
           class="collapsed" onclick="twisty(this)" />

      <!-- the status icon
      <!-  
      <!-  There is no status in the MIB for composite items because they
      <!-  are not real cluster objects.  The src of the image will be
      <!-  updated by code in the client, so leave it as 'none' here. -->
      <img class="status_image"
           onmousedown="top.MODULES['{$module_name}'].handleClick(this,event);">

          <xsl:attribute name="src">
              <xsl:value-of select="concat($imageroot, $image_file)" />
          </xsl:attribute>

          <xsl:attribute name="title">
              <xsl:value-of select="$state_str" />
          </xsl:attribute>

          <!-- The following attributes are used to support -->
          <!-- the custom context menu system.              -->
          <xsl:attribute name="component_type">
              <xsl:value-of select="name(.)" />
          </xsl:attribute>

          <xsl:attribute name="component_name">
              ""
          </xsl:attribute>
      </img>

      <!-- the text/label -->
      <font onmousedown="top.MODULES['{$module_name}'].handleClick(this,event);"
            onMouseOver="JavaScript: this.className += ' highlight'; window.status=''; return true;"
            onmouseout="JavaScript: this.className = this.className.replace(/\s?highlight/g, '');">

          <!-- Assign an appropriate class, based upon -->
          <!-- the tab that invoked this template.     -->
          <xsl:attribute name="class">
              <xsl:if test="$tab_name='RGs'">nav_swtree</xsl:if>
              <xsl:if test="$tab_name='NnN'">nav_hwtree</xsl:if>
              <xsl:text> </xsl:text>
              <xsl:value-of select="name(.)"/>
          </xsl:attribute>

          <xsl:attribute name="name">
              <xsl:value-of select="name(.)" />
          </xsl:attribute>

          <xsl:attribute name="composite">1</xsl:attribute>

          <xsl:attribute name="component_type">
              <xsl:value-of select="name(.)" />
          </xsl:attribute>

          <xsl:attribute name="title">
              <xsl:value-of select="name(.)" />
          </xsl:attribute>

          <xsl:attribute name="component_name">
              ""
          </xsl:attribute>

          <xsl:attribute name="type_code">
              ""
          </xsl:attribute>

          <xsl:attribute name="type_name">
              ""
          </xsl:attribute>

          <xsl:value-of select="name(.)"/>
      </font>

  </xsl:template>
  <!-- ##  End of the "composite" template  ## -->

</xsl:stylesheet>

<!-- vim:set shiftwidth=2: -->
