<?xml version="1.0"?>
<!-- @(#)90       1.8 src/43haes/usr/sbin/cluster/wsm/modules/navigation/sw/status.xsl, hacmp.wsm, 52haes_r541, 0736B_hacmp541 8/31/07 16:12:01 -->

<xsl:stylesheet
  xmlns:statemap="Translation of status codes"
  xmlns:imgmap="Translation of generic status to filenames"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xalan="http://xml.apache.org/xalan"
  exclude-result-prefixes="statemap imgmap"
  version="1.0">

  <!-- exclude-result-prefixes: prevents unwanted namespace declarations -->

  <xsl:include href="../status_common.xsl" />

  <!-- Create variables to more easily access the tables and to reduce the
  <!-  calls to the document function. -->
  <xsl:variable name="statetbl"
    select="document('../../status_table.xml')/statetable" />
  <xsl:variable name="imgtbl"
    select="document('../status_images.xml')/imagetable" />

  <!-- This language parameter can be set on the command line, with english as
  <!-  the default. -->
  <xsl:param name="language" select="'en'" />

  <!-- Declare what kind of output we are producing: HTML with indentation.
  <!-  -->
  <xsl:output method="html" indent="yes" />

  <!-- The name of this module -->
  <xsl:variable name="module_name" select="'Software'" />

  <!-- Where our images are located on the server. -->
  <xsl:variable name="imageroot" select="$imgtbl/root" />


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-
  <!-  Template: main (sorta)
  <!-
  <!-  Description:
  <!-
  <!-       This is the starting point of the processing, since it matches the
  <!-       root of the document.
  <!-
  <!-  -->
  <xsl:template match="/">
    <div class="twisty_tree" id="SoftwareNavTree">

      <ul class="twistyRoot">

        <xsl:apply-templates select="status/cluster" />

      </ul>
    </div>
  </xsl:template>




  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: cluster
  <!-
  <!-  -->
  <xsl:template match="cluster">
    <li>
      <xsl:call-template name="lineitem">
          <xsl:with-param name="tab_name" select="'RGs'" />
      </xsl:call-template>

      <ul class="collapsed">
        <xsl:apply-templates select="../resource_groups/resource_group" />
      </ul>
    </li>
  </xsl:template>



  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-
  <!-  Template: resource_group
  <!-
  <!-  -->
  <xsl:template match="resource_group">
    <xsl:param name="node_id" />

    <xsl:variable name="DOWN" select="'4'" />
    <xsl:variable name="UP"   select="'2'" />

    <!-- Determine the state for the resource group
    <!-
    <!-  Non-concurrent RGs:
    <!-
    <!-     If the resource group is up anywhere, then it is UP.
    <!-
    <!-  Concurrent:
    <!-     UP on all participating nodes:    UP
    <!-     DOWN on all participating nodes:  DOWN
    <!-     ERROR on all participating nodes: ERROR
    <!-     otherwise:                        MIXED
    <!-
    <!-  -->
    <li>
      <!-- <xsl:variable name="statecode"> -->
        <xsl:choose>

          <!-- Online on all available nodes (concurrent) -->
          <xsl:when test="StartupPolicy = 4">

            <!-- When an RG is a composite, the Javascript code will base its
            <!-  status icon on its status on all participating nodes. -->
            <xsl:call-template name="lineitem" >
              <!-- The type_name parameter is used to supply further
              <!-  information on resources.  But since this is not a resource,
              <!-  we can use it to indicate the type of resource GROUP. -->
              <xsl:with-param name="type_name" select="'concurrent'" />

              <xsl:with-param name="tab_name" select="'RGs'" />
              <xsl:with-param name="node_id"  select="$node_id" />
            </xsl:call-template>

          </xsl:when>

          <!-- 
          <!-  Non-concurrent
          <!- 
          <!-     UP on any node:    UP
          <!-     ERROR on any node: ERROR
          <!-     otherwise:         DOWN
          <!-
          <! -->
          <xsl:otherwise>

            <xsl:variable name="upNode" select="nodes/node[NodeState=$UP]" />
            <xsl:variable name="downNodes" select="nodes/node[NodeState != $DOWN]" />

            <xsl:choose>

              <!-- 
              <!-  If the RG is UP on at least one node, use that state 
              <!-->
              <xsl:when test="$upNode" >
                <xsl:variable name="state" select="$statetbl/resource_group/state[@code=$upNode/NodeState]" />

                <!-- Call the lineitem template with the appropriate state -->
                <xsl:call-template name="lineitem">
                  <xsl:with-param name="state" select="$state" />
                  <xsl:with-param name="tab_name" select="'RGs'" />
                  <xsl:with-param name="node_id"  select="$node_id" />
                </xsl:call-template>

              </xsl:when>

              <!--
              <!-  If the RG is DOWN on all nodes, then use that state 
              <!-->
              <xsl:otherwise>
                <!-- DOWN on all nodes, so return DOWN -->
                <xsl:variable name="state" select="$statetbl/resource_group/state[@code=$DOWN]" />

                <!-- Call the lineitem template with the appropriate state -->
                <xsl:call-template name="lineitem">
                  <xsl:with-param name="state" select="$state" />
                  <xsl:with-param name="tab_name" select="'RGs'" />
                  <xsl:with-param name="node_id"  select="$node_id" />
                </xsl:call-template>

              </xsl:otherwise>

            </xsl:choose>
          </xsl:otherwise>

        </xsl:choose>

        <!-- RGs do not expand in the N&N view, but keep this around until it can
        <!-  be used in the RG tab. -->
          <ul class="collapsed">
            <xsl:apply-templates select="resources/resource" />
            <xsl:apply-templates select="nodes/node" />
          </ul>
      </li>
    </xsl:template>


  <xsl:template match="network">
    <xsl:variable name="type" select="$typestbl/resource[@code=current()/Type]" />

    <li>
      <!-- Output the lineitem for this node with the appropriate parameters -->
      <xsl:call-template name="lineitem">
        <xsl:with-param name="state" select="$statetbl/generic/state[@name='none']" />
        <xsl:with-param name="leaf" select="true()" />

        <!-- Prefix the item with its type -->
        <xsl:with-param name="prefix">
          <!-- Get the type node and use the abbreviation -->
          <xsl:value-of select="$type/abbrev[@lang=$language]" />
        </xsl:with-param>

        <xsl:with-param name="type_code" select="Type" />
        <xsl:with-param name="type_name" select="$type/name" />

        <xsl:with-param name="tab_name" select="'RGs'" />
      </xsl:call-template>
    </li>

  </xsl:template>

  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-
  <!-  Template: resource
  <!-
  <!-  -->
  <xsl:template match="resource">
    <xsl:variable name="type" select="$typestbl/resource[@code=current()/Type]" />

    <li>
      <!-- Output the lineitem for this node with the appropriate parameters -->
      <xsl:call-template name="lineitem">
        <xsl:with-param name="state" select="$statetbl/generic/state[@name='none']" />
        <xsl:with-param name="leaf" select="true()" />

        <!-- Prefix the item with its type -->
        <xsl:with-param name="prefix">
          <!-- Get the type node and use the abbreviation -->
          <xsl:value-of select="$type/abbrev[@lang=$language]" />
        </xsl:with-param>

        <xsl:with-param name="type_code" select="Type" />
        <xsl:with-param name="type_name" select="$type/name" />

        <xsl:with-param name="tab_name" select="'RGs'" />
      </xsl:call-template>
    </li>

  </xsl:template>


  <!-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  <!-  
  <!-  Template: node
  <!-
  <!-  -->
  <xsl:template match="node">
    <xsl:variable name="id" select="@id" />
    <li>
      <xsl:call-template name="lineitem">
        <xsl:with-param name="leaf" select="true()" />
        <xsl:with-param name="state" select="$statetbl/node/state[@code=current()/NodeState]" />
        <xsl:with-param name="name" select="/status/nodes/node[@id=current()/@id]/Name" />
        <xsl:with-param name="prefix" select="'Node'" />

        <xsl:with-param name="tab_name" select="'RGs'" />
      </xsl:call-template>
    </li>
  </xsl:template>


</xsl:stylesheet>

<!-- vim:set sw=2: -->

