/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/* 52haes_r541 src/43haes/usr/sbin/cluster/wsm/modules/tooltips.js 1.5    */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* COPYRIGHT International Business Machines Corp. 2006,2007              */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* @(#)31       1.5 src/43haes/usr/sbin/cluster/wsm/modules/tooltips.js, hacmp.wsm, 52haes_r541, 0736A_hacmp541 5/31/07 08:26:28 */

/**************************************************************************
 *
 * File: tooltips.js
 *
 * Description:
 *
 *      This file defines the top-level TOOLTIP object that can be used by any
 *      module to manipulate the global tooltip HTML element and display
 *      tooltips for a given item.
 *
 *
 *************************************************************************/

if (is_undefined(TOOLTIP)) {

    /*
     * Define the global object
     */
    var TOOLTIP = {

        /* Data */
        div            : undefined,
        content        : undefined,

        /* Type-objects for the tooltip factory (See below) */
        cluster        : new Object(),
        site           : new Object(),
        nodes          : new Object(),
        node           : new Object(),
        network        : new Object(),
        service_label  : new Object(),
        ip_label       : new Object(),
        volume_group   : new Object(),
        resource_group : new Object(),
        application    : new Object()

    };

    
    /***********************************************************************
     * Method: mDiv
     *
     * Description:
     *
     *      Returns a reference to the DIV element for the global tooltip
     *
     */
    TOOLTIP.mDiv = function() {
        
        if (this.div == undefined) {
                this.div = top.document.getElementById('ttip');
            }
        return this.div;
    };


    /***********************************************************************
     * Method: mContent
     *
     * Description:
     *
     *      Returns a reference to the content DIV for the tooltip
     *
     */
    TOOLTIP.mContent = function() {
        if (this.content == undefined) {
            this.content = top.document.getElementById('ttcontent');
        }
        return this.content;
    };


    /***********************************************************************
     * Method: show
     *
     * Description:
     *
     *      Places the tooltip according to where the event took place (ex.
     *      near the mouse click) and displays it.  
     *
     * Arguments:
     *
     *      e : the event object causing the tooltip to be displayed
     *      x : optional x coordinate to override the location of the event
     *      y : optional y coordinate to override the location of the event
     */
    TOOLTIP.show = function(e, x, y) {

        if (isIE()) e = event;

        var MIN_BOX_WIDTH = 200; /* This should be the same as in the style sheet */
        var MAX_BOX_WIDTH = 300; /* This should be the same as in the style sheet */
        var SHIFT_X       = 10;
        var SHIFT_Y       = -5;
        var winWidth      = BRIL.getInnerWinWidth(top);
        var winHeight     = BRIL.getInnerWinHeight(top);

        /* Add a link to close the tip. */


        /* If the coordinates were not passed in, then use those from the
         * event.  */
        if ( !x ) x = e.clientX;
        if ( !y ) y = e.clientY;

        /* Get the framework frame we are in.  This needs a loop so we can
         * figure it out from within iframes of any depth. */
        var frame = window;
        while ( !frame.parent.HeaderFrame && frame != top ) {
            frame = frame.parent;
        }

        /* Set the tooltip location to be relative to the framework.  If we're
         * in the navigation frame, adjust x.  If we're in the activity frame,
         * adjust both. */
        switch (frame.name) {
        case 'ActivityFrame':
            x += BRIL.getInnerWinWidth(parent.NavigationFrame);
        case 'NavigationFrame':
            y += BRIL.getInnerWinHeight(parent.HeaderFrame);
            break;
        default:
        }


        /* Adjust the 'x' axis.  Keep the tooltip from being scrunched against
         * the right side of the window by making sure there is room for the
         * max box width.
         *
         * This also circumvents another shortcomming of IE: it doesn't support
         * the "min-width" style.  If it did, we could just set the minimum
         * width of the tooltip to something reasonable, then reposition it
         * accordingly.  But instead, we'll just give it enough room ahead of
         * time, so it shouldn't become too narrow.
         *
         */
        if (x + MAX_BOX_WIDTH > winWidth) {
            x = winWidth - MAX_BOX_WIDTH;
        }

        /* Position the tooltip */
        this.mDiv().style.left = x + SHIFT_X + 'px';
        this.mDiv().style.top  = y + SHIFT_Y + 'px';

        /* Show the tooltip.  NOTE: We can't get the calculated width or height
         * when the display is still 'hidden'.  But we need those values in
         * order to ensure the tooltip box is in plain view. */
        this.mDiv().style.display = 'block';


        /* 
         * Get the dimensions of the tooltip
         */
        if (isMoz()) {
            var view = document.defaultView;
            var ttStyle = view.getComputedStyle(this.mDiv(), null);

            var ttleft   = parseInt(ttStyle.getPropertyValue("left"));
            var tttop    = parseInt(ttStyle.getPropertyValue("top"));
            var ttwidth  = this.mDiv().offsetWidth;
            var ttheight = this.mDiv().clientHeight;
            var ttright  = ttleft + ttwidth;
            var ttbottom = tttop + ttheight;
        }
        else {
            var ttleft   = parseInt(this.mDiv().currentStyle.left);
            var tttop    = parseInt(this.mDiv().currentStyle.top);
            var ttwidth  = this.mDiv().offsetWidth;
            var ttheight = this.mDiv().offsetHeight;
            var ttright  = ttleft + ttwidth;
            var ttbottom = tttop + ttheight;
        }

        /* Now fix the placement of the tip, if needed */

        var space = winWidth - x;  /* space between the mouse and the right edge */
        var EDGE_BUFFER = 5;

        var bottomBuffer = winHeight - ttbottom;
        if (bottomBuffer < EDGE_BUFFER) {
            top.debug("*** adjusting tooltip bottom for edge");
            if (bottomBuffer < 0 && tttop < -bottomBuffer) {
                tttop = EDGE_BUFFER;
                this.mDiv().style.top = tttop;
            }
            else {
                tttop = winHeight - ttheight - EDGE_BUFFER;
                this.mDiv().style.top = tttop;
            }
        }

        /* Here's another kludge for IE.  'Select' input objects (dropdowns)
         * are "windowed" objects, which means they are drawn above
         * "non-windowed" objects, which is just about everything else.  The
         * only thing that can cover them (via z-index) is an iframe.  That
         * means a plain ol' div can't cover them, which means our tooltip
         * can't cover them.
         *
         * So, in order to show tooltips above everything, including dropdowns,
         * we need to "back" it with an empty iframe.  The iframe CAN cover
         * dropdowns, and our tooltip div CAN cover the iframe. */
        if (isIE()) {
            this.ttiframe = top.document.getElementById('IE_tt_iframe');

            this.ttiframe.style.display = 'block';
            this.ttiframe.style.left = ttleft;
            this.ttiframe.style.top = tttop;
            this.ttiframe.width = ttwidth;
            this.ttiframe.height = ttheight;
        }

    };

    /***********************************************************************
     * Method: createElement
     *
     * Description:
     *
     *      Utility function to create a DOM element for the tooltip
     *
     */
    TOOLTIP.createElement = function(type) {
        /* IE needs the 'top' */
        return top.document.createElement(type);
    };


    /***********************************************************************
     * Method: clear
     *
     * Description:
     *
     *      Clear the content of the tooltip.
     *
     */
    TOOLTIP.clear = function() {
        var tt = this.mContent();
        while (tt.firstChild) {
            tt.removeChild(tt.firstChild);
        }
    };


    /***********************************************************************
     * Method: hide
     *
     * Description:
     *
     *      Utility function to hide the tooltip
     *
     */
    TOOLTIP.hide = function() {

        /* Hide the IE-only backer frame before hiding the tooltip itself */
        if (isIE()) {
            this.ttiframe = top.document.getElementById('IE_tt_iframe');
            this.ttiframe.style.display = 'none';
        }

        this.mDiv().style.display = 'none';
    };



    /***********************************************************************
     * Method: toolipContent
     *
     */
    TOOLTIP.tooltipContent = function(target) {

        var tt_type = target.getAttribute('component_type');

        if ('resource' == tt_type) {
            tt_type = target.getAttribute('type_name');
        }

        /* Sanity checks */
        if ( ! tt_type ) {
            throw("Undefined 'component_type' in item");
            return null;
        }
        if ( ! this[tt_type] ) {
            top.debug("Cannot create tooltip for item of type '"
                      + tt_type + "'");
            return null;
        }

        top.debug("Tooltip request for item of type: " + tt_type);

        /* Get the array of actions associated with the item. */
        var actions = this[tt_type].mktt(target);

        if ( !actions.length ) {
            return null;
        }

        var content = TOOLTIP.createElement('div');

        /* This function will be used as the onclick handler for all tooltip
         * actions. */
        var ttselect = function(e) {
            var tgt = BRIL.etarget(e);
            (new Message_SMITSelect(tgt.href)).send();
            BRIL.stopEventBubble(e);
            TOOLTIP.hide();
            return false;
        };

        /* Process the tooltip actions and build the tooltip */
        var elem;
        for (var i in actions) {
            elem = TOOLTIP.createElement('a');
            elem.innerHTML = actions[i].text;
            elem.href = actions[i].href;
            elem.onclick = ttselect;
            content.appendChild(elem);
            content.appendChild(TOOLTIP.createElement('br'));
        }

        return content;
    };



    /***********************************************************************
     * Method: tooltipAction
     *
     * Description:
     *
     *      Handle a click on an item in the tooltip.  Use the event object
     *      parameter to get information about the item clicked.
     */
    TOOLTIP.tooltipAction = function(e) {
        var obj = BRIL.etarget(e);
        (new Message_SMITSelect(obj.href)).send();
        BRIL.stopEventBubble(e);
        TOOLTIP.hide();
        return false;
    };



    /* **********************************************************************
     * **********************************************************************
     *
     * Tooltip Factory
     *
     * The global tooltip object (TOOLTIP) contains an object for each type of
     * cluster component that has tooltip actions associated with it.  Each of
     * those objects (currently) contains only a function that builds the
     * tooltip on the fly.
     *
     * These dynamic tooltips allow us to inject special logic that can make
     * them truely dynamic.  The only example at this point is resource groups:
     * if it is online, then there will be an action to take it offline, and
     * vice-versa.
     *
     */

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for the cluster
     *
     */
    TOOLTIP.cluster.mktt = function(elem) {

        var a;
        var actions = new Array();
        var clustername = elem.getAttribute('component_name');

        /*
         * Add/Change/Show
         */
        actions.push( (a = new Object()) );
        a.text = 'Add/Change/Show an HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_add_change_show_an_hacmp_cluster.dialog';
        a.href += '&cmd_to_classify=cl_resgrp_move_node.select';
        a.href += '&cname=' + clustername;

        /*
         * Remove an HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove an HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_delete_an_hacmp_cluster_delete.dialog';
        a.href += '&cname=' + clustername;

        /*
         * Add a Site
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Site';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_add_site';

        /*
         * Add a Node to the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Node to the HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_add_a_node_to_the_hacmp_cluster_dialog';

        /*
         * Start Cluster Services
         */
        actions.push( (a = new Object()) );
        a.text = 'Start Cluster Services';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clstart';
        a.href += '&how=ALLNODES';

        /*
         * Stop Cluster Services
         */
        actions.push( (a = new Object()) );
        a.text = 'Stop Cluster Services';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clstop';
        a.href += '&how=ALLNODES';

        /*
         * Show Cluster Services
         */
        actions.push( (a = new Object()) );
        a.text = 'Show Cluster Services';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clshowsrv.dialog';

        /*
         * Discover HACMP-related Information from Configured Nodes
         */
        actions.push( (a = new Object()) );
        a.text = 'Discover HACMP-related Information from Configured Nodes';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_discover_hacmp_related_information_from_configured_nodes_menu';

        /*
         * Verify and Synchronize HACMP Configuration
         */
        actions.push( (a = new Object()) );
        a.text = 'Verify and Synchronize HACMP Configuration';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_ver_and_sync.select';

        return actions;
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for sites
     *
     */
    TOOLTIP.site.mktt = function(elem) {
        var a;
        var actions = new Array();
        var sitename = elem.getAttribute('component_name');


        /*
         * Add a Site
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Site';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_add_site';

        /*
         * Change/Show a Site
         */
        actions.push( (a = new Object()) );
        a.text = 'Change/Show a Site';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_change_site.cmdhdr';
        a.href += '&cmd_to_classify=cm_change_site.select';
        a.href += '&raw_field_name=site';
        a.href += '&site=' + sitename;

        /*
         * Remove a Site
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove a Site';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_del_site.cmdhdr';
        a.href += '&cmd_to_classify=cm_del_site.select';
        a.href += '&_rawname=' + sitename;
        a.href += '&site=' + sitename;

        return actions;
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for the nodes composite
     *
     */
    TOOLTIP.nodes.mktt = function(elem) {
        var a;
        var actions = new Array();
        var clustername = elem.getAttribute('component_name');

        /*
         * Start Cluster Services
         */
        actions.push( (a = new Object()) );
        a.text = 'Start Cluster Services';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clstart';
        a.href += '&how=ALLNODES';

        /*
         * Stop Cluster Services
         */
        actions.push( (a = new Object()) );
        a.text = 'Stop Cluster Services';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clstop';
        a.href += '&how=ALLNODES';

        /*
         * Remove Nodes from the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove Nodes from the HACMP Cluster';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_delete_a_node_in_the_hacmp_cluster_select';

        /*
         * Add a Node to the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Node to the HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_add_a_node_to_the_hacmp_cluster_dialog';

        return actions;
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for individual nodes
     *
     */
    TOOLTIP.node.mktt = function(elem) {
        var a;
        var actions = new Array();
        var nodename = elem.getAttribute('component_name');

        /*
         * Start Cluster Services
         */
        actions.push( (a = new Object()) );
        a.text = 'Start Cluster Services';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clstart';
        a.href += '&_rawname=' + nodename;
        a.href += '&how=' + nodename;

        /*
         * Stop Cluster Services
         */
        actions.push( (a = new Object()) );
        a.text = 'Stop Cluster Services';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clstop';
        a.href += '&_rawname=' + nodename;
        a.href += '&how=' + nodename;

        /*
         * Change/Show a Node in the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Change/Show a Node in the HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_change_show_a_node_in_the_hacmp_cluster.dialog';
        a.href += '&cmd_to_classify=cm_change_show_a_node_in_the_hacmp_cluster_select';
        a.href += '&_rawname=' + nodename;

        /*
         * Remove a Node from the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove a Node from the HACMP Cluster';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_delete_a_node_in_the_hacmp_cluster_select';

        /*
         * Add a Node to the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Node to the HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_add_a_node_to_the_hacmp_cluster_dialog';

        return actions;
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for networks
     *
     */
    TOOLTIP.network.mktt = function(elem) {
        var a;
        var actions = new Array();
        var netname = elem.getAttribute('component_name');

        /*
         * Add a Network to the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Network to the HACMP Cluster';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&name=cm_add_a_network_to_the_hacmp_cluster_select';

        /*
         * Change/Show a Network in the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Change/Show a Network in the HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_change_show_a_networkdialog.';
        a.href += '&cmd_to_classify=cm_change_show_a_network_in_the_hacmp_cluster_select';
        a.href += '&_rawname=' + netname;

        /*
         * Remove a Network from the HACMP Cluster
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove a Network from the HACMP Cluster';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_delete_a_network_from_the_hacmp_cluster_dialog';
        a.href += '&cmd_to_classify=cm_delete_a_network_from_the_hacmp_cluster_select';
        a.href += '&_rawname=' + netname;

        /*
         * Add Communications Interfaces/Devices
         */
        actions.push( (a = new Object()) );
        a.text = 'Add Communications Interfaces/Devices';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&name=cm_add_communication_interfaces_devices.select';

        return actions;
    };



    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for SVC IP labels
     *
     */
    TOOLTIP.service_label.mktt = function(elem) {
        var a;
        var actions = new Array();
        var svcname = elem.getAttribute('component_name');

        /*
         * Add a Service IP Label/Address
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Service IP Label/Address';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_add_a_service_ip_label_address.select';

        /*
         * Change/Show a Service IP Label/Address
         */
        actions.push( (a = new Object()) );
        a.text = 'Change/Show a Service IP Label/Address';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_change_show_a_service_ip_label_address.dialog.';
        a.href += '&cmd_to_classify=cm_change_show_a_service_ip_label_address.select';
        a.href += '&_rawname=' + svcname;

        /*
         * Remove Service IP Label(s)/Address(es)
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove Service IP Label(s)/Address(es)';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_delete_service_ip.dialog';
        a.href += '&cmd_to_classify=cm_delete_service_ip.select';
        a.href += '&_rawname=' + svcname;

        return actions;
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for applications
     *
     */
    TOOLTIP.application.mktt = function(elem) {
        var a;
        var actions = new Array();

        var appname = elem.getAttribute('component_name');

        /*
         * Add Application Server
         */
        actions.push( (a = new Object()) );
        a.text = 'Add Application Server';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=claddserv.dialog';

        /*
         * Change/Show an Application Server
         */
        actions.push( (a = new Object()) );
        a.text = 'Change/Show an Application Server';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=clchserv.dialog';
        a.href += '&cmd_to_classify=clchserv.select';
        a.href += '&_rawname=' + appname;

        /*
         * Remove an Application Server
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove an Application Server';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_config_server.del';
        a.href += '&cmd_to_classify=cm_config_server.del.select';
        a.href += '&_rawname=' + appname;

        /*
         * Suspend Application Monitor(s) for Application Server
         */
        actions.push( (a = new Object()) );
        a.text = 'Suspend Application Monitor(s) for Application Server';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_suspend_appmon.dialog';
        a.href += '&cmd_to_classify=cm_suspend_appmon.select';
        a.href += '&_rawname=' + appname;

        /*
         * Resume Application Monitoring
         */
        actions.push( (a = new Object()) );
        a.text = 'Resume Application Monitoring';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_resume_appmon.dialog';
        a.href += '&cmd_to_classify=cm_resume_appmon.select';
        a.href += '&_rawname=' + appname;

        return actions;
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for IP labels
     *
     */
    TOOLTIP.ip_label.mktt = function(elem) {
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for volume groups
     *
     */
    TOOLTIP.volume_group.mktt = function(elem) {
        var a;
        var actions = new Array();

        var vgname = elem.getAttribute('component_name');

        /*
         * Add a Volume to a Shared Volume Group
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Volume to a Shared Volume Group';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cl_extendvg1';
        a.href += '&cmd_to_classify=cl_extendvg';
        a.href += '&_rawname=' + vgname;

        /*
         * Remove a Volume from a Shared Volume Group
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove a Volume from a Shared Volume Group';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cl_reducevg1';
        a.href += '&cmd_to_classify=cl_reducevg';
        a.href += '&_rawname=' + vgname;

        /*
         * Synchronize a Shared Volume Group Definition
         */
        actions.push( (a = new Object()) );
        a.text = 'Synchronize a Shared Volume Group Definition';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cl_updatevg.hdr';

        return actions;
    };


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *
     * Tooltip for resource_groups
     *
     */
    TOOLTIP.resource_group.mktt = function(elem) {

        var a;
        var statusImg = undefined;
        var online = false;
        var actions = new Array();

        var groupname = elem.getAttribute('component_name');

        /* If we were called from the activity frame, we can't get the status
         * of the clicked item (not supported yet). */
        if (NAVIGATION == frameType()) {
            statusImg = ModuleStatus.prototype.getStatusImage(elem.parentNode);
            online = /ERROR|OFFLINE/.test(statusImg.title)
        }

        /*
         * Add a Resource Group
         */
        actions.push( (a = new Object()) );
        a.text = 'Add a Resource Group';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
        a.href += '&name=cm_add_a_resource_group_dialog.custom';

        /*
         * Remove a Resource Group
         */
        actions.push( (a = new Object()) );
        a.text = 'Remove a Resource Group';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_del_grp';
        a.href += '&cmd_to_classify=cm_delete_a_resource_group_select';
        a.href += '&_rawname=' + groupname;

        /*
         * Change/Show a Resource Group
         */
        actions.push( (a = new Object()) );
        a.text = 'Change/Show a Resource Group';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_chg_grp.custom.';
        a.href += '&cmd_to_classify=cm_change_a_resource_group_select';
        a.href += '&_rawname=' + groupname;

        /*
         * Change/Show Resources and Attributes for a Resource Group
         */
        actions.push( (a = new Object()) );
        a.text = 'Change/Show Resources and Attributes for a Resource Group';
        a.href = '/cgi-bin/wsm_cmd_hdr.cgi?tt_id=1';
        a.href += '&panel_id=cm_change_a_resource_group_dialog.';
        a.href += '&cmd_to_classify=cm_change_resources_for_a_rg_select';
        a.href += '&_rawname=' + groupname;

        /* 
         * Move a Resource Group to Another Node
         */
        actions.push( (a = new Object()) );
        a.text = 'Move a Resource Group to Another Node';
        a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1'
        a.href += '&panel_id=cl_resgrp_move.select_node';
        a.href += '&cmd_to_classify=cl_resgrp_move_node.select';
        a.href += '&raw_field_name=group_name';
        a.href += '&group_name=' + groupname;

        /*
         * Bring a Resource Group Online / Offline
         */
        actions.push( (a = new Object()) );
        a.text = 'Bring a Resource Group ';

        if (statusImg) {
            if (online) {
                a.text += 'Online';
                a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
                a.href += '&panel_id=cl_resgrp_start.select_node_';
                a.href += '&cmd_to_classify=cl_resgrp_start.select';
                a.href += '&raw_field_name=group_name';
                a.href += '&group_name=' + groupname;
            }
            else {
                a.text += 'Offline';
                a.href = '/cgi-bin/wsm_name_hdr.cgi?tt_id=1';
                a.href += '&name=cl_resgrp_stop.select_node';
                a.href += '&cmd_to_classify=cl_resgrp_stop.select';
                a.href += '&raw_field_name=group_name';
                a.href += '&group_name=' + groupname;
            }
        }
        else {
            a.text = 'HACMP Resource Group and Application Management';
            a.href = '/cgi-bin/wsm_menu_opt.cgi?tt_id=1';
            a.href += '&name=cm_hacmp_resource_group_and_application_management_menu';
        }

        return actions;
    };

}
