# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/utils/cldata.pm 1.8 
#  
# Licensed Materials - Property of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2006,2007 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
#
# File: cldata.pm
#
# Description:
#
#       This module contains functions for retrieving information about the
#       cluster.
#
#

# @(#)96       1.8 src/43haes/usr/sbin/cluster/wsm/utils/cldata.pm, hacmp.wsm, 52haes_r541, 0736A_hacmp541 7/11/07 16:37:55

package cldata;
require Exporter;

use lib "/usr/es/sbin/cluster/wsm/utils";
use wsm_config qw(get_simulator_mode);

our @EXPORT_OK = qw();

sub getActiveNode(;$);
sub getNodeCommPath($);
sub getSnmpInformation($);

###########################################################################
# Function:  getActiveNode
#
# Description:
#
#       Gets an active node from which we can then request MIB information.  If
#       the local node is active, that will be returned.
#
# Returns:
#  
#	    The local nodename, if active.  Otherwise, the first in the list of
#	    active nodes.
#
# Arguments:
#
#       $1 - Optional boolean specifying whether or not to use the cached
#            value.  Default: 0.
#
sub getActiveNode(;$)
{
    my $node       = "";
    my $nocache    = 0;
    my $local_mgr  = 0;
    my $sessionPrg = "/usr/es/sbin/cluster/wsm/cgi-bin/wsm_session";
    my $localnode  = qx(/usr/es/sbin/cluster/utilities/get_local_nodename);

    ($localnode) = $localnode =~ m/(\w*)/; # Untaint and remove traling '\n'

    unless ($localnode) { return ""; }

    # Get the optional 'no cache' arg
    if (@_) { $nocache = shift; }

    # See if the local cluster manager is active.
    $_ = getClusterMgrState();
    unless (/ST_INIT/) {
        $local_mgr = 1;
    }

    # If the 'nocache' flag was not passed, use the cached value (if any).
    # This will reduce the number of IPC calls generated by clgetactivenodes.
    #
    # If the cached node is not the local node but the local cluster mgr is
    # active, then do not use the cached value; we should be able to revert to
    # using the local node in that case.
    #
    unless ($nocache) {
        $node = qx($sessionPrg -g activeNode);
        chomp $node;
        ($node) = $node =~ m/(\w*)/;
        if ($node) {
            if ($node eq $localnode || ! $local_mgr) {
                return $node;
            }
        }
    }

    # If we've gotten to this point, then we're not using the cached value, and
    # we need to find an active node.  We should always prefer the local node,
    # so the list of active nodes is initialized with the results from the
    # local node.

    my @nodes = qx(/usr/es/sbin/cluster/utilities/clnodename);
    my @activeNodes = qx(/usr/es/sbin/cluster/utilities/clgetactivenodes -n $localnode);
    chomp (@nodes);
    chomp $localnode;

    # If we got a list from the localnode, then it's active.  Otherwise, try
    # all cluster nodes until we get a list of active nodes.
    if (@activeNodes) {
        $node = $localnode;
    }
    else {
        foreach (@nodes) { 
            ($_) = m/(\w*)/;  # untaint
            @activeNodes = qx(/usr/es/sbin/cluster/utilities/clgetactivenodes -n $_);
            if (@activeNodes) {
                $node = $activeNodes[0];
                ($node) = $node =~ m/(\w*)/;  # untaint
                last;
            }
        }
    }

    # Cache the active node to the session.
    system $sessionPrg, "-s", "activeNode=$node";

    #================================================================
    # To improve the odds of debugging a problem, tie the session
    # to the host that is generating it (i.e. where the web browser
    # is running) by caching the remote system's IP address.
    #================================================================
    if (defined($ENV{REMOTE_ADDR})) {
        'NULL' =~ /()/;   # Initialize the "$1" variable
        $ENV{REMOTE_ADDR} =~ /^(\d+\.\d+\.\d+\.\d+)$/; # untaint
        system $sessionPrg, "-s", "remote_addr=$1" if ($1 ne "");
    }

    return $node;
}

###########################################################################
# Function:  getSnmpInformation
#
# Description:
#
#	Gathers cluster information from the MIB via snmpinfo.  The active node
#	used for the request is determined automatically.
#
# Arguments:
#
#   $1 = the name of the MIB variable to request
#
# Returns:
#  
#	An array of SNMP lines, as returned by snmpinfo.
#
#
sub getSnmpInformation($)
{
    my $variable = shift;
    my $node;
    my $nocache = 0;
    my @snmpinfo;
    my @activeHost;
    my $communityName;
    my $platform = qx(uname -s);
    chomp ($platform);

    # If we are in simulator mode, we bypass everything and go directly to 
    # using the clsnmpinfo tool to get the cluster state.
    if (wsm_config::get_simulator_mode() == 1)
    {
       @snmpinfo = qx(/usr/es/sbin/cluster/utilities/clsnmpinfo -l -v -d /var/hacmp/simulator/objrepos -m dump -o /usr/es/sbin/cluster/hacmp.defs $variable);
       return (@snmpinfo);
    }

    # Request the MIB data.  If we did not get it the first time, then it may
    # be because the cached 'active node' is no longer active.  So, the next
    # time around, the cache will be reset and an new active node will be used.
    until (@snmpinfo || $nocache > 1) {
        $node = getActiveNode($nocache);
        unless ($node) { next; }

        if ($platform == "Linux") {
            @snmpinfo = qx(/usr/es/sbin/cluster/utilities/clrexec $node SNMPINFO -v $variable);
        }
        else {
            $communityName = qx(/usr/es/sbin/cluster/utilities/cl_community_name);
            chomp ($communityName);
            ($communityName) = $communityName =~ m/(\w*)/;  # untaint

            $activeHost = getNodeCommPath($node);
            @snmpinfo = qx(/usr/sbin/snmpinfo -h $activeHost -v $communityName -m dump -o /usr/es/sbin/cluster/hacmp.defs $variable);
        }
    }
    continue {
        # If there was no data returned, then incrementing to 1 should work.
        # If not, then it will be 2, and we will fall through without data.
        unless (@snmpinfo) { $nocache++; }
    }

    if ( $node && ! @snmpinfo ) {
        print STDERR "Can't read MIB from node '$node'!\n";
        return (1);       
    }

    return (@snmpinfo);
}

###########################################################################
# Function: getNodeCommPath
#
# Description:
#
#   Utility function to get the communication path of a node from the ODM.  The
#   node name may not be resolvable, or may not be the same as the hostname, so
#   the value returned by this function should be used to communicate with
#   other nodes.
#
sub getNodeCommPath($)
{
    my $node = shift;

    # Get the communication path for the node, since the node name may not
    # necessarily be the hostname.
    my $HACMPnode = new ODM("HACMPnode");
    my @commpaths = $HACMPnode->get( -q => {
            "name"   => $node,
            "object" => "COMMUNICATION_PATH"
        } );
    return $commpaths[0]->{value};
}

###########################################################################
# Function: getClusterMgrState
#
# Description:
#
#   Utility function to return the state string of the clstrmgr process, as
#   reported by the SRC.
#
#   Example states: ST_STABLE, ST_INIT.
#
#
sub getClusterMgrState {
    foreach (qx(lssrc -ls clstrmgrES)) {
        if (/Current state: (.*)/) {
            return $1;
        }
    }
}

1;
