#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/utils/p3.pm 1.3 
#  
# Licensed Materials - Property of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2006,2007 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)95       1.3 src/43haes/usr/sbin/cluster/wsm/utils/p3.pm, hacmp.wsm, 52haes_r541, 0736A_hacmp541 5/29/07 15:24:50



############################################################################
#
# Perl Pre-Processor (p3)
#
#
# Pre-process a file, executing embedded Perl along the way.
#
# This is a poor-man's PHP or ASP, but with Perl.  Perl code can be embedded
# into any file, and its result will be printed in its place.
#
# The embedded Perl code is surrounded by identifying tokens (like '<%p3' and
# '%>') and can span multiple lines.  The value RETURNED by that code will be
# inserted into the document; so, do not use 'print' statements or things will
# probably not look the way you think they will.
#
# Misc notes on the embedded code:
#
#   - All functions defined by this module are available to embedded code
#   - CGI environment variables are available
#   - Always RETURN what should be part of the document.  If the code is not
#     meant to add output (maybe it just defines a function for later use),
#     then a simple 'return;' at the end of a block will return an undef.  A
#     null string can also be returned in this case.
#   - Messages can be logged to error_log by printing to STDERR
#   - Messages can be logged using the logger module
#
#
#
# Functions:
#
#   proc_file   - pre-process a file by name
#   get_parsed  - ditto
#   get_file    - get the contents of a file as-is
#
#
# The main motivation was so authentication could be verified before any file is
# returned by the web server.  This would normally be done BY apache/IHS, but
# we are doing our own system-level authentication and authorization.
#
#

package p3;
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(
    procfile
    get_file
    get_parsed
);

use strict;
#use warnings;

use File::Basename;
use Cwd;
use subs;

use logger;  # Should already be in the path if this module has been loaded.


my $pre = "P3";

#
# Utility functions that may be used from the embedded code.
#
sub get_file($);
sub get_parsed($);

# The (case-insensitive) tokens that mark the embedded code.  They are "public
# members", meaning they can be overridden.
our $START_TOKEN = qr/(?i)<%p3/;
our $STOP_TOKEN = qr/%>/;



# Function: procfile
#
# Description:
#
#       Pre-process a file and print the result.
#
#
sub procfile($) {
    print get_parsed shift;
}


# Function: get_parsed
#
# Description:
#
#       Pre-process the given file and return the processed contents.  Takes
#       the file as the only argument.
#
#
sub get_parsed($) {

    my $input = shift;
    local *P3;

    # Make sure the provided file is valid and open it for reading.
    if (defined $input) {
        unless (-f $input) {
            die "$pre: Cannot process file '$input'";
        }
        # Print to STDERR so we can more easily associate errors with the file
        # whose embedded perl caused on error/warning.
        logger("$pre: Processing file '$input'");
    }
    else { die "$pre: Cannot process; filename not provided."; }

    open P3, "<", $input or die "Cannot open file '$input'\n";


    # Change to the directory of the parsed file.  This will allow for
    # embedded code to do things in a way that is relative to that file (as
    # opposed to being relative to THIS file).  This should not fail, since
    # we've already checked $input.  Store the current directory so we can
    # return to it when we're done; this is needed for recursive/embedded
    # calls.
    my $origdir = getcwd();
    chdir(dirname $input);


    # Slurp the file.  This will allow for one replace pass, instead of
    # building a string from multiple lines when there is a multi-line
    # embedded script.
    { local $/; $_ = <P3>; }

    s{ ($START_TOKEN)(.*?)($STOP_TOKEN) }{

        # Evaluate the embedded code
        my $res=eval($2);

        # Quit with an error if the evaluated code was bad
        die "ERROR in file '$input': $@" if $@;

        # Allow empty return at end of p3 block
        $res if defined $res;

    }xsge;

    chdir $origdir;  # Restore our current location
    return $_;
}

# Function: get_file
#
# Description:
#
#       Include the straight/un-parsed contents of a file.
#
# Returns:
#
#       The contents of the file.
#
#
sub get_file($) {

    my $input = shift;
    my $out = '';
    local *IN;  # local file handle

    if (defined $input) {
        unless (-f $input) {
            die "$pre: ERROR: Cannot include file '$input'";
        }
        logger("$pre: Including file '$input'");
    }
    else {
        die "$pre: ERROR: Cannot include; filename not provided.";
    }

    if ( !open IN, "<", $input ) {
        die "$pre: ERROR: Cannot open file '$input' for reading\n";
    }
    while (<IN>) { $out .= $_; }
    close IN;

    return $out;
}

1;
