# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/utils/wsm_auth.pm 1.9 
#  
# Licensed Materials - Property of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2006,2007 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

# @(#)96       1.9 src/43haes/usr/sbin/cluster/wsm/utils/wsm_auth.pm, hacmp.wsm, 52haes_r541, 0736A_hacmp541 5/31/07 13:30:12

package wsm_auth;

use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
    access_denied
    check_authorized_port
    print_authorized_page_header
    check_https_redirect
    process_login
    log_msg
);

use logger;  # Should already be in the path if this module has been loaded.


##############################################################################
# Function: access_denied
#
# Description: Prints out an access denied page.
#
# Arguments: None
#
# Returns: None
##############################################################################
sub access_denied ( ) 
{
    print <<END;
Status: 401
Content-type: text/html

<HTML>
<HEAD><TITLE>Access Denied</TITLE></HEAD>
<BODY>
<FONT FACE="arial,helvetica">
<B>Error 401: Access denied.</B>
</FONT>
</BODY>
</HTML>
END
    exit 1;
} # End of "access_denied()"


##############################################################################
# Function: check_authorized_port
#
# Description:
# Checks the requested port number against the AUTHORIZED_PORT variable.
# If the requested port is different than AUTHORIZED_PORT, then an access
# denied page is displayed.  If AUTHORIZED_PORT is 0 or not set, then we
# don't run this check.
#
# Arguments: None
#
# Returns: Exits 1 if incorrect port.
##############################################################################
sub check_authorized_port ( )
{
    if (($ENV{'AUTHORIZED_PORT'} ne "") && ($ENV{'AUTHORIZED_PORT'} != 0)) {
        my $port = $ENV{'SERVER_PORT'};
        my $authport = $ENV{'AUTHORIZED_PORT'};
        if ($port != $authport) {
            log_msg("Server port $port not equal to authorized port $authport.\n");
            access_denied();
            exit 1;
        }
    }
} # End of "check_authorized_port()"


##############################################################################
# Function: check_https_redirect
#
# Description:
# If the REDIRECT_TO_HTTPS variable is set to 1 then we will check to see if
# the connection came in over HTTPS.  If it didn't come in over HTTPS, then
# we will redirect the user to the root web document via HTTPS over the
# port that is specified by the AUTHORIZED_PORT variable.  If AUTHORIZED_PORT
# is not set, then we will use the current port, unless the current port is
# 80 (default HTTP) in which case we will use 443 (default HTTPS).
#
# Arguments: None
#
# Returns: Exits 1 if redirecting.
##############################################################################
sub check_https_redirect ( )
{
    my $port = 443;
    (my $server) = $ENV{'SERVER_NAME'} =~ /([\w.]+)(:\d+)?/;

    #
    # From the apache2 documentation:
    #
    #     REDIRECT_ environment variables are created from the CGI environment
    #     variables which existed prior to the redirect, they are renamed with a
    #     REDIRECT_  prefix, i.e., HTTP_USER_AGENT becomes
    #     REDIRECT_HTTP_USER_AGENT. In addition to these new variables, Apache
    #     will define REDIRECT_URL  and REDIRECT_STATUS.
    #
    # There may be multiple redirects, so this loop builds a variable prefix
    # string based on how many redirects there have been.  So, we'll end up with
    # a string like "REDIRECT_REDIRECT_", which we can then use to check for the
    # original value of the HTTPS variable.
    #
    my $v   = "STATUS";    # The variable we're checking
    my $R   = "REDIRECT_"; # The prefix
    my $pre = "";          # The whole prefix, based on how many redirects there were
    while (defined $ENV{"$R$pre$v"}) {
        $pre = $R . $pre;
    }

    # If HTTPS is not being used, redirect to https.
    if ( $ENV{'REDIRECT_TO_HTTPS'} == 1 &&
         $ENV{'HTTPS'} !~ /on/i         &&
         $ENV{"${pre}HTTPS"} !~ /on/i )
    {
        if ($ENV{'AUTHORIZED_PORT'} ne "") {
            $port = $ENV{'AUTHORIZED_PORT'};
        } elsif ($ENV{'SERVER_PORT'} != 80) {
            $port = $ENV{'SERVER_PORT'};
        }

        # Printing a "Location" header will cause the webserver to print out the
        # rest of the required headers and appropriately redirect the web browser.
        print "Location: https://$server:$port/\n\n";
        exit 1;
    }
} # End of "check_https_redirect()"


##############################################################################
# Function: print_authorized_page_header
#
# Description:
# Checks the session cookie to see if it is valid.
# If htaccess is being used, or REQUIRE_AUTHENTICATION is not 1, then we
# don't run this check.
# If the session cookie timed out or doesn't exist, then the user will be
# given a login page.
# If the session cookie is invalid, then an access denied page will be
# displayed.
# If the session cookie is valid, then a new cookie will be generated and
# set in the response.
#
# Arguments:
#
#	site (optional): this is the site that should be loaded if and only
#	                 if the user is successfully authenticated.
#
# Returns: Exits 1 if invalid session.
#          Returns if valid
#
##############################################################################
sub print_authorized_page_header (;$)
{
    my($site) = @_;

    if ($site) {
        chomp($site = "?site=$site");
    }

    my $cookie = "";
    my $rc = 0;

    # If htaccess is setup or REQUIRE_AUTHENTICATION is not 1, then we don't
    # check for an authorized user.
    if (($ENV{'REMOTE_USER'} ne "") || ($ENV{'REQUIRE_AUTHENTICATION'} != 1)) {
        log_msg("Skipping authentication.");
        return;
    }

    # We need to get the user's password hash from the system in order to
    # verify the cookie, so the suid root wsm_cmd_exec program will do that.
    $cookie = qx(/usr/es/sbin/cluster/wsm/cgi-bin/wsm_cmd_exec -c);

    $rc = $? >> 8;
    if ($rc == 2) {
        # Return of 2 means that the user needs to re-login because
        # the cookie timed out or because the cookie didn't exist.
        log_msg("\n\twsm_auth.pm: print_authorized_page_header():\n" .
                "\tThe return code from wsm_cmd_exec was $rc.\n"   .
                "\tThe cookie timed out, or didn't exist. Logging out.");
        print <<LOGOUT;
Content-type: text/html

<BODY>
<SCRIPT TYPE="text/javascript">
top.location.href = '/cgi-bin/wsm_login.cgi$site';
</SCRIPT>
</BODY>
</HTML>
LOGOUT
        exit 1;

    } elsif ($rc == 0) {

        # Return of 0 means that the user's cookie was invalid.
        log_msg("\n\twsm_auth.pm: print_authorized_page_header():\n" .
                "\tThe return code from wsm_cmd_exec was $rc.\n"   .
                "\tThe user's cookie was invalid. Logging out.");
        print <<LOGOUT;
Content-type: text/html

<BODY>
<SCRIPT TYPE="text/javascript">
top.location.href = '/cgi-bin/wsm_login.cgi$site';
</SCRIPT>
</BODY>
</HTML>
LOGOUT
        exit 1;

    } elsif ($rc == 1) {
        # If we got here, then the user's cookie was valid, and we have
        # a new cookie with a new timestamp that we need to set.
        # We need 2 newlines here to end the HTTP headers.
        log_msg("\n\twsm_auth.pm: print_authorized_page_header():\n" .
                "\tSuccessful authentication. User access is authorized.");
        print "Set-Cookie: $cookie; path=/\n";
        return;
    }

    exit 1;
} # End of "print_authorized_page_header()"


##############################################################################
# Function: process_login
#
# Description:
# This function will process the login form post data, extract the username
# and password from the data, and check if the username and password is valid.
# If valid, the user will be redirected to the index page and a session
# cookie will be set.
# If invalid, the user will be given an access denied page.
#
# Arguments: None
#
# Returns: Exits 1 if invalid login.
#          Exits 0 if valid login.
##############################################################################
sub process_login (;$)
{
    my($site) = @_;
    chomp $site;

    my ($formdata, $user, $passwd, $cookie) = ("", "", "", "");
    my $rc = 0;

    # Read in the form data from the login page, it uses the POST method.
    read(STDIN, $formdata, $ENV{'CONTENT_LENGTH'});
    $formdata =~ m/user=(.*)&passwd=(.*)/;
    $user = $1;
    $passwd = $2;

    # Undo HTTP encoding.
    $user =~ tr/+/ /;
    $user =~ s/%0D%0A/\n/g;
    $user =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    if (defined($ENV{VERBOSE_LOGGING}) and
        $ENV{VERBOSE_LOGGING} eq "high")
    {
        print STDERR "wsm_auth.pm: process_login(): processing user \"$user\".\n";
    }

    # Undo HTTP encoding.
    $passwd =~ tr/+/ /;
    $passwd =~ s/%0D%0A/\n/g;
    $passwd =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

    # We need to get the user's password hash from the system in order to
    # verify the login and make the new cookie, so the suid root wsm_cmd_exec
    # program will do that.
    $cookie = qx(/usr/es/sbin/cluster/wsm/cgi-bin/wsm_cmd_exec -v -u '$user' -p '$passwd');
    $rc = $? >> 8;
    if (!$rc) {
        log_msg("Couldn't validate user $user.\n");
        access_denied();
        exit 1;
    } elsif ($rc == 1) {
        # If we got here, then the user's login was valid, and we have a 
        # new cookie with a new timestamp that we need to set.
        if (defined($ENV{VERBOSE_LOGGING}) and
            $ENV{VERBOSE_LOGGING} eq "high")
        {
            print STDERR "wsm_auth.pm: process_login(): establishing session cookies.\n";
        }
        print "Set-Cookie: $cookie; path=/\n";

        # Give the user a unique, persistent ID number to use for sessions
        my $curTime = time();
        print "Set-Cookie: wsm_id=$curTime; path=/\n";

        # Perform some garbage cleanup on the session files
        if (defined($ENV{VERBOSE_LOGGING}) and
            $ENV{VERBOSE_LOGGING} eq "high")
        {
            print STDERR "wsm_auth.pm: process_login(): cleaning session files.\n";
        }
        system("/usr/es/sbin/cluster/wsm/cgi-bin/wsm_session -clean");

        my $type = "http";
        if ($ENV{'HTTPS'} =~ /on/i) {
            $type = "https";
        }
        my ($host, $port) = $ENV{'HTTP_HOST'} =~ /([\w.]+)(:\d+)?/;
        if (! defined $port) {
            if (defined $ENV{'SERVER_PORT'}) {
                $port = ':' . $ENV{'SERVER_PORT'};
            } else {
                $port = '';
            }
        }

        my ($shost) = $ENV{'SERVER_NAME'} =~ /([\w.]+)(:\d+)?/;
        if ($host ne $shost) {
            log_msg("
\tWARNING: the hostname being used (\"$host\") does not
\t         match the name defined in the HTTP server's configuration file
\t         (\"$shost\"). This is not necessarily a problem,
\t         and is mentioned here for informational purposes only.
");
        }
        
        ## We need two newlines here to end the HTTP headers.
        ## print "Location: $type://$host:$port/cgi-bin/wsm_index.cgi\n\n";

        # Load the appropriate page; the specified site (if any),
        # or the main, WebSMIT application by default.
        if ($site) {
            if (defined($ENV{VERBOSE_LOGGING}) and
                $ENV{VERBOSE_LOGGING} eq "high")
            {
                print STDERR "wsm_auth.pm: process_login(): a site has been detected. Redirecting to '$type://$host$port/$site'.\n";
            }
            log_msg("Redirecting to '$type://$host$port/$site'");
            print "Location: $type://$host$port/$site\n\n";
        } else {
            if (defined($ENV{VERBOSE_LOGGING}) and
                $ENV{VERBOSE_LOGGING} eq "high")
            {
                print STDERR "wsm_auth.pm: process_login(): no site was detected. Redirecting to '$type://$host$port/index.html'.\n";
            }
            log_msg("Redirecting to '$type://$host$port/index.html'");
            print "Location: $type://$host$port/index.html\n\n";
        }

        if (defined($ENV{VERBOSE_LOGGING}) and
            $ENV{VERBOSE_LOGGING} eq "high")
        {
            print STDERR "wsm_auth.pm: process_login(): exiting with a value of 0 (zero).\n";
        }
        exit 0;
    }

    if (defined($ENV{VERBOSE_LOGGING}) and
        $ENV{VERBOSE_LOGGING} eq "high")
    {
        print STDERR "wsm_auth.pm: process_login(): exiting with a value of 1.\n";
    }
    exit 1;

} # End of "process_login()"


###############################################################################
# Function: log_msg
#
# Description:
# Prints out a log message to standard error.  The format of the log
# message is:
# TIMESTAMP: PROGNAME: Remote Address IPADDR: MSG
#
# Arguments:  msg : Message string.
#
# Returns:  None
###############################################################################
sub log_msg ( $ )
{
    my $msg = shift;
    my $time = localtime;
    my $addr = $ENV{'REMOTE_ADDR'};
    logger("$time: wsm_auth_run: Remote Address $addr: $msg");
} # End of "log_msg()"

# vim:set ft=perl:
