# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
# 52haes_r541 src/43haes/usr/sbin/cluster/wsm/utils/wsm_config.pm 1.8 
#  
# Licensed Materials - Property of IBM 
#  
# COPYRIGHT International Business Machines Corp. 2006,2007 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

# @(#)97       1.8 src/43haes/usr/sbin/cluster/wsm/utils/wsm_config.pm, hacmp.wsm, 52haes_r541, 0747B_hacmp541 11/15/07 11:10:26

package wsm_config;

use strict;
use warnings;

use Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(get_config load_config get_simulator_mode get_odmdir);

my $master_config = '/usr/es/sbin/cluster/wsm/wsm_smit.conf';
my $default_odm_dir = '/etc/objrepos';
my $simulator_odm_dir = '/var/hacmp/simulator/objrepos';
my %CFG;


#
# Function: get_config
#
# Description:
#
#       Returns a hash containing the configuration values in the master config
#       file.  Lines beginning with '#' and empty lines are ignored.
#
# Call using "my $CFG = wsm_config::get_config('<CONFIG_FILE>');".
# Access using "$$CFG{<VAR_NAME>};".
#
sub get_config {
    # Initialize the hash
    %CFG = ();
    my $file = shift;
    unless (defined $file && $file) {
        $file = $master_config;
    }
    unless (-f $file) {
        warn "No such config file '$file'";
        return {};  # return empty hash
    }

    #============================================
    # Load the system-wide default NLS settings
    #============================================
    if (open ENVFILE, '<', '/etc/environment') {
        my @sysdef = grep(!/^\s*#/, <ENVFILE>);
        close ENVFILE;

        my @nlspath = grep(/^\s*NLPATH=/, @sysdef);
        $CFG{NLSPATH} = $nlspath[$#nlspath] if (@nlspath);
        $CFG{NLSPATH} = '/usr/lib/nls/msg/%L/%N:/usr/lib/nls/msg/%L/%N.cat' if (! $CFG{NLSPATH});
        $CFG{NLSPATH} = "$CFG{NLSPATH}:" . '/usr/es/lib/nls/msg/%L/%N:/usr/es/lib/nls/msg/%L/%N.cat'; # For Linux

        my @lang = grep(/^\s*LANG=/, @sysdef);
        if (@lang) {
            chomp(my ($var, $val) = split(/=/, $lang[$#lang]));
            $CFG{LANG} = $val;
        }
        my @lc_all = grep(/^\s*LC_ALL=/, @sysdef);
        if (@lc_all) {
            chomp(my ($var, $val) = split(/=/, $lang[$#lc_all]));
            $CFG{LC_ALL} = $val;
        }

        if (! $CFG{LC_ALL} and ! $CFG{LANG}) {
            $CFG{LC_ALL} = 'en_US';
            $CFG{LANG} = 'en_US';
        } elsif(! $CFG{LANG}) {
            $CFG{LANG} = $CFG{LC_ALL};
        } elsif(! $CFG{LC_ALL}) {
            $CFG{LC_ALL} = $CFG{LANG};
        }
    }

    #=====================================================================
    # Load the specified file, overriding any competing default settings
    #=====================================================================
    if (open CFILE, '<', $file) {
        while (<CFILE>) {
            next if (/^\s*#/);    # Skip all commented lines
            next if ( !/^\w+=/ ); # Only process assignments; skip code

            (my $name, my $val) = split /=/;
            next if ($val =~ /\$/); # Avoid calculated assignments

            $name =~ s/^\s+//;  # Eliminate any leading whitespace
            $val =~ s/^["']//;  # Eliminate leading quotes
            $val =~ s/['"]$//;  # Eliminate trailing quotes
            $val =~ s/\s+$//;   # Eliminate trailing whitespace/newline

            if ($name eq 'NLSPATH' and exists($CFG{NLSPATH})) {
                $CFG{NLSPATH} = "$CFG{NLSPATH}:$val";
            } else {
                $CFG{$name} = $val;
            }
        }
        close CFILE;
    }

    return \%CFG;  # Return reference to main hash
} # End of "get_config()"


#
# Function: load_config
#
# Description:
#
#       Load the config from the given file into the current environment
#
#
sub load_config {
    get_config(shift);
    for my $k (keys %CFG) { $ENV{$k} = $CFG{$k}; } 
}


############################################################################
#
# Function: set_wsm_locale
#
# Description:
#
#       This function detects the current language setting in the browser
#       that is accessing SUIT. If it is determined to be an available and
#       valid language, then it is used, and any global settings in the
#       "wsm_smit.conf" file are overidden. If it is *not* a usable language,
#       then an appropriate error message is provided.
#
#       This function serves the secondary purpose of validating the global
#       LANG setting, found in "wsm_smit.conf". If it is determined to be
#       non-English *and* does not work, then the English strings are used,
#       and an appropriate error message is displayed.
#
#       NOTE: currently, this function is hard-coded to check for a specific
#             message in a specific catalog to determine the validity of the
#             LANG setting. A future enhancement might be to make that the
#             default behavior, but allow an override.
#
#       NOTE: since this function can be invoked *before* the configuration
#             file has been loaded, it is necessary to determine if that is
#             the case. Otherwise, if NLSPATH has not been set, this function
#             will always fail, and will always bark a bogus warning message
#             into the "wsm_smit.log" file.
#
# Arguments:
#
#       None.
#
# Expected environment:
#
#       Running in an HTTP server.
#
# Side-effects:
#
#       If necessary, sets LANG.
#
############################################################################
sub set_wsm_locale
{
    if ($ENV{HTTP_ACCEPT_LANGUAGE} and $ENV{ACCEPTED_USERS} and $ENV{NLSPATH}) {

        #================================================================
        # Determine the candidate language, as requested by the browser
        #================================================================
        my $HAL = lc($ENV{HTTP_ACCEPT_LANGUAGE});
        my($LCAND) = split(/,/, $HAL);
        if ($LCAND =~ /-/) {
            my($LG, $LC) = split(/-/, $LCAND);
            $LC = uc($LC);
            $LCAND = "${LG}_${LC}";
        } elsif ($LCAND =~ /_/) {
            my($LG, $LC) = split(/_/, $LCAND);
            $LC = uc($LC);
            $LCAND = "${LG}_${LC}";
        } elsif ($LCAND eq 'en') {
            $LCAND = 'en_US';
            if ($HAL =~ /gb/) {
                $LCAND = 'en_GB';
            }
        }

        if ($LCAND eq 'c') {
            $LCAND = 'en_US';
            if ($HAL =~ /gb/) {
                $LCAND = 'en_GB';
            }
        } elsif ($LCAND eq "ja") { $LCAND = 'Ja_JP ja_JP'; }
          elsif ($LCAND eq "ko") { $LCAND = 'ko_KR'; }
          elsif ($LCAND eq "pt") { $LCAND = 'pt_BR'; }
          elsif ($LCAND eq "zh") { $LCAND = 'zh_TW'; };

        #==========================================================
        # Determine if the message catalogs for this language are
        # installed. If not, then force the use of en_US, and log
        # an appropriate message.
        #==========================================================
        my $ORIG_LANG = "$ENV{LANG}";
        my $ORIG_LC_ALL = "$ENV{LC_ALL}";
        foreach (split(/\s+/, $LCAND)) {
            my $LCAND = $_;

            $ENV{LANG} = $LCAND;
            $ENV{LC_ALL} = $LCAND;
            my $MSG = `dspmsg -s 40 cluster.cat 59 "LANG_NOT_INSTALLED"`;
            if ($MSG eq 'LANG_NOT_INSTALLED') {
                $ENV{LANG} = $ORIG_LANG;
                $ENV{LC_ALL} = $ORIG_LC_ALL;
                print STDERR <<EOD;
WARNING: a WebSMIT access from $ENV{REMOTE_ADDR} was received that requested the
         "$LCAND" language. That language does not appear to be available
         on this server, so "en_US" was used instead.
EOD
            } else {
                last;
            }
        }
    }

    #==========================================================
    # Do what you can to ensure that the LANG value will work
    #==========================================================
    if (! $ENV{LANG}) {  # If no LANG, use en_US as the default
        $ENV{LANG} = 'en_US';
        $ENV{LC_ALL} = 'en_US';
    } elsif ($ENV{LANG} ne 'en_US' and $ENV{ACCEPTED_USERS} and $ENV{NLSPATH}) {
        #==========================================================
        # Determine if the message catalogs for this language are
        # installed. If not, then force the use of en_US, and log
        # an appropriate message.
        #==========================================================
        my $MSG = `dspmsg -s 40 cluster.cat 59 "LANG_NOT_INSTALLED"`;
        if ($MSG eq 'LANG_NOT_INSTALLED') {
            print STDERR <<EOD;
WARNING: a WebSMIT access from $ENV{REMOTE_ADDR} was received that requested the
         "$ENV{LANG}" language. That language does not appear to be available
         on this server, so "en_US" was used instead.
EOD
            $ENV{LANG} = 'en_US';
            $ENV{LC_ALL} = 'en_US';
        }
    }

} # End of "set_wsm_locale()"


#
# Function: get_simulator_mode
#
# Description:
#
#       Load config and return SIMULATE_MODE
#
#
sub get_simulator_mode {
    get_config(shift);

    return 1 if $CFG{'SIMULATE_MODE'} && $CFG{'SIMULATE_MODE'} eq 1;

    return 0;
}


#
# Function: get_odmdir
#
# Description:
#
#       Load config and return the simulator ODM dir, or default ODM
#       depending on if wsm is running in simulator mode or not
#
#
sub get_odmdir {
    return $simulator_odm_dir if get_simulator_mode() == 1;
    return $default_odm_dir;
}

1;
